@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2022 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -94,21 +94,22 @@ static void init_data_memory(uint32_t dim, const dnnl_dim_t *dims,
         dnnl_format_tag_t user_tag, dnnl_engine_t engine, float *data,
         dnnl_memory_t *memory) {
     dnnl_memory_desc_t user_md;
-    CHECK(dnnl_memory_desc_init_by_tag(
+    CHECK(dnnl_memory_desc_create_with_tag(
             &user_md, dim, dims, dnnl_f32, user_tag));
-    CHECK(dnnl_memory_create(memory, &user_md, engine, DNNL_MEMORY_ALLOCATE));
+    CHECK(dnnl_memory_create(memory, user_md, engine, DNNL_MEMORY_ALLOCATE));
+    CHECK(dnnl_memory_desc_destroy(user_md));
     write_to_dnnl_memory(data, *memory);
 }
 
 dnnl_status_t prepare_reorder(dnnl_memory_t *user_memory, // in
-        const dnnl_memory_desc_t *prim_memory_md, // in
+        const_dnnl_memory_desc_t prim_memory_md, // in
         dnnl_engine_t prim_engine, // in: primitive's engine
         int dir_is_user_to_prim, // in: user -> prim or prim -> user
         dnnl_memory_t *prim_memory, // out: primitive's memory created
         dnnl_primitive_t *reorder, // out: reorder primitive created
         uint32_t *net_index, // primitive index in net (inc if reorder created)
         dnnl_primitive_t *net, args_t *net_args) { // net params
-    const dnnl_memory_desc_t *user_memory_md;
+    const_dnnl_memory_desc_t user_memory_md;
     dnnl_memory_get_memory_desc(*user_memory, &user_memory_md);
 
     dnnl_engine_t user_mem_engine;
@@ -155,14 +156,17 @@ void simple_net() {
     dnnl_primitive_t net_fwd[10], net_bwd[10];
     args_t net_fwd_args[10], net_bwd_args[10];
 
-    dnnl_dim_t net_src_sizes[4] = {BATCH, IC, CONV_IH, CONV_IW};
-    dnnl_dim_t net_dst_sizes[4] = {BATCH, OC, POOL_OH, POOL_OW};
+    const int ndims = 4;
+    dnnl_dims_t net_src_sizes = {BATCH, IC, CONV_IH, CONV_IW};
+    dnnl_dims_t net_dst_sizes = {BATCH, OC, POOL_OH, POOL_OW};
 
-    float *net_src = (float *)malloc(product(net_src_sizes, 4) * sizeof(float));
-    float *net_dst = (float *)malloc(product(net_dst_sizes, 4) * sizeof(float));
+    float *net_src
+            = (float *)malloc(product(net_src_sizes, ndims) * sizeof(float));
+    float *net_dst
+            = (float *)malloc(product(net_dst_sizes, ndims) * sizeof(float));
 
-    init_net_data(net_src, 4, net_src_sizes);
-    memset(net_dst, 0, product(net_dst_sizes, 4) * sizeof(float));
+    init_net_data(net_src, ndims, net_src_sizes);
+    memset(net_dst, 0, product(net_dst_sizes, ndims) * sizeof(float));
 
     //----------------------------------------------------------------------
     //----------------- Forward Stream -------------------------------------
@@ -170,28 +174,31 @@ void simple_net() {
     // {BATCH, IC, CONV_IH, CONV_IW} (x) {OC, IC, 11, 11} ->
     // {BATCH, OC, CONV_OH, CONV_OW}
     // strides: {CONV_STRIDE, CONV_STRIDE}
-    dnnl_dim_t *conv_user_src_sizes = net_src_sizes;
-    dnnl_dim_t conv_user_weights_sizes[4] = {OC, IC, 11, 11};
-    dnnl_dim_t conv_bias_sizes[4] = {OC};
-    dnnl_dim_t conv_user_dst_sizes[4] = {BATCH, OC, CONV_OH, CONV_OW};
-    dnnl_dim_t conv_strides[2] = {CONV_STRIDE, CONV_STRIDE};
-    dnnl_dim_t conv_padding[2] = {CONV_PAD, CONV_PAD};
+    dnnl_dims_t conv_user_src_sizes;
+    for (int i = 0; i < ndims; i++)
+        conv_user_src_sizes[i] = net_src_sizes[i];
+    dnnl_dims_t conv_user_weights_sizes = {OC, IC, 11, 11};
+    dnnl_dims_t conv_bias_sizes = {OC};
+    dnnl_dims_t conv_user_dst_sizes = {BATCH, OC, CONV_OH, CONV_OW};
+    dnnl_dims_t conv_strides = {CONV_STRIDE, CONV_STRIDE};
+    dnnl_dims_t conv_dilation = {0, 0};
+    dnnl_dims_t conv_padding = {CONV_PAD, CONV_PAD};
 
     float *conv_src = net_src;
     float *conv_weights = (float *)malloc(
-            product(conv_user_weights_sizes, 4) * sizeof(float));
+            product(conv_user_weights_sizes, ndims) * sizeof(float));
     float *conv_bias
             = (float *)malloc(product(conv_bias_sizes, 1) * sizeof(float));
 
-    init_net_data(conv_weights, 4, conv_user_weights_sizes);
+    init_net_data(conv_weights, ndims, conv_user_weights_sizes);
     init_net_data(conv_bias, 1, conv_bias_sizes);
 
     // create memory for user data
     dnnl_memory_t conv_user_src_memory, conv_user_weights_memory,
             conv_user_bias_memory;
-    init_data_memory(4, conv_user_src_sizes, dnnl_nchw, engine, conv_src,
+    init_data_memory(ndims, conv_user_src_sizes, dnnl_nchw, engine, conv_src,
             &conv_user_src_memory);
-    init_data_memory(4, conv_user_weights_sizes, dnnl_oihw, engine,
+    init_data_memory(ndims, conv_user_weights_sizes, dnnl_oihw, engine,
             conv_weights, &conv_user_weights_memory);
     init_data_memory(1, conv_bias_sizes, dnnl_x, engine, conv_bias,
             &conv_user_bias_memory);
@@ -203,30 +210,31 @@ void simple_net() {
         // create data descriptors for convolution w/ no specified format
         dnnl_memory_desc_t conv_src_md, conv_weights_md, conv_bias_md,
                 conv_dst_md;
-        CHECK(dnnl_memory_desc_init_by_tag(&conv_src_md, 4, conv_user_src_sizes,
-                dnnl_f32, dnnl_format_tag_any));
-        CHECK(dnnl_memory_desc_init_by_tag(&conv_weights_md, 4,
+        CHECK(dnnl_memory_desc_create_with_tag(&conv_src_md, ndims,
+                conv_user_src_sizes, dnnl_f32, dnnl_format_tag_any));
+        CHECK(dnnl_memory_desc_create_with_tag(&conv_weights_md, ndims,
                 conv_user_weights_sizes, dnnl_f32, dnnl_format_tag_any));
-        CHECK(dnnl_memory_desc_init_by_tag(
+        CHECK(dnnl_memory_desc_create_with_tag(
                 &conv_bias_md, 1, conv_bias_sizes, dnnl_f32, dnnl_x));
-        CHECK(dnnl_memory_desc_init_by_tag(&conv_dst_md, 4, conv_user_dst_sizes,
-                dnnl_f32, dnnl_format_tag_any));
+        CHECK(dnnl_memory_desc_create_with_tag(&conv_dst_md, ndims,
+                conv_user_dst_sizes, dnnl_f32, dnnl_format_tag_any));
 
-        dnnl_convolution_desc_t conv_any_desc;
-        CHECK(dnnl_convolution_forward_desc_init(&conv_any_desc, dnnl_forward,
-                dnnl_convolution_direct, &conv_src_md, &conv_weights_md,
-                &conv_bias_md, &conv_dst_md, conv_strides, conv_padding,
-                conv_padding));
+        CHECK(dnnl_convolution_forward_primitive_desc_create(&conv_pd, engine,
+                dnnl_forward, dnnl_convolution_direct, conv_src_md,
+                conv_weights_md, conv_bias_md, conv_dst_md, conv_strides,
+                conv_dilation, conv_padding, conv_padding, NULL));
 
-        CHECK(dnnl_primitive_desc_create(
-                &conv_pd, &conv_any_desc, NULL, engine, NULL));
+        CHECK(dnnl_memory_desc_destroy(conv_src_md));
+        CHECK(dnnl_memory_desc_destroy(conv_weights_md));
+        CHECK(dnnl_memory_desc_destroy(conv_bias_md));
+        CHECK(dnnl_memory_desc_destroy(conv_dst_md));
     }
 
     dnnl_memory_t conv_internal_src_memory, conv_internal_weights_memory,
             conv_internal_dst_memory;
 
     // create memory for dst data, we don't need to reorder it to user data
-    const dnnl_memory_desc_t *conv_dst_md
+    const_dnnl_memory_desc_t conv_dst_md
             = dnnl_primitive_desc_query_md(conv_pd, dnnl_query_dst_md, 0);
     CHECK(dnnl_memory_create(&conv_internal_dst_memory, conv_dst_md, engine,
             DNNL_MEMORY_ALLOCATE));
@@ -235,13 +243,13 @@ void simple_net() {
     // if required
     dnnl_primitive_t conv_reorder_src, conv_reorder_weights;
 
-    const dnnl_memory_desc_t *conv_src_md
+    const_dnnl_memory_desc_t conv_src_md
             = dnnl_primitive_desc_query_md(conv_pd, dnnl_query_src_md, 0);
     CHECK(prepare_reorder(&conv_user_src_memory, conv_src_md, engine, 1,
             &conv_internal_src_memory, &conv_reorder_src, &n_fwd, net_fwd,
             net_fwd_args));
 
-    const dnnl_memory_desc_t *conv_weights_md
+    const_dnnl_memory_desc_t conv_weights_md
             = dnnl_primitive_desc_query_md(conv_pd, dnnl_query_weights_md, 0);
     CHECK(prepare_reorder(&conv_user_weights_memory, conv_weights_md, engine, 1,
             &conv_internal_weights_memory, &conv_reorder_weights, &n_fwd,
@@ -274,20 +282,17 @@ void simple_net() {
 
     // keep memory format of source same as the format of convolution
     // output in order to avoid reorder
-    const dnnl_memory_desc_t *relu_src_md = conv_dst_md;
+    const_dnnl_memory_desc_t relu_src_md = conv_dst_md;
+    const_dnnl_memory_desc_t relu_dst_md = relu_src_md;
 
     // create a relu primitive descriptor
-    dnnl_eltwise_desc_t relu_desc;
-    CHECK(dnnl_eltwise_forward_desc_init(&relu_desc, dnnl_forward,
-            dnnl_eltwise_relu, relu_src_md, negative_slope, 0));
-
     dnnl_primitive_desc_t relu_pd;
-    CHECK(dnnl_primitive_desc_create(&relu_pd, &relu_desc, NULL, engine, NULL));
+    CHECK(dnnl_eltwise_forward_primitive_desc_create(&relu_pd, engine,
+            dnnl_forward, dnnl_eltwise_relu, relu_src_md, relu_dst_md,
+            negative_slope, 0, NULL));
 
     // create relu dst memory
     dnnl_memory_t relu_dst_memory;
-    const dnnl_memory_desc_t *relu_dst_md
-            = dnnl_primitive_desc_query_md(relu_pd, dnnl_query_dst_md, 0);
     CHECK(dnnl_memory_create(
             &relu_dst_memory, relu_dst_md, engine, DNNL_MEMORY_ALLOCATE));
 
@@ -314,27 +319,24 @@ void simple_net() {
 
     // create lrn src memory descriptor using dst memory descriptor
     //  from previous primitive
-    const dnnl_memory_desc_t *lrn_src_md = relu_dst_md;
+    const_dnnl_memory_desc_t lrn_src_md = relu_dst_md;
+    const_dnnl_memory_desc_t lrn_dst_md = lrn_src_md;
 
     // create a lrn primitive descriptor
-    dnnl_lrn_desc_t lrn_desc;
-    CHECK(dnnl_lrn_forward_desc_init(&lrn_desc, dnnl_forward,
-            dnnl_lrn_across_channels, lrn_src_md, local_size, alpha, beta, k));
-
     dnnl_primitive_desc_t lrn_pd;
-    CHECK(dnnl_primitive_desc_create(&lrn_pd, &lrn_desc, NULL, engine, NULL));
+    CHECK(dnnl_lrn_forward_primitive_desc_create(&lrn_pd, engine, dnnl_forward,
+            dnnl_lrn_across_channels, lrn_src_md, lrn_dst_md, local_size, alpha,
+            beta, k, NULL));
 
     // create primitives for lrn dst and workspace memory
     dnnl_memory_t lrn_dst_memory, lrn_ws_memory;
 
-    const dnnl_memory_desc_t *lrn_dst_md
-            = dnnl_primitive_desc_query_md(lrn_pd, dnnl_query_dst_md, 0);
     CHECK(dnnl_memory_create(
             &lrn_dst_memory, lrn_dst_md, engine, DNNL_MEMORY_ALLOCATE));
 
     // create workspace only in training and only for forward primitive
     // query lrn_pd for workspace, this memory will be shared with forward lrn
-    const dnnl_memory_desc_t *lrn_ws_md
+    const_dnnl_memory_desc_t lrn_ws_md
             = dnnl_primitive_desc_query_md(lrn_pd, dnnl_query_workspace_md, 0);
     CHECK(dnnl_memory_create(
             &lrn_ws_memory, lrn_ws_md, engine, DNNL_MEMORY_ALLOCATE));
@@ -354,11 +356,13 @@ void simple_net() {
     // kernel: {3, 3}
     // strides: {POOL_STRIDE, POOL_STRIDE}
     // dilation: {0, 0}
-    dnnl_dim_t *pool_dst_sizes = net_dst_sizes;
-    dnnl_dim_t pool_kernel[2] = {3, 3};
-    dnnl_dim_t pool_strides[2] = {POOL_STRIDE, POOL_STRIDE};
-    dnnl_dim_t pool_padding[2] = {POOL_PAD, POOL_PAD};
-    dnnl_dim_t pool_dilation[2] = {0, 0};
+    dnnl_dims_t pool_dst_sizes;
+    for (int i = 0; i < ndims; i++)
+        pool_dst_sizes[i] = net_dst_sizes[i];
+    dnnl_dims_t pool_kernel = {3, 3};
+    dnnl_dims_t pool_strides = {POOL_STRIDE, POOL_STRIDE};
+    dnnl_dims_t pool_padding = {POOL_PAD, POOL_PAD};
+    dnnl_dims_t pool_dilation = {0, 0};
 
     // create memory for user dst data
     dnnl_memory_t pool_user_dst_memory;
@@ -371,25 +375,23 @@ void simple_net() {
     {
         // create pooling src memory descriptor using dst descriptor
         //  from previous primitive
-        const dnnl_memory_desc_t *pool_src_md = lrn_dst_md;
+        const_dnnl_memory_desc_t pool_src_md = lrn_dst_md;
 
         // create descriptors for dst pooling data
         dnnl_memory_desc_t pool_dst_md;
-        CHECK(dnnl_memory_desc_init_by_tag(&pool_dst_md, 4, pool_dst_sizes,
+        CHECK(dnnl_memory_desc_create_with_tag(&pool_dst_md, 4, pool_dst_sizes,
                 dnnl_f32, dnnl_format_tag_any));
 
-        dnnl_pooling_v2_desc_t pool_desc;
-        CHECK(dnnl_pooling_v2_forward_desc_init(&pool_desc, dnnl_forward,
-                dnnl_pooling_max, pool_src_md, &pool_dst_md, pool_strides,
-                pool_kernel, pool_dilation, pool_padding, pool_padding));
-
-        CHECK(dnnl_primitive_desc_create(
-                &pool_pd, &pool_desc, NULL, engine, NULL));
+        CHECK(dnnl_pooling_forward_primitive_desc_create(&pool_pd, engine,
+                dnnl_forward, dnnl_pooling_max, pool_src_md, pool_dst_md,
+                pool_strides, pool_kernel, pool_dilation, pool_padding,
+                pool_padding, NULL));
+        CHECK(dnnl_memory_desc_destroy(pool_dst_md));
     }
 
     // create memory for workspace
     dnnl_memory_t pool_ws_memory;
-    const dnnl_memory_desc_t *pool_ws_md
+    const_dnnl_memory_desc_t pool_ws_md
             = dnnl_primitive_desc_query_md(pool_pd, dnnl_query_workspace_md, 0);
     CHECK(dnnl_memory_create(
             &pool_ws_memory, pool_ws_md, engine, DNNL_MEMORY_ALLOCATE));
@@ -398,7 +400,7 @@ void simple_net() {
     // if required
     dnnl_primitive_t pool_reorder_dst;
     dnnl_memory_t pool_internal_dst_memory;
-    const dnnl_memory_desc_t *pool_dst_md
+    const_dnnl_memory_desc_t pool_dst_md
             = dnnl_primitive_desc_query_md(pool_pd, dnnl_query_dst_md, 0);
     n_fwd += 1; // tentative workaround: preserve space for pooling that should
             // happen before the reorder
@@ -440,21 +442,17 @@ void simple_net() {
 
     // Pooling Backward
     // pooling diff src memory descriptor
-    const dnnl_memory_desc_t *pool_diff_src_md = lrn_dst_md;
+    const_dnnl_memory_desc_t pool_diff_src_md = lrn_dst_md;
 
     // pooling diff dst memory descriptor
-    const dnnl_memory_desc_t *pool_diff_dst_md = pool_dst_md;
-
-    // create backward pooling descriptor
-    dnnl_pooling_v2_desc_t pool_bwd_desc;
-    CHECK(dnnl_pooling_v2_backward_desc_init(&pool_bwd_desc, dnnl_pooling_max,
-            pool_diff_src_md, pool_diff_dst_md, pool_strides, pool_kernel,
-            pool_dilation, pool_padding, pool_padding));
+    const_dnnl_memory_desc_t pool_diff_dst_md = pool_dst_md;
 
     // backward primitive descriptor needs to hint forward descriptor
     dnnl_primitive_desc_t pool_bwd_pd;
-    CHECK(dnnl_primitive_desc_create(
-            &pool_bwd_pd, &pool_bwd_desc, NULL, engine, pool_pd));
+    CHECK(dnnl_pooling_backward_primitive_desc_create(&pool_bwd_pd, engine,
+            dnnl_pooling_max, pool_diff_src_md, pool_diff_dst_md, pool_strides,
+            pool_kernel, pool_dilation, pool_padding, pool_padding, pool_pd,
+            NULL));
 
     // create reorder primitive between user diff dst and pool diff dst
     // if required
@@ -486,21 +484,17 @@ void simple_net() {
     n_bwd++;
 
     // Backward lrn
-    const dnnl_memory_desc_t *lrn_diff_dst_md = pool_diff_src_md;
+    const_dnnl_memory_desc_t lrn_diff_dst_md = pool_diff_src_md;
+    const_dnnl_memory_desc_t lrn_diff_src_md = lrn_diff_dst_md;
 
     // create backward lrn descriptor
-    dnnl_lrn_desc_t lrn_bwd_desc;
-    CHECK(dnnl_lrn_backward_desc_init(&lrn_bwd_desc, dnnl_lrn_across_channels,
-            lrn_diff_dst_md, lrn_src_md, local_size, alpha, beta, k));
-
     dnnl_primitive_desc_t lrn_bwd_pd;
-    CHECK(dnnl_primitive_desc_create(
-            &lrn_bwd_pd, &lrn_bwd_desc, NULL, engine, lrn_pd));
+    CHECK(dnnl_lrn_backward_primitive_desc_create(&lrn_bwd_pd, engine,
+            dnnl_lrn_across_channels, lrn_diff_src_md, lrn_diff_dst_md,
+            lrn_src_md, local_size, alpha, beta, k, lrn_pd, NULL));
 
     // create memory for lrn diff src
     dnnl_memory_t lrn_diff_src_memory;
-    const dnnl_memory_desc_t *lrn_diff_src_md = dnnl_primitive_desc_query_md(
-            lrn_bwd_pd, dnnl_query_diff_src_md, 0);
     CHECK(dnnl_memory_create(&lrn_diff_src_memory, lrn_diff_src_md, engine,
             DNNL_MEMORY_ALLOCATE));
 
@@ -518,21 +512,17 @@ void simple_net() {
     n_bwd++;
 
     // Backward relu
-    const dnnl_memory_desc_t *relu_diff_dst_md = lrn_diff_src_md;
+    const_dnnl_memory_desc_t relu_diff_src_md = lrn_diff_src_md;
+    const_dnnl_memory_desc_t relu_diff_dst_md = lrn_diff_src_md;
 
     // create backward relu descriptor
-    dnnl_eltwise_desc_t relu_bwd_desc;
-    CHECK(dnnl_eltwise_backward_desc_init(&relu_bwd_desc, dnnl_eltwise_relu,
-            relu_diff_dst_md, relu_src_md, negative_slope, 0));
-
     dnnl_primitive_desc_t relu_bwd_pd;
-    CHECK(dnnl_primitive_desc_create(
-            &relu_bwd_pd, &relu_bwd_desc, NULL, engine, relu_pd));
+    CHECK(dnnl_eltwise_backward_primitive_desc_create(&relu_bwd_pd, engine,
+            dnnl_eltwise_relu, relu_diff_src_md, relu_diff_dst_md, relu_src_md,
+            negative_slope, 0, relu_pd, NULL));
 
     // create memory for relu diff src
     dnnl_memory_t relu_diff_src_memory;
-    const dnnl_memory_desc_t *relu_diff_src_md = dnnl_primitive_desc_query_md(
-            relu_bwd_pd, dnnl_query_diff_src_md, 0);
     CHECK(dnnl_memory_create(&relu_diff_src_memory, relu_diff_src_md, engine,
             DNNL_MEMORY_ALLOCATE));
 
@@ -569,24 +559,26 @@ void simple_net() {
         // weights to chose the format it prefers for best performance
         dnnl_memory_desc_t conv_diff_src_md, conv_diff_weights_md,
                 conv_diff_bias_md, conv_diff_dst_md;
-        CHECK(dnnl_memory_desc_init_by_tag(&conv_diff_src_md, 4,
+        CHECK(dnnl_memory_desc_create_with_tag(&conv_diff_src_md, 4,
                 conv_user_src_sizes, dnnl_f32, dnnl_format_tag_any));
-        CHECK(dnnl_memory_desc_init_by_tag(&conv_diff_weights_md, 4,
+        CHECK(dnnl_memory_desc_create_with_tag(&conv_diff_weights_md, 4,
                 conv_user_weights_sizes, dnnl_f32, dnnl_format_tag_any));
-        CHECK(dnnl_memory_desc_init_by_tag(
+        CHECK(dnnl_memory_desc_create_with_tag(
                 &conv_diff_bias_md, 1, conv_bias_sizes, dnnl_f32, dnnl_x));
-        CHECK(dnnl_memory_desc_init_by_tag(&conv_diff_dst_md, 4,
+        CHECK(dnnl_memory_desc_create_with_tag(&conv_diff_dst_md, 4,
                 conv_user_dst_sizes, dnnl_f32, dnnl_format_tag_any));
 
         // create backward convolution descriptor
-        dnnl_convolution_desc_t conv_bwd_weights_desc;
-        CHECK(dnnl_convolution_backward_weights_desc_init(
-                &conv_bwd_weights_desc, dnnl_convolution_direct,
-                &conv_diff_src_md, &conv_diff_weights_md, &conv_diff_bias_md,
-                &conv_diff_dst_md, conv_strides, conv_padding, conv_padding));
+        CHECK(dnnl_convolution_backward_weights_primitive_desc_create(
+                &conv_bwd_weights_pd, engine, dnnl_convolution_direct,
+                conv_diff_src_md, conv_diff_weights_md, conv_diff_bias_md,
+                conv_diff_dst_md, conv_strides, conv_dilation, conv_padding,
+                conv_padding, conv_pd, NULL));
 
-        CHECK(dnnl_primitive_desc_create(&conv_bwd_weights_pd,
-                &conv_bwd_weights_desc, NULL, engine, conv_pd));
+        CHECK(dnnl_memory_desc_destroy(conv_diff_src_md));
+        CHECK(dnnl_memory_desc_destroy(conv_diff_weights_md));
+        CHECK(dnnl_memory_desc_destroy(conv_diff_bias_md));
+        CHECK(dnnl_memory_desc_destroy(conv_diff_dst_md));
     }
 
     // for best performance convolution backward might chose
@@ -597,7 +589,7 @@ void simple_net() {
     // format chosen by backward convolution
     dnnl_primitive_t conv_bwd_reorder_src;
     dnnl_memory_t conv_bwd_internal_src_memory;
-    const dnnl_memory_desc_t *conv_diff_src_md = dnnl_primitive_desc_query_md(
+    const_dnnl_memory_desc_t conv_diff_src_md = dnnl_primitive_desc_query_md(
             conv_bwd_weights_pd, dnnl_query_src_md, 0);
     CHECK(prepare_reorder(&conv_src_memory, conv_diff_src_md, engine, 1,
             &conv_bwd_internal_src_memory, &conv_bwd_reorder_src, &n_bwd,
@@ -611,7 +603,7 @@ void simple_net() {
     // and format preferred by conv_diff_weights
     dnnl_primitive_t conv_reorder_diff_dst;
     dnnl_memory_t conv_internal_diff_dst_memory;
-    const dnnl_memory_desc_t *conv_diff_dst_md = dnnl_primitive_desc_query_md(
+    const_dnnl_memory_desc_t conv_diff_dst_md = dnnl_primitive_desc_query_md(
             conv_bwd_weights_pd, dnnl_query_diff_dst_md, 0);
 
     CHECK(prepare_reorder(&relu_diff_src_memory, conv_diff_dst_md, engine, 1,
@@ -625,7 +617,7 @@ void simple_net() {
     // create reorder primitives for conv diff weights memory
     dnnl_primitive_t conv_reorder_diff_weights;
     dnnl_memory_t conv_internal_diff_weights_memory;
-    const dnnl_memory_desc_t *conv_diff_weights_md
+    const_dnnl_memory_desc_t conv_diff_weights_md
             = dnnl_primitive_desc_query_md(
                     conv_bwd_weights_pd, dnnl_query_diff_weights_md, 0);
     n_bwd += 1; // tentative workaround: preserve space for conv_bwd_weights
@@ -642,7 +634,7 @@ void simple_net() {
 
     // create memory for diff bias memory
     dnnl_memory_t conv_diff_bias_memory;
-    const dnnl_memory_desc_t *conv_diff_bias_md = dnnl_primitive_desc_query_md(
+    const_dnnl_memory_desc_t conv_diff_bias_md = dnnl_primitive_desc_query_md(
             conv_bwd_weights_pd, dnnl_query_diff_weights_md, 1);
     CHECK(dnnl_memory_create(&conv_diff_bias_memory, conv_diff_bias_md, engine,
             DNNL_MEMORY_ALLOCATE));
