@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@
 ///
 /// > Example code: @ref performance_profiling.cpp
 ///
-/// This example uses [DNNL_VERBOSE](@ref dev_guide_verbose) trace output
+/// This example uses [ONEDNN_VERBOSE](@ref dev_guide_verbose) trace output
 /// to tune oneDNN code to align
 /// with the [best practices](@ref dev_guide_inference).
 ///
@@ -66,10 +66,10 @@
 /// ./program.exe [cpu|gpu] [implementation]
 /// ~~~
 ///
-/// Before you run the program, set your `DNNL_VERBOSE` environment
-/// variable to 1:
+/// Before you run the program, set your `ONEDNN_VERBOSE` environment
+/// variable to `profile_exec`:
 /// ~~~sh
-/// export DNNL_VERBOSE=1
+/// export ONEDNN_VERBOSE=profile_exec
 /// ~~~
 ///
 /// The program starts by creating oneDNN memory objects in **NCHW**
@@ -118,9 +118,9 @@ void create_and_execute_relu(memory &data, engine &eng, stream &s) {
     // relu operates on whatever data format is given to it
 
     // create a primitive
-    auto relu_d = eltwise_forward::desc(prop_kind::forward_inference,
-            algorithm::eltwise_relu, data.get_desc(), 0.f, 0.f);
-    auto relu_pd = eltwise_forward::primitive_desc(relu_d, eng);
+    auto relu_pd = eltwise_forward::primitive_desc(eng,
+            prop_kind::forward_inference, algorithm::eltwise_relu,
+            data.get_desc(), data.get_desc(), 0.f, 0.f);
     auto relu = eltwise_forward(relu_pd);
 
     // execute it (in-place)
@@ -132,7 +132,7 @@ void create_and_execute_relu(memory &data, engine &eng, stream &s) {
 primitive_attr create_attr_with_relu_post_op() {
     // create a post-op with relu
     post_ops ops;
-    ops.append_eltwise(1.f, algorithm::eltwise_relu, 0.f, 0.f);
+    ops.append_eltwise(algorithm::eltwise_relu, 0.f, 0.f);
 
     // create an attribute and set the corresponding post op
     primitive_attr attr;
@@ -157,7 +157,7 @@ void conv_relu_naive(const memory &user_src, const memory &user_wei,
     /// First it sets the dimensions and format for convolution memory
     /// descriptors (`_md`) to match `user_` values--one `md` each for source,
     /// destination, and weight data. Then it uses those `md` to create the
-    /// convolution descriptor `conv_d`, which tells oneDNN to use
+    /// convolution primitive descriptor `conv_pd`, which tells oneDNN to use
     /// plain format (NCHW) for the convolution.
     /// @page performance_profiling_cpp
     /// @snippet performance_profiling.cpp Create mem_desc
@@ -168,13 +168,6 @@ void conv_relu_naive(const memory &user_src, const memory &user_wei,
     auto conv_dst_md = memory::desc(user_dst.get_desc());
     // [Create mem_desc]
     /// @page performance_profiling_cpp
-    /// @snippet performance_profiling.cpp Create conv_desc
-    // [Create conv_desc]
-    // create a convolution descriptor
-    auto conv_d = convolution_forward::desc(prop_kind::forward_inference,
-            algorithm::convolution_direct, conv_src_md, conv_wei_md,
-            conv_dst_md, strides, padding, padding);
-    // [Create conv_desc]
     /// Next the program creates a convolution primitive descriptor `conv_pd`
     /// and convolution primitive `conv`. These structs will inherit
     /// NCHW format from `md` by way of the `conv_d`. Finally it creates
@@ -184,7 +177,9 @@ void conv_relu_naive(const memory &user_src, const memory &user_wei,
     /// @snippet performance_profiling.cpp Create conv_prim_desc
     // [Create conv_prim_desc]
     // create a convolution primitive descriptor
-    auto conv_pd = convolution_forward::primitive_desc(conv_d, eng);
+    auto conv_pd = convolution_forward::primitive_desc(eng,
+            prop_kind::forward_inference, algorithm::convolution_direct,
+            conv_src_md, conv_wei_md, conv_dst_md, strides, padding, padding);
     // [Create conv_prim_desc]
     /// @page performance_profiling_cpp
     /// @snippet performance_profiling.cpp Create conv_primitive
@@ -214,14 +209,14 @@ void conv_relu_naive(const memory &user_src, const memory &user_wei,
     /// the input data format is at the time it is called.
     ///
     /// Using NCHW data format may result in suboptimal performance for compute
-    /// intensive primitives, as shown in the following DNNL_VERBOSE output
+    /// intensive primitives, as shown in the following ONEDNN_VERBOSE output
     /// by the convolution and relu execution
     /// times of 38.3 and 2.9 milliseconds, respectively.
     ///
-    /// *DNNL_VERBOSE output (see configuration notice\*):*
+    /// *ONEDNN_VERBOSE output (see configuration notice\*):*
     /// ~~~sh
-    /// dnnl_verbose,exec,cpu,convolution,gemm:jit,forward_inference,src_f32::blocked:abcd:f0 wei_f32::blocked:abcd:f0 bia_undef::undef::f0 dst_f32::blocked:abcd:f0,,alg:convolution_direct,mb128_ic3oc96_ih227oh55kh11sh4dh0ph0_iw227ow55kw11sw4dw0pw0,38.314
-    /// dnnl_verbose,exec,cpu,eltwise,jit:avx512_common,forward_inference,data_f32::blocked:abcd:f0 diff_undef::undef::f0,,alg:eltwise_relu alpha:0 beta:0,128x96x55x55,2.87695
+    /// onednn_verbose,v0,exec,cpu,convolution,gemm:jit,forward_inference,src_f32::blocked:abcd:f0 wei_f32::blocked:abcd:f0 bia_undef::undef::f0 dst_f32::blocked:abcd:f0,,alg:convolution_direct,mb128_ic3oc96_ih227oh55kh11sh4dh0ph0_iw227ow55kw11sw4dw0pw0,38.314
+    /// onednn_verbose,v0,exec,cpu,eltwise,jit:avx512_common,forward_inference,data_f32::blocked:abcd:f0 diff_undef::undef::f0,,alg:eltwise_relu alpha:0 beta:0,128x96x55x55,2.87695
     /// ~~~
     /// In *Blocked format implementation*, we will incorporate the best
     /// practice of letting oneDNN determine the optimal format
@@ -243,30 +238,22 @@ void conv_relu_blocked(memory user_src, memory user_wei, memory user_dst,
     ///
     /// First it creates the md as in **naive implementation**. Next it changes
     /// the dnnl::memory::format_tag for each md to `ANY`. Then it uses those
-    /// md to create the convolution descriptor conv_d, which tells oneDNN
-    /// to use whatever format it recommends for the convolution.
+    /// md to create the convolution primitive  descriptor conv_pd, which tells
+    /// oneDNN to use whatever format it recommends for the convolution.
     /// oneDNN will choose a friendly blocked format.
     /// @page performance_profiling_cpp
     /// @snippet performance_profiling.cpp Create mem_desc with tag=any
     // [Create mem_desc with tag=any]
-    // copy the dimensions and format from user's memory
-    auto conv_src_md = memory::desc(user_src.get_desc());
-    auto conv_wei_md = memory::desc(user_wei.get_desc());
-    auto conv_dst_md = memory::desc(user_dst.get_desc());
-
-    // reset format to "any" to allow convolution to pick the best implementation
-    conv_src_md.data.format_kind = dnnl_format_kind_any;
-    conv_wei_md.data.format_kind = dnnl_format_kind_any;
-    conv_dst_md.data.format_kind = dnnl_format_kind_any;
+    // copy the dimensions and data type from user's memory and set format tag
+    // to "any" to allow convolution to pick the best implementation
+    auto conv_src_md = memory::desc(user_src.get_desc().get_dims(),
+            user_src.get_desc().get_data_type(), memory::format_tag::any);
+    auto conv_wei_md = memory::desc(user_wei.get_desc().get_dims(),
+            user_wei.get_desc().get_data_type(), memory::format_tag::any);
+    auto conv_dst_md = memory::desc(user_dst.get_desc().get_dims(),
+            user_dst.get_desc().get_data_type(), memory::format_tag::any);
     // [Create mem_desc with tag=any]
-    /// @page performance_profiling_cpp
-    /// @snippet performance_profiling.cpp Create conv_desc implementation2
-    // [Create conv_desc implementation2]
-    // create a convolution descriptor
-    auto conv_d = convolution_forward::desc(prop_kind::forward_inference,
-            algorithm::convolution_direct, conv_src_md, conv_wei_md,
-            conv_dst_md, strides, padding, padding);
-    // [Create conv_desc implementation2]
+
     /// Next the program creates a convolution primitive descriptor conv_pd and
     /// convolution primitive conv as in naive implementation.
     /// However, in this implementation the structs will inherit blocked format
@@ -275,7 +262,9 @@ void conv_relu_blocked(memory user_src, memory user_wei, memory user_dst,
     /// @snippet performance_profiling.cpp Create conv_prim_desc implementation2
     // [Create conv_prim_desc implementation2]
     // create a convolution primitive descriptor and primitive
-    auto conv_pd = convolution_forward::primitive_desc(conv_d, eng);
+    auto conv_pd = convolution_forward::primitive_desc(eng,
+            prop_kind::forward_inference, algorithm::convolution_direct,
+            conv_src_md, conv_wei_md, conv_dst_md, strides, padding, padding);
     // [Create conv_prim_desc implementation2]
     /// Since the resulting convolution primitive will expect
     /// blocked source data, conditional reorders are inserted to convert
@@ -339,7 +328,7 @@ void conv_relu_blocked(memory user_src, memory user_wei, memory user_dst,
     /// @page performance_profiling_cpp
     /// Blocked memory format is recommended for oneDNN primitive
     /// execution and provides better performance, as shown in the
-    /// DNNL_VERBOSE output by the convolution and relu execution times of
+    /// ONEDNN_VERBOSE output by the convolution and relu execution times of
     /// 18.3 and 2.7 milliseconds (down from 38.3 and 2.9 in
     /// *naive implementation*), respectively.
     /// In this implementation, there is an additional reorder operation that
@@ -350,16 +339,16 @@ void conv_relu_blocked(memory user_src, memory user_wei, memory user_dst,
     /// and one at the end of the chain, and only pay the reorder penalty at
     /// those points in the execution.
     ///
-    /// *DNNL_VERBOSE output (see configuration notice\*):*
+    /// *ONEDNN_VERBOSE output (see configuration notice\*):*
     /// ~~~sh
-    /// dnnl_verbose,exec,cpu,reorder,jit:uni,undef,src_f32::blocked:abcd:f0 dst_f32::blocked:Acdb16a:f0,,,96x3x11x11,0.0310059
-    /// dnnl_verbose,exec,cpu,convolution,jit:avx512_common,forward_inference,src_f32::blocked:abcd:f0 wei_f32::blocked:Acdb16a:f0 bia_undef::undef::f0 dst_f32::blocked:aBcd16b:f0,,alg:convolution_direct,mb128_ic3oc96_ih227oh55kh11sh4dh0ph0_iw227ow55kw11sw4dw0pw0,18.3101
-    /// dnnl_verbose,exec,cpu,eltwise,jit:avx512_common,forward_inference,data_f32::blocked:aBcd16b:f0 diff_undef::undef::f0,,alg:eltwise_relu alpha:0 beta:0,128x96x55x55,2.66895
-    /// dnnl_verbose,exec,cpu,reorder,jit:uni,undef,src_f32::blocked:aBcd16b:f0 dst_f32::blocked:abcd:f0,,,128x96x55x55,4.80396
+    /// onednn_verbose,v0,exec,cpu,reorder,jit:uni,undef,src_f32::blocked:abcd:f0 dst_f32::blocked:Acdb16a:f0,,,96x3x11x11,0.0310059
+    /// onednn_verbose,v0,exec,cpu,convolution,jit:avx512_common,forward_inference,src_f32::blocked:abcd:f0 wei_f32::blocked:Acdb16a:f0 bia_undef::undef::f0 dst_f32::blocked:aBcd16b:f0,,alg:convolution_direct,mb128_ic3oc96_ih227oh55kh11sh4dh0ph0_iw227ow55kw11sw4dw0pw0,18.3101
+    /// onednn_verbose,v0,exec,cpu,eltwise,jit:avx512_common,forward_inference,data_f32::blocked:aBcd16b:f0 diff_undef::undef::f0,,alg:eltwise_relu alpha:0 beta:0,128x96x55x55,2.66895
+    /// onednn_verbose,v0,exec,cpu,reorder,jit:uni,undef,src_f32::blocked:aBcd16b:f0 dst_f32::blocked:abcd:f0,,,128x96x55x55,4.80396
     /// ~~~
     /// This inference implementation is closer to best practices than
     /// *naive implementation* because it uses oneDNN recommended memory
-    /// format. *fused implementation* will futher optimize the performance by
+    /// format. *fused implementation* will further optimize the performance by
     /// fusing convolution with ReLU using oneDNN
     /// [post-ops](@ref dev_guide_attributes_post_ops).
     // reorder data to the user's format if needed.
@@ -379,23 +368,18 @@ void conv_relu_fused(memory user_src, memory user_wei, memory user_dst,
     /// `conv_relu_fused()`.
     /// @page performance_profiling_cpp
     ///
-    /// First the memory descriptors and convolution descriptor are created as
-    /// in *naive implementation*.
-    // copy the dimensions and format from user's memory
-    auto conv_src_md = memory::desc(user_src.get_desc());
-    auto conv_wei_md = memory::desc(user_wei.get_desc());
-    auto conv_dst_md = memory::desc(user_dst.get_desc());
+    /// First the memory descriptors and convolution primitive descriptor are
+    /// created as in *naive implementation*.
+    // copy the dimensions data type from user's memory and set format tag
+    // to any to allow convolution to pick the best implementation
+    auto conv_src_md = memory::desc(user_src.get_desc().get_dims(),
+            user_src.get_desc().get_data_type(), memory::format_tag::any);
+    auto conv_wei_md = memory::desc(user_wei.get_desc().get_dims(),
+            user_wei.get_desc().get_data_type(), memory::format_tag::any);
+    auto conv_dst_md = memory::desc(user_dst.get_desc().get_dims(),
+            user_dst.get_desc().get_data_type(), memory::format_tag::any);
 
-    // reset format to any to allow convolution to pick the best implementation
-    conv_src_md.data.format_kind = dnnl_format_kind_any;
-    conv_wei_md.data.format_kind = dnnl_format_kind_any;
-    conv_dst_md.data.format_kind = dnnl_format_kind_any;
-
-    // create a convolution descriptor
-    auto conv_d = convolution_forward::desc(prop_kind::forward_inference,
-            algorithm::convolution_direct, conv_src_md, conv_wei_md,
-            conv_dst_md, strides, padding, padding);
-    /// Then in preparation for the convolution prim desctiptor, a ReLU post-op
+    /// Then in preparation for the convolution prim descriptor, a ReLU post-op
     /// is built and added to the primitive attribute `attr`:
     /// @page performance_profiling_cpp
     /// @snippet performance_profiling.cpp Create post_op attr with relu
@@ -409,7 +393,10 @@ void conv_relu_fused(memory user_src, memory user_wei, memory user_dst,
     auto attr = create_attr_with_relu_post_op();
 
     // create a convolution primitive descriptor
-    auto conv_pd = convolution_forward::primitive_desc(conv_d, attr, eng);
+    auto conv_pd = convolution_forward::primitive_desc(eng,
+            prop_kind::forward_inference, algorithm::convolution_direct,
+            conv_src_md, conv_wei_md, conv_dst_md, strides, padding, padding,
+            attr);
     // [Create prim_desc with attr]
     /// Then conditional reorders are applied as in *blocked format
     /// implementation* to convert `user_` format NCHW to blocked. Finally, it
@@ -465,11 +452,11 @@ void conv_relu_fused(memory user_src, memory user_wei, memory user_dst,
     /// The consequence to following best practices can be seen in the execution
     /// time of the fused primitive of 18.0 milliseconds.
     ///
-    /// *DNNL_VERBOSE output (see configuration notice\*):*
+    /// *ONEDNN_VERBOSE output (see configuration notice\*):*
     /// ~~~sh
-    /// dnnl_verbose,exec,cpu,reorder,jit:uni,undef,src_f32::blocked:abcd:f0 dst_f32::blocked:Acdb16a:f0,,,96x3x11x11,0.0148926
-    /// dnnl_verbose,exec,cpu,convolution,jit:avx512_common,forward_inference,src_f32::blocked:abcd:f0 wei_f32::blocked:Acdb16a:f0 bia_undef::undef::f0 dst_f32::blocked:aBcd16b:f0,post_ops:'eltwise_relu;';,alg:convolution_direct,mb128_ic3oc96_ih227oh55kh11sh4dh0ph0_iw227ow55kw11sw4dw0pw0,17.968
-    /// dnnl_verbose,exec,cpu,reorder,jit:uni,undef,src_f32::blocked:aBcd16b:f0 dst_f32::blocked:abcd:f0,,,128x96x55x55,4.66797
+    /// onednn_verbose,v0,exec,cpu,reorder,jit:uni,undef,src_f32::blocked:abcd:f0 dst_f32::blocked:Acdb16a:f0,,,96x3x11x11,0.0148926
+    /// onednn_verbose,v0,exec,cpu,convolution,jit:avx512_common,forward_inference,src_f32::blocked:abcd:f0 wei_f32::blocked:Acdb16a:f0 bia_undef::undef::f0 dst_f32::blocked:aBcd16b:f0,post_ops:'eltwise_relu;';,alg:convolution_direct,mb128_ic3oc96_ih227oh55kh11sh4dh0ph0_iw227ow55kw11sw4dw0pw0,17.968
+    /// onednn_verbose,v0,exec,cpu,reorder,jit:uni,undef,src_f32::blocked:aBcd16b:f0 dst_f32::blocked:abcd:f0,,,128x96x55x55,4.66797
     /// ~~~
 }
 
