@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
 *******************************************************************************/
 
 #include "common/memory_storage.hpp"
+#include "common/engine.hpp"
 #include "common/memory.hpp"
 
 #include <assert.h>
@@ -34,5 +35,15 @@ status_t memory_storage_t::init(unsigned flags, size_t size, void *handle) {
     return set_data_handle(handle);
 }
 
+memory_storage_t::memory_storage_t(
+        engine_t *engine, const memory_storage_t *root_storage)
+    : engine_(engine), root_storage_(root_storage) {
+    if (engine_) engine_->retain();
+}
+
+memory_storage_t::~memory_storage_t() {
+    if (engine_) engine_->release();
+};
+
 } // namespace impl
 } // namespace dnnl
