@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@
 
 #include <atomic>
 #include <cassert>
+#include <climits>
 #include <cmath>
 #include <cstddef>
 #include <cstdint>
@@ -66,10 +67,27 @@ namespace impl {
 // Sanity check for 64 bits
 static_assert(sizeof(void *) == 8, "oneDNN supports 64-bit architectures only");
 
+// Note: if `f` has any explicit templated arguments, e.g., func<A, B>, then
+// compiler returns `error: macro "CHECK" passed 2 arguments, but takes just 1`.
+// The solution is to use an alias, e.g. `using func_alias = func<A, B>;` and
+// use `func_alias` in CHECK, then it compiles.
 #define CHECK(f) \
     do { \
-        status_t _status_ = f; \
-        if (_status_ != status::success) return _status_; \
+        dnnl::impl::status_t _status_ = f; \
+        if (_status_ != dnnl::impl::status::success) return _status_; \
+    } while (0)
+
+#define CHECK_BOOL(f) \
+    do { \
+        dnnl::impl::status_t _status_ = f; \
+        if (_status_ != dnnl::impl::status::success) return false; \
+    } while (0)
+
+#define UNUSED_STATUS(f) \
+    do { \
+        dnnl::impl::status_t _status_ = f; \
+        assert(_status_ == dnnl::impl::status::success); \
+        MAYBE_UNUSED(_status_); \
     } while (0)
 
 #define IMPLICATION(cause, effect) (!(cause) || !!(effect))
@@ -159,16 +177,16 @@ inline typename remove_reference<T>::type zero() {
 }
 
 template <typename T, typename... Args>
-std::unique_ptr<T> make_unique(Args &&... args) {
+std::unique_ptr<T> make_unique(Args &&...args) {
     return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
 }
 
 template <typename T, typename P>
-inline bool everyone_is(T val, P item) {
+constexpr bool everyone_is(T val, P item) {
     return val == item;
 }
 template <typename T, typename P, typename... Args>
-inline bool everyone_is(T val, P item, Args... item_others) {
+constexpr bool everyone_is(T val, P item, Args... item_others) {
     return val == item && everyone_is(val, item_others...);
 }
 
@@ -191,15 +209,34 @@ constexpr P map(T pat, P def, T item, P ival, Args... item_others) {
 }
 
 template <typename... Args>
-inline bool any_null(Args... ptrs) {
+constexpr bool any_null(Args... ptrs) {
     return one_of(nullptr, ptrs...);
 }
 
+// For some unknown reason, GCC 11.x and beyond can't compile specific places
+// of the library that involve this routine. It's connected to the fact that
+// this function is inline and defined in a header.
+#if defined(__GNUC__) && __GNUC__ > 8 && !defined(__clang__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wrestrict"
+// /usr/include/bits/string_fortified.h:29:33: warning: ‘void* __builtin_memcpy(
+//     void*, const void*, long unsigned int)’ accessing 18446744056529682432 or
+//     more bytes at offsets 320 and 0 overlaps 9223372002495037441 bytes at
+//     offset -9223372019674906625 [-Wrestrict]
+#pragma GCC diagnostic ignored "-Wstringop-overflow"
+// warning: ‘void* __builtin_memcpy(void*, const void*, long unsigned int)’
+//     specified bound between 18446744056529682432 and 18446744073709551608
+//     exceeds maximum object size 9223372036854775807 [-Wstringop-overflow=]
+#endif
 template <typename T>
 inline void array_copy(T *dst, const T *src, size_t size) {
     for (size_t i = 0; i < size; ++i)
         dst[i] = src[i];
 }
+#if defined(__GNUC__) && __GNUC__ > 8 && !defined(__clang__)
+#pragma GCC diagnostic pop
+#endif
+
 template <typename T>
 inline bool array_cmp(const T *a1, const T *a2, size_t size) {
     for (size_t i = 0; i < size; ++i)
@@ -222,13 +259,13 @@ constexpr int product_impl(const T *arr, int2type<0>) {
 }
 
 template <typename T, size_t num>
-inline T product_impl(const T *arr, int2type<num>) {
+constexpr T product_impl(const T *arr, int2type<num>) {
     return arr[0] * product_impl(arr + 1, int2type<num - 1>());
 }
 } // namespace product_impl
 
 template <size_t num, typename T>
-inline T array_product(const T *arr) {
+constexpr T array_product(const T *arr) {
     return product_impl::product_impl(arr, product_impl::int2type<num - 1>());
 }
 
@@ -240,6 +277,11 @@ inline R array_product(const T *arr, size_t size) {
     return prod;
 }
 
+template <typename T, typename R = T>
+inline R array_product(const std::vector<T> &v) {
+    return array_product<T, R>(v.data(), v.size());
+}
+
 template <typename T, typename R = T>
 inline R array_min(const T *arr, size_t size) {
     R min = std::numeric_limits<R>::max();
@@ -284,7 +326,7 @@ inline void simultaneous_sort(
 }
 
 template <typename T>
-inline const T &saturate(const T &low, const T &upper, const T &a) {
+constexpr const T &saturate(const T &low, const T &upper, const T &a) {
     return nstl::max(low, nstl::min(upper, a));
 }
 
@@ -300,10 +342,29 @@ inline typename remove_reference<T>::type rnd_up(const T a, const U b) {
 }
 
 template <typename T, typename U>
-inline typename remove_reference<T>::type rnd_dn(const T a, const U b) {
+constexpr typename remove_reference<T>::type rnd_dn(const T a, const U b) {
     return static_cast<typename remove_reference<T>::type>((a / b) * b);
 }
 
+template <typename T>
+inline typename remove_reference<T>::type rnd_up_pow2(const T a) {
+    using R = typename remove_reference<T>::type;
+    if (a <= 0)
+        return static_cast<R>(1);
+    else {
+        T b = a - 1;
+        for (size_t v = 1; v < sizeof(T) * CHAR_BIT; v <<= 1)
+            b |= (b >> v);
+        return static_cast<R>(b + 1);
+    }
+}
+
+template <typename T>
+inline typename remove_reference<T>::type rnd_down_pow2(const T a) {
+    auto ret = rnd_up_pow2(a);
+    return ret == a ? ret : ret / 2;
+}
+
 template <typename T, typename U>
 inline typename remove_reference<T>::type max_div(const T a, const U b) {
     U div = b;
@@ -314,6 +375,11 @@ inline typename remove_reference<T>::type max_div(const T a, const U b) {
     return static_cast<typename remove_reference<T>::type>(div);
 }
 
+template <typename T>
+inline typename remove_reference<T>::type max_pow2_div(const T a) {
+    return static_cast<typename remove_reference<T>::type>(((a - 1) & ~a) + 1);
+}
+
 template <typename T>
 T *align_ptr(T *ptr, uintptr_t alignment) {
     return (T *)(((uintptr_t)ptr + alignment - 1) & ~(alignment - 1));
@@ -337,7 +403,7 @@ inline T nd_iterator_init(T start) {
     return start;
 }
 template <typename T, typename U, typename W, typename... Args>
-inline T nd_iterator_init(T start, U &x, const W &X, Args &&... tuple) {
+inline T nd_iterator_init(T start, U &x, const W &X, Args &&...tuple) {
     start = nd_iterator_init(start, utils::forward<Args>(tuple)...);
     x = start % X;
     return start / X;
@@ -347,7 +413,7 @@ inline bool nd_iterator_step() {
     return true;
 }
 template <typename U, typename W, typename... Args>
-inline bool nd_iterator_step(U &x, const W &X, Args &&... tuple) {
+inline bool nd_iterator_step(U &x, const W &X, Args &&...tuple) {
     if (nd_iterator_step(utils::forward<Args>(tuple)...)) {
         if (++x - X == 0) {
             x = 0;
@@ -373,7 +439,7 @@ inline bool nd_iterator_jump(U &cur, const U end, W &x, const Y &X) {
 }
 template <typename U, typename W, typename Y, typename... Args>
 inline bool nd_iterator_jump(
-        U &cur, const U end, W &x, const Y &X, Args &&... tuple) {
+        U &cur, const U end, W &x, const Y &X, Args &&...tuple) {
     if (nd_iterator_jump(cur, end, utils::forward<Args>(tuple)...)) {
         if (++x - X == 0) {
             x = 0;
@@ -384,11 +450,11 @@ inline bool nd_iterator_jump(
 }
 
 template <typename T>
-inline T pick(size_t i, const T &x0) {
+constexpr T pick(size_t i, const T &x0) {
     return x0;
 }
 template <typename T, typename... Args>
-inline T pick(size_t i, const T &x0, Args &&... args) {
+constexpr T pick(size_t i, const T &x0, Args &&...args) {
     return i == 0 ? x0 : pick(i - 1, utils::forward<Args>(args)...);
 }
 
@@ -417,32 +483,37 @@ struct array_offset_calculator {
     array_offset_calculator(Telem *base, Targs... Fargs) : _dims {Fargs...} {
         _base_ptr = base;
     }
+
     template <typename... Targs>
-    inline Telem &operator()(Targs... Fargs) {
+    array_offset_calculator(std::nullptr_t, Targs... Fargs) = delete;
+
+    template <typename... Targs>
+    inline Telem &operator()(Targs... Fargs) const {
+        assert(static_cast<bool>(_base_ptr));
         return *(_base_ptr + _offset(1, Fargs...));
     }
 
 private:
     template <typename... Targs>
-    inline size_t _offset(size_t const dimension, size_t element) {
+    inline size_t _offset(size_t const dimension, size_t element) const {
         return element;
     }
 
     template <typename... Targs>
     inline size_t _offset(
-            size_t const dimension, size_t theta, size_t element) {
+            size_t const dimension, size_t theta, size_t element) const {
         return element + (_dims[dimension] * theta);
     }
 
     template <typename... Targs>
     inline size_t _offset(size_t const dimension, size_t theta, size_t element,
-            Targs... Fargs) {
+            Targs... Fargs) const {
         size_t t_prime = element + (_dims[dimension] * theta);
         return _offset(dimension + 1, t_prime, Fargs...);
     }
 
     Telem *_base_ptr;
-    const int _dims[Tdims];
+    const dim_t _dims[Tdims];
 };
 
 template <typename derived_type, typename base_type>
@@ -467,7 +538,14 @@ const char *format_cvt_impl(T &&t) {
 
 template <typename... Args>
 std::string format_impl(const char *fmt, Args... args) {
+    // volatile here is a workaround for GCC 8 format-truncation warning e.g.:
+    // ‘%d’ directive output truncated writing 1 byte into a region of size 0
+    // triggered by overaggressive optmization in '-O3'; fixed in GCC 9+
+#if defined(__GNUC__) && __GNUC__ == 8 && !defined(__clang__)
+    volatile size_t sz = snprintf(nullptr, 0, fmt, args...);
+#else
     size_t sz = snprintf(nullptr, 0, fmt, args...);
+#endif
     std::string buf(sz + 1, '\0');
     snprintf(&buf[0], sz + 1, fmt, args...);
     buf.resize(sz);
@@ -475,10 +553,23 @@ std::string format_impl(const char *fmt, Args... args) {
 }
 
 template <typename... Args>
-std::string format(const char *fmt, Args &&... args) {
+std::string format(const char *fmt, Args &&...args) {
     return format_impl(fmt, format_cvt_impl(std::forward<Args>(args))...);
 }
 
+inline bool need_src_or_dst_check(
+        bool is_fwd, dim_t o, dim_t i, dim_t k, dim_t p, dim_t s, dim_t d) {
+    if (is_fwd) {
+        dim_t i_min = -p;
+        dim_t i_max = (o - 1) * s - p + (k - 1) * (1 + d);
+        return (i_min < 0) || (i_max >= i);
+    }
+    // Backward.
+    dim_t os_min = p - (k - 1) * (1 + d);
+    dim_t os_max = (i - 1) + p;
+    return (os_min < 0) || (os_max >= o * s);
+}
+
 // transforms @param l(ogical)_offset into a @param dims_pos based on input
 // dimensions @param dims and @param ndims.
 inline void l_dims_by_l_offset(
@@ -496,22 +587,34 @@ inline void l_dims_by_l_offset(
     }
 }
 
-inline int get_dims_mask(const dims_t dims1, const dims_t dims2, int ndims) {
+inline int get_dims_mask(const dims_t dims1, const dims_t dims2, int ndims,
+        bool skip_dim_of_one = false) {
     int mask = 0;
-    for (int d = 0; d < ndims; ++d)
-        mask += dims1[d] == dims2[d] ? (1 << d) : 0;
+    for (int d = 0; d < ndims; ++d) {
+        // Disable mask_bit for dimensions of `1` by request.
+        int mask_bit = skip_dim_of_one && dims1[d] == 1 ? 0 : (1 << d);
+        mask += dims1[d] == dims2[d] ? mask_bit : 0;
+    }
     return mask;
 };
 
-inline void copy_dims_with_mask(
-        dims_t ddims, const dims_t sdims, int ndims, int mask) {
+// The function can be used to get dimensions for memory descriptors or
+// dimensions for logical offset. First ones are happy to have ones when mask
+// is not applied. This allows to initialize them with existing functions using
+// tags/strides. Latter ones are not nappy with ones and must have zeros as
+// logical offsets starts with 0. `fill_with_one` flag regulates the behavior
+// between them.
+inline void copy_dims_with_mask(dims_t ddims, const dims_t sdims, int ndims,
+        int mask, bool fill_with_one = false) {
     for (int d = 0; d < ndims; ++d) {
-        ddims[d] = (mask & (1 << d)) ? sdims[d] : 0;
+        ddims[d] = (mask & (1 << d)) ? sdims[d]
+                                     : static_cast<dim_t>(fill_with_one);
     }
 }
 
-inline void apply_mask_on_dims(dims_t dims, int ndims, int mask) {
-    copy_dims_with_mask(dims, dims, ndims, mask);
+inline void apply_mask_on_dims(
+        dims_t dims, int ndims, int mask, bool fill_with_one = false) {
+    copy_dims_with_mask(dims, dims, ndims, mask, fill_with_one);
 }
 
 inline void dim_iterator(const dims_t dims, dims_t indices, int ndims) {
@@ -520,10 +623,22 @@ inline void dim_iterator(const dims_t dims, dims_t indices, int ndims) {
     }
 }
 
+template <typename T, size_t S>
+inline size_t array_size(T (&t)[S]) {
+    return S;
+}
+
+inline bool validate_dims(int ndims, const dims_t dims) {
+    for (int d = 0; d < ndims; ++d)
+        if (dims[d] <= 0) return false;
+    return true;
+}
+
 } // namespace utils
 
 int32_t fetch_and_add(int32_t *dst, int32_t val);
 inline void yield_thread() {}
+bool is_destroying_cache_safe();
 
 // Reads an environment variable 'name' and stores its string value in the
 // 'buffer' of 'buffer_size' bytes (including the terminating zero) on
@@ -549,14 +664,35 @@ inline void yield_thread() {}
 // retrieve the length of the environment variable value string.
 //
 int getenv(const char *name, char *buffer, int buffer_size);
-// Reads an integer from the environment
+// Reads an integer from the environment. For internal needs.
 int getenv_int(const char *name, int default_value = 0);
+// Reads an integer from user environment. Takes a var name without
+// prefix and checks both supported variants - with "ONEDNN_" (primary) and
+// "DNNL_" (secondary) prefixes.
+int getenv_int_user(const char *name, int default_value = 0);
+// Reads a string literal from user environment. Takes a var name without
+// prefix and checks both supported variants - with "ONEDNN_" (primary) and
+// "DNNL_" (secondary) prefixes.
+std::string getenv_string_user(const char *name);
+
+// Various getter for profiling info
 bool get_jit_dump();
 unsigned get_jit_profiling_flags();
 std::string get_jit_profiling_jitdumpdir();
+// Checks if the filepath is a valid path and not a symlink to ensure
+// the application only processes secure files.
+status_t check_for_symlinks(const char *filename, bool *res);
 FILE *fopen(const char *filename, const char *mode);
 int getpagesize();
 
+// return current library fpmath_mode
+fpmath_mode_t get_fpmath_mode();
+// checks if an fpmath_mode is valid
+status_t check_fpmath_mode(fpmath_mode_t mode);
+// Returns true if values reprensented by type sub_dt can all be
+// represented in dt. return false eotherwise
+bool is_fpsubtype(data_type_t sub_dt, data_type_t dt);
+
 constexpr int msan_enabled = MSAN_ENABLED;
 inline void msan_unpoison(void *ptr, size_t size) {
 #if MSAN_ENABLED
@@ -564,6 +700,15 @@ inline void msan_unpoison(void *ptr, size_t size) {
 #endif
 }
 
+// Helper to avoid #ifdefs for DNNL_DEV_MODE related code
+static constexpr bool is_dev_mode() {
+#ifdef DNNL_DEV_MODE
+    return true;
+#else
+    return false;
+#endif
+}
+
 // std::optional? std::maybe? std::whatever
 template <typename T>
 struct setting_t {
@@ -572,8 +717,8 @@ private:
     bool initialized_;
 
 public:
-    setting_t() : initialized_ {false} {}
-    setting_t(const T init) : value_ {init}, initialized_ {false} {}
+    constexpr setting_t() : value_ {}, initialized_ {false} {}
+    constexpr setting_t(const T init) : value_ {init}, initialized_ {false} {}
     bool initialized() { return initialized_; }
     T get() { return value_; }
     void set(T new_value) {
@@ -596,31 +741,9 @@ inline int float2int(float x) {
     return utils::bit_cast<int>(x);
 }
 
-// XXX: Currently SYCL doesn't provide an API to get device UUID but
-// we need to be able to distinguish OpenCL device from Level0 device.
-// As a temporary solution the compound ID will be used for that.
-// Below is a table explaning what the numbers are for different backends:
-//
-// -------------------------------------------------------------
-//  Backend      | Compound ID
-// -------------------------------------------------------------
-//  Host         | <backend_t::host, 0, 0>
-//  OpenCL       | <backend_t::opencl, cl_device, 0>
-//  NVIDIA       | <backend_t::nvidia, cuDevice, 0>
-//  Level0       | <backend_t::level0, uuid[0-63], uuid[64-127]>
-//  Pure CPU     | <0, 0, 0>
-//  Pure GPU     | <0, cl_device, 0>
-using device_id_t = std::tuple<int, uint64_t, uint64_t>;
-
-struct device_id_hash_t {
-    size_t operator()(const device_id_t &id) const {
-        size_t result = 0;
-        result = hash_combine(result, std::get<0>(id));
-        result = hash_combine(result, std::get<1>(id));
-        result = hash_combine(result, std::get<2>(id));
-        return result;
-    }
-};
+inline float int2float(int x) {
+    return utils::bit_cast<float>(x);
+}
 
 // A setting (basically a value) that can be set() multiple times until the
 // time first time the get() method is called. The set() method is expected to
@@ -629,42 +752,152 @@ struct device_id_hash_t {
 // get() method also has a 'soft' mode when the setting is not locked for
 // re-setting. This is used for testing purposes.
 template <typename T>
-struct set_before_first_get_setting_t {
+struct set_once_before_first_get_setting_t {
 private:
     T value_;
     std::atomic<unsigned> state_;
-    enum : unsigned { idle = 0, busy_setting = 1, locked_after_a_get = 2 };
+    enum : unsigned { idle = 0, busy_setting = 1, locked = 2 };
 
 public:
-    set_before_first_get_setting_t(T init) : value_ {init}, state_ {idle} {}
+    set_once_before_first_get_setting_t(T init)
+        : value_ {init}, state_ {idle} {}
 
     bool set(T new_value) {
-        if (state_.load() == locked_after_a_get) return false;
+        if (state_.load() == locked) return false;
 
         while (true) {
             unsigned expected = idle;
             if (state_.compare_exchange_weak(expected, busy_setting)) break;
-            if (expected == locked_after_a_get) return false;
+            if (expected == locked) return false;
         }
 
         value_ = new_value;
-        state_.store(idle);
+        state_.store(locked);
         return true;
     }
 
     T get(bool soft = false) {
-        if (!soft && state_.load() != locked_after_a_get) {
+        if (!soft && state_.load() != locked) {
             while (true) {
                 unsigned expected = idle;
-                if (state_.compare_exchange_weak(expected, locked_after_a_get))
-                    break;
-                if (expected == locked_after_a_get) break;
+                if (state_.compare_exchange_weak(expected, locked)) break;
+                if (expected == locked) break;
             }
         }
         return value_;
     }
 };
 
+inline bool is_native_runtime(runtime_kind_t kind) {
+    return utils::one_of(kind, runtime_kind::seq, runtime_kind::omp,
+            runtime_kind::tbb, runtime_kind::threadpool);
+}
+
+// Convenience wrapper to choose at compile-time between std::unique_ptr's
+// default deleter and a no-op one.
+//
+// This is useful for static pointers to objects with non-trivial destructors.
+// In some environments (e.g. tests where not all threads are joined at exit
+// time) these destructors can result in sanitizer failures (e.g. races in
+// thread sanitizer) when destructing unique_ptr's, but not with raw pointers.
+// Of course in a shared library environment using raw pointers (that are
+// therefore never freed) would result in memory leaks; this is why
+// DNNL_MAYBE_UNIQUE_PTR_IS_UNIQUE defaults to 1.
+#ifndef DNNL_MAYBE_UNIQUE_PTR_IS_UNIQUE
+#define DNNL_MAYBE_UNIQUE_PTR_IS_UNIQUE 1
+#endif
+
+#if DNNL_MAYBE_UNIQUE_PTR_IS_UNIQUE
+template <typename T>
+using maybe_unique_ptr = std::unique_ptr<T>;
+#else
+struct nop_deleter_t {
+    template <typename T>
+    void operator()(T const &) const noexcept {}
+};
+template <typename T>
+using maybe_unique_ptr = std::unique_ptr<T, nop_deleter_t>;
+#endif // DNNL_MAYBE_UNIQUE_PTR_IS_UNIQUE
+
+// Common abstraction to manipulate nibbles in memory as pairs
+struct nibble2_t {
+
+    // constructs a nibble pair from a pair of uint8_t values
+    nibble2_t(uint8_t low_, uint8_t high_) {
+        low = low_;
+        high = high_;
+    }
+
+    // constructs a nibble pairs from an uin8_t, taking its low and high part
+    nibble2_t(uint8_t pack_) {
+        low = pack_ & 0xf;
+        high = (pack_ >> 4) & 0xf;
+    }
+
+    // sets low (idx=0) or high (idx=1)  nibble.
+    inline void set(uint8_t val, int idx) {
+        switch (idx) {
+            case 0: low = val; return;
+            case 1: high = val; return;
+            default: assert(!"Out of range index"); return;
+        }
+    }
+
+    // returns low (idx = 0) or high (idx = 1) nibble in a uint8_t
+    inline uint8_t get(int idx) const {
+        switch (idx) {
+            case 0: return low;
+            case 1: return high;
+            default: assert(!"out of range index"); return 0;
+        }
+    }
+
+    // returns pair of nibbles as uint8t
+    inline uint8_t get() const { return static_cast<uint8_t>(high << 4 | low); }
+
+private:
+    uint8_t low : 4;
+    uint8_t high : 4;
+};
+static_assert(sizeof(nibble2_t) == 1, "nibble2_t must be 1 byte");
+
+/// Iterates through a binary integer
+/// usage:
+///
+/// for(int idx : mask_iterator(13)) { // 13 == 1101
+///     printf("%d\t", idx);
+/// }
+/// output: 0  2  3
+class mask_iterator {
+    int mask_;
+    int index_;
+
+public:
+    using iterator_category = std::input_iterator_tag;
+    using difference_type = int;
+    using value_type = int;
+    using pointer = value_type *;
+    using reference = value_type &;
+    mask_iterator() : mask_(0), index_(0) {}
+    mask_iterator(int mask) : mask_(mask), index_(0) {
+        if ((mask_ & 0x1) == 0) { ++(*this); }
+    }
+    mask_iterator &begin() { return *this; }
+    mask_iterator end() const { return 0; }
+    value_type operator*() const { return index_; }
+    mask_iterator &operator++() {
+        do {
+            index_++;
+            mask_ >>= 1;
+        } while ((mask_ & 0x1) == 0 && mask_ != 0);
+        if (mask_ == 0) { index_ = 0; }
+        return *this;
+    }
+    bool operator!=(const mask_iterator &other) const {
+        return mask_ != other.mask_ || index_ != other.index_;
+    }
+};
+
 } // namespace impl
 } // namespace dnnl
 
