@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -20,7 +20,9 @@
 
 #include "common/math_utils.hpp"
 
+#include "cpu/platform.hpp"
 #include "cpu/primitive_attr_postops.hpp"
+#include "cpu/ref_io_helper.hpp"
 #include "cpu/simple_q10n.hpp"
 
 #if DNNL_X64
@@ -37,28 +39,36 @@ namespace gemm_x8s8s32x_convolution_utils {
 template <typename dst_data_t>
 struct ref_pp_ker_t : pp_ker_t {
     ref_pp_ker_t(const convolution_pd_t *pd, const conv_gemm_conf_t &jcp)
-        : pp_ker_t(pd, jcp) {
-        if (jcp.with_eltwise || jcp.with_binary) {
-            ref_post_ops_.reset(new ref_post_ops_t(jcp.post_ops));
-        }
-    }
+        : pp_ker_t(pd, jcp), dst_md_(pd->dst_md()) {}
 
     using acc_data_t = pp_ker_t::acc_data_t;
 
     void operator()(void *dst, const acc_data_t *acc, const char *bias,
-            const float *scales, float sum_scale, float signed_scale, int g,
-            size_t start, size_t end, const zero_point_call_params_t &zp,
+            const float *scales, float dst_scale, float sum_scale,
+            float signed_scale, int g, size_t start, size_t end,
+            const zero_point_call_params_t &zp,
             const void *post_ops_binary_rhs_arg_vec, const void *dst_orig,
             const exec_ctx_t &ctx, const memory_desc_t &dst_md,
             const single_gemm_conv_chunk_desc_t &chunk_desc) const override;
 
+    status_t create_kernel() override {
+        if (this->jcp_.with_eltwise || this->jcp_.with_binary) {
+            ref_post_ops_
+                    = utils::make_unique<ref_post_ops_t>(this->jcp_.post_ops);
+            if (!ref_post_ops_) return status::out_of_memory;
+            return ref_post_ops_->init(dst_md_);
+        }
+        return status::success;
+    }
+
 private:
     std::unique_ptr<ref_post_ops_t> ref_post_ops_;
+    const memory_desc_t *dst_md_;
 };
 
 template <typename dst_data_t>
 void ref_pp_ker_t<dst_data_t>::operator()(void *void_dst, const acc_data_t *acc,
-        const char *bias, const float *scales, float sum_scale,
+        const char *bias, const float *scales, float dst_scale, float sum_scale,
         float signed_scale, int g, size_t start, size_t end,
         const zero_point_call_params_t &zp,
         const void * /* post_ops_binary_rhs_arg_vec */,
@@ -69,7 +79,6 @@ void ref_pp_ker_t<dst_data_t>::operator()(void *void_dst, const acc_data_t *acc,
     if (end <= start) return;
 
     assert(data_traits<dst_data_t>::data_type == jcp_.dst_data_type);
-    dst_data_t *dst = (dst_data_t *)void_dst;
 
     const lldiv_t dv_start = std::div((long long)start, (long long)jcp_.oc);
     const lldiv_t dv_end = std::div((long long)(end - 1), (long long)jcp_.oc);
@@ -101,20 +110,29 @@ void ref_pp_ker_t<dst_data_t>::operator()(void *void_dst, const acc_data_t *acc,
 
             if (jcp_.signed_input) data *= signed_scale;
 
-            if (jcp_.with_bias)
-                data += math::get_bias(
-                        bias, g * jcp_.oc + oc, jcp_.bias_data_type);
-
+            // dequantize data
             data *= scales[(g * jcp_.oc + oc) * jcp_.scale_idx_mult];
-            if (jcp_.with_sum) data += sum_scale * dst[dst_off];
+
+            if (jcp_.with_bias) {
+                const float b = io::load_float_value(
+                        jcp_.bias_data_type, bias, g * jcp_.oc + oc);
+                data += b;
+            }
+
+            if (jcp_.with_sum)
+                data += sum_scale
+                        * io::load_float_value(
+                                jcp_.sum_data_type, void_dst, dst_off);
             if (jcp_.with_eltwise || jcp_.with_binary) {
                 args.l_offset = (g * jcp_.oc + oc) * jcp_.os;
                 ref_post_ops_->execute(data, args);
             }
 
-            if (jcp_.zp.dst_exists) data += zp_dst_val;
+            // quantize data
+            if (jcp_.with_dst_scale) data *= dst_scale;
+            if (jcp_.zp.dst_exists) data += static_cast<float>(zp_dst_val);
 
-            dst[dst_off] = qz_a1b0<float, dst_data_t>()(data);
+            io::store_float_value(jcp_.dst_data_type, data, void_dst, dst_off);
         }
     }
 }
@@ -133,6 +151,7 @@ pp_ker_t *pp_ker_t::create(
 #endif
     switch (pd->dst_md()->data_type) {
         case data_type::f32: return new ref_pp_ker_t<float>(pd, jcp);
+        case data_type::bf16: return new ref_pp_ker_t<bfloat16_t>(pd, jcp);
         case data_type::s32: return new ref_pp_ker_t<int32_t>(pd, jcp);
         case data_type::s8: return new ref_pp_ker_t<int8_t>(pd, jcp);
         case data_type::u8: return new ref_pp_ker_t<uint8_t>(pd, jcp);
@@ -148,7 +167,7 @@ bool post_ops_ok(const post_ops_t &post_ops, const memory_desc_wrapper *dst_d) {
     return std::all_of(post_ops.entry_.cbegin(), post_ops.entry_.cend(),
             [](const dnnl_post_ops::entry_t &post_op) {
                 return post_op.is_eltwise() || post_op.is_sum()
-                        || post_op.is_binary();
+                        || post_op.is_binary() || post_op.is_prelu();
             });
 }
 
@@ -157,9 +176,9 @@ bool post_ops_ok(const post_ops_t &post_ops, const memory_desc_t *dst_d) {
     return post_ops_ok(post_ops, &dst_md);
 }
 
-bool mayiuse_jit_pp_kernel() noexcept {
+bool mayiuse_jit_pp_kernel(data_type_t dst_dt) noexcept {
 #if DNNL_X64
-    return x64::gemm_x8s8s32x_convolution_utils::mayiuse_jit_pp_kernel();
+    return x64::gemm_x8s8s32x_convolution_utils::mayiuse_jit_pp_kernel(dst_dt);
 #else
     return false;
 #endif
