@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2021 Intel Corporation
+* Copyright 2017-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -45,21 +45,38 @@ struct nchw_pooling_fwd_t : public primitive_t {
             const format_tag_t desired_fmt_tag = utils::pick(ndims() - 3,
                     format_tag::ncw, format_tag::nchw, format_tag::ncdhw);
 
-            const bool ok = is_fwd()
-                    && utils::one_of(desc()->alg_kind, alg_kind::pooling_max,
+            VDISPATCH_POOLING(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_POOLING(
+                    utils::one_of(desc()->alg_kind, alg_kind::pooling_max,
                             alg_kind::pooling_avg_include_padding,
-                            alg_kind::pooling_avg_exclude_padding)
-                    && utils::everyone_is(
-                            d_type, src_md()->data_type, dst_md()->data_type)
-                    && platform::has_data_type_support(d_type)
-                    && !has_zero_dim_memory()
-                    && set_default_params() == status::success
-                    && attr()->has_default_values(
-                            primitive_attr_t::skip_mask_t::post_ops, d_type)
-                    && memory_desc_matches_tag(*src_md(), desired_fmt_tag)
-                    && memory_desc_matches_tag(*dst_md(), desired_fmt_tag)
-                    && !is_dilated();
-            if (!ok) return status::unimplemented;
+                            alg_kind::pooling_avg_exclude_padding),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_POOLING(utils::everyone_is(d_type, src_md()->data_type,
+                                      dst_md()->data_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(platform::has_data_type_support(d_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_POOLING(!is_dilated(), VERBOSE_UNSUPPORTED_FEATURE,
+                    "does not support dilations");
+            VDISPATCH_POOLING(
+                    attr()->has_default_values(
+                            primitive_attr_t::skip_mask_t::post_ops, d_type),
+                    VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_POOLING(
+                    ref_post_ops_t::primitive_kind_ok(attr()->post_ops_),
+                    VERBOSE_UNSUPPORTED_POSTOP);
+            VDISPATCH_POOLING(set_default_params() == status::success,
+                    VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_POOLING(
+                    memory_desc_matches_tag(*src_md(), desired_fmt_tag),
+                    VERBOSE_UNSUPPORTED_TAG_S, "src");
+            VDISPATCH_POOLING(
+                    memory_desc_matches_tag(*dst_md(), desired_fmt_tag),
+                    VERBOSE_UNSUPPORTED_TAG_S, "dst");
+            VDISPATCH_POOLING(
+                    attr_.set_default_formats(dst_md(0)) == status::success,
+                    VERBOSE_UNSUPPORTED_POSTOP);
 
             const bool is_training
                     = desc_.prop_kind == prop_kind::forward_training;
@@ -74,18 +91,26 @@ struct nchw_pooling_fwd_t : public primitive_t {
     private:
         void init_scratchpad() {
             using namespace memory_tracking::names;
-            if (src_md()->data_type == data_type::bf16) {
-                const size_t src_sz_ = ID() * IH() * IW() * C() * MB();
+            if (src_md()->data_type != data_type::f32) {
+                const size_t src_sz_ = ID() * IH() * IW() * IC() * MB();
                 auto scratchpad = scratchpad_registry().registrar();
                 scratchpad.template book<float>(key_pool_src_bf16cvt, src_sz_);
             }
         }
     };
 
-    nchw_pooling_fwd_t(const pd_t *apd);
+    nchw_pooling_fwd_t(const pd_t *apd) : primitive_t(apd) {}
 
     using data_t = typename prec_traits<d_type>::type;
 
+    status_t init(engine_t *engine) override {
+        ref_post_ops_
+                = utils::make_unique<ref_post_ops_t>(pd()->attr()->post_ops_);
+        if (!ref_post_ops_) return status::out_of_memory;
+        CHECK(ref_post_ops_->init(pd()->dst_md()));
+        return status::success;
+    }
+
     status_t execute(const exec_ctx_t &ctx) const override {
         return execute_forward(ctx);
     }
@@ -93,7 +118,7 @@ struct nchw_pooling_fwd_t : public primitive_t {
 private:
     status_t execute_forward(const exec_ctx_t &ctx) const;
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
-    const ref_post_ops_t ref_post_ops_;
+    std::unique_ptr<ref_post_ops_t> ref_post_ops_;
 };
 
 template <data_type_t d_type>
@@ -109,36 +134,40 @@ struct nchw_pooling_bwd_t : public primitive_t {
 
             using namespace prop_kind;
             using namespace alg_kind;
-            bool ok = !is_fwd()
-                    && utils::one_of(desc()->alg_kind, alg_kind::pooling_max,
+            VDISPATCH_POOLING(!is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_POOLING(
+                    utils::one_of(desc()->alg_kind, alg_kind::pooling_max,
                             alg_kind::pooling_avg_include_padding,
-                            alg_kind::pooling_avg_exclude_padding)
-                    && utils::everyone_is(d_type, diff_dst_md()->data_type,
-                            diff_src_md()->data_type)
-                    && platform::has_data_type_support(d_type)
-                    && !has_zero_dim_memory()
-                    && set_default_params() == status::success
-                    && attr()->has_default_values()
-                    && memory_desc_matches_tag(*diff_dst_md(), desired_fmt_tag)
-                    && memory_desc_matches_tag(*diff_src_md(), desired_fmt_tag)
-                    && !is_dilated();
-            if (!ok) return status::unimplemented;
+                            alg_kind::pooling_avg_exclude_padding),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_POOLING(
+                    utils::everyone_is(d_type, diff_dst_md()->data_type,
+                            diff_src_md()->data_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(platform::has_data_type_support(d_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+            VDISPATCH_POOLING(set_default_params() == status::success,
+                    VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_POOLING(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_POOLING(
+                    memory_desc_matches_tag(*diff_dst_md(), desired_fmt_tag),
+                    VERBOSE_UNSUPPORTED_TAG_S, "diff_dst");
+            VDISPATCH_POOLING(
+                    memory_desc_matches_tag(*diff_src_md(), desired_fmt_tag),
+                    VERBOSE_UNSUPPORTED_TAG_S, "diff_src");
+            VDISPATCH_POOLING(!is_dilated(), VERBOSE_UNSUPPORTED_FEATURE,
+                    "does not support dilations");
 
             if (desc()->alg_kind == pooling_max) {
-                bool ws_ok
-                        = true && hint_fwd_pd_ && hint_fwd_pd_->workspace_md();
-                if (!ws_ok) return status::unimplemented;
-
-                const auto &ws_blk
-                        = hint_fwd_pd_->workspace_md()->format_desc.blocking;
-                ws_ok = ws_ok && ws_blk.inner_nblks <= 1
-                        && IMPLICATION(ws_blk.inner_nblks == 1,
-                                ws_blk.inner_idxs[0] == 1);
-                if (!ws_ok) return status::unimplemented;
-
-                ws_md_ = *hint_fwd_pd_->workspace_md();
+                const auto ws_dt = hint_fwd_pd_->workspace_md()->data_type;
+                init_default_ws(ws_dt);
+                VDISPATCH_POOLING(
+                        compare_ws(hint_fwd_pd_), VERBOSE_WS_MISMATCH);
             }
 
+            nthr_ = dnnl_get_max_threads();
             calculate_channel_block_size();
             init_scratchpad();
 
@@ -146,20 +175,20 @@ struct nchw_pooling_bwd_t : public primitive_t {
         }
 
         dim_t channel_block_size_;
+        int nthr_; // To not exceed the limit in execute used for set up.
 
     private:
         void init_scratchpad() {
             using namespace memory_tracking::names;
-            if (diff_dst_md()->data_type == data_type::bf16) {
+            if (diff_dst_md()->data_type != data_type::f32) {
                 size_t dst_sz_ = OD() * OH() * OW();
                 size_t src_sz_ = ID() * IH() * IW();
-                size_t nthrs = dnnl_get_max_threads();
                 auto scratchpad = scratchpad_registry().registrar();
 
                 scratchpad.template book<float>(key_pool_src_bf16cvt,
-                        src_sz_ * nthrs * channel_block_size_);
+                        src_sz_ * nthr_ * channel_block_size_);
                 scratchpad.template book<float>(key_pool_dst_bf16cvt,
-                        dst_sz_ * nthrs * channel_block_size_);
+                        dst_sz_ * nthr_ * channel_block_size_);
             }
         }
 
@@ -169,8 +198,7 @@ struct nchw_pooling_bwd_t : public primitive_t {
             // spatial
             dim_t dst_sz_ = OD() * OH() * OW();
             dim_t src_sz_ = ID() * IH() * IW();
-            dim_t nthrs = dnnl_get_max_threads();
-            dim_t C_per_thr = nstl::min(MB() * C() / nthrs, C());
+            dim_t C_per_thr = nstl::min(MB() * IC() / nthr_, IC());
             const dim_t max_block_size
                     = platform::get_per_core_cache_size(1) / 2;
             dim_t data_size_per_ch = (dst_sz_ + src_sz_) * 6; // f32 + bf16
