@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -20,7 +20,9 @@
 #include <algorithm>
 
 #include "common/c_types_map.hpp"
+#include "common/compiler_workarounds.hpp"
 #include "common/dnnl_thread.hpp"
+#include "common/math_utils.hpp"
 #include "common/type_helpers.hpp"
 
 #include "cpu/platform.hpp"
@@ -29,13 +31,6 @@
 
 #include "cpu/nspc_batch_normalization.hpp"
 
-// clang 6 and 7 generate incorrect code with OMP_SIMD in some particular cases
-#if (defined __clang_major__) && (__clang_major__ >= 6)
-#define SAFE_TO_USE_OMP_SIMD 0
-#else
-#define SAFE_TO_USE_OMP_SIMD 1
-#endif
-
 namespace dnnl {
 namespace impl {
 namespace cpu {
@@ -44,13 +39,16 @@ using namespace memory_tracking::names;
 using namespace data_type;
 
 template <data_type_t d_type>
-void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
+status_t nspc_batch_normalization_fwd_t<d_type>::execute_forward(
         const exec_ctx_t &ctx) const {
     const bool save_stats = pd()->is_training();
     const bool is_training = pd()->is_training();
     const bool fuse_norm_relu = pd()->fuse_norm_relu();
     const bool calculate_stats = !pd()->stats_is_src();
-    const bool with_relu = pd()->with_relu_post_op();
+    const bool with_relu = pd()->with_relu_post_op(is_training);
+
+    const auto use_scale = pd()->use_scale();
+    const auto use_shift = pd()->use_shift();
 
     auto scratchpad = ctx.get_scratchpad_grantor();
     auto tmp_mean = scratchpad.template get<acc_data_t>(key_bnorm_tmp_mean);
@@ -58,7 +56,8 @@ void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
     auto *ws_reduce = scratchpad.template get<acc_data_t>(key_bnorm_reduction);
 
     auto src = CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
-    auto scaleshift = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_SCALE_SHIFT);
+    auto scale = CTX_IN_MEM(const float *, DNNL_ARG_SCALE);
+    auto shift = CTX_IN_MEM(const float *, DNNL_ARG_SHIFT);
 
     acc_data_t *mean, *variance;
     if (!calculate_stats) {
@@ -78,9 +77,7 @@ void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
 
     auto dst = CTX_OUT_MEM(data_t *, DNNL_ARG_DST);
     auto ws = CTX_OUT_MEM(uint8_t *, DNNL_ARG_WORKSPACE);
-    acc_data_t *tmp_data_ = d_type == bf16
-            ? scratchpad.template get<acc_data_t>(key_bnorm_bf16cvt)
-            : nullptr;
+    acc_data_t *tmp_data_ = scratchpad.template get<acc_data_t>(key_bnorm_cvt);
 
     const dim_t N = pd()->MB();
     const dim_t C = pd()->C();
@@ -89,10 +86,11 @@ void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
     const dim_t SP = pd()->H() * pd()->W() * pd()->D();
 
     const float eps = pd()->desc()->batch_norm_epsilon;
-    const bool use_scaleshift = pd()->use_scaleshift();
-    auto maybe_post_op
-            = [&](acc_data_t res) { return (with_relu && res < 0) ? 0 : res; };
-    int nthr = dnnl_get_max_threads();
+    auto maybe_post_op = [&](acc_data_t res) {
+        if (with_relu) return math::relu_fwd(res, pd()->alpha());
+        return res;
+    };
+    const int nthr = pd()->nthr_;
 
     if (calculate_stats) {
         parallel(nthr, [&](const int ithr, const int nthr) {
@@ -106,11 +104,10 @@ void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
                 for (dim_t sp = 0; sp < SP; sp++) {
                     const acc_data_t *_src;
                     const size_t s_off = (size_t)n * SP * C + sp * C;
-                    if (d_type == bf16) {
-                        // convert src from b16 to f32
+                    if (utils::one_of(d_type, bf16, f16)) {
+                        // convert src from xf16 to f32
                         acc_data_t *tmp_src = tmp_data_ + ithr * C_align;
-                        cvt_bfloat16_to_float(
-                                tmp_src, (bfloat16_t *)src + s_off, C);
+                        types::cvt_to_float(tmp_src, src + s_off, C);
                         _src = tmp_src;
                     } else {
                         _src = reinterpret_cast<const acc_data_t *>(
@@ -135,20 +132,22 @@ void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
 
             acc_data_t *mean_loc = tmp_mean + nstl::max(C, (dim_t)16) * ithr;
 
-            for (dim_t c = 0; c < C; c++) {
-                mean_loc[c] = mean[c];
-                ws_reduce[C * ithr + c] = 0.;
+            if (ithr > 0 || save_stats) {
+                for (dim_t c = 0; c < C; c++)
+                    mean_loc[c] = mean[c];
             }
 
+            for (dim_t c = 0; c < C; c++)
+                ws_reduce[C * ithr + c] = 0.;
+
             for (dim_t n = N_s; n < N_e; n++) {
                 for (dim_t sp = 0; sp < SP; sp++) {
                     const acc_data_t *_src;
                     const size_t s_off = (size_t)n * SP * C + sp * C;
-                    if (d_type == bf16) {
-                        // convert src from b16 to f32
+                    if (utils::one_of(d_type, bf16, f16)) {
+                        // convert src from xf16 to f32
                         acc_data_t *tmp_src = tmp_data_ + ithr * C_align;
-                        cvt_bfloat16_to_float(
-                                tmp_src, (bfloat16_t *)src + s_off, C);
+                        types::cvt_to_float(tmp_src, src + s_off, C);
                         _src = tmp_src;
                     } else {
                         _src = reinterpret_cast<const acc_data_t *>(
@@ -170,8 +169,10 @@ void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
         });
         parallel(nthr, [&](const int ithr, const int nthr) {
             acc_data_t *variance_loc = tmp_var + nstl::max(C, (dim_t)16) * ithr;
-            for (dim_t c = 0; c < C; c++)
-                variance_loc[c] = variance[c];
+            if (ithr > 0 || save_stats) {
+                for (dim_t c = 0; c < C; c++)
+                    variance_loc[c] = variance[c];
+            }
         });
     }
 
@@ -193,31 +194,29 @@ void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
                 acc_data_t *_dst;
                 const acc_data_t *_src;
                 const size_t s_off = (size_t)n * SP * C + sp * C;
-                if (d_type == bf16) {
+                if (utils::one_of(d_type, bf16, f16)) {
                     // store dst to f32 buffer
                     _dst = tmp_data_ + ithr * C_align;
-                    // convert src from b16 to f32
+                    // convert src from xf16 to f32
                     acc_data_t *tmp_src = tmp_data_ + (nthr + ithr) * C_align;
-                    cvt_bfloat16_to_float(
-                            tmp_src, (bfloat16_t *)src + s_off, C);
+                    types::cvt_to_float(tmp_src, src + s_off, C);
                     _src = tmp_src;
                 } else {
                     _dst = reinterpret_cast<acc_data_t *>(dst + s_off);
                     _src = reinterpret_cast<const acc_data_t *>(src + s_off);
                 }
-#if SAFE_TO_USE_OMP_SIMD
+#if CLANG_WA_02_SAFE_TO_USE_OMP_SIMD
                 PRAGMA_OMP_SIMD()
 #endif
                 for (int c = 0; c < C; c++) {
                     const size_t c_off = s_off + c;
                     acc_data_t sqrt_variance = static_cast<acc_data_t>(
                             sqrtf(variance_loc[c] + eps));
-                    acc_data_t sm = (use_scaleshift ? (acc_data_t)scaleshift[c]
-                                                    : (acc_data_t)1.0f)
+                    acc_data_t sm = (use_scale ? (acc_data_t)scale[c]
+                                               : (acc_data_t)1.0f)
                             / sqrt_variance;
-                    acc_data_t sv = use_scaleshift
-                            ? (acc_data_t)scaleshift[C + c]
-                            : (acc_data_t)0;
+                    acc_data_t sv
+                            = use_shift ? (acc_data_t)shift[c] : (acc_data_t)0;
                     acc_data_t bn_res = sm * (_src[c] - mean_loc[c]) + sv;
                     if (fuse_norm_relu) {
                         if (bn_res <= 0) {
@@ -229,51 +228,57 @@ void nspc_batch_normalization_fwd_t<d_type>::execute_forward(
                     }
                     _dst[c] = maybe_post_op(bn_res);
                 }
-                if (d_type == bf16) {
-                    // convert dst from f32 to b16
-                    cvt_float_to_bfloat16((bfloat16_t *)dst + s_off, _dst, C);
+                if (utils::one_of(d_type, bf16, f16)) {
+                    // convert dst from f32 to xf16
+                    types::cvt_from_float(dst + s_off, _dst, C);
                 }
             }
         }
     });
+    return status::success;
 }
 
 template struct nspc_batch_normalization_fwd_t<f32>;
 template struct nspc_batch_normalization_fwd_t<bf16>;
+template struct nspc_batch_normalization_fwd_t<f16>;
 
 template <data_type_t d_type>
-void nspc_batch_normalization_bwd_t<d_type>::execute_backward(
+status_t nspc_batch_normalization_bwd_t<d_type>::execute_backward(
         const exec_ctx_t &ctx) const {
+
+    const auto use_scale = pd()->use_scale();
+
     auto src = CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
     auto mean = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_MEAN);
     auto variance = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_VARIANCE);
+    auto scale = CTX_IN_MEM(acc_data_t *, DNNL_ARG_SCALE);
     auto diff_dst = CTX_IN_MEM(const data_t *, DNNL_ARG_DIFF_DST);
-    auto scaleshift = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_SCALE_SHIFT);
     auto ws = CTX_IN_MEM(const uint8_t *, DNNL_ARG_WORKSPACE);
 
     auto diff_src = CTX_OUT_MEM(data_t *, DNNL_ARG_DIFF_SRC);
-    auto diff_scaleshift = CTX_OUT_MEM(acc_data_t *, DNNL_ARG_DIFF_SCALE_SHIFT);
+    auto diff_scale = CTX_OUT_MEM(acc_data_t *, DNNL_ARG_DIFF_SCALE);
+    auto diff_shift = CTX_OUT_MEM(acc_data_t *, DNNL_ARG_DIFF_SHIFT);
 
     auto scratchpad = ctx.get_scratchpad_grantor();
     auto tmp_diff_ss
             = scratchpad.template get<acc_data_t>(key_bnorm_tmp_diff_ss);
 
-    if (diff_scaleshift == nullptr) diff_scaleshift = tmp_diff_ss;
+    const size_t scratch_diff_shift_off = diff_scale ? 0 : pd()->C();
+    if (diff_scale == nullptr) diff_scale = tmp_diff_ss;
+    if (diff_shift == nullptr)
+        diff_shift = &tmp_diff_ss[scratch_diff_shift_off];
 
     const dim_t N = pd()->MB();
     const dim_t C = pd()->C();
     const int simd_w = 16;
     const dim_t C_align = utils::rnd_up(C, simd_w);
     const dim_t SP = pd()->D() * pd()->H() * pd()->W();
-    acc_data_t *diff_gamma = diff_scaleshift, *diff_beta = diff_scaleshift + C;
+    acc_data_t *diff_gamma = diff_scale, *diff_beta = diff_shift;
     acc_data_t *ws_reduce
             = scratchpad.template get<acc_data_t>(key_bnorm_reduction);
-    acc_data_t *tmp_data_ = d_type == bf16
-            ? scratchpad.template get<acc_data_t>(key_bnorm_bf16cvt)
-            : nullptr;
+    acc_data_t *tmp_data_ = scratchpad.template get<acc_data_t>(key_bnorm_cvt);
 
     const float eps = pd()->desc()->batch_norm_epsilon;
-    const bool use_scaleshift = pd()->use_scaleshift();
     const bool calculate_diff_stats = !pd()->use_global_stats();
     const bool fuse_norm_relu = pd()->fuse_norm_relu();
 
@@ -283,7 +288,7 @@ void nspc_batch_normalization_bwd_t<d_type>::execute_backward(
             platform::get_vector_register_size() / (int)sizeof(float), 8);
     const dim_t tail = C % c_blk;
     const dim_t nb_c_blk = (size_t)C / c_blk;
-    int nthr = dnnl_get_max_threads();
+    const int nthr = pd()->nthr_;
 
     parallel(nthr, [&](const int ithr, const int nthr) {
         dim_t N_s = 0, N_e = 0;
@@ -299,23 +304,21 @@ void nspc_batch_normalization_bwd_t<d_type>::execute_backward(
                 const acc_data_t *_diff_dst;
                 const acc_data_t *_src;
                 const size_t s_off = (size_t)n * SP * C + sp * C;
-                if (d_type == bf16) {
-                    // convert diff_dst from b16 to f32
+                if (utils::one_of(d_type, bf16, f16)) {
+                    // convert diff_dst to f32
                     acc_data_t *tmp_diff_dst = tmp_data_ + ithr * C_align;
-                    cvt_bfloat16_to_float(
-                            tmp_diff_dst, (bfloat16_t *)diff_dst + s_off, C);
+                    types::cvt_to_float(tmp_diff_dst, diff_dst + s_off, C);
                     _diff_dst = tmp_diff_dst;
-                    // convert src from b16 to f32
+                    // convert src to f32
                     acc_data_t *tmp_src = tmp_data_ + (nthr + ithr) * C_align;
-                    cvt_bfloat16_to_float(
-                            tmp_src, (bfloat16_t *)src + s_off, C);
+                    types::cvt_to_float(tmp_src, src + s_off, C);
                     _src = tmp_src;
                 } else {
                     _diff_dst = reinterpret_cast<const acc_data_t *>(
                             diff_dst + s_off);
                     _src = reinterpret_cast<const acc_data_t *>(src + s_off);
                 }
-#if SAFE_TO_USE_OMP_SIMD
+#if CLANG_WA_02_SAFE_TO_USE_OMP_SIMD
                 PRAGMA_OMP_SIMD()
 #endif
                 for (dim_t c = 0; c < C; c++) {
@@ -362,20 +365,18 @@ void nspc_batch_normalization_bwd_t<d_type>::execute_backward(
                 const acc_data_t *_diff_dst;
                 const acc_data_t *_src;
                 const size_t s_off = (size_t)n * SP * C + sp * C;
-                if (d_type == bf16) {
+                if (utils::one_of(d_type, bf16, f16)) {
                     // store diff_src to f32 buffer
                     _diff_src = tmp_data_ + ithr * C_align;
-                    // convert diff_dst from b16 to f32
+                    // convert diff_dst to f32
                     acc_data_t *tmp_diff_dst = tmp_data_ + ithr * C_align;
-                    cvt_bfloat16_to_float(
-                            tmp_diff_dst, (bfloat16_t *)diff_dst + s_off, C);
+                    types::cvt_to_float(tmp_diff_dst, diff_dst + s_off, C);
                     _diff_dst = tmp_diff_dst;
                     if (calculate_diff_stats) {
-                        // convert src from b16 to f32
+                        // convert src to f32
                         acc_data_t *tmp_src
                                 = tmp_data_ + (2 * nthr + ithr) * C_align;
-                        cvt_bfloat16_to_float(
-                                tmp_src, (bfloat16_t *)src + s_off, C);
+                        types::cvt_to_float(tmp_src, src + s_off, C);
                         _src = tmp_src;
                     } else
                         _src = nullptr; // to avoid compiler warning w/ gcc483
@@ -387,12 +388,12 @@ void nspc_batch_normalization_bwd_t<d_type>::execute_backward(
                     _src = reinterpret_cast<const acc_data_t *>(src + s_off);
                 }
 
-#if SAFE_TO_USE_OMP_SIMD
+#if CLANG_WA_02_SAFE_TO_USE_OMP_SIMD
                 PRAGMA_OMP_SIMD(simdlen(16))
 #endif
                 for (dim_t c = 0; c < nb_c_blk * c_blk; c++) {
                     const size_t c_off = s_off + c;
-                    acc_data_t gamma = use_scaleshift ? scaleshift[c] : 1;
+                    acc_data_t gamma = use_scale ? scale[c] : 1;
                     acc_data_t sqrt_variance = static_cast<acc_data_t>(
                             1.0f / sqrtf(variance[c] + eps));
                     acc_data_t v_diff_src;
@@ -410,9 +411,8 @@ void nspc_batch_normalization_bwd_t<d_type>::execute_backward(
                 }
                 for (dim_t c = 0; c < tail; c++) {
                     const size_t c_off = s_off + nb_c_blk * c_blk + c;
-                    acc_data_t gamma = use_scaleshift
-                            ? scaleshift[nb_c_blk * c_blk + c]
-                            : 1;
+                    acc_data_t gamma
+                            = use_scale ? scale[nb_c_blk * c_blk + c] : 1;
                     acc_data_t sqrt_variance = static_cast<acc_data_t>(
                             1.0f / sqrtf(variance[nb_c_blk * c_blk + c] + eps));
                     acc_data_t v_diff_src;
@@ -431,18 +431,19 @@ void nspc_batch_normalization_bwd_t<d_type>::execute_backward(
                     v_diff_src *= gamma * sqrt_variance;
                     _diff_src[nb_c_blk * c_blk + c] = v_diff_src;
                 }
-                if (d_type == bf16) {
-                    // convert diff_src from f32 to b16
-                    cvt_float_to_bfloat16(
-                            (bfloat16_t *)diff_src + s_off, _diff_src, C);
+                if (utils::one_of(d_type, bf16, f16)) {
+                    // convert diff_src from f32
+                    types::cvt_from_float(diff_src + s_off, _diff_src, C);
                 }
             }
         }
     });
+    return status::success;
 }
 
 template struct nspc_batch_normalization_bwd_t<f32>;
 template struct nspc_batch_normalization_bwd_t<bf16>;
+template struct nspc_batch_normalization_bwd_t<f16>;
 } // namespace cpu
 } // namespace impl
 } // namespace dnnl
