@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -63,8 +63,12 @@ status_t ref_batch_normalization_fwd_t<d_type>::execute_forward(
 
     status_t status = status::success;
 
+    const memory_desc_wrapper data_d(pd()->src_md());
+    const memory_desc_wrapper ss_d(pd()->weights_md());
+
     auto src = CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
-    auto scaleshift = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_SCALE_SHIFT);
+    auto scale = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_SCALE);
+    auto shift = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_SHIFT);
 
     auto mean = pd()->stats_is_src()
             ? const_cast<acc_data_t *>(CTX_IN_MEM(const float *, DNNL_ARG_MEAN))
@@ -81,9 +85,6 @@ status_t ref_batch_normalization_fwd_t<d_type>::execute_forward(
     auto ws = CTX_OUT_CLEAN_MEM(uint8_t *, DNNL_ARG_WORKSPACE, status);
     CHECK(status);
 
-    const memory_desc_wrapper data_d(pd()->src_md());
-    const memory_desc_wrapper scaleshift_d(pd()->weights_md());
-
     const auto ndims = data_d.ndims();
     const auto N = pd()->MB();
     const auto C = pd()->C();
@@ -92,7 +93,6 @@ status_t ref_batch_normalization_fwd_t<d_type>::execute_forward(
     const auto W = pd()->W();
 
     const auto eps = pd()->desc()->batch_norm_epsilon;
-    const auto use_scaleshift = pd()->use_scaleshift();
     const auto calculate_stats = !pd()->stats_is_src();
     const auto fuse_norm_relu = pd()->fuse_norm_relu();
     const auto save_stats = pd()->is_training();
@@ -108,9 +108,10 @@ status_t ref_batch_normalization_fwd_t<d_type>::execute_forward(
         return status::success;
     }
 
-    const bool with_relu = pd()->with_relu_post_op();
+    const bool with_relu = pd()->with_relu_post_op(is_training);
     auto maybe_post_op = [&](acc_data_t res) {
-        return (with_relu && res < 0.0f) ? 0.0f : res;
+        if (with_relu) return math::relu_fwd(res, pd()->alpha());
+        return res;
     };
 
     parallel_nd(C, [&](dim_t c) {
@@ -138,10 +139,8 @@ status_t ref_batch_normalization_fwd_t<d_type>::execute_forward(
         }
 
         acc_data_t sqrt_variance = sqrtf(v_variance + eps);
-        acc_data_t sm
-                = (use_scaleshift ? scaleshift[scaleshift_d.off(0, c)] : 1.0f)
-                / sqrt_variance;
-        acc_data_t sv = use_scaleshift ? scaleshift[scaleshift_d.off(1, c)] : 0;
+        acc_data_t sm = (scale ? scale[ss_d.off(c)] : 1.0f) / sqrt_variance;
+        acc_data_t sv = shift ? shift[ss_d.off(c)] : 0;
 
         for_(dim_t n = 0; n < N; ++n)
         for_(dim_t d = 0; d < D; ++d)
@@ -159,7 +158,8 @@ status_t ref_batch_normalization_fwd_t<d_type>::execute_forward(
                 }
             }
             if (d_type == s8)
-                dst[d_off] = qz_a1b0<float, data_t>()(maybe_post_op(bn_res));
+                dst[d_off]
+                        = q10n::qz_a1b0<float, data_t>()(maybe_post_op(bn_res));
             else
                 dst[d_off] = maybe_post_op(bn_res);
         }
@@ -177,28 +177,34 @@ status_t ref_batch_normalization_fwd_t<d_type>::execute_forward(
 template struct ref_batch_normalization_fwd_t<s8>;
 template struct ref_batch_normalization_fwd_t<f32>;
 template struct ref_batch_normalization_fwd_t<bf16>;
+template struct ref_batch_normalization_fwd_t<f16>;
 
 template <impl::data_type_t d_type>
 status_t ref_batch_normalization_bwd_t<d_type>::execute_backward(
         const exec_ctx_t &ctx) const {
     status_t status = status::success;
+
+    const memory_desc_wrapper data_d(pd()->src_md());
+    const memory_desc_wrapper diff_data_d(pd()->diff_src_md());
+    const memory_desc_wrapper ss_d(pd()->weights_md());
+    const memory_desc_wrapper diff_ss_d(pd()->diff_weights_md());
+
     auto src = CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
     auto mean = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_MEAN);
     auto variance = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_VARIANCE);
     auto diff_dst = CTX_IN_MEM(const data_t *, DNNL_ARG_DIFF_DST);
-    auto scaleshift = CTX_IN_MEM(const acc_data_t *, DNNL_ARG_SCALE_SHIFT);
     auto ws = CTX_IN_MEM(const uint8_t *, DNNL_ARG_WORKSPACE);
 
     auto diff_src = CTX_OUT_CLEAN_MEM(data_t *, DNNL_ARG_DIFF_SRC, status);
     CHECK(status);
-    auto diff_scaleshift = CTX_OUT_CLEAN_MEM(
-            acc_data_t *, DNNL_ARG_DIFF_SCALE_SHIFT, status);
-    CHECK(status);
 
-    const memory_desc_wrapper data_d(pd()->src_md());
-    const memory_desc_wrapper diff_data_d(pd()->diff_src_md());
-    const memory_desc_wrapper scaleshift_d(pd()->weights_md());
-    const memory_desc_wrapper diff_scaleshift_d(pd()->diff_weights_md());
+    auto scale = CTX_IN_MEM(acc_data_t *, DNNL_ARG_SCALE);
+    auto diff_scale
+            = CTX_OUT_CLEAN_MEM(acc_data_t *, DNNL_ARG_DIFF_SCALE, status);
+    CHECK(status);
+    auto diff_shift
+            = CTX_OUT_CLEAN_MEM(acc_data_t *, DNNL_ARG_DIFF_SHIFT, status);
+    CHECK(status);
 
     const auto ndims = data_d.ndims();
     const auto N = pd()->MB();
@@ -208,16 +214,19 @@ status_t ref_batch_normalization_bwd_t<d_type>::execute_backward(
     const auto W = pd()->W();
 
     const auto eps = pd()->desc()->batch_norm_epsilon;
-    const auto use_scaleshift = pd()->use_scaleshift();
     const auto calculate_diff_stats = !pd()->use_global_stats();
     const auto fuse_norm_relu = pd()->fuse_norm_relu();
 
     /* fast return */
     if (this->pd()->has_zero_dim_memory()) {
-        if (diff_scaleshift) {
+        if (diff_scale) {
             for (dim_t c = 0; c < C; ++c) {
-                diff_scaleshift[diff_scaleshift_d.off(0, c)] = 0;
-                diff_scaleshift[diff_scaleshift_d.off(1, c)] = 0;
+                diff_scale[diff_ss_d.off(c)] = 0.0f;
+            }
+        }
+        if (diff_shift) {
+            for (dim_t c = 0; c < C; ++c) {
+                diff_shift[diff_ss_d.off(c)] = 0.0f;
             }
         }
         return status::success;
@@ -228,8 +237,7 @@ status_t ref_batch_normalization_bwd_t<d_type>::execute_backward(
         acc_data_t v_variance = variance[c];
         acc_data_t sqrt_variance
                 = static_cast<acc_data_t>(1.0f / sqrtf(v_variance + eps));
-        acc_data_t gamma
-                = use_scaleshift ? scaleshift[scaleshift_d.off(0, c)] : 1;
+        acc_data_t gamma = scale ? scale[ss_d.off(c)] : 1.0f;
         acc_data_t diff_gamma = 0;
         acc_data_t diff_beta = 0;
 
@@ -249,10 +257,8 @@ status_t ref_batch_normalization_bwd_t<d_type>::execute_backward(
         }
         diff_gamma *= sqrt_variance;
 
-        if (diff_scaleshift) {
-            diff_scaleshift[diff_scaleshift_d.off(0, c)] = diff_gamma;
-            diff_scaleshift[diff_scaleshift_d.off(1, c)] = diff_beta;
-        }
+        if (diff_scale) diff_scale[diff_ss_d.off(c)] = diff_gamma;
+        if (diff_shift) diff_shift[diff_ss_d.off(c)] = diff_beta;
 
         for_(dim_t n = 0; n < N; ++n)
         for_(dim_t d = 0; d < D; ++d)
@@ -280,6 +286,7 @@ status_t ref_batch_normalization_bwd_t<d_type>::execute_backward(
 
 template struct ref_batch_normalization_bwd_t<f32>;
 template struct ref_batch_normalization_bwd_t<bf16>;
+template struct ref_batch_normalization_bwd_t<f16>;
 
 } // namespace cpu
 } // namespace impl
