@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2021 Intel Corporation
+* Copyright 2018-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -22,32 +22,17 @@
 #include "common/type_helpers.hpp"
 
 #include "cpu/cpu_primitive.hpp"
-#include "cpu/simple_q10n.hpp"
+#include "cpu/ref_io_helper.hpp"
 
+#include "cpu/ref_convolution_utils.hpp"
 #include "cpu/ref_deconvolution.hpp"
 
 namespace dnnl {
 namespace impl {
 namespace cpu {
 
-namespace {
-dim_t get_data_off(const memory_desc_wrapper &mdw, int ndims, dim_t mb, dim_t c,
-        dim_t id, dim_t ih, dim_t iw) {
-    switch (ndims) {
-        case 5: return mdw.off(mb, c, id, ih, iw);
-        case 4: return mdw.off(mb, c, ih, iw);
-        case 3: return mdw.off(mb, c, iw);
-        default: assert(!"unsupported ndims"); return dim_t(0);
-    }
-}
-
-} // namespace
-
-template <data_type_t dst_type>
 void ref_deconvolution_fwd_t::compute_fwd_bias_common(const exec_ctx_t &ctx,
-        typename prec_traits<dst_type>::type *dst,
-        const float *conv_output) const {
-    using dst_data_t = typename prec_traits<dst_type>::type;
+        void *dst, const float *conv_output, bool non_default_attr) const {
     const auto bias = CTX_IN_MEM(const void *, DNNL_ARG_BIAS);
     const memory_desc_wrapper dst_d(pd()->dst_md());
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
@@ -63,18 +48,18 @@ void ref_deconvolution_fwd_t::compute_fwd_bias_common(const exec_ctx_t &ctx,
     parallel_nd(MB, G, OC, OD, OH, OW,
             [&](dim_t mb, dim_t g, dim_t oc, dim_t od, dim_t oh, dim_t ow) {
                 const dim_t c = g * OC + oc;
-                const dim_t off = get_data_off(dst_d, ndims, mb, c, od, oh, ow);
-                float b = types::get_float_value(bias_d.data_type(), bias, c);
+                const dim_t off = ref_conv_utils::get_data_off(
+                        dst_d, ndims, mb, c, od, oh, ow);
+                float b = io::load_float_value(bias_d.data_type(), bias, c);
                 float d = conv_output[off];
-                dst[off] = cpu::saturate_and_round<dst_data_t>(d + b);
+                // Use f32 if attributes happen after bias to get precise answer
+                auto dt = non_default_attr ? data_type::f32 : dst_d.data_type();
+                io::store_float_value(dt, d + b, dst, off);
             });
 }
 
-template <data_type_t dst_type>
 void ref_deconvolution_fwd_t::compute_fwd_bias_ncdhw(const exec_ctx_t &ctx,
-        typename prec_traits<dst_type>::type *dst,
-        const float *conv_output) const {
-    using dst_data_t = typename prec_traits<dst_type>::type;
+        void *dst, const float *conv_output, bool non_default_attr) const {
     const auto bias = CTX_IN_MEM(const void *, DNNL_ARG_BIAS);
     const memory_desc_wrapper dst_d(pd()->dst_md());
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
@@ -85,20 +70,19 @@ void ref_deconvolution_fwd_t::compute_fwd_bias_ncdhw(const exec_ctx_t &ctx,
 
     parallel_nd(MB, OC, [&](dim_t mb, dim_t oc) {
         const dim_t off = (mb * OC + oc) * SP;
-        float b = types::get_float_value(bias_d.data_type(), bias, oc);
+        float b = io::load_float_value(bias_d.data_type(), bias, oc);
         PRAGMA_OMP_SIMD()
         for (dim_t sp = 0; sp < SP; ++sp) {
             float d = conv_output[off + sp];
-            dst[off + sp] = cpu::saturate_and_round<dst_data_t>(d + b);
+            // Use f32 if attributes happen after bias to get precise answer.
+            auto dt = non_default_attr ? data_type::f32 : dst_d.data_type();
+            io::store_float_value(dt, d + b, dst, off + sp);
         }
     });
 }
 
-template <data_type_t dst_type>
 void ref_deconvolution_fwd_t::compute_fwd_bias_ndhwc(const exec_ctx_t &ctx,
-        typename prec_traits<dst_type>::type *dst,
-        const float *conv_output) const {
-    using dst_data_t = typename prec_traits<dst_type>::type;
+        void *dst, const float *conv_output, bool non_default_attr) const {
     const auto bias = CTX_IN_MEM(const void *, DNNL_ARG_BIAS);
     const memory_desc_wrapper dst_d(pd()->dst_md());
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
@@ -111,18 +95,18 @@ void ref_deconvolution_fwd_t::compute_fwd_bias_ndhwc(const exec_ctx_t &ctx,
         const dim_t off = (mb * SP + sp) * OC;
         PRAGMA_OMP_SIMD()
         for (dim_t oc = 0; oc < OC; ++oc) {
-            float b = types::get_float_value(bias_d.data_type(), bias, oc);
+            float b = io::load_float_value(bias_d.data_type(), bias, oc);
             float d = conv_output[off + oc];
-            dst[off + oc] = cpu::saturate_and_round<dst_data_t>(d + b);
+            // Use f32 if attributes happen after bias to get precise answer.
+            auto dt = non_default_attr ? data_type::f32 : dst_d.data_type();
+            io::store_float_value(dt, d + b, dst, off + oc);
         }
     });
 }
 
-template <data_type_t dst_type, dim_t blk_size>
+template <dim_t blk_size>
 void ref_deconvolution_fwd_t::compute_fwd_bias_nCdhwXc(const exec_ctx_t &ctx,
-        typename prec_traits<dst_type>::type *dst,
-        const float *conv_output) const {
-    using dst_data_t = typename prec_traits<dst_type>::type;
+        void *dst, const float *conv_output, bool non_default_attr) const {
     const auto bias = CTX_IN_MEM(const void *, DNNL_ARG_BIAS);
     const memory_desc_wrapper dst_d(pd()->dst_md());
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
@@ -139,101 +123,157 @@ void ref_deconvolution_fwd_t::compute_fwd_bias_nCdhwXc(const exec_ctx_t &ctx,
                 const dim_t blk = nstl::min(blk_size, OC - oc);
 
                 PRAGMA_OMP_SIMD()
-                for (dim_t i = 0; i < blk; ++i) {
-                    float b = types::get_float_value(
-                            bias_d.data_type(), bias, oc + i);
+                for (dim_t i = 0; i < blk_size; ++i) {
+                    float b = i < blk ? io::load_float_value(
+                                      bias_d.data_type(), bias, oc + i)
+                                      : 0;
                     float d = conv_output[off + i];
-                    dst[off + i] = cpu::saturate_and_round<dst_data_t>(d + b);
+                    // Use f32 if attributes happen after bias to get precise
+                    // answer.
+                    auto dt = non_default_attr ? data_type::f32
+                                               : dst_d.data_type();
+                    io::store_float_value(dt, d + b, dst, off + i);
                 }
             });
 }
 
-template <data_type_t dst_type>
-void ref_deconvolution_fwd_t::compute_fwd_bias(const exec_ctx_t &ctx,
-        typename prec_traits<dst_type>::type *dst,
-        const float *conv_output) const {
+void ref_deconvolution_fwd_t::compute_fwd_bias(const exec_ctx_t &ctx, void *dst,
+        const float *conv_output, bool non_default_attr) const {
     using namespace format_tag;
     switch (pd()->dst_tag_) {
         case ncdhw:
         case nchw:
         case ncw:
-            compute_fwd_bias_ncdhw<dst_type>(ctx, dst, conv_output);
+            compute_fwd_bias_ncdhw(ctx, dst, conv_output, non_default_attr);
             break;
         case ndhwc:
         case nhwc:
         case nwc:
-            compute_fwd_bias_ndhwc<dst_type>(ctx, dst, conv_output);
+            compute_fwd_bias_ndhwc(ctx, dst, conv_output, non_default_attr);
             break;
         case nCdhw8c:
         case nChw8c:
         case nCw8c:
-            assert(dst_type != data_type::bf16);
-            compute_fwd_bias_nCdhwXc<dst_type, 8>(ctx, dst, conv_output);
+            compute_fwd_bias_nCdhwXc<8>(
+                    ctx, dst, conv_output, non_default_attr);
             break;
         case nCdhw16c:
         case nChw16c:
         case nCw16c:
-            compute_fwd_bias_nCdhwXc<dst_type, 16>(ctx, dst, conv_output);
+            compute_fwd_bias_nCdhwXc<16>(
+                    ctx, dst, conv_output, non_default_attr);
             break;
         default:
-            compute_fwd_bias_common<dst_type>(ctx, dst, conv_output);
+            compute_fwd_bias_common(ctx, dst, conv_output, non_default_attr);
             break;
     }
 }
 
-template <data_type_t dst_type>
+status_t ref_deconvolution_fwd_t::compute_oscale(
+        const exec_ctx_t &ctx, float *dst) const {
+
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    const int wei_scale_mask
+            = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+
+    const memory_desc_wrapper dst_d(pd()->dst_md());
+
+    const auto MB = pd()->MB();
+    const auto OH = pd()->OH();
+    const auto OW = pd()->OW();
+    const auto OD = pd()->OD();
+    const auto OC = pd()->OC();
+    const auto OCP = dst_d.padded_dims()[1];
+    const auto ndims = pd()->desc()->src_desc.ndims;
+
+    const auto maybe_oscale = [](float &d, dim_t oc, const float *src_scales,
+                                      const float *wei_scales, int wei_mask) {
+        // scale_idx_mult = 1 for per_oc scales and 0, otherwise
+        const int wei_scale_idx_mult = wei_mask != 0;
+        d *= src_scales[0] * wei_scales[oc * wei_scale_idx_mult];
+    };
+
+    parallel_nd(MB, OCP, OD, OH, OW,
+            [&](dim_t mb, int ocp, dim_t od, dim_t oh, dim_t ow) {
+                auto dst_off = ref_conv_utils::get_data_off(
+                        dst_d, ndims, mb, ocp, od, oh, ow);
+                float tmp_result = 0;
+
+                if (ocp < OC) {
+                    tmp_result = dst[dst_off];
+                    maybe_oscale(tmp_result, ocp, src_scales, wei_scales,
+                            wei_scale_mask);
+                    dst[dst_off] = tmp_result;
+                }
+            });
+
+    return status_t::dnnl_success;
+}
+
 status_t ref_deconvolution_fwd_t::compute_ref_attrs(const exec_ctx_t &ctx,
-        const float *conv_output,
-        typename prec_traits<dst_type>::type *original_dst) const {
-    using dst_data_t = typename prec_traits<dst_type>::type;
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+        const float *conv_output, void *original_dst) const {
+    auto dst = CTX_OUT_MEM(void *, DNNL_ARG_DST);
+
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+    const int dst_scale_mask = pd()->attr()->scales_.get(DNNL_ARG_DST).mask_;
+
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
     const bool is_dst_zp_common
             = pd()->attr()->zero_points_.common(DNNL_ARG_DST);
 
     const memory_desc_wrapper dst_d(pd()->dst_md());
 
-    const auto G = pd()->G();
     const auto MB = pd()->MB();
     const auto OH = pd()->OH();
     const auto OW = pd()->OW();
     const auto OD = pd()->OD();
-    const auto OC = pd()->OC() / G;
+    const auto OC = pd()->OC();
+    const auto OCP = dst_d.padded_dims()[1];
     const auto ndims = pd()->desc()->src_desc.ndims;
 
-    const auto maybe_oscale = [=](float &d, dim_t g, dim_t oc) {
-        // scale_idx_mult = 1 for per_oc scales and 0, otherwise
-        const int scale_idx_mult
-                = pd()->attr()->output_scales_.mask_ == (1 << 1);
-        const float *scales = pd()->attr()->output_scales_.scales_;
-        d *= scales[(g * OC + oc) * scale_idx_mult];
-    };
-
-    const auto maybe_dst_zero_point = [=](float &result, dim_t g, dim_t oc) {
+    const auto maybe_dst_zero_point = [=](float &result, dim_t oc) {
         if (is_dst_zp_common)
             result += dst_zero_point[0];
         else
-            result += dst_zero_point[g * OC + oc];
+            result += dst_zero_point[oc];
     };
 
-    parallel_nd(MB, G, OC, OD, OH, OW,
-            [&](dim_t mb, dim_t g, dim_t oc, dim_t od, dim_t oh, dim_t ow) {
-                auto dst_off = get_data_off(
-                        dst_d, ndims, mb, g * OC + oc, od, oh, ow);
-                dim_t dst_l_off = (mb * OC * G + g * OC + oc) * OD * OH * OW
-                        + od * OH * OW + oh * OW + ow;
-                float tmp_result = conv_output[dst_off];
-                maybe_oscale(tmp_result, g, oc);
+    const auto maybe_scale
+            = [](float &d, dim_t oc, const float *scales, int mask) {
+                  // scale_idx_mult = 1 for per_oc scales and 0, otherwise
+                  const int scale_idx_mult = mask != 0;
+                  d *= scales[oc * scale_idx_mult];
+              };
 
-                ref_post_ops_t::args_t args;
-                if (pd()->attr()->post_ops_.find(primitive_kind::sum) != -1)
-                    args.dst_val = (float)original_dst[dst_off];
-                args.ctx = &ctx;
-                args.l_offset = dst_l_off;
-                args.dst_md = pd()->dst_md();
-                ref_post_ops->execute(tmp_result, args);
-                maybe_dst_zero_point(tmp_result, g, oc);
-                dst[dst_off] = cpu::saturate_and_round<dst_data_t>(tmp_result);
+    const auto sum_dt = pd()->attr()->post_ops_.get_sum_dt(dst_d.data_type());
+
+    parallel_nd(MB, OCP, OD, OH, OW,
+            [&](dim_t mb, int ocp, dim_t od, dim_t oh, dim_t ow) {
+                auto dst_off = ref_conv_utils::get_data_off(
+                        dst_d, ndims, mb, ocp, od, oh, ow);
+                float tmp_result = 0;
+
+                if (ocp < OC) {
+                    dim_t dst_l_off = (mb * OC + ocp) * OD * OH * OW
+                            + od * OH * OW + oh * OW + ow;
+                    msan_unpoison(
+                            (void *)(&conv_output[dst_off]), sizeof(float));
+                    tmp_result = conv_output[dst_off];
+
+                    ref_post_ops_t::args_t args;
+                    if (pd()->attr()->post_ops_.find(primitive_kind::sum) != -1)
+                        args.dst_val = io::load_float_value(
+                                sum_dt, original_dst, dst_off);
+                    args.ctx = &ctx;
+                    args.l_offset = dst_l_off;
+                    args.dst_md = pd()->dst_md();
+                    ref_post_ops->execute(tmp_result, args);
+                    maybe_scale(tmp_result, ocp, dst_scales, dst_scale_mask);
+                    maybe_dst_zero_point(tmp_result, ocp);
+                }
+                io::store_float_value(
+                        dst_d.data_type(), tmp_result, dst, dst_off);
             });
 
     return status_t::dnnl_success;
@@ -293,13 +333,13 @@ static void compute_src_zp_compensation(const exec_ctx_t &ctx,
                 const int32_t wei32 = static_cast<int32_t>(wei[weights_offset]);
 
                 if (is_src_zp_common)
-                    acc += wei32;
+                    acc += wei32 * src_zero_point[0];
                 else
                     acc += wei32 * src_zero_point[g * IC + ic];
             }
         }
 
-        zp_compensation[out_offset] = acc * src_zero_point[0];
+        zp_compensation[out_offset] = acc;
     });
 }
 
@@ -416,8 +456,8 @@ static status_t apply_src_zero_point(const exec_ctx_t &ctx,
             [&](const dim_t mb, const dim_t g, const dim_t oc, const dim_t od,
                     const dim_t oh, const dim_t ow) {
                 const auto oc_off = g * OC + oc;
-                const auto dst_off
-                        = get_data_off(dst_d, ndims, mb, oc_off, od, oh, ow);
+                const auto dst_off = ref_conv_utils::get_data_off(
+                        dst_d, ndims, mb, oc_off, od, oh, ow);
                 int32_t conv_result
                         = conv_output[dst_off] - zp_src_compensation[oc_off];
 
@@ -447,9 +487,11 @@ status_t ref_deconvolution_fwd_t::execute(const exec_ctx_t &ctx) const {
 
     // Create intermediate memory for f32 output if needed.
     auto dst = args.at(DNNL_ARG_DST);
-    memory_t tmp_memory(dst.mem->engine(), pd()->conv_pd_->diff_src_md(),
-            scratchpad.get_memory_storage(key_deconv_bias));
-    memory_arg_t tmp_conv_output = {&tmp_memory, false};
+    std::unique_ptr<memory_t, memory_deleter_t> tmp_memory;
+    CHECK(safe_ptr_assign(tmp_memory,
+            new memory_t(dst.mem->engine(), pd()->conv_pd_->diff_src_md(),
+                    scratchpad.get_memory_storage(key_deconv_bias))));
+    memory_arg_t tmp_conv_output = {tmp_memory.get(), false};
 
     conv_args[DNNL_ARG_DIFF_SRC]
             = ref_bias || non_default_attr ? tmp_conv_output : dst;
@@ -480,9 +522,6 @@ status_t ref_deconvolution_fwd_t::execute(const exec_ctx_t &ctx) const {
     auto status = conv_p_->execute(conv_ctx);
     if (status != status::success) return status;
 
-    conv_args[DNNL_ARG_DIFF_SRC]
-            = ref_bias || non_default_attr ? tmp_conv_output : dst;
-
     using namespace data_type;
 
     if (!pd()->attr()->zero_points_.has_default_values(DNNL_ARG_SRC)) {
@@ -495,54 +534,25 @@ status_t ref_deconvolution_fwd_t::execute(const exec_ctx_t &ctx) const {
         }
     }
 
-    auto dst_type = pd()->dst_md()->data_type;
+    float *conv_output = scratchpad.get<float>(key_deconv_bias);
+
+    const auto &arg_scales = pd()->attr()->scales_;
+    const auto &src_scales = arg_scales.get(DNNL_ARG_SRC);
+    const auto &wei_scales = arg_scales.get(DNNL_ARG_WEIGHTS);
+
+    if (!src_scales.has_default_values() || !wei_scales.has_default_values()) {
+        compute_oscale(ctx, conv_output);
+    }
 
     if (ref_bias) {
-        float *conv_output = scratchpad.get<float>(key_deconv_bias);
-        if (non_default_attr) {
-            // Overwrite conv_output since further attr computations still need
-            // f32 output.
-            compute_fwd_bias<f32>(ctx, conv_output, conv_output);
-        } else {
-
-#define CASE(DT) \
-    case (DT): { \
-        using dst_data_t = typename prec_traits<DT>::type; \
-        dst_data_t *dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST); \
-        compute_fwd_bias<DT>(ctx, dst, conv_output); \
-    } break
-
-            switch (dst_type) {
-                CASE(f32);
-                CASE(bf16);
-                CASE(s32);
-                CASE(s8);
-                CASE(u8);
-                default: assert(!"unsupported data type");
-            }
-#undef CASE
-        }
+        void *dst = CTX_OUT_MEM(void *, DNNL_ARG_DST);
+        void *tmp_output = non_default_attr ? conv_output : dst;
+        compute_fwd_bias(ctx, tmp_output, conv_output, non_default_attr);
     }
 
     if (non_default_attr) {
-        float *conv_output = scratchpad.get<float>(key_deconv_bias);
-
-#define CASE(DT) \
-    case (DT): { \
-        using dst_data_t = typename prec_traits<DT>::type; \
-        dst_data_t *original_dst = scratchpad.get<dst_data_t>(key_deconv_sum); \
-        compute_ref_attrs<DT>(ctx, conv_output, original_dst); \
-    } break
-
-        switch (dst_type) {
-            CASE(f32);
-            CASE(bf16);
-            CASE(s32);
-            CASE(s8);
-            CASE(u8);
-            default: assert(!"unsupported data type");
-        }
-#undef CASE
+        void *original_dst = scratchpad.get<void>(key_deconv_sum);
+        compute_ref_attrs(ctx, conv_output, original_dst);
     }
 
     return status::success;
@@ -581,7 +591,7 @@ void ref_deconvolution_bwd_weights_t::compute_bwd_bias(
         for_(dim_t od = 0; od < OD; ++od)
         for_(dim_t oh = 0; oh < OH; ++oh)
         for (dim_t ow = 0; ow < OW; ++ow) {
-            const auto d_dst_off = get_data_off(
+            const auto d_dst_off = ref_conv_utils::get_data_off(
                     diff_dst_d, ndims, mb, g * OC + oc, od, oh, ow);
             db += diff_dst[d_dst_off];
         }
@@ -732,6 +742,10 @@ status_t ref_deconvolution_bwd_weights_t::execute(const exec_ctx_t &ctx) const {
             compute_bias<bf16, bf16>(ctx);
         else if (dbia_type == f32 && ddst_type == bf16)
             compute_bias<f32, bf16>(ctx);
+        else if (utils::everyone_is(f16, dbia_type, ddst_type))
+            compute_bias<f16, f16>(ctx);
+        else if (dbia_type == f32 && ddst_type == f16)
+            compute_bias<f32, f16>(ctx);
         else {
             assert(!"unsupported data type");
             return status::runtime_error;
@@ -748,6 +762,10 @@ template void ref_deconvolution_bwd_weights_t::compute_bias<f32, bf16>(
         const exec_ctx_t &ctx) const;
 template void ref_deconvolution_bwd_weights_t::compute_bias<bf16, bf16>(
         const exec_ctx_t &ctx) const;
+template void ref_deconvolution_bwd_weights_t::compute_bias<f32, f16>(
+        const exec_ctx_t &ctx) const;
+template void ref_deconvolution_bwd_weights_t::compute_bias<f16, f16>(
+        const exec_ctx_t &ctx) const;
 } // namespace cpu
 } // namespace impl
 } // namespace dnnl
