@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2018-2021 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
+* Copyright 2022 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -22,7 +23,7 @@
 
 #include "common/c_types_map.hpp"
 #include "common/primitive.hpp"
-#include "common/primitive_iterator.hpp"
+#include "common/primitive_desc_iterator.hpp"
 #include "common/stream.hpp"
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
@@ -43,11 +44,12 @@ static status_t weights_axes_permutation(
         perm[d] = d;
     nstl::swap(perm[0 + with_groups], perm[1 + with_groups]);
 
-    return dnnl_memory_desc_permute_axes(o_md, i_md, perm);
+    return memory_desc_permute_axes(*o_md, *i_md, perm);
 }
 
 static status_t conv_descr_create(const deconvolution_desc_t *dd,
-        convolution_desc_t *cd, data_type_t src_dt = data_type::undef) {
+        convolution_desc_t *cd, const memory_desc_t *bias_md = nullptr,
+        data_type_t src_dt = data_type::undef) {
     using namespace prop_kind;
     alg_kind_t alg_kind = dd->alg_kind == alg_kind::deconvolution_direct
             ? alg_kind::convolution_direct
@@ -60,7 +62,8 @@ static status_t conv_descr_create(const deconvolution_desc_t *dd,
     if (utils::one_of(dd->prop_kind, forward_training, forward_inference)) {
         prop_kind = backward_data;
         assert(src_dt != data_type::undef);
-        memory_desc_init_by_md_and_dt(src_md_patched, dd->dst_desc, src_dt);
+        CHECK(memory_desc_init_by_md_and_dt(
+                src_md_patched, dd->dst_desc, src_dt));
         src_md = &src_md_patched;
         dst_md = &dd->src_desc;
         d_weights_d = &dd->weights_desc;
@@ -84,25 +87,24 @@ static status_t conv_descr_create(const deconvolution_desc_t *dd,
     CHECK(weights_axes_permutation(&c_weights_d, d_weights_d, with_groups));
 
     return conv_desc_init(cd, prop_kind, alg_kind, src_md, &c_weights_d,
-            prop_kind != backward_weights ? &dd->bias_desc : nullptr, dst_md,
-            dd->strides, dd->dilates, dd->padding[0], dd->padding[1]);
+            bias_md, dst_md, dd->strides, dd->dilates, dd->padding[0],
+            dd->padding[1]);
 }
 
 struct ref_deconvolution_fwd_t : public primitive_t {
     struct pd_t : public cpu_deconvolution_fwd_pd_t {
-        pd_t(const deconvolution_desc_t *adesc, const primitive_attr_t *attr,
-                const deconvolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_deconvolution_fwd_pd_t(adesc, attr, hint_fwd_pd) {}
+        using cpu_deconvolution_fwd_pd_t::cpu_deconvolution_fwd_pd_t;
 
         pd_t(const pd_t &other)
             : cpu_deconvolution_fwd_pd_t(other)
             , conv_pd_(other.conv_pd_->clone())
             , conv_supports_bias_(other.conv_supports_bias_)
-            , dst_tag_(other.dst_tag_) {}
+            , dst_tag_(other.dst_tag_)
+            , name_(other.name_) {}
 
         ~pd_t() = default;
 
-        DECLARE_COMMON_PD_T(conv_pd_->name(), ref_deconvolution_fwd_t);
+        DECLARE_COMMON_PD_T(name_.c_str(), ref_deconvolution_fwd_t);
 
         status_t init_convolution(engine_t *engine) {
             using namespace format_tag;
@@ -112,7 +114,6 @@ struct ref_deconvolution_fwd_t : public primitive_t {
             // impl available and apply post-ops and/or bias update later in
             // this impl via simple loop.
             primitive_attr_t conv_attr;
-            conv_attr.set_scratchpad_mode(scratchpad_mode::user);
 
             convolution_desc_t cd;
             // When no attributes were requested, try to find a bwd_d conv impl
@@ -121,13 +122,14 @@ struct ref_deconvolution_fwd_t : public primitive_t {
             // diff_src for conv for correct result. If attributes are
             // requested, enforce conv impl to return f32 output no matter what.
             if (attr()->has_default_values()) {
-                CHECK(conv_descr_create(desc(), &cd, dst_md()->data_type));
-                dnnl_primitive_desc_iterator it(
+                CHECK(conv_descr_create(
+                        desc(), &cd, weights_md(1), dst_md()->data_type));
+                primitive_desc_iterator_t it(
                         engine, (op_desc_t *)&cd, &conv_attr, nullptr);
                 if (!it.is_initialized()) return status::out_of_memory;
 
                 while (++it != it.end()) {
-                    conv_pd_.reset(it.fetch_once());
+                    conv_pd_ = *it;
                     if (with_bias()) {
                         conv_supports_bias_ = utils::downcast<
                                 cpu_convolution_bwd_data_pd_t *>(conv_pd_.get())
@@ -139,34 +141,55 @@ struct ref_deconvolution_fwd_t : public primitive_t {
                 }
             }
 
-            // Enforce f32 dt for diff src and work with f32 output for bias
-            // update or post ops after conv execution.
-            CHECK(conv_descr_create(desc(), &cd, data_type::f32));
-            dnnl_primitive_desc_iterator it(
-                    engine, (op_desc_t *)&cd, &conv_attr, nullptr);
-            if (!it.is_initialized()) return status::out_of_memory;
+            // Intermediate f32 buffer is supported only for given condition.
+            if (!attr()->has_default_values() || with_bias()) {
+                // Enforce f32 dt for diff src and work with f32 output for bias
+                // update or post ops after conv execution.
+                CHECK(conv_descr_create(desc(), &cd, nullptr, data_type::f32));
+                primitive_desc_iterator_t it(
+                        engine, (op_desc_t *)&cd, &conv_attr, nullptr);
+                if (!it.is_initialized()) return status::out_of_memory;
 
-            while (++it != it.end()) {
-                conv_pd_.reset(it.fetch_once());
-                bool ok = conv_pd_->weights_md()->extra.flags == 0;
-                if (ok) return status::success;
+                while (++it != it.end()) {
+                    conv_pd_ = *it;
+                    bool ok = conv_pd_->weights_md()->extra.flags == 0;
+                    if (ok) return status::success;
+                }
             }
             return status::unimplemented;
         }
 
         status_t init(engine_t *engine) {
             using namespace format_tag;
+            using namespace data_type;
             using smask_t = primitive_attr_t::skip_mask_t;
+            auto skip_mask = smask_t::post_ops | smask_t::sum_dt;
+            if (utils::one_of(desc()->src_desc.data_type, s8, u8))
+                skip_mask |= smask_t::scales_runtime
+                        | smask_t::zero_points_runtime;
 
-            const bool ok = is_fwd()
-                    && utils::one_of(desc()->alg_kind,
-                            alg_kind::deconvolution_direct,
-                            alg_kind::deconvolution_winograd)
-                    && attr()->has_default_values(smask_t::oscale
-                            | smask_t::post_ops | smask_t::zero_points_runtime)
-                    && output_scales_mask_ok() && post_ops_ok()
-                    && zero_points_ok();
-            if (!ok) return status::unimplemented;
+            VDISPATCH_DECONVOLUTION(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_DECONVOLUTION(utils::one_of(desc()->alg_kind,
+                                            alg_kind::deconvolution_direct,
+                                            alg_kind::deconvolution_winograd),
+                    VERBOSE_BAD_ALGORITHM);
+            // This implementation will check data types requirements through
+            // an underlying convolution implementation, however, convolution
+            // might be called without bias, thus, need to check bias data type
+            // if it was requested.
+            if (with_bias()) {
+                const auto bia_type = invariant_wei_md(1)->data_type;
+                VDISPATCH_DECONVOLUTION(utils::one_of(bia_type, f32, bf16, f16,
+                                                f8_e5m2, f8_e4m3),
+                        VERBOSE_UNSUPPORTED_DT);
+            }
+            VDISPATCH_DECONVOLUTION(attr()->has_default_values(skip_mask),
+                    VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_DECONVOLUTION(
+                    attr_scales_ok(), VERBOSE_UNSUPPORTED_SCALES_CFG);
+            VDISPATCH_DECONVOLUTION(post_ops_ok(), VERBOSE_UNSUPPORTED_POSTOP);
+            VDISPATCH_DECONVOLUTION(
+                    zero_points_ok(), VERBOSE_UNSUPPORTED_ZP_CFG);
 
             CHECK(init_convolution(engine));
 
@@ -190,15 +213,20 @@ struct ref_deconvolution_fwd_t : public primitive_t {
                     utils::pick(ndims() - 3, nCw8c, nChw8c, nCdhw8c),
                     utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c));
 
+            init_name();
             init_scratchpad();
-            return status::success;
+            return attr_.set_default_formats(dst_md(0));
         }
 
-        std::unique_ptr<primitive_desc_t> conv_pd_;
+        std::shared_ptr<primitive_desc_t> conv_pd_;
         bool conv_supports_bias_ = false;
         format_tag_t dst_tag_;
 
     private:
+        std::string name_ = "conv:any+"; // convolution-based deconvolution
+
+        void init_name() { name_.append(conv_pd_->name()); }
+
         void init_scratchpad() {
             using namespace memory_tracking::names;
             auto scratchpad = scratchpad_registry().registrar();
@@ -227,23 +255,19 @@ struct ref_deconvolution_fwd_t : public primitive_t {
             }
         }
 
-        bool output_scales_mask_ok() const {
-            using namespace data_type;
-            const auto &mask = attr()->output_scales_.mask_;
-            return IMPLICATION(!utils::one_of(src_md()->data_type, s8, u8),
-                           attr()->output_scales_.has_default_values())
-                    && (mask == 0 || mask == 1 << 1);
-        }
-
         bool post_ops_ok() const {
-            return attr()->post_ops_.find(primitive_kind::convolution) == -1;
+            using namespace data_type;
+            const bool is_int8 = utils::one_of(src_md()->data_type, s8, u8);
+            return attr()->post_ops_.check_sum_consistency(
+                           dst_md()->data_type, is_int8)
+                    && ref_post_ops_t::primitive_kind_ok(attr()->post_ops_);
         }
 
         bool zero_points_ok() const {
             using namespace data_type;
             int mask_src = 0, mask_dst = 0;
-            attr()->zero_points_.get(DNNL_ARG_SRC, nullptr, &mask_src, nullptr);
-            attr()->zero_points_.get(DNNL_ARG_DST, nullptr, &mask_dst, nullptr);
+            attr()->zero_points_.get(DNNL_ARG_SRC, &mask_src);
+            attr()->zero_points_.get(DNNL_ARG_DST, &mask_dst);
 
             return IMPLICATION(!utils::one_of(src_md()->data_type, s8, u8),
                            attr()->zero_points_.has_default_values())
@@ -261,40 +285,33 @@ struct ref_deconvolution_fwd_t : public primitive_t {
         ref_post_ops
                 = utils::make_unique<ref_post_ops_t>(pd()->attr()->post_ops_);
         if (!ref_post_ops) return status::out_of_memory;
+        CHECK(ref_post_ops->init(pd()->dst_md()));
         return status::success;
     }
 
     status_t execute(const exec_ctx_t &ctx) const override;
 
 private:
-    template <data_type_t dst_type>
-    void compute_fwd_bias_common(const exec_ctx_t &ctx,
-            typename prec_traits<dst_type>::type *dst,
-            const float *conv_output) const;
+    void compute_fwd_bias_common(const exec_ctx_t &ctx, void *dst,
+            const float *conv_output, bool non_default_attr) const;
 
-    template <data_type_t dst_type>
-    void compute_fwd_bias_ncdhw(const exec_ctx_t &ctx,
-            typename prec_traits<dst_type>::type *dst,
-            const float *conv_output) const;
+    void compute_fwd_bias_ncdhw(const exec_ctx_t &ctx, void *dst,
+            const float *conv_output, bool non_default_attr) const;
 
-    template <data_type_t dst_type>
-    void compute_fwd_bias_ndhwc(const exec_ctx_t &ctx,
-            typename prec_traits<dst_type>::type *dst,
-            const float *conv_output) const;
+    void compute_fwd_bias_ndhwc(const exec_ctx_t &ctx, void *dst,
+            const float *conv_output, bool non_default_attr) const;
 
-    template <data_type_t dst_type, dim_t blk_size>
-    void compute_fwd_bias_nCdhwXc(const exec_ctx_t &ctx,
-            typename prec_traits<dst_type>::type *dst,
-            const float *conv_output) const;
+    template <dim_t blk_size>
+    void compute_fwd_bias_nCdhwXc(const exec_ctx_t &ctx, void *dst,
+            const float *conv_output, bool non_default_attr) const;
 
-    template <data_type_t dst_type>
-    void compute_fwd_bias(const exec_ctx_t &ctx,
-            typename prec_traits<dst_type>::type *dst,
-            const float *conv_output) const;
+    status_t compute_oscale(const exec_ctx_t &ctx, float *dst) const;
+
+    void compute_fwd_bias(const exec_ctx_t &ctx, void *dst,
+            const float *conv_output, bool non_default_attr) const;
 
-    template <data_type_t dst_type>
     status_t compute_ref_attrs(const exec_ctx_t &ctx, const float *conv_output,
-            typename prec_traits<dst_type>::type *original_dst) const;
+            void *original_dst) const;
 
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
     std::shared_ptr<primitive_t> conv_p_;
@@ -303,17 +320,16 @@ private:
 
 struct ref_deconvolution_bwd_data_t : public primitive_t {
     struct pd_t : public cpu_deconvolution_bwd_data_pd_t {
-        pd_t(const deconvolution_desc_t *adesc, const primitive_attr_t *attr,
-                const deconvolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_deconvolution_bwd_data_pd_t(adesc, attr, hint_fwd_pd) {}
+        using cpu_deconvolution_bwd_data_pd_t::cpu_deconvolution_bwd_data_pd_t;
 
         pd_t(const pd_t &other)
             : cpu_deconvolution_bwd_data_pd_t(other)
-            , conv_pd_(other.conv_pd_->clone()) {}
+            , conv_pd_(other.conv_pd_->clone())
+            , name_(other.name_) {}
 
         ~pd_t() = default;
 
-        DECLARE_COMMON_PD_T(conv_pd_->name(), ref_deconvolution_bwd_data_t);
+        DECLARE_COMMON_PD_T(name_.c_str(), ref_deconvolution_bwd_data_t);
 
         status_t init_convolution(engine_t *engine) {
             using namespace types;
@@ -323,13 +339,12 @@ struct ref_deconvolution_bwd_data_t : public primitive_t {
             if (status != status::success) return status;
             primitive_attr_t conv_attr(*attr());
             if (!conv_attr.is_initialized()) return status::out_of_memory;
-            conv_attr.set_scratchpad_mode(scratchpad_mode::user);
 
-            dnnl_primitive_desc_iterator it(
+            primitive_desc_iterator_t it(
                     engine, (op_desc_t *)&cd, &conv_attr, nullptr);
             if (!it.is_initialized()) return status::out_of_memory;
             while (++it != it.end()) {
-                conv_pd_.reset(it.fetch_once());
+                conv_pd_ = *it;
                 if (conv_pd_->weights_md()->extra.flags == 0)
                     return status::success;
             }
@@ -342,35 +357,44 @@ struct ref_deconvolution_bwd_data_t : public primitive_t {
             auto dsrc_type = desc()->diff_src_desc.data_type;
             auto wei_type = desc()->weights_desc.data_type;
             auto ddst_type = desc()->diff_dst_desc.data_type;
-            bool ok = true && desc()->prop_kind == prop_kind::backward_data
-                    && (utils::everyone_is(f32, dsrc_type, wei_type, ddst_type)
-                            || (utils::one_of(dsrc_type, f32, bf16)
-                                    && utils::everyone_is(
-                                            bf16, wei_type, ddst_type)))
-                    && utils::one_of(desc()->alg_kind,
-                            alg_kind::deconvolution_direct,
-                            alg_kind::deconvolution_winograd)
-                    && attr()->has_default_values();
 
-            if (ok) {
-                CHECK(init_convolution(engine));
-                if (weights_md_.format_kind == format_kind::any)
-                    CHECK(weights_axes_permutation(&weights_md_,
-                            conv_pd_->weights_md(), with_groups()));
-                if (diff_src_md_.format_kind == format_kind::any)
-                    diff_src_md_ = *conv_pd_->dst_md();
-                if (diff_dst_md_.format_kind == format_kind::any)
-                    diff_dst_md_ = *conv_pd_->src_md();
-                init_scratchpad();
-                return status::success;
-            }
+            VDISPATCH_DECONVOLUTION(
+                    desc()->prop_kind == prop_kind::backward_data,
+                    VERBOSE_BAD_PROPKIND);
+            VDISPATCH_DECONVOLUTION(utils::one_of(wei_type, f32, bf16, f16),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_DECONVOLUTION(ddst_type == wei_type,
+                    VERBOSE_INCONSISTENT_DT, "diff_dst", "weights");
+            VDISPATCH_DECONVOLUTION(utils::one_of(dsrc_type, wei_type, f32),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_DECONVOLUTION(utils::one_of(desc()->alg_kind,
+                                            alg_kind::deconvolution_direct,
+                                            alg_kind::deconvolution_winograd),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_DECONVOLUTION(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
 
-            return status::unimplemented;
+            CHECK(init_convolution(engine));
+            if (weights_md_.format_kind == format_kind::any)
+                CHECK(weights_axes_permutation(
+                        &weights_md_, conv_pd_->weights_md(), with_groups()));
+            if (diff_src_md_.format_kind == format_kind::any)
+                diff_src_md_ = *conv_pd_->dst_md();
+            if (diff_dst_md_.format_kind == format_kind::any)
+                diff_dst_md_ = *conv_pd_->src_md();
+
+            init_name();
+            init_scratchpad();
+            return status::success;
         }
 
-        std::unique_ptr<primitive_desc_t> conv_pd_;
+        std::shared_ptr<primitive_desc_t> conv_pd_;
 
     private:
+        std::string name_ = "conv:any+"; // convolution-based deconvolution
+
+        void init_name() { name_.append(conv_pd_->name()); }
+
         void init_scratchpad() {
             auto scratchpad = scratchpad_registry().registrar();
             scratchpad.book(memory_tracking::names::key_nested,
@@ -386,6 +410,14 @@ struct ref_deconvolution_bwd_data_t : public primitive_t {
         return pd()->conv_pd_->create_primitive(conv_p_, engine);
     }
 
+#if DNNL_AARCH64 && DNNL_AARCH64_USE_ACL
+    status_t create_resource(
+            engine_t *engine, resource_mapper_t &mapper) const override {
+        CHECK(conv_p_->create_resource(engine, mapper));
+        return status::success;
+    }
+#endif
+
     status_t execute(const exec_ctx_t &ctx) const override;
 
 private:
@@ -395,18 +427,18 @@ private:
 
 struct ref_deconvolution_bwd_weights_t : public primitive_t {
     struct pd_t : public cpu_deconvolution_bwd_weights_pd_t {
-        pd_t(const deconvolution_desc_t *adesc, const primitive_attr_t *attr,
-                const deconvolution_fwd_pd_t *hint_fwd_pd)
-            : cpu_deconvolution_bwd_weights_pd_t(adesc, attr, hint_fwd_pd) {}
+        using cpu_deconvolution_bwd_weights_pd_t::
+                cpu_deconvolution_bwd_weights_pd_t;
 
         pd_t(const pd_t &other)
             : cpu_deconvolution_bwd_weights_pd_t(other)
             , conv_pd_(other.conv_pd_->clone())
-            , dst_tag_(other.dst_tag_) {}
+            , dst_tag_(other.dst_tag_)
+            , name_(other.name_) {}
 
         ~pd_t() = default;
 
-        DECLARE_COMMON_PD_T(conv_pd_->name(), ref_deconvolution_bwd_weights_t);
+        DECLARE_COMMON_PD_T(name_.c_str(), ref_deconvolution_bwd_weights_t);
 
         status_t init_convolution(engine_t *engine) {
             using namespace types;
@@ -417,13 +449,12 @@ struct ref_deconvolution_bwd_weights_t : public primitive_t {
             if (status != status::success) return status;
             primitive_attr_t conv_attr(*attr());
             if (!conv_attr.is_initialized()) return status::out_of_memory;
-            conv_attr.set_scratchpad_mode(scratchpad_mode::user);
 
-            dnnl_primitive_desc_iterator it(
+            primitive_desc_iterator_t it(
                     engine, (op_desc_t *)&cd, &conv_attr, nullptr);
             if (!it.is_initialized()) return status::out_of_memory;
             while (++it != it.end()) {
-                conv_pd_.reset(it.fetch_once());
+                conv_pd_ = *it;
                 bool bf16_ref_deconv_supports_bias = IMPLICATION(with_bias()
                                 && desc()->src_desc.data_type
                                         == data_type::bf16,
@@ -443,47 +474,68 @@ struct ref_deconvolution_bwd_weights_t : public primitive_t {
         status_t init(engine_t *engine) {
             using namespace format_tag;
             using namespace data_type;
-            auto src_type = desc()->src_desc.data_type;
-            auto dwei_type = desc()->diff_weights_desc.data_type;
-            auto ddst_type = desc()->diff_dst_desc.data_type;
-            bool ok = true && desc()->prop_kind == prop_kind::backward_weights
-                    && (utils::everyone_is(f32, src_type, dwei_type, ddst_type)
-                            || (utils::one_of(dwei_type, f32, bf16)
-                                    && utils::everyone_is(
-                                            bf16, src_type, ddst_type)))
-                    && utils::one_of(desc()->alg_kind,
-                            alg_kind::deconvolution_direct,
-                            alg_kind::deconvolution_winograd)
-                    && attr()->has_default_values();
-
-            if (ok) {
-                CHECK(init_convolution(engine));
-                if (diff_weights_md_.format_kind == format_kind::any)
-                    CHECK(weights_axes_permutation(&diff_weights_md_,
-                            conv_pd_->diff_weights_md(), with_groups()));
-                if (src_md_.format_kind == format_kind::any)
-                    src_md_ = *conv_pd_->diff_dst_md();
-                if (diff_dst_md_.format_kind == format_kind::any)
-                    diff_dst_md_ = *conv_pd_->src_md();
-                if (diff_bias_md_.format_kind == format_kind::any)
-                    CHECK(memory_desc_init_by_tag(diff_bias_md_, x));
-
-                dst_tag_ = memory_desc_matches_one_of_tag(diff_dst_md_,
-                        utils::pick(ndims() - 3, ncw, nchw, ncdhw),
-                        utils::pick(ndims() - 3, nwc, nhwc, ndhwc),
-                        utils::pick(ndims() - 3, nCw8c, nChw8c, nCdhw8c),
-                        utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c));
-                init_scratchpad();
-                return status::success;
+            auto src_type = invariant_src_md()->data_type;
+            auto wei_type = invariant_wei_md(0)->data_type;
+            auto dst_type = invariant_dst_md()->data_type;
+            VDISPATCH_DECONVOLUTION(
+                    desc()->prop_kind == prop_kind::backward_weights,
+                    VERBOSE_BAD_PROPKIND);
+            VDISPATCH_DECONVOLUTION(utils::one_of(src_type, f32, bf16, f16),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_DECONVOLUTION(dst_type == src_type,
+                    VERBOSE_INCONSISTENT_DT, "diff_dst", "src");
+            VDISPATCH_DECONVOLUTION(utils::one_of(wei_type, src_type, f32),
+                    VERBOSE_UNSUPPORTED_DT);
+            // This implementation will check data types requirements through
+            // an underlying convolution implementation, however, convolution
+            // might be called without bias, thus, need to check bias data type
+            // if it was requested.
+            if (with_bias()) {
+                const auto bia_type = invariant_wei_md(1)->data_type;
+                VDISPATCH_DECONVOLUTION(utils::one_of(bia_type, f32, bf16, f16)
+                                && (bia_type == dst_type
+                                        || (bia_type == f32
+                                                && utils::one_of(
+                                                        dst_type, bf16, f16))),
+                        VERBOSE_UNSUPPORTED_DT);
             }
+            VDISPATCH_DECONVOLUTION(utils::one_of(desc()->alg_kind,
+                                            alg_kind::deconvolution_direct,
+                                            alg_kind::deconvolution_winograd),
+                    VERBOSE_BAD_ALGORITHM);
+            VDISPATCH_DECONVOLUTION(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
 
-            return status::unimplemented;
+            CHECK(init_convolution(engine));
+            if (diff_weights_md_.format_kind == format_kind::any)
+                CHECK(weights_axes_permutation(&diff_weights_md_,
+                        conv_pd_->diff_weights_md(), with_groups()));
+            if (src_md_.format_kind == format_kind::any)
+                src_md_ = *conv_pd_->diff_dst_md();
+            if (diff_dst_md_.format_kind == format_kind::any)
+                diff_dst_md_ = *conv_pd_->src_md();
+            if (diff_bias_md_.format_kind == format_kind::any)
+                CHECK(memory_desc_init_by_tag(diff_bias_md_, x));
+
+            dst_tag_ = memory_desc_matches_one_of_tag(diff_dst_md_,
+                    utils::pick(ndims() - 3, ncw, nchw, ncdhw),
+                    utils::pick(ndims() - 3, nwc, nhwc, ndhwc),
+                    utils::pick(ndims() - 3, nCw8c, nChw8c, nCdhw8c),
+                    utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c));
+
+            init_name();
+            init_scratchpad();
+            return status::success;
         }
 
-        std::unique_ptr<primitive_desc_t> conv_pd_;
+        std::shared_ptr<primitive_desc_t> conv_pd_;
         format_tag_t dst_tag_;
 
     private:
+        std::string name_ = "conv:any+"; // convolution-based deconvolution
+
+        void init_name() { name_.append(conv_pd_->name()); }
+
         void init_scratchpad() {
             auto scratchpad = scratchpad_registry().registrar();
             scratchpad.book(memory_tracking::names::key_nested,
