@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -38,7 +38,7 @@ namespace cpu {
                                                             : (f).off(n, c, d, \
                                                                     h, w))))
 
-template <impl::data_type_t data_type>
+template <data_type_t data_type>
 status_t ref_eltwise_fwd_t<data_type>::execute_forward_nCspBc_padded(
         const exec_ctx_t &ctx) const {
     status_t status = status::success;
@@ -46,13 +46,13 @@ status_t ref_eltwise_fwd_t<data_type>::execute_forward_nCspBc_padded(
     auto dst = CTX_OUT_CLEAN_MEM(data_t *, DNNL_ARG_DST, status);
     CHECK(status);
 
-    const memory_desc_wrapper data_d(pd()->data_md());
-    const blocking_desc_t &blk = data_d.blocking_desc();
+    const memory_desc_wrapper src_d(pd()->src_md());
+    const blocking_desc_t &blk = src_d.blocking_desc();
     const dim_t block = blk.inner_blks[0];
 
     const dim_t MB = pd()->MB();
     const dim_t C = pd()->C() / block;
-    const dim_t C_PADDED = data_d.padded_dims()[1] / block;
+    const dim_t C_PADDED = src_d.padded_dims()[1] / block;
     const dim_t tail = pd()->C() % block;
     const dim_t SP = pd()->D() * pd()->H() * pd()->W();
     const auto alg_kind = pd()->desc()->alg_kind;
@@ -61,7 +61,7 @@ status_t ref_eltwise_fwd_t<data_type>::execute_forward_nCspBc_padded(
 
     auto ker = [=](data_t &d, data_t s) {
         float res = compute_eltwise_scalar_fwd(alg_kind, s, alpha, beta);
-        d = cpu::saturate_and_round<data_t>(res);
+        d = cpu::q10n::saturate_and_round<data_t>(res);
     };
 
     parallel_nd(MB, C_PADDED, SP, [&](dim_t n, dim_t c, dim_t sp) {
@@ -78,7 +78,7 @@ status_t ref_eltwise_fwd_t<data_type>::execute_forward_nCspBc_padded(
     return status::success;
 }
 
-template <impl::data_type_t data_type>
+template <data_type_t data_type>
 status_t ref_eltwise_fwd_t<data_type>::execute_forward_generic(
         const exec_ctx_t &ctx) const {
     /* fast return */
@@ -89,7 +89,7 @@ status_t ref_eltwise_fwd_t<data_type>::execute_forward_generic(
     auto dst = CTX_OUT_CLEAN_MEM(data_t *, DNNL_ARG_DST, status);
     CHECK(status);
 
-    const memory_desc_wrapper data_d(pd()->data_md());
+    const memory_desc_wrapper src_d(pd()->src_md());
 
     const dim_t MB = pd()->MB();
     const dim_t C = pd()->C();
@@ -99,11 +99,11 @@ status_t ref_eltwise_fwd_t<data_type>::execute_forward_generic(
     const auto alg_kind = pd()->desc()->alg_kind;
     const float alpha = pd()->desc()->alpha;
     const float beta = pd()->desc()->beta;
-    const int ndims = pd()->desc()->data_desc.ndims;
+    const int ndims = pd()->ndims();
 
     parallel_nd(
             MB, C, D, H, W, [&](dim_t n, dim_t c, dim_t d, dim_t h, dim_t w) {
-                auto data_p_off = DATA_OFF(data_d, n, c, d, h, w);
+                auto data_p_off = DATA_OFF(src_d, n, c, d, h, w);
                 float res = compute_eltwise_scalar_fwd(
                         alg_kind, src[data_p_off], alpha, beta);
                 dim_t data_l_off = (((n * C + c) * D + d) * H + h) * W + w;
@@ -114,12 +114,12 @@ status_t ref_eltwise_fwd_t<data_type>::execute_forward_generic(
                 args.dst_md = pd()->dst_md();
                 ref_post_ops->execute(res, args);
 
-                dst[data_p_off] = cpu::saturate_and_round<data_t>(res);
+                dst[data_p_off] = cpu::q10n::saturate_and_round<data_t>(res);
             });
     return status::success;
 }
 
-template <impl::data_type_t data_type>
+template <data_type_t data_type>
 status_t ref_eltwise_fwd_t<data_type>::execute_forward_dense(
         const exec_ctx_t &ctx) const {
     status_t status = status::success;
@@ -127,41 +127,41 @@ status_t ref_eltwise_fwd_t<data_type>::execute_forward_dense(
     auto dst = CTX_OUT_CLEAN_MEM(data_t *, DNNL_ARG_DST, status);
     CHECK(status);
 
-    const memory_desc_wrapper data_d(pd()->data_md());
+    const memory_desc_wrapper src_d(pd()->src_md());
 
-    const auto nelems = data_d.nelems(true);
+    const auto nelems = src_d.nelems(true);
     const auto alg_kind = pd()->desc()->alg_kind;
     const float alpha = pd()->desc()->alpha;
     const float beta = pd()->desc()->beta;
 
-    src += data_d.offset0();
-    dst += data_d.offset0();
+    src += src_d.offset0();
+    dst += src_d.offset0();
 
     // a fast path for relu as the most popular activation
     if (alg_kind == alg_kind::eltwise_relu && alpha == 0) {
         parallel_nd(nelems, [&](dim_t e) {
             float res = math::relu_fwd(src[e], alpha);
-            dst[e] = cpu::saturate_and_round<data_t>(res);
+            dst[e] = cpu::q10n::saturate_and_round<data_t>(res);
         });
         return status::success;
     }
 
     parallel_nd(nelems, [&](dim_t e) {
         float res = compute_eltwise_scalar_fwd(alg_kind, src[e], alpha, beta);
-        dst[e] = cpu::saturate_and_round<data_t>(res);
+        dst[e] = cpu::q10n::saturate_and_round<data_t>(res);
     });
     return status::success;
 }
 
-template <impl::data_type_t data_type>
+template <data_type_t data_type>
 status_t ref_eltwise_bwd_t<data_type>::execute_backward_generic(
         const exec_ctx_t &ctx) const {
     /* fast return */
     if (pd()->has_zero_dim_memory()) return status::success;
 
     status_t status = status::success;
-    auto src = pd()->use_dst() ? CTX_IN_MEM(const data_t *, DNNL_ARG_DST)
-                               : CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
+    auto src = CTX_IN_MEM(
+            const data_t *, pd()->use_dst() ? DNNL_ARG_DST : DNNL_ARG_SRC);
     auto diff_dst = CTX_IN_MEM(const data_t *, DNNL_ARG_DIFF_DST);
     auto diff_src = CTX_OUT_CLEAN_MEM(data_t *, DNNL_ARG_DIFF_SRC, status);
     CHECK(status);
@@ -177,7 +177,7 @@ status_t ref_eltwise_bwd_t<data_type>::execute_backward_generic(
     const auto alg_kind = pd()->desc()->alg_kind;
     const float alpha = pd()->desc()->alpha;
     const float beta = pd()->desc()->beta;
-    const int ndims = pd()->desc()->data_desc.ndims;
+    const int ndims = pd()->ndims();
 
     parallel_nd(
             MB, C, D, H, W, [&](dim_t n, dim_t c, dim_t d, dim_t h, dim_t w) {
@@ -191,14 +191,14 @@ status_t ref_eltwise_bwd_t<data_type>::execute_backward_generic(
     return status::success;
 }
 
-template <>
-status_t ref_eltwise_bwd_t<data_type::f32>::execute_backward_dense(
+template <data_type_t data_type>
+status_t ref_eltwise_bwd_t<data_type>::execute_backward_dense(
         const exec_ctx_t &ctx) const {
     status_t status = status::success;
-    auto src = pd()->use_dst() ? CTX_IN_MEM(const data_t *, DNNL_ARG_DST)
-                               : CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
-    auto diff_dst = CTX_IN_MEM(const data_t *, DNNL_ARG_DIFF_DST);
-    auto diff_src = CTX_OUT_CLEAN_MEM(data_t *, DNNL_ARG_DIFF_SRC, status);
+    const void *src = pd()->use_dst() ? CTX_IN_MEM(const void *, DNNL_ARG_DST)
+                                      : CTX_IN_MEM(const void *, DNNL_ARG_SRC);
+    const void *diff_dst = CTX_IN_MEM(const void *, DNNL_ARG_DIFF_DST);
+    void *diff_src = CTX_OUT_CLEAN_MEM(void *, DNNL_ARG_DIFF_SRC, status);
     CHECK(status);
 
     const memory_desc_wrapper data_d(pd()->data_md());
@@ -209,77 +209,77 @@ status_t ref_eltwise_bwd_t<data_type::f32>::execute_backward_dense(
     const float alpha = pd()->desc()->alpha;
     const float beta = pd()->desc()->beta;
 
-    src += data_d.offset0();
-    diff_dst += diff_data_d.offset0();
-    diff_src += diff_data_d.offset0();
+    if (data_type == data_type::f32) {
+        const float *src_ptr = static_cast<const float *>(src);
+        const float *diff_dst_ptr = static_cast<const float *>(diff_dst);
+        float *diff_src_ptr = static_cast<float *>(diff_src);
 
-    parallel(0, [&](const int ithr, const int nthr) {
-        dim_t start = 0, end = 0;
-        balance211(nelems, nthr, ithr, start, end);
-        if (start == end) return;
+        src_ptr += data_d.offset0();
+        diff_dst_ptr += diff_data_d.offset0();
+        diff_src_ptr += diff_data_d.offset0();
 
-        for (dim_t i = start; i < end; i++) {
-            diff_src[i] = compute_eltwise_scalar_bwd(
-                    alg_kind, diff_dst[i], src[i], alpha, beta);
-        }
-    });
-    return status::success;
-}
+        parallel(0, [&](const int ithr, const int nthr) {
+            dim_t start = 0, end = 0;
+            balance211(nelems, nthr, ithr, start, end);
+            if (start == end) return;
 
-template <>
-status_t ref_eltwise_bwd_t<data_type::bf16>::execute_backward_dense(
-        const exec_ctx_t &ctx) const {
-    using namespace memory_tracking::names;
+            for (dim_t i = start; i < end; i++) {
+                diff_src_ptr[i] = compute_eltwise_scalar_bwd(
+                        alg_kind, diff_dst_ptr[i], src_ptr[i], alpha, beta);
+            }
+        });
+    } else if (utils::one_of(data_type, data_type::bf16, data_type::f16)) {
+        const data_t *src_ptr = static_cast<const data_t *>(src);
+        const data_t *diff_dst_ptr = static_cast<const data_t *>(diff_dst);
+        data_t *diff_src_ptr = static_cast<data_t *>(diff_src);
 
-    status_t status = status::success;
-    auto src = pd()->use_dst() ? CTX_IN_MEM(const data_t *, DNNL_ARG_DST)
-                               : CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
-    auto diff_dst = CTX_IN_MEM(const data_t *, DNNL_ARG_DIFF_DST);
-    auto diff_src = CTX_OUT_CLEAN_MEM(data_t *, DNNL_ARG_DIFF_SRC, status);
-    CHECK(status);
+        src_ptr += data_d.offset0();
+        diff_dst_ptr += diff_data_d.offset0();
+        diff_src_ptr += diff_data_d.offset0();
 
-    auto scratchpad = ctx.get_scratchpad_grantor();
-    auto s_f = scratchpad.template get<float>(key_eltwise_src);
-    auto dd_f = scratchpad.template get<float>(key_eltwise_diff_dst);
+        using namespace memory_tracking::names;
+        auto scratchpad = ctx.get_scratchpad_grantor();
+        auto *src_f32 = scratchpad.template get<float>(key_eltwise_src);
+        auto *diff_dst_f32
+                = scratchpad.template get<float>(key_eltwise_diff_dst);
 
-    const memory_desc_wrapper data_d(pd()->data_md());
-    const memory_desc_wrapper diff_data_d(pd()->diff_src_md());
+        parallel(0, [&](const int ithr, const int nthr) {
+            dim_t start = 0, end = 0;
+            balance211(nelems, nthr, ithr, start, end);
+            if (start == end) return;
 
-    const auto nelems = data_d.nelems(true);
-    const auto alg_kind = pd()->desc()->alg_kind;
-    const float alpha = pd()->desc()->alpha;
-    const float beta = pd()->desc()->beta;
+            types::cvt_to_float(src_f32 + start, src_ptr + start, end - start);
+            types::cvt_to_float(
+                    diff_dst_f32 + start, diff_dst_ptr + start, end - start);
 
-    src += data_d.offset0();
-    diff_dst += diff_data_d.offset0();
-    diff_src += diff_data_d.offset0();
+            for (dim_t i = start; i < end; i++) {
+                diff_dst_f32[i] = compute_eltwise_scalar_bwd(
+                        alg_kind, diff_dst_f32[i], src_f32[i], alpha, beta);
+            }
 
-    parallel(0, [&](const int ithr, const int nthr) {
-        dim_t start = 0, end = 0;
-        balance211(nelems, nthr, ithr, start, end);
-        if (start == end) return;
-
-        cvt_bfloat16_to_float(s_f + start, src + start, end - start);
-        cvt_bfloat16_to_float(dd_f + start, diff_dst + start, end - start);
-
-        for (dim_t i = start; i < end; i++) {
-            dd_f[i] = compute_eltwise_scalar_bwd(
-                    alg_kind, dd_f[i], s_f[i], alpha, beta);
-        }
-
-        cvt_float_to_bfloat16(diff_src + start, dd_f + start, end - start);
-    });
+            types::cvt_from_float(
+                    diff_src_ptr + start, diff_dst_f32 + start, end - start);
+        });
+    } else {
+        assert(!"unsupported data type");
+    }
     return status::success;
 }
 
 template struct ref_eltwise_fwd_t<data_type::f32>;
 template struct ref_eltwise_fwd_t<data_type::bf16>;
+template struct ref_eltwise_fwd_t<data_type::f16>;
 template struct ref_eltwise_fwd_t<data_type::s32>;
 template struct ref_eltwise_fwd_t<data_type::s8>;
 template struct ref_eltwise_fwd_t<data_type::u8>;
+template struct ref_eltwise_fwd_t<data_type::f8_e5m2>;
+template struct ref_eltwise_fwd_t<data_type::f8_e4m3>;
 
 template struct ref_eltwise_bwd_t<data_type::f32>;
 template struct ref_eltwise_bwd_t<data_type::bf16>;
+template struct ref_eltwise_bwd_t<data_type::f16>;
+template struct ref_eltwise_bwd_t<data_type::f8_e5m2>;
+template struct ref_eltwise_bwd_t<data_type::f8_e4m3>;
 
 } // namespace cpu
 } // namespace impl
