@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -43,13 +43,24 @@ struct ref_pooling_fwd_t : public primitive_t {
         status_t init(engine_t *engine) {
             using sm = primitive_attr_t::skip_mask_t;
 
-            bool ok = platform::has_data_type_support(data_type)
-                    && set_default_params() == status::success && is_fwd()
-                    && utils::everyone_is(
-                            data_type, src_md()->data_type, dst_md()->data_type)
-                    && desc()->accum_data_type == acc_type
-                    && attr()->has_default_values(sm::post_ops);
-            if (!ok) return status::unimplemented;
+            VDISPATCH_POOLING(platform::has_data_type_support(data_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(set_default_params() == status::success,
+                    VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_POOLING(is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_POOLING(utils::everyone_is(data_type, src_md()->data_type,
+                                      dst_md()->data_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(desc()->accum_data_type == acc_type,
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(attr()->has_default_values(sm::post_ops),
+                    VERBOSE_UNSUPPORTED_ATTR);
+            VDISPATCH_POOLING(
+                    ref_post_ops_t::primitive_kind_ok(attr()->post_ops_),
+                    VERBOSE_UNSUPPORTED_POSTOP);
+            VDISPATCH_POOLING(
+                    attr_.set_default_formats(dst_md(0)) == status::success,
+                    VERBOSE_UNSUPPORTED_POSTOP);
 
             bool is_training = desc_.prop_kind == prop_kind::forward_training;
             if (desc()->alg_kind == alg_kind::pooling_max && is_training)
@@ -65,6 +76,7 @@ struct ref_pooling_fwd_t : public primitive_t {
         ref_post_ops
                 = utils::make_unique<ref_post_ops_t>(pd()->attr()->post_ops_);
         if (!ref_post_ops) return status::out_of_memory;
+        CHECK(ref_post_ops->init(pd()->dst_md()));
         return status::success;
     }
 
@@ -81,7 +93,6 @@ private:
     std::unique_ptr<ref_post_ops_t> ref_post_ops;
 };
 
-template <impl::data_type_t data_type>
 struct ref_pooling_bwd_t : public primitive_t {
     struct pd_t : public cpu_pooling_bwd_pd_t {
         using cpu_pooling_bwd_pd_t::cpu_pooling_bwd_pd_t;
@@ -89,31 +100,56 @@ struct ref_pooling_bwd_t : public primitive_t {
         DECLARE_COMMON_PD_T("ref:any", ref_pooling_bwd_t);
 
         status_t init(engine_t *engine) {
-            bool ok = platform::has_data_type_support(data_type)
-                    && set_default_params() == status::success && !is_fwd()
-                    && utils::everyone_is(data_type, diff_dst_md()->data_type,
-                            diff_src_md()->data_type)
-                    && attr()->has_default_values();
-            if (!ok) return status::unimplemented;
+            using namespace data_type;
+            const auto diff_src_type = diff_src_md(0)->data_type;
+            const auto diff_dst_type = diff_dst_md(0)->data_type;
+
+            VDISPATCH_POOLING(!is_fwd(), VERBOSE_BAD_PROPKIND);
+            VDISPATCH_POOLING(platform::has_data_type_support(diff_src_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(platform::has_data_type_support(diff_dst_type),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(utils::one_of(diff_src_type, f32, bf16, f16),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(utils::one_of(diff_dst_type, f32, bf16, f16),
+                    VERBOSE_UNSUPPORTED_DT);
+            VDISPATCH_POOLING(set_default_params() == status::success,
+                    VERBOSE_UNSUPPORTED_TAG);
+            VDISPATCH_POOLING(
+                    attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
 
             if (desc()->alg_kind == alg_kind::pooling_max) {
-                init_default_ws();
-                if (!compare_ws(hint_fwd_pd_)) return status::unimplemented;
+                const auto ws_dt = hint_fwd_pd_->workspace_md()->data_type;
+                init_default_ws(ws_dt);
+                VDISPATCH_POOLING(
+                        compare_ws(hint_fwd_pd_), VERBOSE_WS_MISMATCH);
             }
 
+            nthr_ = dnnl_get_max_threads();
+            init_scratchpad();
+
             return status::success;
         }
+
+        int nthr_; // To not exceed the limit in execute used for set up.
+
+    private:
+        void init_scratchpad() {
+            using namespace memory_tracking::names;
+            auto scratchpad = scratchpad_registry().registrar();
+            if (diff_src_md()->data_type != data_type::f32) {
+                const memory_desc_wrapper diff_src_d(diff_src_md());
+                scratchpad.template book<float>(
+                        key_pool_src_bf16cvt, diff_src_d.nelems(true));
+            }
+        }
     };
 
     ref_pooling_bwd_t(const pd_t *apd) : primitive_t(apd) {}
-    typedef typename prec_traits<data_type>::type data_t;
 
-    status_t execute(const exec_ctx_t &ctx) const override {
-        return execute_backward(ctx);
-    }
+    status_t execute(const exec_ctx_t &ctx) const override;
 
 private:
-    status_t execute_backward(const exec_ctx_t &ctx) const;
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
 };
 
