@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #include <math.h>
 
 #include "common/c_types_map.hpp"
+#include "common/dnnl_thread.hpp"
 #include "common/nstl.hpp"
 
 #include "cpu/simple_q10n.hpp"
@@ -151,7 +152,7 @@ status_t ref_reduction_t<src_type, dst_type, acc_type>::execute_ref(
         args.dst_md = pd()->dst_md();
         ref_post_ops->execute(acc_f32, args);
 
-        dst[dst_off] = saturate_and_round<dst_t>(acc_f32);
+        dst[dst_off] = q10n::saturate_and_round<dst_t>(acc_f32);
     });
 
     return status::success;
@@ -161,12 +162,14 @@ using namespace data_type;
 template struct ref_reduction_t<f32, f32, f32>;
 template struct ref_reduction_t<bf16, bf16, f32>;
 template struct ref_reduction_t<bf16, f32, f32>;
+template struct ref_reduction_t<f16, f16, f32>;
+template struct ref_reduction_t<f16, f32, f32>;
 template struct ref_reduction_t<s8, s8, s32>;
 template struct ref_reduction_t<s8, s32, s32>;
-template struct ref_reduction_t<s8, f32, f32>;
+template struct ref_reduction_t<s8, f32, s32>;
 template struct ref_reduction_t<u8, u8, s32>;
 template struct ref_reduction_t<u8, s32, s32>;
-template struct ref_reduction_t<u8, f32, f32>;
+template struct ref_reduction_t<u8, f32, s32>;
 
 } // namespace cpu
 } // namespace impl
