@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -22,14 +22,21 @@ namespace cpu {
 
 // clang-format off
 
-const impl_list_map_t regular_f32_f16_impl_list_map {
-    // f32 -> f16
-    {{f32, f16, 0}, {
-        REG_SR(f32, any, f16, any, fmt_order::any, spec::reference),
+const impl_list_map_t &regular_f32_f16_impl_list_map() {
+    static const impl_list_map_t the_map = REG_REORDER_P({
+        // f32 -> f16
+        {{f32, f16, 0}, {
+            DNNL_X64_ONLY(CPU_REORDER_INSTANCE(x64::jit_uni_reorder_direct_copy_t))
+            DNNL_X64_ONLY(CPU_REORDER_INSTANCE(x64::jit_blk_reorder_t))
+            DNNL_X64_ONLY(CPU_REORDER_INSTANCE(x64::jit_uni_reorder_t))
 
-        nullptr,
-    }},
-};
+            REG_SR(f32, any, f16, any, fmt_order::any, spec::reference)
+
+            nullptr,
+        }},
+    });
+    return the_map;
+}
 
 // clang-format on
 
