@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ namespace cpu {
 namespace x64 {
 
 jit_avx2_f32_copy_an_kern::jit_avx2_f32_copy_an_kern()
-    : jit_generator(nullptr, F32_COPY_KERNEL_CODE_SIZE) {}
+    : jit_generator(jit_name()) {}
 
 void jit_avx2_f32_copy_an_kern::generate() {
 
@@ -60,120 +60,7 @@ void jit_avx2_f32_copy_an_kern::generate() {
 
     inLocalLabel();
     {
-
-        Xbyak::Label l1018;
-        Xbyak::Label l1048;
-        Xbyak::Label l104c;
-        Xbyak::Label l1070;
-        Xbyak::Label l1110;
-        Xbyak::Label l1164;
-        Xbyak::Label l1198;
-        Xbyak::Label l11b8;
-        Xbyak::Label l11bc;
-        Xbyak::Label l11e0;
-        Xbyak::Label l127c;
-        Xbyak::Label l12d0;
-        Xbyak::Label l1304;
-        Xbyak::Label l1324;
-        Xbyak::Label l1328;
-        Xbyak::Label l134c;
-        Xbyak::Label l13e8;
-        Xbyak::Label l143c;
-        Xbyak::Label l1470;
-        Xbyak::Label l1490;
-        Xbyak::Label l1494;
-        Xbyak::Label l14b8;
-        Xbyak::Label l1554;
-        Xbyak::Label l15a8;
-        Xbyak::Label l15dc;
-        Xbyak::Label l15fc;
-        Xbyak::Label l1600;
-        Xbyak::Label l1608;
-        Xbyak::Label l1614;
-        Xbyak::Label l162c;
-        Xbyak::Label l1808;
-        Xbyak::Label l18f4;
-        Xbyak::Label l196c;
-        Xbyak::Label l19ac;
-        Xbyak::Label l19bc;
-        Xbyak::Label l19e0;
-        Xbyak::Label l1b1c;
-        Xbyak::Label l1bbc;
-        Xbyak::Label l1c10;
-        Xbyak::Label l1c40;
-        Xbyak::Label l1c44;
-        Xbyak::Label l1c68;
-        Xbyak::Label l1d08;
-        Xbyak::Label l1d5c;
-        Xbyak::Label l1d90;
-        Xbyak::Label l1db0;
-        Xbyak::Label l1db4;
-        Xbyak::Label l1dd8;
-        Xbyak::Label l1e74;
-        Xbyak::Label l1e8;
-        Xbyak::Label l1ec8;
-        Xbyak::Label l1efc;
-        Xbyak::Label l1f1c;
-        Xbyak::Label l1f20;
-        Xbyak::Label l1f44;
-        Xbyak::Label l1fe0;
-        Xbyak::Label l2034;
-        Xbyak::Label l2068;
-        Xbyak::Label l2088;
-        Xbyak::Label l208c;
-        Xbyak::Label l20b0;
-        Xbyak::Label l214c;
-        Xbyak::Label l21a0;
-        Xbyak::Label l21d4;
-        Xbyak::Label l21f4;
-        Xbyak::Label l21f8;
-        Xbyak::Label l2a4;
-        Xbyak::Label l304;
-        Xbyak::Label l338;
-        Xbyak::Label l348;
-        Xbyak::Label l36c;
-        Xbyak::Label l468;
-        Xbyak::Label l4e4;
-        Xbyak::Label l528;
-        Xbyak::Label l54;
-        Xbyak::Label l550;
-        Xbyak::Label l554;
-        Xbyak::Label l574;
-        Xbyak::Label l5f0;
-        Xbyak::Label l634;
-        Xbyak::Label l660;
-        Xbyak::Label l67c;
-        Xbyak::Label l680;
-        Xbyak::Label l6a0;
-        Xbyak::Label l6c;
-        Xbyak::Label l718;
-        Xbyak::Label l75c;
-        Xbyak::Label l788;
-        Xbyak::Label l7a4;
-        Xbyak::Label l7a8;
-        Xbyak::Label l7c8;
-        Xbyak::Label l840;
-        Xbyak::Label l884;
-        Xbyak::Label l8b0;
-        Xbyak::Label l8cc;
-        Xbyak::Label l8d0;
-        Xbyak::Label l8f0;
-        Xbyak::Label l968;
-        Xbyak::Label l9ac;
-        Xbyak::Label l9d8;
-        Xbyak::Label l9f4;
-        Xbyak::Label l9f8;
-        Xbyak::Label la00;
-        Xbyak::Label la1c;
-        Xbyak::Label la34;
-        Xbyak::Label lc10;
-        Xbyak::Label lcfc;
-        Xbyak::Label ld74;
-        Xbyak::Label ldb4;
-        Xbyak::Label ldc4;
-        Xbyak::Label lde8;
-        Xbyak::Label lf24;
-        Xbyak::Label lfc4;
+        std::vector<Xbyak::Label> labels(113);
 
         preamble();
 #ifdef _WIN32
@@ -198,21 +85,21 @@ void jit_avx2_f32_copy_an_kern::generate() {
         vpslld(xmm4, xmm4, 0x1f);
         vperm2f128(ymm4, ymm4, ymm4, 0x20);
         vucomiss(xmm6, xmm3);
-        jne(la00, T_NEAR);
+        jne(labels[102], T_NEAR);
         cmp(N, 0x18);
-        jl(l348, T_NEAR);
+        jl(labels[69], T_NEAR);
         align(4);
 
-        L(l54);
+        L(labels[74]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x60);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1e8, T_NEAR);
+        jle(labels[49], T_NEAR);
         align(4);
 
-        L(l6c);
+        L(labels[84]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -265,12 +152,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -768);
         dec(I);
-        jg(l6c, T_NEAR);
+        jg(labels[84], T_NEAR);
         align(4);
 
-        L(l1e8);
+        L(labels[49]);
         test(M, 0x4);
-        jle(l2a4, T_NEAR);
+        jle(labels[66], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -299,9 +186,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -384);
         align(4);
 
-        L(l2a4);
+        L(labels[66]);
         test(M, 0x2);
-        jle(l304, T_NEAR);
+        jle(labels[67], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -318,9 +205,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -192);
         align(4);
 
-        L(l304);
+        L(labels[67]);
         test(M, 0x1);
-        jle(l338, T_NEAR);
+        jle(labels[68], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -330,24 +217,24 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -96);
         align(4);
 
-        L(l338);
+        L(labels[68]);
         sub(N, 0x18);
         cmp(N, 0x18);
-        jge(l54, T_NEAR);
+        jge(labels[74], T_NEAR);
         align(4);
 
-        L(l348);
+        L(labels[69]);
         cmp(N, 0x10);
-        jl(l554, T_NEAR);
+        jl(labels[76], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x40);
         mov(I, M);
         sar(I, 0x3);
-        jle(l468, T_NEAR);
+        jle(labels[71], T_NEAR);
         align(4);
 
-        L(l36c);
+        L(labels[70]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -384,12 +271,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -512);
         dec(I);
-        jg(l36c, T_NEAR);
+        jg(labels[70], T_NEAR);
         align(4);
 
-        L(l468);
+        L(labels[71]);
         test(M, 0x4);
-        jle(l4e4, T_NEAR);
+        jle(labels[72], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -410,9 +297,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -256);
         align(4);
 
-        L(l4e4);
+        L(labels[72]);
         test(M, 0x2);
-        jle(l528, T_NEAR);
+        jle(labels[73], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -425,9 +312,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l528);
+        L(labels[73]);
         test(M, 0x1);
-        jle(l550, T_NEAR);
+        jle(labels[75], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 - 0x60]);
@@ -435,22 +322,22 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l550);
+        L(labels[75]);
         sub(N, 0x10);
         align(4);
 
-        L(l554);
+        L(labels[76]);
         cmp(N, 0x8);
-        jl(l680, T_NEAR);
+        jl(labels[82], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x20);
         mov(I, M);
         sar(I, 0x3);
-        jle(l5f0, T_NEAR);
+        jle(labels[78], T_NEAR);
         align(4);
 
-        L(l574);
+        L(labels[77]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 + LDA * 1 - 0x80]);
@@ -471,12 +358,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -256);
         dec(I);
-        jg(l574, T_NEAR);
+        jg(labels[77], T_NEAR);
         align(4);
 
-        L(l5f0);
+        L(labels[78]);
         test(M, 0x4);
-        jle(l634, T_NEAR);
+        jle(labels[79], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 + LDA * 1 - 0x80]);
@@ -489,9 +376,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l634);
+        L(labels[79]);
         test(M, 0x2);
-        jle(l660, T_NEAR);
+        jle(labels[80], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         vmovups(ymm0, yword[A1 + LDA * 1 - 0x80]);
@@ -500,30 +387,30 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l660);
+        L(labels[80]);
         test(M, 0x1);
-        jle(l67c, T_NEAR);
+        jle(labels[81], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmovups(yword[B - 0x80], ymm0);
         sub(B, -32);
         align(4);
 
-        L(l67c);
+        L(labels[81]);
         sub(N, 0x8);
         align(4);
 
-        L(l680);
+        L(labels[82]);
         cmp(N, 0x4);
-        jl(l7a8, T_NEAR);
+        jl(labels[89], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x10);
         mov(I, M);
         sar(I, 0x3);
-        jle(l718, T_NEAR);
+        jle(labels[85], T_NEAR);
         align(4);
 
-        L(l6a0);
+        L(labels[83]);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmovups(xword[B - 0x80], xmm0);
         vmovups(xmm0, xword[A1 + LDA * 1 - 0x80]);
@@ -544,12 +431,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -128);
         dec(I);
-        jg(l6a0, T_NEAR);
+        jg(labels[83], T_NEAR);
         align(4);
 
-        L(l718);
+        L(labels[85]);
         test(M, 0x4);
-        jle(l75c, T_NEAR);
+        jle(labels[86], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmovups(xword[B - 0x80], xmm0);
         vmovups(xmm0, xword[A1 + LDA * 1 - 0x80]);
@@ -562,9 +449,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l75c);
+        L(labels[86]);
         test(M, 0x2);
-        jle(l788, T_NEAR);
+        jle(labels[87], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmovups(xword[B - 0x80], xmm0);
         vmovups(xmm0, xword[A1 + LDA * 1 - 0x80]);
@@ -573,30 +460,30 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l788);
+        L(labels[87]);
         test(M, 0x1);
-        jle(l7a4, T_NEAR);
+        jle(labels[88], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmovups(xword[B - 0x80], xmm0);
         sub(B, -16);
         align(4);
 
-        L(l7a4);
+        L(labels[88]);
         sub(N, 0x4);
         align(4);
 
-        L(l7a8);
+        L(labels[89]);
         cmp(N, 0x2);
-        jl(l8d0, T_NEAR);
+        jl(labels[95], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x8);
         mov(I, M);
         sar(I, 0x3);
-        jle(l840, T_NEAR);
+        jle(labels[91], T_NEAR);
         align(4);
 
-        L(l7c8);
+        L(labels[90]);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmovlps(qword[B - 0x80], xmm0);
         vmovsd(xmm0, qword[A1 + LDA * 1 - 0x80]);
@@ -617,12 +504,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -64);
         dec(I);
-        jg(l7c8, T_NEAR);
+        jg(labels[90], T_NEAR);
         align(4);
 
-        L(l840);
+        L(labels[91]);
         test(M, 0x4);
-        jle(l884, T_NEAR);
+        jle(labels[92], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmovlps(qword[B - 0x80], xmm0);
         vmovsd(xmm0, qword[A1 + LDA * 1 - 0x80]);
@@ -635,9 +522,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l884);
+        L(labels[92]);
         test(M, 0x2);
-        jle(l8b0, T_NEAR);
+        jle(labels[93], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmovlps(qword[B - 0x80], xmm0);
         vmovsd(xmm0, qword[A1 + LDA * 1 - 0x80]);
@@ -646,30 +533,30 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l8b0);
+        L(labels[93]);
         test(M, 0x1);
-        jle(l8cc, T_NEAR);
+        jle(labels[94], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmovlps(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(l8cc);
+        L(labels[94]);
         sub(N, 0x2);
         align(4);
 
-        L(l8d0);
+        L(labels[95]);
         cmp(N, 0x1);
-        jl(l9f8, T_NEAR);
+        jl(labels[101], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x4);
         mov(I, M);
         sar(I, 0x3);
-        jle(l968, T_NEAR);
+        jle(labels[97], T_NEAR);
         align(4);
 
-        L(l8f0);
+        L(labels[96]);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmovss(dword[B - 0x80], xmm0);
         vmovss(xmm0, dword[A1 + LDA * 1 - 0x80]);
@@ -690,12 +577,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -32);
         dec(I);
-        jg(l8f0, T_NEAR);
+        jg(labels[96], T_NEAR);
         align(4);
 
-        L(l968);
+        L(labels[97]);
         test(M, 0x4);
-        jle(l9ac, T_NEAR);
+        jle(labels[98], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmovss(dword[B - 0x80], xmm0);
         vmovss(xmm0, dword[A1 + LDA * 1 - 0x80]);
@@ -708,9 +595,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l9ac);
+        L(labels[98]);
         test(M, 0x2);
-        jle(l9d8, T_NEAR);
+        jle(labels[99], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmovss(dword[B - 0x80], xmm0);
         vmovss(xmm0, dword[A1 + LDA * 1 - 0x80]);
@@ -719,41 +606,41 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l9d8);
+        L(labels[99]);
         test(M, 0x1);
-        jle(l9f4, T_NEAR);
+        jle(labels[100], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmovss(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(l9f4);
+        L(labels[100]);
         sub(N, 0x1);
         align(4);
 
-        L(l9f8);
-        jmp(l21f8, T_NEAR);
+        L(labels[101]);
+        jmp(labels[65], T_NEAR);
         align(4);
 
-        L(la00);
+        L(labels[102]);
         vxorps(xmm3, xmm3, xmm4);
         vucomiss(xmm6, xmm3);
-        jne(l1608, T_NEAR);
+        jne(labels[27], T_NEAR);
         vmovaps(ymm6, ymm4);
         cmp(N, 0x18);
-        jl(ldc4, T_NEAR);
+        jl(labels[109], T_NEAR);
         align(4);
 
-        L(la1c);
+        L(labels[103]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x60);
         mov(I, M);
         sar(I, 0x3);
-        jle(lc10, T_NEAR);
+        jle(labels[105], T_NEAR);
         align(4);
 
-        L(la34);
+        L(labels[104]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -830,12 +717,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -768);
         dec(I);
-        jg(la34, T_NEAR);
+        jg(labels[104], T_NEAR);
         align(4);
 
-        L(lc10);
+        L(labels[105]);
         test(M, 0x4);
-        jle(lcfc, T_NEAR);
+        jle(labels[106], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -876,9 +763,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -384);
         align(4);
 
-        L(lcfc);
+        L(labels[106]);
         test(M, 0x2);
-        jle(ld74, T_NEAR);
+        jle(labels[107], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -901,9 +788,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -192);
         align(4);
 
-        L(ld74);
+        L(labels[107]);
         test(M, 0x1);
-        jle(ldb4, T_NEAR);
+        jle(labels[108], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -916,24 +803,24 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -96);
         align(4);
 
-        L(ldb4);
+        L(labels[108]);
         sub(N, 0x18);
         cmp(N, 0x18);
-        jge(la1c, T_NEAR);
+        jge(labels[103], T_NEAR);
         align(4);
 
-        L(ldc4);
+        L(labels[109]);
         cmp(N, 0x10);
-        jl(l104c, T_NEAR);
+        jl(labels[2], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x40);
         mov(I, M);
         sar(I, 0x3);
-        jle(lf24, T_NEAR);
+        jle(labels[111], T_NEAR);
         align(4);
 
-        L(lde8);
+        L(labels[110]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -986,12 +873,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -512);
         dec(I);
-        jg(lde8, T_NEAR);
+        jg(labels[110], T_NEAR);
         align(4);
 
-        L(lf24);
+        L(labels[111]);
         test(M, 0x4);
-        jle(lfc4, T_NEAR);
+        jle(labels[112], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1020,9 +907,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -256);
         align(4);
 
-        L(lfc4);
+        L(labels[112]);
         test(M, 0x2);
-        jle(l1018, T_NEAR);
+        jle(labels[0], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1039,9 +926,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l1018);
+        L(labels[0]);
         test(M, 0x1);
-        jle(l1048, T_NEAR);
+        jle(labels[1], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1051,22 +938,22 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l1048);
+        L(labels[1]);
         sub(N, 0x10);
         align(4);
 
-        L(l104c);
+        L(labels[2]);
         cmp(N, 0x8);
-        jl(l11bc, T_NEAR);
+        jl(labels[8], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x20);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1110, T_NEAR);
+        jle(labels[4], T_NEAR);
         align(4);
 
-        L(l1070);
+        L(labels[3]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1095,12 +982,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -256);
         dec(I);
-        jg(l1070, T_NEAR);
+        jg(labels[3], T_NEAR);
         align(4);
 
-        L(l1110);
+        L(labels[4]);
         test(M, 0x4);
-        jle(l1164, T_NEAR);
+        jle(labels[5], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1117,9 +1004,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l1164);
+        L(labels[5]);
         test(M, 0x2);
-        jle(l1198, T_NEAR);
+        jle(labels[6], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1130,31 +1017,31 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l1198);
+        L(labels[6]);
         test(M, 0x1);
-        jle(l11b8, T_NEAR);
+        jle(labels[7], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vxorps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
         sub(B, -32);
         align(4);
 
-        L(l11b8);
+        L(labels[7]);
         sub(N, 0x8);
         align(4);
 
-        L(l11bc);
+        L(labels[8]);
         cmp(N, 0x4);
-        jl(l1328, T_NEAR);
+        jl(labels[14], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x10);
         mov(I, M);
         sar(I, 0x3);
-        jle(l127c, T_NEAR);
+        jle(labels[10], T_NEAR);
         align(4);
 
-        L(l11e0);
+        L(labels[9]);
         vmovups(xmm0, xword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1183,12 +1070,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -128);
         dec(I);
-        jg(l11e0, T_NEAR);
+        jg(labels[9], T_NEAR);
         align(4);
 
-        L(l127c);
+        L(labels[10]);
         test(M, 0x4);
-        jle(l12d0, T_NEAR);
+        jle(labels[11], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1205,9 +1092,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l12d0);
+        L(labels[11]);
         test(M, 0x2);
-        jle(l1304, T_NEAR);
+        jle(labels[12], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1218,31 +1105,31 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l1304);
+        L(labels[12]);
         test(M, 0x1);
-        jle(l1324, T_NEAR);
+        jle(labels[13], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
         sub(B, -16);
         align(4);
 
-        L(l1324);
+        L(labels[13]);
         sub(N, 0x4);
         align(4);
 
-        L(l1328);
+        L(labels[14]);
         cmp(N, 0x2);
-        jl(l1494, T_NEAR);
+        jl(labels[20], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x8);
         mov(I, M);
         sar(I, 0x3);
-        jle(l13e8, T_NEAR);
+        jle(labels[16], T_NEAR);
         align(4);
 
-        L(l134c);
+        L(labels[15]);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1271,12 +1158,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -64);
         dec(I);
-        jg(l134c, T_NEAR);
+        jg(labels[15], T_NEAR);
         align(4);
 
-        L(l13e8);
+        L(labels[16]);
         test(M, 0x4);
-        jle(l143c, T_NEAR);
+        jle(labels[17], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1293,9 +1180,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l143c);
+        L(labels[17]);
         test(M, 0x2);
-        jle(l1470, T_NEAR);
+        jle(labels[18], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1306,31 +1193,31 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l1470);
+        L(labels[18]);
         test(M, 0x1);
-        jle(l1490, T_NEAR);
+        jle(labels[19], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(l1490);
+        L(labels[19]);
         sub(N, 0x2);
         align(4);
 
-        L(l1494);
+        L(labels[20]);
         cmp(N, 0x1);
-        jl(l1600, T_NEAR);
+        jl(labels[26], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x4);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1554, T_NEAR);
+        jle(labels[22], T_NEAR);
         align(4);
 
-        L(l14b8);
+        L(labels[21]);
         vmovss(xmm0, dword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1359,12 +1246,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -32);
         dec(I);
-        jg(l14b8, T_NEAR);
+        jg(labels[21], T_NEAR);
         align(4);
 
-        L(l1554);
+        L(labels[22]);
         test(M, 0x4);
-        jle(l15a8, T_NEAR);
+        jle(labels[23], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1381,9 +1268,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l15a8);
+        L(labels[23]);
         test(M, 0x2);
-        jle(l15dc, T_NEAR);
+        jle(labels[24], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -1394,38 +1281,38 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l15dc);
+        L(labels[24]);
         test(M, 0x1);
-        jle(l15fc, T_NEAR);
+        jle(labels[25], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vxorps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(l15fc);
+        L(labels[25]);
         sub(N, 0x1);
         align(4);
 
-        L(l1600);
-        jmp(l21f8, T_NEAR);
+        L(labels[26]);
+        jmp(labels[65], T_NEAR);
         align(4);
 
-        L(l1608);
+        L(labels[27]);
         cmp(N, 0x18);
-        jl(l19bc, T_NEAR);
+        jl(labels[34], T_NEAR);
         align(4);
 
-        L(l1614);
+        L(labels[28]);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x60);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1808, T_NEAR);
+        jle(labels[30], T_NEAR);
         align(4);
 
-        L(l162c);
+        L(labels[29]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1502,12 +1389,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -768);
         dec(I);
-        jg(l162c, T_NEAR);
+        jg(labels[29], T_NEAR);
         align(4);
 
-        L(l1808);
+        L(labels[30]);
         test(M, 0x4);
-        jle(l18f4, T_NEAR);
+        jle(labels[31], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1548,9 +1435,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -384);
         align(4);
 
-        L(l18f4);
+        L(labels[31]);
         test(M, 0x2);
-        jle(l196c, T_NEAR);
+        jle(labels[32], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1573,9 +1460,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -192);
         align(4);
 
-        L(l196c);
+        L(labels[32]);
         test(M, 0x1);
-        jle(l19ac, T_NEAR);
+        jle(labels[33], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1588,24 +1475,24 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -96);
         align(4);
 
-        L(l19ac);
+        L(labels[33]);
         sub(N, 0x18);
         cmp(N, 0x18);
-        jge(l1614, T_NEAR);
+        jge(labels[28], T_NEAR);
         align(4);
 
-        L(l19bc);
+        L(labels[34]);
         cmp(N, 0x10);
-        jl(l1c44, T_NEAR);
+        jl(labels[40], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x40);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1b1c, T_NEAR);
+        jle(labels[36], T_NEAR);
         align(4);
 
-        L(l19e0);
+        L(labels[35]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1658,12 +1545,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -512);
         dec(I);
-        jg(l19e0, T_NEAR);
+        jg(labels[35], T_NEAR);
         align(4);
 
-        L(l1b1c);
+        L(labels[36]);
         test(M, 0x4);
-        jle(l1bbc, T_NEAR);
+        jle(labels[37], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1692,9 +1579,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -256);
         align(4);
 
-        L(l1bbc);
+        L(labels[37]);
         test(M, 0x2);
-        jle(l1c10, T_NEAR);
+        jle(labels[38], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1711,9 +1598,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l1c10);
+        L(labels[38]);
         test(M, 0x1);
-        jle(l1c40, T_NEAR);
+        jle(labels[39], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1723,22 +1610,22 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l1c40);
+        L(labels[39]);
         sub(N, 0x10);
         align(4);
 
-        L(l1c44);
+        L(labels[40]);
         cmp(N, 0x8);
-        jl(l1db4, T_NEAR);
+        jl(labels[46], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x20);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1d08, T_NEAR);
+        jle(labels[42], T_NEAR);
         align(4);
 
-        L(l1c68);
+        L(labels[41]);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1767,12 +1654,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -256);
         dec(I);
-        jg(l1c68, T_NEAR);
+        jg(labels[41], T_NEAR);
         align(4);
 
-        L(l1d08);
+        L(labels[42]);
         test(M, 0x4);
-        jle(l1d5c, T_NEAR);
+        jle(labels[43], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1789,9 +1676,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -128);
         align(4);
 
-        L(l1d5c);
+        L(labels[43]);
         test(M, 0x2);
-        jle(l1d90, T_NEAR);
+        jle(labels[44], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
@@ -1802,31 +1689,31 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l1d90);
+        L(labels[44]);
         test(M, 0x1);
-        jle(l1db0, T_NEAR);
+        jle(labels[45], T_NEAR);
         vmovups(ymm0, yword[A1 - 0x80]);
         vmulps(ymm0, ymm6, ymm0);
         vmovups(yword[B - 0x80], ymm0);
         sub(B, -32);
         align(4);
 
-        L(l1db0);
+        L(labels[45]);
         sub(N, 0x8);
         align(4);
 
-        L(l1db4);
+        L(labels[46]);
         cmp(N, 0x4);
-        jl(l1f20, T_NEAR);
+        jl(labels[53], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x10);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1e74, T_NEAR);
+        jle(labels[48], T_NEAR);
         align(4);
 
-        L(l1dd8);
+        L(labels[47]);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1855,12 +1742,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -128);
         dec(I);
-        jg(l1dd8, T_NEAR);
+        jg(labels[47], T_NEAR);
         align(4);
 
-        L(l1e74);
+        L(labels[48]);
         test(M, 0x4);
-        jle(l1ec8, T_NEAR);
+        jle(labels[50], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1877,9 +1764,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l1ec8);
+        L(labels[50]);
         test(M, 0x2);
-        jle(l1efc, T_NEAR);
+        jle(labels[51], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
@@ -1890,31 +1777,31 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l1efc);
+        L(labels[51]);
         test(M, 0x1);
-        jle(l1f1c, T_NEAR);
+        jle(labels[52], T_NEAR);
         vmovups(xmm0, xword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovups(xword[B - 0x80], xmm0);
         sub(B, -16);
         align(4);
 
-        L(l1f1c);
+        L(labels[52]);
         sub(N, 0x4);
         align(4);
 
-        L(l1f20);
+        L(labels[53]);
         cmp(N, 0x2);
-        jl(l208c, T_NEAR);
+        jl(labels[59], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x8);
         mov(I, M);
         sar(I, 0x3);
-        jle(l1fe0, T_NEAR);
+        jle(labels[55], T_NEAR);
         align(4);
 
-        L(l1f44);
+        L(labels[54]);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1943,12 +1830,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -64);
         dec(I);
-        jg(l1f44, T_NEAR);
+        jg(labels[54], T_NEAR);
         align(4);
 
-        L(l1fe0);
+        L(labels[55]);
         test(M, 0x4);
-        jle(l2034, T_NEAR);
+        jle(labels[56], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1965,9 +1852,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l2034);
+        L(labels[56]);
         test(M, 0x2);
-        jle(l2068, T_NEAR);
+        jle(labels[57], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
@@ -1978,31 +1865,31 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l2068);
+        L(labels[57]);
         test(M, 0x1);
-        jle(l2088, T_NEAR);
+        jle(labels[58], T_NEAR);
         vmovsd(xmm0, qword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovlps(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(l2088);
+        L(labels[58]);
         sub(N, 0x2);
         align(4);
 
-        L(l208c);
+        L(labels[59]);
         cmp(N, 0x1);
-        jl(l21f8, T_NEAR);
+        jl(labels[65], T_NEAR);
         mov(A1, A);
         lea(A2, ptr[A1 + LDA * 4]);
         add(A, 0x4);
         mov(I, M);
         sar(I, 0x3);
-        jle(l214c, T_NEAR);
+        jle(labels[61], T_NEAR);
         align(4);
 
-        L(l20b0);
+        L(labels[60]);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -2031,12 +1918,12 @@ void jit_avx2_f32_copy_an_kern::generate() {
         lea(A2, ptr[A2 + LDA * 8]);
         sub(B, -32);
         dec(I);
-        jg(l20b0, T_NEAR);
+        jg(labels[60], T_NEAR);
         align(4);
 
-        L(l214c);
+        L(labels[61]);
         test(M, 0x4);
-        jle(l21a0, T_NEAR);
+        jle(labels[62], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -2053,9 +1940,9 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l21a0);
+        L(labels[62]);
         test(M, 0x2);
-        jle(l21d4, T_NEAR);
+        jle(labels[63], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
@@ -2066,20 +1953,20 @@ void jit_avx2_f32_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l21d4);
+        L(labels[63]);
         test(M, 0x1);
-        jle(l21f4, T_NEAR);
+        jle(labels[64], T_NEAR);
         vmovss(xmm0, dword[A1 - 0x80]);
         vmulps(xmm0, xmm6, xmm0);
         vmovss(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(l21f4);
+        L(labels[64]);
         sub(N, 0x1);
         align(4);
 
-        L(l21f8);
+        L(labels[65]);
 
         postamble();
     }
