@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 #include <atomic>
 #include <cmath>
+#include <memory>
 #include <mutex>
 
 #include "common/dnnl_thread.hpp"
@@ -25,6 +26,7 @@
 
 #include "cpu/gemm/f32/gemm_utils_f32.hpp"
 #include "cpu/gemm/f32/ref_gemm_f32.hpp"
+#include "cpu/gemm/gemm.hpp"
 #include "cpu/gemm/gemm_msan_unpoison.hpp"
 
 #include "cpu/x64/jit_generator.hpp"
@@ -42,9 +44,10 @@ namespace x64 {
 
 #define STACKSIZE get_size_of_abi_save_regs()
 #ifdef _WIN32
-#define STACK_K_CAPACITY 32
+#define STACK_CAPACITY 6400
 #else
-#define STACK_K_CAPACITY 2048
+// Roughly 4 4kB pages for kernel.
+#define STACK_CAPACITY (4 * PAGE_4K)
 #endif
 #define SIZE 4
 #define OFFSET 128
@@ -59,23 +62,16 @@ using namespace gemm_utils;
 struct xbyak_gemm_t : public jit_generator {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_avx512_common_gemm_f32_xbyak_gemm)
 
-    xbyak_gemm_t(char isTransA, char isTransB, float beta, bool hasBias = false,
-            void *code_ptr = nullptr,
-            size_t code_size = 80 * Xbyak::DEFAULT_MAX_CODE_SIZE)
-        : jit_generator(code_ptr, code_size)
+    xbyak_gemm_t(char isTransA, char isTransB, float beta, bool hasBias = false)
+        : jit_generator(jit_name())
         , isTransA(isTransA)
         , isTransB(isTransB)
         , beta(beta)
-        , hasBias(hasBias) {}
+        , hasBias(hasBias)
+        , STACK_K_CAPACITY((STACK_CAPACITY - 256) / (SIZE * UNROLL_M)) {}
 
     void generate() override {
         using namespace Xbyak;
-
-        enum {
-            ver_avx512_core,
-            ver_avx512_mic
-        } ver = mayiuse(avx512_core) ? ver_avx512_core : ver_avx512_mic;
-
         bool isBeta0 = (beta == 0.0);
         bool isBetaN = (!isBeta0 && beta != 1.0);
 
@@ -139,7 +135,7 @@ struct xbyak_gemm_t : public jit_generator {
         auto VBIAS2 = zmm2;
         auto VBIAS3 = zmm3;
 
-        auto PREFETCHSIZEA = ver == ver_avx512_core ? 48 : 80;
+        auto PREFETCHSIZEA = 48;
         auto PREFETCHSIZEB = 16;
 
         Zmm regs[] = {zmm8, zmm9, zmm10, zmm11, zmm12, zmm13, zmm14, zmm15,
@@ -513,150 +509,44 @@ struct xbyak_gemm_t : public jit_generator {
         // Loop with unroll_n - 2 FMAs; called by innerkernel
         auto fmaloop = [&](int unroll_m, int unroll_n, int iteration) {
             for (int i = 2; i < unroll_n; i++) {
-                if (ver == ver_avx512_core) {
-                    if (!isTransB) {
-                        switch (i) {
-                            case 2:
-                                vbroadcastss(zmm3,
-                                        ptr[BO1 + LDB * 2
-                                                + (iteration - OFFSET) * SIZE]);
-                                break;
-                            case 3:
-                                vbroadcastss(zmm3,
-                                        ptr[BO1 + LDB3
-                                                + (iteration - OFFSET) * SIZE]);
-                                break;
-                            case 4:
-                                vbroadcastss(zmm3,
-                                        ptr[BO2 + (iteration - OFFSET) * SIZE]);
-                                break;
-                            case 5:
-                                vbroadcastss(zmm3,
-                                        ptr[BO2 + LDB * 1
-                                                + (iteration - OFFSET) * SIZE]);
-                                break;
-                            case 6:
-                                vbroadcastss(zmm3,
-                                        ptr[BO2 + LDB * 2
-                                                + (iteration - OFFSET) * SIZE]);
-                                break;
-                            case 7:
-                                vbroadcastss(zmm3,
-                                        ptr[BO2 + LDB3
-                                                + (iteration - OFFSET) * SIZE]);
-                                break;
-                        }
-                    } else {
-                        vbroadcastss(zmm3, ptr[BO1 + (i - OFFSET) * SIZE]);
+                if (!isTransB) {
+                    switch (i) {
+                        case 2:
+                            vbroadcastss(zmm3,
+                                    ptr[BO1 + LDB * 2
+                                            + (iteration - OFFSET) * SIZE]);
+                            break;
+                        case 3:
+                            vbroadcastss(zmm3,
+                                    ptr[BO1 + LDB3
+                                            + (iteration - OFFSET) * SIZE]);
+                            break;
+                        case 4:
+                            vbroadcastss(zmm3,
+                                    ptr[BO2 + (iteration - OFFSET) * SIZE]);
+                            break;
+                        case 5:
+                            vbroadcastss(zmm3,
+                                    ptr[BO2 + LDB * 1
+                                            + (iteration - OFFSET) * SIZE]);
+                            break;
+                        case 6:
+                            vbroadcastss(zmm3,
+                                    ptr[BO2 + LDB * 2
+                                            + (iteration - OFFSET) * SIZE]);
+                            break;
+                        case 7:
+                            vbroadcastss(zmm3,
+                                    ptr[BO2 + LDB3
+                                            + (iteration - OFFSET) * SIZE]);
+                            break;
                     }
-                    vfmadd231ps(regs[i], zmm3, zmm0);
-                    if (unroll_m >= 32) vfmadd231ps(regs[i + 8], zmm3, zmm1);
-                    if (unroll_m >= 48) vfmadd231ps(regs[i + 16], zmm3, zmm2);
                 } else {
-                    if (!isTransB) {
-                        switch (i) {
-                            case 2:
-                                vfmadd231ps(regs[i], zmm0,
-                                        zword_b[BO1 + LDB * 2
-                                                + (iteration - OFFSET) * SIZE]);
-                                if (unroll_m >= 32)
-                                    vfmadd231ps(regs[i + 8], zmm1,
-                                            zword_b[BO1 + LDB * 2
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                if (unroll_m >= 48)
-                                    vfmadd231ps(regs[i + 16], zmm2,
-                                            zword_b[BO1 + LDB * 2
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                break;
-                            case 3:
-                                vfmadd231ps(regs[i], zmm0,
-                                        zword_b[BO1 + LDB3
-                                                + (iteration - OFFSET) * SIZE]);
-                                if (unroll_m >= 32)
-                                    vfmadd231ps(regs[i + 8], zmm1,
-                                            zword_b[BO1 + LDB3
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                if (unroll_m >= 48)
-                                    vfmadd231ps(regs[i + 16], zmm2,
-                                            zword_b[BO1 + LDB3
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                break;
-                            case 4:
-                                vfmadd231ps(regs[i], zmm0,
-                                        zword_b[BO2
-                                                + (iteration - OFFSET) * SIZE]);
-                                if (unroll_m >= 32)
-                                    vfmadd231ps(regs[i + 8], zmm1,
-                                            zword_b[BO2
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                if (unroll_m >= 48)
-                                    vfmadd231ps(regs[i + 16], zmm2,
-                                            zword_b[BO2
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                break;
-                            case 5:
-                                vfmadd231ps(regs[i], zmm0,
-                                        zword_b[BO2 + LDB * 1
-                                                + (iteration - OFFSET) * SIZE]);
-                                if (unroll_m >= 32)
-                                    vfmadd231ps(regs[i + 8], zmm1,
-                                            zword_b[BO2 + LDB * 1
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                if (unroll_m >= 48)
-                                    vfmadd231ps(regs[i + 16], zmm2,
-                                            zword_b[BO2 + LDB * 1
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                break;
-                            case 6:
-                                vfmadd231ps(regs[i], zmm0,
-                                        zword_b[BO2 + LDB * 2
-                                                + (iteration - OFFSET) * SIZE]);
-                                if (unroll_m >= 32)
-                                    vfmadd231ps(regs[i + 8], zmm1,
-                                            zword_b[BO2 + LDB * 2
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                if (unroll_m >= 48)
-                                    vfmadd231ps(regs[i + 16], zmm2,
-                                            zword_b[BO2 + LDB * 2
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                break;
-                            case 7:
-                                vfmadd231ps(regs[i], zmm0,
-                                        zword_b[BO2 + LDB3
-                                                + (iteration - OFFSET) * SIZE]);
-                                if (unroll_m >= 32)
-                                    vfmadd231ps(regs[i + 8], zmm1,
-                                            zword_b[BO2 + LDB3
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                if (unroll_m >= 48)
-                                    vfmadd231ps(regs[i + 16], zmm2,
-                                            zword_b[BO2 + LDB3
-                                                    + (iteration - OFFSET)
-                                                            * SIZE]);
-                                break;
-                        }
-                    } else {
-                        vfmadd231ps(regs[i], zmm0,
-                                zword_b[BO1 + (i - OFFSET) * SIZE]);
-                        if (unroll_m >= 32)
-                            vfmadd231ps(regs[i + 8], zmm1,
-                                    zword_b[BO1 + (i - OFFSET) * SIZE]);
-                        if (unroll_m >= 48)
-                            vfmadd231ps(regs[i + 16], zmm2,
-                                    zword_b[BO1 + (i - OFFSET) * SIZE]);
-                    }
+                    vbroadcastss(zmm3, ptr[BO1 + (i - OFFSET) * SIZE]);
                 }
+                vfmadd231ps(regs[i], zmm3, zmm0);
+                if (unroll_m >= 32) vfmadd231ps(regs[i + 8], zmm3, zmm1);
+                if (unroll_m >= 48) vfmadd231ps(regs[i + 16], zmm3, zmm2);
             }
         };
 
@@ -757,36 +647,14 @@ struct xbyak_gemm_t : public jit_generator {
                     add(AO1, LDA);
                 }
 
-                if (ver == ver_avx512_core) {
-                    if (!isTransB) {
-                        vbroadcastss(zmm3, ptr[BO1 + (i - OFFSET) * SIZE]);
-                    } else {
-                        vbroadcastss(zmm3, ptr[BO1 + (0 - OFFSET) * SIZE]);
-                    }
-                    vfmadd231ps(regs[0], zmm3, zmm0);
-                    if (unroll_m >= 32) vfmadd231ps(regs[0 + 8], zmm3, zmm1);
-                    if (unroll_m >= 48) vfmadd231ps(regs[0 + 16], zmm3, zmm2);
+                if (!isTransB) {
+                    vbroadcastss(zmm3, ptr[BO1 + (i - OFFSET) * SIZE]);
                 } else {
-                    if (!isTransB) {
-                        vfmadd231ps(regs[0], zmm0,
-                                zword_b[BO1 + (i - OFFSET) * SIZE]);
-                        if (unroll_m >= 32)
-                            vfmadd231ps(regs[0 + 8], zmm1,
-                                    zword_b[BO1 + (i - OFFSET) * SIZE]);
-                        if (unroll_m >= 48)
-                            vfmadd231ps(regs[0 + 16], zmm2,
-                                    zword_b[BO1 + (i - OFFSET) * SIZE]);
-                    } else {
-                        vfmadd231ps(regs[0], zmm0,
-                                zword_b[BO1 + (0 - OFFSET) * SIZE]);
-                        if (unroll_m >= 32)
-                            vfmadd231ps(regs[0 + 8], zmm1,
-                                    zword_b[BO1 + (0 - OFFSET) * SIZE]);
-                        if (unroll_m >= 48)
-                            vfmadd231ps(regs[0 + 16], zmm2,
-                                    zword_b[BO1 + (0 - OFFSET) * SIZE]);
-                    }
+                    vbroadcastss(zmm3, ptr[BO1 + (0 - OFFSET) * SIZE]);
                 }
+                vfmadd231ps(regs[0], zmm3, zmm0);
+                if (unroll_m >= 32) vfmadd231ps(regs[0 + 8], zmm3, zmm1);
+                if (unroll_m >= 48) vfmadd231ps(regs[0 + 16], zmm3, zmm2);
 
                 if (unroll_n >= i + 1) {
                     if (!isTransB) {
@@ -828,42 +696,15 @@ struct xbyak_gemm_t : public jit_generator {
                 }
 
                 if (unroll_n >= 2) {
-                    if (ver == ver_avx512_core) {
-                        if (!isTransB) {
-                            vbroadcastss(zmm3,
-                                    ptr[BO1 + LDB * 1 + (i - OFFSET) * SIZE]);
-                        } else {
-                            vbroadcastss(zmm3, ptr[BO1 + (1 - OFFSET) * SIZE]);
-                        }
-                        vfmadd231ps(regs[1], zmm3, zmm0);
-                        if (unroll_m >= 32)
-                            vfmadd231ps(regs[1 + 8], zmm3, zmm1);
-                        if (unroll_m >= 48)
-                            vfmadd231ps(regs[1 + 16], zmm3, zmm2);
+                    if (!isTransB) {
+                        vbroadcastss(
+                                zmm3, ptr[BO1 + LDB * 1 + (i - OFFSET) * SIZE]);
                     } else {
-                        if (!isTransB) {
-                            vfmadd231ps(regs[1], zmm0,
-                                    zword_b[BO1 + LDB * 1
-                                            + (i - OFFSET) * SIZE]);
-                            if (unroll_m >= 32)
-                                vfmadd231ps(regs[1 + 8], zmm1,
-                                        zword_b[BO1 + LDB * 1
-                                                + (i - OFFSET) * SIZE]);
-                            if (unroll_m >= 48)
-                                vfmadd231ps(regs[1 + 16], zmm2,
-                                        zword_b[BO1 + LDB * 1
-                                                + (i - OFFSET) * SIZE]);
-                        } else {
-                            vfmadd231ps(regs[1], zmm0,
-                                    zword_b[BO1 + (1 - OFFSET) * SIZE]);
-                            if (unroll_m >= 32)
-                                vfmadd231ps(regs[1 + 8], zmm1,
-                                        zword_b[BO1 + (1 - OFFSET) * SIZE]);
-                            if (unroll_m >= 48)
-                                vfmadd231ps(regs[1 + 16], zmm2,
-                                        zword_b[BO1 + (1 - OFFSET) * SIZE]);
-                        }
+                        vbroadcastss(zmm3, ptr[BO1 + (1 - OFFSET) * SIZE]);
                     }
+                    vfmadd231ps(regs[1], zmm3, zmm0);
+                    if (unroll_m >= 32) vfmadd231ps(regs[1 + 8], zmm3, zmm1);
+                    if (unroll_m >= 48) vfmadd231ps(regs[1 + 16], zmm3, zmm2);
                 }
 
                 if (isCopy) {
@@ -909,43 +750,24 @@ struct xbyak_gemm_t : public jit_generator {
                 fmaloop(unroll_m, unroll_n, i);
 
                 if (i == 1) {
-                    if (doCPrefetch) {
-                        if (ver == ver_avx512_core)
-                            prefetchw(ptr[CO2 + 0 * 16 * SIZE]);
-                        else
-                            prefetcht0(ptr[CO2 + 0 * 16 * SIZE]);
-                    }
+                    if (doCPrefetch) { prefetchw(ptr[CO2 + 0 * 16 * SIZE]); }
                 }
                 if (i == 3) {
                     if (doCPrefetch && unroll_m >= 32) {
-                        if (ver == ver_avx512_core)
-                            prefetchw(ptr[CO2 + 1 * 16 * SIZE]);
-                        else
-                            prefetcht0(ptr[CO2 + 1 * 16 * SIZE]);
-                    }
-                    if (!isTransA) {
-                        if (ver == ver_avx512_core)
-                            prefetcht0(ptr[AA + 16 * 0 * SIZE]);
-                        else
-                            prefetcht2(ptr[AA + 16 * 0 * SIZE]);
+                        prefetchw(ptr[CO2 + 1 * 16 * SIZE]);
                     }
+                    if (!isTransA) { prefetcht0(ptr[AA + 16 * 0 * SIZE]); }
                 }
                 if (i == 5) {
                     if (doCPrefetch) {
                         if (unroll_m >= 48) {
-                            if (ver == ver_avx512_core)
-                                prefetchw(ptr[CO2 + 2 * 16 * SIZE]);
-                            else
-                                prefetcht0(ptr[CO2 + 2 * 16 * SIZE]);
+                            prefetchw(ptr[CO2 + 2 * 16 * SIZE]);
                         }
                         add(CO2, LDC);
                     }
                     if (!isTransA) {
                         if (unroll_m >= 32) {
-                            if (ver == ver_avx512_core)
-                                prefetcht0(ptr[AA + 16 * 1 * SIZE]);
-                            else
-                                prefetcht2(ptr[AA + 16 * 1 * SIZE]);
+                            prefetcht0(ptr[AA + 16 * 1 * SIZE]);
                         }
                     }
                 }
@@ -961,12 +783,7 @@ struct xbyak_gemm_t : public jit_generator {
                 if (unroll_n >= 4) sub(BO2, -8 * SIZE);
             }
             if (!isTransA) {
-                if (unroll_m >= 48) {
-                    if (ver == ver_avx512_core)
-                        prefetcht0(ptr[AA + 16 * 2 * SIZE]);
-                    else
-                        prefetcht2(ptr[AA + 16 * 2 * SIZE]);
-                }
+                if (unroll_m >= 48) { prefetcht0(ptr[AA + 16 * 2 * SIZE]); }
                 lea(AA, ptr[AA + LDA]);
             }
 
@@ -1024,7 +841,7 @@ struct xbyak_gemm_t : public jit_generator {
             if (isCopy) {
                 lea(LDA4, ptr[rsp + 128 + OFFSET * SIZE]);
             } else {
-                auto step = ver == ver_avx512_core ? 2 : 4;
+                auto step = 2;
                 lea(LDA4, ptr[LDA * step + (16 - 1 - OFFSET) * SIZE]);
             }
 
@@ -1682,7 +1499,7 @@ struct xbyak_gemm_t : public jit_generator {
         // needed
         mov(rax, LDA);
         or_(rax, A);
-        and_(rax, ver == ver_avx512_core ? 0x07 : 0x3f);
+        and_(rax, 0x07);
         mov(FLAG, rax);
 
         for (int i = 8; i < 16; i++) {
@@ -1726,11 +1543,15 @@ struct xbyak_gemm_t : public jit_generator {
         postamble();
     }
 
+public:
+    dim_t stack_k_capacity() const { return STACK_K_CAPACITY; }
+
 private:
     const char isTransA;
     const char isTransB;
     const float beta;
     const bool hasBias;
+    const dim_t STACK_K_CAPACITY;
 };
 
 xbyak_gemm_t *get_xbyak_gemm(
@@ -1740,9 +1561,9 @@ xbyak_gemm_t *get_xbyak_gemm(
     };
 
     // Kernel table [isTransA][isTransB][hasBias][beta (0, 1, other)]
-    static xbyak_gemm_t *kernel_table[2][2][2][3];
+    static maybe_unique_ptr<xbyak_gemm_t> kernel_table[2][2][2][3];
     static std::once_flag initialized;
-    dnnl_status_t st = dnnl_success;
+    static std::atomic<dnnl_status_t> st(dnnl_success);
     std::call_once(initialized, [&] {
         for (bool isTransA : {false, true})
             for (bool isTransB : {false, true})
@@ -1753,8 +1574,8 @@ xbyak_gemm_t *get_xbyak_gemm(
                         auto &kern = kernel_table[isTransA][isTransB][hasBias]
                                                  [beta_idx(beta)];
 
-                        kern = new xbyak_gemm_t(
-                                isTransA, isTransB, beta, hasBias);
+                        kern.reset(new xbyak_gemm_t(
+                                isTransA, isTransB, beta, hasBias));
                         if (kern->create_kernel() != dnnl_success) {
                             st = dnnl_runtime_error;
                             return;
@@ -1763,14 +1584,14 @@ xbyak_gemm_t *get_xbyak_gemm(
     });
 
     return (st == dnnl_success)
-            ? kernel_table[isTransA][isTransB][hasBias][beta_idx(beta)]
+            ? kernel_table[isTransA][isTransB][hasBias][beta_idx(beta)].get()
             : nullptr;
 }
 
 dnnl_status_t sgemm_nocopy_driver(const char *transa, const char *transb,
         dim_t m, dim_t n, dim_t k, const float *alpha, const float *a,
         dim_t lda, const float *b, dim_t ldb, const float *beta, float *c,
-        dim_t ldc, const float *bias, float *ws) {
+        dim_t ldc, const float *bias) {
 
     bool isTransA = (*transa == 'T' || *transa == 't');
     bool isTransB = (*transb == 'T' || *transb == 't');
@@ -1806,7 +1627,7 @@ dnnl_status_t sgemm_nocopy_driver(const char *transa, const char *transb,
     if (utils::any_null(ker_bn, ker_b1, ker_b0)) return dnnl_runtime_error;
 
     dim_t BM = 4032, BN, BK;
-    if (mayiuse(avx512_core)) {
+    if (mayiuse(avx512_core) && __BUILD_GEMM_AVX512) {
         BN = isTransA ? 384 : 64;
         BK = 384;
     } else {
@@ -1814,6 +1635,19 @@ dnnl_status_t sgemm_nocopy_driver(const char *transa, const char *transb,
         BK = isTransB ? 96 : 192;
         if (!isTransA && !isTransB) BK = 128;
     }
+
+    float *ws = nullptr;
+    bool use_heap_mem = BK > ker_b1->stack_k_capacity();
+    if (use_heap_mem) {
+        // Kernel uses sizeK * unroll_m + 64 + unroll_m elements as workspace.
+        const dim_t max_sizeK = BK;
+        const size_t ws_size
+                = sizeof *ws * (max_sizeK * UNROLL_M + 64 + UNROLL_M);
+
+        ws = (float *)malloc(ws_size, PAGE_4K);
+        if (!ws) return dnnl_out_of_memory;
+    }
+
     const float *curA, *curB, *curBias = nullptr;
     float *curC;
 
@@ -1873,6 +1707,8 @@ dnnl_status_t sgemm_nocopy_driver(const char *transa, const char *transb,
             }
         }
     }
+
+    free(ws);
     msan_unpoison_matrix(c, m, n, ldc, sizeof(*c));
 
     return dnnl_success;
@@ -1880,11 +1716,11 @@ dnnl_status_t sgemm_nocopy_driver(const char *transa, const char *transb,
 
 } // namespace avx512_common_gemm_f32
 
-dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
-        const dim_t *p_m, const dim_t *p_n, const dim_t *p_k,
-        const float *p_alpha, const float *A, const dim_t *p_lda,
-        const float *B, const dim_t *p_ldb, const float *p_beta, float *C,
-        const dim_t *p_ldc, const float *bias) {
+dnnl_status_t jit_avx512_common_gemm_f32(int nthrs, const char *transa,
+        const char *transb, const dim_t *p_m, const dim_t *p_n,
+        const dim_t *p_k, const float *p_alpha, const float *A,
+        const dim_t *p_lda, const float *B, const dim_t *p_ldb,
+        const float *p_beta, float *C, const dim_t *p_ldc, const float *bias) {
 
     using namespace dnnl::impl::utils;
     using namespace avx512_common_gemm_f32;
@@ -1894,7 +1730,8 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
         return ref_gemm(transa, transb, p_m, p_n, p_k, p_alpha, A, p_lda, B,
                 p_lda, p_beta, C, p_ldc, bias);
 
-    int nthr_to_use = (dnnl_in_parallel()) ? 1 : dnnl_get_max_threads();
+    int nthr_max = dnnl_get_current_num_threads();
+    int nthr_to_use = nstl::min(nthrs, nthr_max);
 
     dim_t m = *p_m;
     dim_t n = *p_n;
@@ -1912,9 +1749,7 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
             m, n, k, nthr_to_use, &nthr_m, &nthr_n, &nthr_k, &MB, &NB, &KB);
     assert(IMPLICATION(!dnnl_thr_syncable(), nthr_k == 1));
 
-    // May not happen, but just in case
-    if (nthr_to_use < nthr_m * nthr_n * nthr_k)
-        nthr_to_use = nthr_m * nthr_n * nthr_k;
+    nthr_to_use = nthr_m * nthr_n * nthr_k;
 
     nthr_mn = nthr_m * nthr_n;
 
@@ -1922,7 +1757,6 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
     unsigned char volatile *ompstatus = nullptr;
 
     float *c_buffers = nullptr;
-    float *ws_buffers = nullptr;
 
     if (nthr_k > 1) {
         ompstatus_ = (unsigned char *)malloc(
@@ -1944,23 +1778,19 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
         }
     }
 
-    const size_t ws_elems_per_thr
-            = (size_t)rnd_up(div_up(k, nthr_k), KB) * 48 + 64;
-    const size_t ws_size_per_thr
-            = rnd_up(ws_elems_per_thr * sizeof(float), PAGE_4K);
-    if (k > STACK_K_CAPACITY) {
-        ws_buffers = (float *)malloc(nthr_to_use * ws_size_per_thr, PAGE_4K);
-        if (!ws_buffers) {
-            free(ompstatus_);
-            free(c_buffers);
-            return dnnl_out_of_memory;
-        }
+    if (nthr_to_use == 1) {
+        auto status = sgemm_nocopy_driver(transa, transb, m, n, k, p_alpha, A,
+                lda, B, ldb, p_beta, C, ldc, bias);
+        return status;
     }
 
-    std::atomic<dnnl_status_t> st(dnnl_success);
+    // Always use the maximum number of threads to avoid OMP overhead that can
+    // occur due to change thread counts.
+    int nthr_spawn = dnnl_thr_syncable() ? nthr_max : nthr_to_use;
 
-    parallel(nthr_to_use, [&](int ithr, int nthr) {
-        assert(nthr == nthr_to_use);
+    std::atomic<dnnl_status_t> st(dnnl_success);
+    parallel(nthr_spawn, [&](int ithr, int nthr) {
+        assert(nthr == nthr_spawn);
         MAYBE_UNUSED(nthr);
 
         int ithr_m, ithr_n, ithr_k, ithr_mn;
@@ -1970,9 +1800,6 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
         int cbase, ibase;
         const float *myA, *myB, *myBias = nullptr;
         float *myC = C, myBeta;
-        float *ws = ws_buffers
-                ? ws_buffers + ithr * ws_size_per_thr / sizeof(float)
-                : nullptr;
         dim_t ld = ldc;
 
         int sum_later = nthr < nthr_m * nthr_n * nthr_k;
@@ -2034,7 +1861,7 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
 
                 dnnl_status_t st_thr = sgemm_nocopy_driver(transa, transb, myM,
                         myN, myK, p_alpha, myA, lda, myB, ldb, &myBeta, myC, ld,
-                        myBias, ws);
+                        myBias);
                 if (st_thr != dnnl_success) {
                     st = st_thr;
                     return;
@@ -2078,13 +1905,17 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
         }
     });
 
-    CHECK(st);
+    if (st != dnnl_success) {
+        free(ompstatus_);
+        free(c_buffers);
+        return st;
+    }
 
     // handle C summation later
     if (nthr_k > 1 && ompstatus[0] == 0) {
 
-        parallel(nthr_to_use, [&](int ithr, int nthr) {
-            assert(nthr == nthr_to_use);
+        parallel(nthr_spawn, [&](int ithr, int nthr) {
+            assert(nthr == nthr_spawn);
             MAYBE_UNUSED(nthr);
 
             int ithr_m, ithr_n, ithr_k, ithr_mn;
@@ -2151,7 +1982,6 @@ dnnl_status_t jit_avx512_common_gemm_f32(const char *transa, const char *transb,
 
     free(c_buffers);
     free(ompstatus_);
-    free(ws_buffers);
 
     return dnnl_success;
 }
