@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -82,7 +82,7 @@ static inline bool use_reference_igemm(void) {
             && data_traits<a_dt>::data_type == data_type::s8
             && data_traits<b_dt>::data_type == data_type::u8;
     if (is_s8u8)
-        return !mayiuse(sse41) || mayiuse(avx512_mic);
+        return !mayiuse(sse41);
     else
         return !mayiuse(avx512_core);
 }
@@ -319,7 +319,7 @@ dnnl_status_t sgemm_pack(const char *identifier, const char *transa,
             *K, *alpha, src, ld, dst);
     return dnnl_success;
 #else
-    gemm_pack_storage_t pack_dst {dst};
+    gemm_pack_storage_t pack_dst(dst, false);
 
     return gemm_pack_driver<float, float, float>(identifier, transa, transb, M,
             N, K, alpha, lda, ldb, src, &pack_dst, false);
@@ -338,7 +338,7 @@ dnnl_status_t gemm_bf16bf16f32_pack(const char *identifier, const char *transa,
             identifier, transa, transb, M, N, K, alpha, lda, ldb, src, dst);
     if (result != dnnl_success) return result;
 
-    gemm_pack_storage_t pack_dst {dst};
+    gemm_pack_storage_t pack_dst(dst, false);
 
     return gemm_pack_driver<bfloat16_t, bfloat16_t, float>(identifier, transa,
             transb, M, N, K, alpha, lda, ldb, src, &pack_dst, false);
@@ -368,7 +368,7 @@ dnnl_status_t gemm_x8x8s32_pack(const char *identifier, const char *transa,
         return dnnl_success;
     }
 #endif
-    gemm_pack_storage_t pack_dst {dst};
+    gemm_pack_storage_t pack_dst(dst, false);
 
     if (!use_reference_igemm<a_dt, b_dt>()) {
         return gemm_pack_driver<a_dt, b_dt, int32_t>(identifier, transa, transb,
@@ -476,8 +476,20 @@ dnnl_status_t gemm_x8x8s32_compute(const char *transa, const char *transb,
     auto transa_eff = *transa, transb_eff = *transb;
 
     if (!use_reference_igemm<a_dt, b_dt>()) {
-        return gemm_s8x8s32(&transa_eff, &transb_eff, offsetc, M, N, K, alpha,
-                A, &lda_eff, ao, B, &ldb_eff, bo, beta, C, ldc, co);
+        if (std::is_same<b_dt, int8_t>::value) {
+            return gemm_s8s8s32(&transa_eff, &transb_eff, offsetc, M, N, K,
+                    alpha, A, &lda_eff, ao, reinterpret_cast<const int8_t *>(B),
+                    &ldb_eff, reinterpret_cast<const int8_t *>(bo), beta, C,
+                    ldc, co);
+        } else if (std::is_same<b_dt, uint8_t>::value) {
+            return gemm_s8u8s32(&transa_eff, &transb_eff, offsetc, M, N, K,
+                    alpha, A, &lda_eff, ao,
+                    reinterpret_cast<const uint8_t *>(B), &ldb_eff,
+                    reinterpret_cast<const uint8_t *>(bo), beta, C, ldc, co);
+        } else {
+            assert(!"unexpected b_dt");
+            return status::runtime_error;
+        }
     } else {
         dim_t ld, td;
 
@@ -501,8 +513,20 @@ dnnl_status_t gemm_x8x8s32_compute(const char *transa, const char *transb,
             transb_eff = trans == no_trans ? 'N' : 'T';
         }
 
-        return gemm_s8x8s32(&transa_eff, &transb_eff, offsetc, M, N, K, alpha,
-                A, &lda_eff, ao, B, &ldb_eff, bo, beta, C, ldc, co);
+        if (std::is_same<b_dt, int8_t>::value) {
+            return gemm_s8s8s32(&transa_eff, &transb_eff, offsetc, M, N, K,
+                    alpha, A, &lda_eff, ao, reinterpret_cast<const int8_t *>(B),
+                    &ldb_eff, reinterpret_cast<const int8_t *>(bo), beta, C,
+                    ldc, co);
+        } else if (std::is_same<b_dt, uint8_t>::value) {
+            return gemm_s8u8s32(&transa_eff, &transb_eff, offsetc, M, N, K,
+                    alpha, A, &lda_eff, ao,
+                    reinterpret_cast<const uint8_t *>(B), &ldb_eff,
+                    reinterpret_cast<const uint8_t *>(bo), beta, C, ldc, co);
+        } else {
+            assert(!"unexpected b_dt");
+            return status::runtime_error;
+        }
     }
 }
 
