@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ namespace cpu {
 namespace x64 {
 
 jit_avx512_core_u8_copy_an_kern::jit_avx512_core_u8_copy_an_kern()
-    : jit_generator(nullptr, U8_COPY_KERNEL_CODE_SIZE) {}
+    : jit_generator(jit_name()) {}
 
 void jit_avx512_core_u8_copy_an_kern::generate() {
 
@@ -62,53 +62,7 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
 
     inLocalLabel();
     {
-
-        Xbyak::Label l170;
-        Xbyak::Label l1f0;
-        Xbyak::Label l20;
-        Xbyak::Label l224;
-        Xbyak::Label l234;
-        Xbyak::Label l240;
-        Xbyak::Label l254;
-        Xbyak::Label l32c;
-        Xbyak::Label l34;
-        Xbyak::Label l388;
-        Xbyak::Label l3b0;
-        Xbyak::Label l3c0;
-        Xbyak::Label l3cc;
-        Xbyak::Label l3dc;
-        Xbyak::Label l454;
-        Xbyak::Label l48c;
-        Xbyak::Label l4a8;
-        Xbyak::Label l4b8;
-        Xbyak::Label l4c4;
-        Xbyak::Label l4d8;
-        Xbyak::Label l570;
-        Xbyak::Label l5c4;
-        Xbyak::Label l5f0;
-        Xbyak::Label l60c;
-        Xbyak::Label l61c;
-        Xbyak::Label l628;
-        Xbyak::Label l638;
-        Xbyak::Label l6b0;
-        Xbyak::Label l6f4;
-        Xbyak::Label l720;
-        Xbyak::Label l73c;
-        Xbyak::Label l74c;
-        Xbyak::Label l758;
-        Xbyak::Label l76c;
-        Xbyak::Label l804;
-        Xbyak::Label l858;
-        Xbyak::Label l88c;
-        Xbyak::Label l8a4;
-        Xbyak::Label l8b2;
-        Xbyak::Label l8bc;
-        Xbyak::Label l8cc;
-        Xbyak::Label l944;
-        Xbyak::Label l98c;
-        Xbyak::Label l9b0;
-        Xbyak::Label l9c8;
-        Xbyak::Label l9d8;
+        std::vector<Xbyak::Label> labels(46);
 
         preamble();
 #ifdef _WIN32
@@ -124,18 +78,18 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(A, -128);
         sub(B, -128);
         cmp(N, 0x30);
-        jl(l234, T_NEAR);
+        jl(labels[4], T_NEAR);
         align(4);
 
-        L(l20);
+        L(labels[2]);
         mov(A1, A);
         add(A, 0x30);
         mov(I, M);
         sar(I, 0x2);
-        jle(l170, T_NEAR);
+        jle(labels[0], T_NEAR);
         align(4);
 
-        L(l34);
+        L(labels[8]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -199,12 +153,12 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         movdqu(xword[B + 0x30], xmm2);
         sub(B, -192);
         dec(I);
-        jg(l34, T_NEAR);
+        jg(labels[8], T_NEAR);
         align(4);
 
-        L(l170);
+        L(labels[0]);
         test(M, 0x2);
-        jle(l1f0, T_NEAR);
+        jle(labels[1], T_NEAR);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 - 0x70]);
         movdqu(xmm2, xword[A1 - 0x60]);
@@ -231,9 +185,9 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -96);
         align(4);
 
-        L(l1f0);
+        L(labels[1]);
         test(M, 0x1);
-        jle(l224, T_NEAR);
+        jle(labels[3], T_NEAR);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 - 0x70]);
         movdqu(xmm2, xword[A1 - 0x60]);
@@ -244,26 +198,26 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -48);
         align(4);
 
-        L(l224);
+        L(labels[3]);
         sub(N, 0x30);
         cmp(N, 0x30);
-        jge(l20, T_NEAR);
+        jge(labels[2], T_NEAR);
         align(4);
 
-        L(l234);
+        L(labels[4]);
         cmp(N, 0x20);
-        jl(l3c0, T_NEAR);
+        jl(labels[11], T_NEAR);
         align(4);
 
-        L(l240);
+        L(labels[5]);
         mov(A1, A);
         add(A, 0x20);
         mov(I, M);
         sar(I, 0x2);
-        jle(l32c, T_NEAR);
+        jle(labels[7], T_NEAR);
         align(4);
 
-        L(l254);
+        L(labels[6]);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 + LDA * 1 - 0x80]);
         movdqu(xmm2, xword[A1 + LDA * 2 - 0x80]);
@@ -307,12 +261,12 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         movdqu(xword[B - 0x10], xmm2);
         sub(B, -128);
         dec(I);
-        jg(l254, T_NEAR);
+        jg(labels[6], T_NEAR);
         align(4);
 
-        L(l32c);
+        L(labels[7]);
         test(M, 0x2);
-        jle(l388, T_NEAR);
+        jle(labels[9], T_NEAR);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 - 0x70]);
         add(A1, LDA);
@@ -332,9 +286,9 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -64);
         align(4);
 
-        L(l388);
+        L(labels[9]);
         test(M, 0x1);
-        jle(l3b0, T_NEAR);
+        jle(labels[10], T_NEAR);
         movdqu(xmm0, xword[A1 - 0x80]);
         movdqu(xmm1, xword[A1 - 0x70]);
         add(A1, LDA);
@@ -343,26 +297,26 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l3b0);
+        L(labels[10]);
         sub(N, 0x20);
         cmp(N, 0x20);
-        jge(l240, T_NEAR);
+        jge(labels[5], T_NEAR);
         align(4);
 
-        L(l3c0);
+        L(labels[11]);
         cmp(N, 0x10);
-        jl(l4b8, T_NEAR);
+        jl(labels[17], T_NEAR);
         align(4);
 
-        L(l3cc);
+        L(labels[12]);
         mov(A1, A);
         add(A, 0x10);
         mov(I, M);
         sar(I, 0x2);
-        jle(l454, T_NEAR);
+        jle(labels[14], T_NEAR);
         align(4);
 
-        L(l3dc);
+        L(labels[13]);
         movdqu(xmm0, xword[A1 - 0x80]);
         add(A1, LDA);
         movdqu(xmm1, xword[A1 - 0x80]);
@@ -389,12 +343,12 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         movdqu(xword[B - 0x50], xmm2);
         sub(B, -64);
         dec(I);
-        jg(l3dc, T_NEAR);
+        jg(labels[13], T_NEAR);
         align(4);
 
-        L(l454);
+        L(labels[14]);
         test(M, 0x2);
-        jle(l48c, T_NEAR);
+        jle(labels[15], T_NEAR);
         movdqu(xmm0, xword[A1 - 0x80]);
         add(A1, LDA);
         movdqu(xmm1, xword[A1 - 0x80]);
@@ -407,35 +361,35 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l48c);
+        L(labels[15]);
         test(M, 0x1);
-        jle(l4a8, T_NEAR);
+        jle(labels[16], T_NEAR);
         movdqu(xmm0, xword[A1 - 0x80]);
         add(A1, LDA);
         movdqu(xword[B - 0x80], xmm0);
         sub(B, -16);
         align(4);
 
-        L(l4a8);
+        L(labels[16]);
         sub(N, 0x10);
         cmp(N, 0x10);
-        jge(l3cc, T_NEAR);
+        jge(labels[12], T_NEAR);
         align(4);
 
-        L(l4b8);
+        L(labels[17]);
         cmp(N, 0x8);
-        jl(l61c, T_NEAR);
+        jl(labels[24], T_NEAR);
         align(4);
 
-        L(l4c4);
+        L(labels[18]);
         mov(A1, A);
         add(A, 0x8);
         mov(I, M);
         sar(I, 0x3);
-        jle(l570, T_NEAR);
+        jle(labels[20], T_NEAR);
         align(4);
 
-        L(l4d8);
+        L(labels[19]);
         movq(xmm0, qword[A1 - 0x80]);
         add(A1, LDA);
         movq(xmm1, qword[A1 - 0x80]);
@@ -468,12 +422,12 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         movdqu(xword[B - 0x50], xmm1);
         sub(B, -64);
         dec(I);
-        jg(l4d8, T_NEAR);
+        jg(labels[19], T_NEAR);
         align(4);
 
-        L(l570);
+        L(labels[20]);
         test(M, 0x4);
-        jle(l5c4, T_NEAR);
+        jle(labels[21], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         add(A1, LDA);
         movq(xmm1, qword[A1 - 0x80]);
@@ -492,9 +446,9 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -32);
         align(4);
 
-        L(l5c4);
+        L(labels[21]);
         test(M, 0x2);
-        jle(l5f0, T_NEAR);
+        jle(labels[22], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         add(A1, LDA);
         movq(xmm1, qword[A1 - 0x80]);
@@ -504,35 +458,35 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l5f0);
+        L(labels[22]);
         test(M, 0x1);
-        jle(l60c, T_NEAR);
+        jle(labels[23], T_NEAR);
         movq(xmm0, qword[A1 - 0x80]);
         add(A1, LDA);
         movq(qword[B - 0x80], xmm0);
         sub(B, -8);
         align(4);
 
-        L(l60c);
+        L(labels[23]);
         sub(N, 0x8);
         cmp(N, 0x8);
-        jge(l4c4, T_NEAR);
+        jge(labels[18], T_NEAR);
         align(4);
 
-        L(l61c);
+        L(labels[24]);
         cmp(N, 0x4);
-        jl(l74c, T_NEAR);
+        jl(labels[31], T_NEAR);
         align(4);
 
-        L(l628);
+        L(labels[25]);
         mov(A1, A);
         add(A, 0x4);
         mov(I, M);
         sar(I, 0x3);
-        jle(l6b0, T_NEAR);
+        jle(labels[27], T_NEAR);
         align(4);
 
-        L(l638);
+        L(labels[26]);
         movd(xmm0, dword[A1 - 0x80]);
         add(A1, LDA);
         movd(xmm1, dword[A1 - 0x80]);
@@ -559,12 +513,12 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         movdqu(xword[B - 0x70], xmm0);
         sub(B, -32);
         dec(I);
-        jg(l638, T_NEAR);
+        jg(labels[26], T_NEAR);
         align(4);
 
-        L(l6b0);
+        L(labels[27]);
         test(M, 0x4);
-        jle(l6f4, T_NEAR);
+        jle(labels[28], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         add(A1, LDA);
         movd(xmm1, dword[A1 - 0x80]);
@@ -580,9 +534,9 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -16);
         align(4);
 
-        L(l6f4);
+        L(labels[28]);
         test(M, 0x2);
-        jle(l720, T_NEAR);
+        jle(labels[29], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         add(A1, LDA);
         movd(xmm1, dword[A1 - 0x80]);
@@ -592,34 +546,34 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l720);
+        L(labels[29]);
         test(M, 0x1);
-        jle(l73c, T_NEAR);
+        jle(labels[30], T_NEAR);
         movd(xmm0, dword[A1 - 0x80]);
         movd(dword[B - 0x80], xmm0);
         sub(B, -4);
         align(4);
 
-        L(l73c);
+        L(labels[30]);
         sub(N, 0x4);
         cmp(N, 0x4);
-        jge(l628, T_NEAR);
+        jge(labels[25], T_NEAR);
         align(4);
 
-        L(l74c);
+        L(labels[31]);
         cmp(N, 0x2);
-        jl(l8b2, T_NEAR);
+        jl(labels[38], T_NEAR);
         align(4);
 
-        L(l758);
+        L(labels[32]);
         mov(A1, A);
         add(A, 0x2);
         mov(LDA3, M);
         sar(LDA3, 0x3);
-        jle(l804, T_NEAR);
+        jle(labels[34], T_NEAR);
         align(4);
 
-        L(l76c);
+        L(labels[33]);
         mov(ax, word[A1 - 0x80]);
         add(A1, LDA);
         pinsrw(xmm0, eax, 0x0);
@@ -654,12 +608,12 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         movdqu(xword[B - 0x80], xmm0);
         sub(B, -16);
         dec(LDA3);
-        jg(l76c, T_NEAR);
+        jg(labels[33], T_NEAR);
         align(4);
 
-        L(l804);
+        L(labels[34]);
         test(M, 0x4);
-        jle(l858, T_NEAR);
+        jle(labels[35], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         add(A1, LDA);
         pinsrw(xmm0, eax, 0x0);
@@ -679,9 +633,9 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -8);
         align(4);
 
-        L(l858);
+        L(labels[35]);
         test(M, 0x2);
-        jle(l88c, T_NEAR);
+        jle(labels[36], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         add(A1, LDA);
         pinsrw(xmm0, eax, 0x0);
@@ -693,34 +647,34 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l88c);
+        L(labels[36]);
         test(M, 0x1);
-        jle(l8a4, T_NEAR);
+        jle(labels[37], T_NEAR);
         mov(ax, word[A1 - 0x80]);
         mov(word[B - 0x80], ax);
         sub(B, -2);
         align(4);
 
-        L(l8a4);
+        L(labels[37]);
         sub(N, 0x2);
         cmp(N, 0x2);
-        jge(l758, T_NEAR);
+        jge(labels[32], T_NEAR);
         align(4);
 
-        L(l8b2);
+        L(labels[38]);
         cmp(N, 0x1);
-        jl(l9d8, T_NEAR);
+        jl(labels[45], T_NEAR);
         align(4);
 
-        L(l8bc);
+        L(labels[39]);
         mov(A1, A);
         add(A, 0x1);
         mov(LDA3, M);
         sar(LDA3, 0x3);
-        jle(l944, T_NEAR);
+        jle(labels[41], T_NEAR);
         align(4);
 
-        L(l8cc);
+        L(labels[40]);
         mov(al, byte[A1 - 0x80]);
         add(A1, LDA);
         pinsrb(xmm0, eax, 0x0);
@@ -748,12 +702,12 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         movq(qword[B - 0x80], xmm0);
         sub(B, -8);
         dec(LDA3);
-        jg(l8cc, T_NEAR);
+        jg(labels[40], T_NEAR);
         align(4);
 
-        L(l944);
+        L(labels[41]);
         test(M, 0x4);
-        jle(l98c, T_NEAR);
+        jle(labels[42], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         add(A1, LDA);
         pinsrb(xmm0, eax, 0x0);
@@ -770,9 +724,9 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -4);
         align(4);
 
-        L(l98c);
+        L(labels[42]);
         test(M, 0x2);
-        jle(l9b0, T_NEAR);
+        jle(labels[43], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         add(A1, LDA);
         mov(byte[B - 0x80], al);
@@ -782,21 +736,21 @@ void jit_avx512_core_u8_copy_an_kern::generate() {
         sub(B, -2);
         align(4);
 
-        L(l9b0);
+        L(labels[43]);
         test(M, 0x1);
-        jle(l9c8, T_NEAR);
+        jle(labels[44], T_NEAR);
         mov(al, byte[A1 - 0x80]);
         mov(byte[B - 0x80], al);
         sub(B, -1);
         align(4);
 
-        L(l9c8);
+        L(labels[44]);
         sub(N, 0x1);
         cmp(N, 0x1);
-        jge(l8bc, T_NEAR);
+        jge(labels[39], T_NEAR);
         align(4);
 
-        L(l9d8);
+        L(labels[45]);
 
         postamble();
     }
