@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ namespace cpu {
 namespace x64 {
 
 jit_avx_kernel_b0_b_gemm_s8u8s32_kern::jit_avx_kernel_b0_b_gemm_s8u8s32_kern()
-    : jit_generator(nullptr, S8U8S32_COMPUTE_KERNEL_CODE_SIZE) {}
+    : jit_generator(jit_name()) {}
 
 void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
 
@@ -84,98 +84,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
 
     inLocalLabel();
     {
-
-        Xbyak::Label l1030;
-        Xbyak::Label l1080;
-        Xbyak::Label l10e0;
-        Xbyak::Label l1128;
-        Xbyak::Label l112c;
-        Xbyak::Label l1180;
-        Xbyak::Label l11a8;
-        Xbyak::Label l1230;
-        Xbyak::Label l1244;
-        Xbyak::Label l12c;
-        Xbyak::Label l12cc;
-        Xbyak::Label l1304;
-        Xbyak::Label l1354;
-        Xbyak::Label l13ac;
-        Xbyak::Label l1404;
-        Xbyak::Label l1434;
-        Xbyak::Label l1494;
-        Xbyak::Label l14a0;
-        Xbyak::Label l1500;
-        Xbyak::Label l1528;
-        Xbyak::Label l1568;
-        Xbyak::Label l15b0;
-        Xbyak::Label l15e0;
-        Xbyak::Label l15e4;
-        Xbyak::Label l1638;
-        Xbyak::Label l1660;
-        Xbyak::Label l16e8;
-        Xbyak::Label l16fc;
-        Xbyak::Label l1784;
-        Xbyak::Label l17bc;
-        Xbyak::Label l180c;
-        Xbyak::Label l1864;
-        Xbyak::Label l18bc;
-        Xbyak::Label l18ec;
-        Xbyak::Label l194c;
-        Xbyak::Label l1958;
-        Xbyak::Label l19b8;
-        Xbyak::Label l19e0;
-        Xbyak::Label l1a20;
-        Xbyak::Label l1a68;
-        Xbyak::Label l1a98;
-        Xbyak::Label l1a9c;
-        Xbyak::Label l1af0;
-        Xbyak::Label l1b18;
-        Xbyak::Label l1ba0;
-        Xbyak::Label l1bb4;
-        Xbyak::Label l1c3c;
-        Xbyak::Label l1c74;
-        Xbyak::Label l1cc4;
-        Xbyak::Label l1d1c;
-        Xbyak::Label l1d74;
-        Xbyak::Label l1da4;
-        Xbyak::Label l1e04;
-        Xbyak::Label l1e10;
-        Xbyak::Label l1e70;
-        Xbyak::Label l1e98;
-        Xbyak::Label l1ed8;
-        Xbyak::Label l1f20;
-        Xbyak::Label l1f50;
-        Xbyak::Label l1f54;
-        Xbyak::Label l278;
-        Xbyak::Label l28c;
-        Xbyak::Label l3d8;
-        Xbyak::Label l460;
-        Xbyak::Label l514;
-        Xbyak::Label l5e8;
-        Xbyak::Label l6c0;
-        Xbyak::Label l708;
-        Xbyak::Label l7e0;
-        Xbyak::Label l7f0;
-        Xbyak::Label l8c8;
-        Xbyak::Label l914;
-        Xbyak::Label l98c;
-        Xbyak::Label la28;
-        Xbyak::Label la9c;
-        Xbyak::Label lab0;
-        Xbyak::Label lb0;
-        Xbyak::Label lb04;
-        Xbyak::Label lb30;
-        Xbyak::Label lbf8;
-        Xbyak::Label lc0c;
-        Xbyak::Label lcd4;
-        Xbyak::Label ld28;
-        Xbyak::Label ld98;
-        Xbyak::Label le18;
-        Xbyak::Label le98;
-        Xbyak::Label led4;
-        Xbyak::Label lf4;
-        Xbyak::Label lf60;
-        Xbyak::Label lf70;
-        Xbyak::Label lffc;
+        std::vector<Xbyak::Label> labels(91);
 
         auto stack_alloc_size = 32;
         auto args_offset = stack_alloc_size + get_size_of_abi_save_regs() + 8;
@@ -214,10 +123,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         vpshufd(xmm7, xmm7, 0x0);
         mov(J, M);
         cmp(J, 0x10);
-        jl(lab0, T_NEAR);
+        jl(labels[75], T_NEAR);
         align(4);
 
-        L(lb0);
+        L(labels[77]);
         mov(CO1, C);
         add(C, 0x40);
         mov(BO, B);
@@ -232,10 +141,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         mov(qword[COFFSET_RY], H);
         mov(I, N);
         cmp(I, 0x2);
-        jl(l6c0, T_NEAR);
+        jl(labels[66], T_NEAR);
         align(4);
 
-        L(lf4);
+        L(labels[87]);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm1, xword[AO - 0x70]);
@@ -244,12 +153,12 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l3d8, T_NEAR);
+        jle(labels[62], T_NEAR);
         sub(H, 0x8);
-        jle(l278, T_NEAR);
+        jle(labels[60], T_NEAR);
         align(4);
 
-        L(l12c);
+        L(labels[10]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -319,17 +228,17 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x80);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l12c, T_NEAR);
+        jg(labels[10], T_NEAR);
         align(4);
 
-        L(l278);
+        L(labels[60]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(l3d8, T_NEAR);
+        jle(labels[62], T_NEAR);
         align(4);
 
-        L(l28c);
+        L(labels[61]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -399,13 +308,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x80);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l28c, T_NEAR);
+        jg(labels[61], T_NEAR);
         align(4);
 
-        L(l3d8);
+        L(labels[62]);
         mov(H, K);
         test(H, 0x4);
-        je(l460, T_NEAR);
+        je(labels[63], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -436,10 +345,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(l460);
+        L(labels[63]);
         mov(H, K);
         test(H, 0x2);
-        je(l514, T_NEAR);
+        je(labels[64], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -479,10 +388,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l514);
+        L(labels[64]);
         mov(H, K);
         test(H, 0x1);
-        je(l5e8, T_NEAR);
+        je(labels[65], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -529,7 +438,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l5e8);
+        L(labels[65]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -574,12 +483,12 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(lf4, T_NEAR);
+        jge(labels[87], T_NEAR);
         align(4);
 
-        L(l6c0);
+        L(labels[66]);
         test(I, 0x1);
-        jle(la9c, T_NEAR);
+        jle(labels[74], T_NEAR);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm1, xword[AO - 0x70]);
@@ -588,12 +497,12 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l8c8, T_NEAR);
+        jle(labels[70], T_NEAR);
         sub(H, 0x8);
-        jle(l7e0, T_NEAR);
+        jle(labels[68], T_NEAR);
         align(4);
 
-        L(l708);
+        L(labels[67]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -637,16 +546,16 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x80);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l708, T_NEAR);
+        jg(labels[67], T_NEAR);
         align(4);
 
-        L(l7e0);
+        L(labels[68]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l8c8, T_NEAR);
+        jle(labels[70], T_NEAR);
         align(4);
 
-        L(l7f0);
+        L(labels[69]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -690,13 +599,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x80);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l7f0, T_NEAR);
+        jg(labels[69], T_NEAR);
         align(4);
 
-        L(l8c8);
+        L(labels[70]);
         mov(H, K);
         test(H, 0x4);
-        je(l914, T_NEAR);
+        je(labels[71], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -714,10 +623,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l914);
+        L(labels[71]);
         mov(H, K);
         test(H, 0x2);
-        je(l98c, T_NEAR);
+        je(labels[72], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -744,10 +653,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l98c);
+        L(labels[72]);
         mov(H, K);
         test(H, 0x1);
-        je(la28, T_NEAR);
+        je(labels[73], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -781,7 +690,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(la28);
+        L(labels[73]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -809,16 +718,16 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(la9c);
+        L(labels[74]);
         mov(A, AO);
         sub(J, 0x10);
         cmp(J, 0x10);
-        jge(lb0, T_NEAR);
+        jge(labels[77], T_NEAR);
         align(4);
 
-        L(lab0);
+        L(labels[75]);
         test(J, 0x8);
-        jle(l112c, T_NEAR);
+        jle(labels[4], T_NEAR);
         mov(CO1, C);
         add(C, 0x20);
         mov(BO, B);
@@ -833,22 +742,22 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         mov(qword[COFFSET_RY], H);
         mov(I, N);
         cmp(I, 0x2);
-        jl(le98, T_NEAR);
+        jl(labels[85], T_NEAR);
         align(4);
 
-        L(lb04);
+        L(labels[76]);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm1, xword[AO - 0x70]);
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(lcd4, T_NEAR);
+        jle(labels[81], T_NEAR);
         sub(H, 0x8);
-        jle(lbf8, T_NEAR);
+        jle(labels[79], T_NEAR);
         align(4);
 
-        L(lb30);
+        L(labels[78]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -890,17 +799,17 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x40);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(lb30, T_NEAR);
+        jg(labels[78], T_NEAR);
         align(4);
 
-        L(lbf8);
+        L(labels[79]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(lcd4, T_NEAR);
+        jle(labels[81], T_NEAR);
         align(4);
 
-        L(lc0c);
+        L(labels[80]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -942,13 +851,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x40);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(lc0c, T_NEAR);
+        jg(labels[80], T_NEAR);
         align(4);
 
-        L(lcd4);
+        L(labels[81]);
         mov(H, K);
         test(H, 0x4);
-        je(ld28, T_NEAR);
+        je(labels[82], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -967,10 +876,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(ld28);
+        L(labels[82]);
         mov(H, K);
         test(H, 0x2);
-        je(ld98, T_NEAR);
+        je(labels[83], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -995,10 +904,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(ld98);
+        L(labels[83]);
         mov(H, K);
         test(H, 0x1);
-        je(le18, T_NEAR);
+        je(labels[84], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -1027,7 +936,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(le18);
+        L(labels[84]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -1054,24 +963,24 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(lb04, T_NEAR);
+        jge(labels[76], T_NEAR);
         align(4);
 
-        L(le98);
+        L(labels[85]);
         test(I, 0x1);
-        jle(l1128, T_NEAR);
+        jle(labels[3], T_NEAR);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm1, xword[AO - 0x70]);
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(lffc, T_NEAR);
+        jle(labels[90], T_NEAR);
         sub(H, 0x8);
-        jle(lf60, T_NEAR);
+        jle(labels[88], T_NEAR);
         align(4);
 
-        L(led4);
+        L(labels[86]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1099,16 +1008,16 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x40);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(led4, T_NEAR);
+        jg(labels[86], T_NEAR);
         align(4);
 
-        L(lf60);
+        L(labels[88]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(lffc, T_NEAR);
+        jle(labels[90], T_NEAR);
         align(4);
 
-        L(lf70);
+        L(labels[89]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1136,13 +1045,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x40);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(lf70, T_NEAR);
+        jg(labels[89], T_NEAR);
         align(4);
 
-        L(lffc);
+        L(labels[90]);
         mov(H, K);
         test(H, 0x4);
-        je(l1030, T_NEAR);
+        je(labels[0], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1154,10 +1063,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1030);
+        L(labels[0]);
         mov(H, K);
         test(H, 0x2);
-        je(l1080, T_NEAR);
+        je(labels[1], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -1175,10 +1084,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1080);
+        L(labels[1]);
         mov(H, K);
         test(H, 0x1);
-        je(l10e0, T_NEAR);
+        je(labels[2], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -1200,7 +1109,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(l10e0);
+        L(labels[2]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -1218,13 +1127,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(l1128);
+        L(labels[3]);
         mov(A, AO);
         align(4);
 
-        L(l112c);
+        L(labels[4]);
         test(J, 0x4);
-        jle(l15e4, T_NEAR);
+        jle(labels[23], T_NEAR);
         mov(CO1, C);
         add(C, 0x10);
         mov(BO, B);
@@ -1239,21 +1148,21 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         mov(qword[COFFSET_RY], H);
         mov(I, N);
         cmp(I, 0x2);
-        jl(l1404, T_NEAR);
+        jl(labels[14], T_NEAR);
         align(4);
 
-        L(l1180);
+        L(labels[5]);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l12cc, T_NEAR);
+        jle(labels[9], T_NEAR);
         sub(H, 0x8);
-        jle(l1230, T_NEAR);
+        jle(labels[7], T_NEAR);
         align(4);
 
-        L(l11a8);
+        L(labels[6]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1281,17 +1190,17 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x20);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l11a8, T_NEAR);
+        jg(labels[6], T_NEAR);
         align(4);
 
-        L(l1230);
+        L(labels[7]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(l12cc, T_NEAR);
+        jle(labels[9], T_NEAR);
         align(4);
 
-        L(l1244);
+        L(labels[8]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1319,13 +1228,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x20);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1244, T_NEAR);
+        jg(labels[8], T_NEAR);
         align(4);
 
-        L(l12cc);
+        L(labels[9]);
         mov(H, K);
         test(H, 0x4);
-        je(l1304, T_NEAR);
+        je(labels[11], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1338,10 +1247,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(l1304);
+        L(labels[11]);
         mov(H, K);
         test(H, 0x2);
-        je(l1354, T_NEAR);
+        je(labels[12], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -1359,10 +1268,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1354);
+        L(labels[12]);
         mov(H, K);
         test(H, 0x1);
-        je(l13ac, T_NEAR);
+        je(labels[13], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -1382,7 +1291,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l13ac);
+        L(labels[13]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -1400,23 +1309,23 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(l1180, T_NEAR);
+        jge(labels[5], T_NEAR);
         align(4);
 
-        L(l1404);
+        L(labels[14]);
         test(I, 0x1);
-        jle(l15e0, T_NEAR);
+        jle(labels[22], T_NEAR);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1500, T_NEAR);
+        jle(labels[18], T_NEAR);
         sub(H, 0x8);
-        jle(l1494, T_NEAR);
+        jle(labels[16], T_NEAR);
         align(4);
 
-        L(l1434);
+        L(labels[15]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1436,16 +1345,16 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x20);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1434, T_NEAR);
+        jg(labels[15], T_NEAR);
         align(4);
 
-        L(l1494);
+        L(labels[16]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l1500, T_NEAR);
+        jle(labels[18], T_NEAR);
         align(4);
 
-        L(l14a0);
+        L(labels[17]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1465,13 +1374,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x20);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l14a0, T_NEAR);
+        jg(labels[17], T_NEAR);
         align(4);
 
-        L(l1500);
+        L(labels[18]);
         mov(H, K);
         test(H, 0x4);
-        je(l1528, T_NEAR);
+        je(labels[19], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1480,10 +1389,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1528);
+        L(labels[19]);
         mov(H, K);
         test(H, 0x2);
-        je(l1568, T_NEAR);
+        je(labels[20], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -1497,10 +1406,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1568);
+        L(labels[20]);
         mov(H, K);
         test(H, 0x1);
-        je(l15b0, T_NEAR);
+        je(labels[21], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -1516,7 +1425,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(l15b0);
+        L(labels[21]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -1529,13 +1438,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(l15e0);
+        L(labels[22]);
         mov(A, AO);
         align(4);
 
-        L(l15e4);
+        L(labels[23]);
         test(J, 0x2);
-        jle(l1a9c, T_NEAR);
+        jle(labels[41], T_NEAR);
         mov(CO1, C);
         add(C, 0x8);
         mov(BO, B);
@@ -1550,21 +1459,21 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         mov(qword[COFFSET_RY], H);
         mov(I, N);
         cmp(I, 0x2);
-        jl(l18bc, T_NEAR);
+        jl(labels[32], T_NEAR);
         align(4);
 
-        L(l1638);
+        L(labels[24]);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1784, T_NEAR);
+        jle(labels[28], T_NEAR);
         sub(H, 0x8);
-        jle(l16e8, T_NEAR);
+        jle(labels[26], T_NEAR);
         align(4);
 
-        L(l1660);
+        L(labels[25]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1592,17 +1501,17 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x10);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1660, T_NEAR);
+        jg(labels[25], T_NEAR);
         align(4);
 
-        L(l16e8);
+        L(labels[26]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(l1784, T_NEAR);
+        jle(labels[28], T_NEAR);
         align(4);
 
-        L(l16fc);
+        L(labels[27]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1630,13 +1539,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x10);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l16fc, T_NEAR);
+        jg(labels[27], T_NEAR);
         align(4);
 
-        L(l1784);
+        L(labels[28]);
         mov(H, K);
         test(H, 0x4);
-        je(l17bc, T_NEAR);
+        je(labels[29], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1649,10 +1558,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(l17bc);
+        L(labels[29]);
         mov(H, K);
         test(H, 0x2);
-        je(l180c, T_NEAR);
+        je(labels[30], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -1670,10 +1579,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l180c);
+        L(labels[30]);
         mov(H, K);
         test(H, 0x1);
-        je(l1864, T_NEAR);
+        je(labels[31], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -1693,7 +1602,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1864);
+        L(labels[31]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -1711,23 +1620,23 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(l1638, T_NEAR);
+        jge(labels[24], T_NEAR);
         align(4);
 
-        L(l18bc);
+        L(labels[32]);
         test(I, 0x1);
-        jle(l1a98, T_NEAR);
+        jle(labels[40], T_NEAR);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l19b8, T_NEAR);
+        jle(labels[36], T_NEAR);
         sub(H, 0x8);
-        jle(l194c, T_NEAR);
+        jle(labels[34], T_NEAR);
         align(4);
 
-        L(l18ec);
+        L(labels[33]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1747,16 +1656,16 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x10);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l18ec, T_NEAR);
+        jg(labels[33], T_NEAR);
         align(4);
 
-        L(l194c);
+        L(labels[34]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l19b8, T_NEAR);
+        jle(labels[36], T_NEAR);
         align(4);
 
-        L(l1958);
+        L(labels[35]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1776,13 +1685,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x10);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1958, T_NEAR);
+        jg(labels[35], T_NEAR);
         align(4);
 
-        L(l19b8);
+        L(labels[36]);
         mov(H, K);
         test(H, 0x4);
-        je(l19e0, T_NEAR);
+        je(labels[37], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1791,10 +1700,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l19e0);
+        L(labels[37]);
         mov(H, K);
         test(H, 0x2);
-        je(l1a20, T_NEAR);
+        je(labels[38], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -1808,10 +1717,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1a20);
+        L(labels[38]);
         mov(H, K);
         test(H, 0x1);
-        je(l1a68, T_NEAR);
+        je(labels[39], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -1827,7 +1736,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(l1a68);
+        L(labels[39]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -1840,13 +1749,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(l1a98);
+        L(labels[40]);
         mov(A, AO);
         align(4);
 
-        L(l1a9c);
+        L(labels[41]);
         test(J, 0x1);
-        jle(l1f54, T_NEAR);
+        jle(labels[59], T_NEAR);
         mov(CO1, C);
         add(C, 0x4);
         mov(BO, B);
@@ -1861,21 +1770,21 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         mov(qword[COFFSET_RY], H);
         mov(I, N);
         cmp(I, 0x2);
-        jl(l1d74, T_NEAR);
+        jl(labels[50], T_NEAR);
         align(4);
 
-        L(l1af0);
+        L(labels[42]);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1c3c, T_NEAR);
+        jle(labels[46], T_NEAR);
         sub(H, 0x8);
-        jle(l1ba0, T_NEAR);
+        jle(labels[44], T_NEAR);
         align(4);
 
-        L(l1b18);
+        L(labels[43]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1903,17 +1812,17 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x8);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1b18, T_NEAR);
+        jg(labels[43], T_NEAR);
         align(4);
 
-        L(l1ba0);
+        L(labels[44]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(l1c3c, T_NEAR);
+        jle(labels[46], T_NEAR);
         align(4);
 
-        L(l1bb4);
+        L(labels[45]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1941,13 +1850,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x8);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1bb4, T_NEAR);
+        jg(labels[45], T_NEAR);
         align(4);
 
-        L(l1c3c);
+        L(labels[46]);
         mov(H, K);
         test(H, 0x4);
-        je(l1c74, T_NEAR);
+        je(labels[47], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -1960,10 +1869,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(l1c74);
+        L(labels[47]);
         mov(H, K);
         test(H, 0x2);
-        je(l1cc4, T_NEAR);
+        je(labels[48], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -1981,10 +1890,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1cc4);
+        L(labels[48]);
         mov(H, K);
         test(H, 0x1);
-        je(l1d1c, T_NEAR);
+        je(labels[49], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -2004,7 +1913,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1d1c);
+        L(labels[49]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -2022,23 +1931,23 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(l1af0, T_NEAR);
+        jge(labels[42], T_NEAR);
         align(4);
 
-        L(l1d74);
+        L(labels[50]);
         test(I, 0x1);
-        jle(l1f50, T_NEAR);
+        jle(labels[58], T_NEAR);
         mov(AO, A);
         vmovdqu(xmm0, xword[AO - 0x80]);
         vmovdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1e70, T_NEAR);
+        jle(labels[54], T_NEAR);
         sub(H, 0x8);
-        jle(l1e04, T_NEAR);
+        jle(labels[52], T_NEAR);
         align(4);
 
-        L(l1da4);
+        L(labels[51]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -2058,16 +1967,16 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x8);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1da4, T_NEAR);
+        jg(labels[51], T_NEAR);
         align(4);
 
-        L(l1e04);
+        L(labels[52]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l1e70, T_NEAR);
+        jle(labels[54], T_NEAR);
         align(4);
 
-        L(l1e10);
+        L(labels[53]);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -2087,13 +1996,13 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(AO, 0x8);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1e10, T_NEAR);
+        jg(labels[53], T_NEAR);
         align(4);
 
-        L(l1e70);
+        L(labels[54]);
         mov(H, K);
         test(H, 0x4);
-        je(l1e98, T_NEAR);
+        je(labels[55], T_NEAR);
         vpshufd(xmm4, xmm5, 0x0);
         vpmaddubsw(xmm6, xmm4, xmm0);
         vpmaddwd(xmm6, xmm7, xmm6);
@@ -2102,10 +2011,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1e98);
+        L(labels[55]);
         mov(H, K);
         test(H, 0x2);
-        je(l1ed8, T_NEAR);
+        je(labels[56], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vmovdqu(xmm1, xword[AO - 0x80]);
         vpunpcklwd(xmm0, xmm1, xmm6);
@@ -2119,10 +2028,10 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1ed8);
+        L(labels[56]);
         mov(H, K);
         test(H, 0x1);
-        je(l1f20, T_NEAR);
+        je(labels[57], T_NEAR);
         vxorps(xmm6, xmm6, xmm6);
         vbroadcastss(xmm0, dword[AO - 0x80]);
         vpunpcklbw(xmm0, xmm0, xmm6);
@@ -2138,7 +2047,7 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(l1f20);
+        L(labels[57]);
         mov(H, qword[COFFSET_RY]);
         vbroadcastss(xmm0, dword[H]);
         vpaddd(xmm8, xmm8, xmm0);
@@ -2151,11 +2060,11 @@ void jit_avx_kernel_b0_b_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(l1f50);
+        L(labels[58]);
         mov(A, AO);
         align(4);
 
-        L(l1f54);
+        L(labels[59]);
         add(rsp, stack_alloc_size);
         postamble();
     }
