@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ namespace cpu {
 namespace x64 {
 
 jit_sse41_kernel_b0_gemm_s8u8s32_kern::jit_sse41_kernel_b0_gemm_s8u8s32_kern()
-    : jit_generator(nullptr, S8U8S32_COMPUTE_KERNEL_CODE_SIZE) {}
+    : jit_generator(jit_name()) {}
 
 void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
 
@@ -77,98 +77,7 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
 
     inLocalLabel();
     {
-
-        Xbyak::Label l1020;
-        Xbyak::Label l1058;
-        Xbyak::Label l10b0;
-        Xbyak::Label l1118;
-        Xbyak::Label l1130;
-        Xbyak::Label l1134;
-        Xbyak::Label l116c;
-        Xbyak::Label l1194;
-        Xbyak::Label l1220;
-        Xbyak::Label l1234;
-        Xbyak::Label l12c0;
-        Xbyak::Label l12fc;
-        Xbyak::Label l1350;
-        Xbyak::Label l13b0;
-        Xbyak::Label l13d8;
-        Xbyak::Label l1408;
-        Xbyak::Label l146c;
-        Xbyak::Label l1478;
-        Xbyak::Label l14dc;
-        Xbyak::Label l1504;
-        Xbyak::Label l1544;
-        Xbyak::Label l1590;
-        Xbyak::Label l15a0;
-        Xbyak::Label l15a4;
-        Xbyak::Label l15dc;
-        Xbyak::Label l1604;
-        Xbyak::Label l1690;
-        Xbyak::Label l16a4;
-        Xbyak::Label l1730;
-        Xbyak::Label l176c;
-        Xbyak::Label l17c0;
-        Xbyak::Label l1820;
-        Xbyak::Label l1844;
-        Xbyak::Label l1874;
-        Xbyak::Label l18d8;
-        Xbyak::Label l18e4;
-        Xbyak::Label l1948;
-        Xbyak::Label l1970;
-        Xbyak::Label l19b0;
-        Xbyak::Label l19fc;
-        Xbyak::Label l1a08;
-        Xbyak::Label l1a0c;
-        Xbyak::Label l1a44;
-        Xbyak::Label l1a6c;
-        Xbyak::Label l1af8;
-        Xbyak::Label l1b0c;
-        Xbyak::Label l1b98;
-        Xbyak::Label l1bd4;
-        Xbyak::Label l1c28;
-        Xbyak::Label l1c88;
-        Xbyak::Label l1cb0;
-        Xbyak::Label l1ce0;
-        Xbyak::Label l1d44;
-        Xbyak::Label l1d50;
-        Xbyak::Label l1db4;
-        Xbyak::Label l1ddc;
-        Xbyak::Label l1e1c;
-        Xbyak::Label l1e68;
-        Xbyak::Label l1e78;
-        Xbyak::Label l1e7c;
-        Xbyak::Label l26c;
-        Xbyak::Label l280;
-        Xbyak::Label l400;
-        Xbyak::Label l4a4;
-        Xbyak::Label l578;
-        Xbyak::Label l668;
-        Xbyak::Label l6cc;
-        Xbyak::Label l714;
-        Xbyak::Label l808;
-        Xbyak::Label l818;
-        Xbyak::Label l8c;
-        Xbyak::Label l90c;
-        Xbyak::Label l968;
-        Xbyak::Label l9f0;
-        Xbyak::Label la98;
-        Xbyak::Label lac4;
-        Xbyak::Label lad8;
-        Xbyak::Label lb10;
-        Xbyak::Label lb3c;
-        Xbyak::Label lb4;
-        Xbyak::Label lc18;
-        Xbyak::Label lc2c;
-        Xbyak::Label ld08;
-        Xbyak::Label ld64;
-        Xbyak::Label lde0;
-        Xbyak::Label le70;
-        Xbyak::Label leac;
-        Xbyak::Label lec;
-        Xbyak::Label lee8;
-        Xbyak::Label lf7c;
-        Xbyak::Label lf8c;
+        std::vector<Xbyak::Label> labels(91);
 
         auto stack_alloc_size = 32;
         auto args_offset = stack_alloc_size + get_size_of_abi_save_regs() + 8;
@@ -203,10 +112,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         pshufd(xmm7, xmm7, 0x0);
         mov(J, M);
         cmp(J, 0x10);
-        jl(lad8, T_NEAR);
+        jl(labels[75], T_NEAR);
         align(4);
 
-        L(l8c);
+        L(labels[69]);
         mov(CO1, C);
         add(C, 0x40);
         mov(BO, B);
@@ -215,10 +124,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(AA, ptr[A + AA * 1 + 0x200]);
         mov(I, N);
         cmp(I, 0x2);
-        jl(l6cc, T_NEAR);
+        jl(labels[65], T_NEAR);
         align(4);
 
-        L(lb4);
+        L(labels[78]);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm1, xword[AO - 0x70]);
@@ -227,12 +136,12 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l400, T_NEAR);
+        jle(labels[61], T_NEAR);
         sub(H, 0x8);
-        jle(l26c, T_NEAR);
+        jle(labels[59], T_NEAR);
         align(4);
 
-        L(lec);
+        L(labels[86]);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -314,17 +223,17 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x80);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(lec, T_NEAR);
+        jg(labels[86], T_NEAR);
         align(4);
 
-        L(l26c);
+        L(labels[59]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(l400, T_NEAR);
+        jle(labels[61], T_NEAR);
         align(4);
 
-        L(l280);
+        L(labels[60]);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -406,13 +315,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x80);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l280, T_NEAR);
+        jg(labels[60], T_NEAR);
         align(4);
 
-        L(l400);
+        L(labels[61]);
         mov(H, K);
         test(H, 0x4);
-        je(l4a4, T_NEAR);
+        je(labels[62], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -449,10 +358,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(l4a4);
+        L(labels[62]);
         mov(H, K);
         test(H, 0x2);
-        je(l578, T_NEAR);
+        je(labels[63], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -500,10 +409,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l578);
+        L(labels[63]);
         mov(H, K);
         test(H, 0x1);
-        je(l668, T_NEAR);
+        je(labels[64], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -557,7 +466,7 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l668);
+        L(labels[64]);
         movdqu(xword[CO1], xmm8);
         xorps(xmm8, xmm8);
         movdqu(xword[CO1 + 0x10], xmm10);
@@ -577,12 +486,12 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(lb4, T_NEAR);
+        jge(labels[78], T_NEAR);
         align(4);
 
-        L(l6cc);
+        L(labels[65]);
         test(I, 0x1);
-        jle(lac4, T_NEAR);
+        jle(labels[74], T_NEAR);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm1, xword[AO - 0x70]);
@@ -591,12 +500,12 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l90c, T_NEAR);
+        jle(labels[70], T_NEAR);
         sub(H, 0x8);
-        jle(l808, T_NEAR);
+        jle(labels[67], T_NEAR);
         align(4);
 
-        L(l714);
+        L(labels[66]);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -646,16 +555,16 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x80);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l714, T_NEAR);
+        jg(labels[66], T_NEAR);
         align(4);
 
-        L(l808);
+        L(labels[67]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l90c, T_NEAR);
+        jle(labels[70], T_NEAR);
         align(4);
 
-        L(l818);
+        L(labels[68]);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -705,13 +614,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x80);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l818, T_NEAR);
+        jg(labels[68], T_NEAR);
         align(4);
 
-        L(l90c);
+        L(labels[70]);
         mov(H, K);
         test(H, 0x4);
-        je(l968, T_NEAR);
+        je(labels[71], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -732,10 +641,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l968);
+        L(labels[71]);
         mov(H, K);
         test(H, 0x2);
-        je(l9f0, T_NEAR);
+        je(labels[72], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -767,10 +676,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l9f0);
+        L(labels[72]);
         mov(H, K);
         test(H, 0x1);
-        je(la98, T_NEAR);
+        je(labels[73], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -808,7 +717,7 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(la98);
+        L(labels[73]);
         movdqu(xword[CO1], xmm8);
         xorps(xmm8, xmm8);
         movdqu(xword[CO1 + 0x10], xmm10);
@@ -820,16 +729,16 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(lac4);
+        L(labels[74]);
         mov(A, AO);
         sub(J, 0x10);
         cmp(J, 0x10);
-        jge(l8c, T_NEAR);
+        jge(labels[69], T_NEAR);
         align(4);
 
-        L(lad8);
+        L(labels[75]);
         test(J, 0x8);
-        jle(l1134, T_NEAR);
+        jle(labels[4], T_NEAR);
         mov(CO1, C);
         add(C, 0x20);
         mov(BO, B);
@@ -838,22 +747,22 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(AA, ptr[A + AA * 1 + 0x200]);
         mov(I, N);
         cmp(I, 0x2);
-        jl(leac, T_NEAR);
+        jl(labels[85], T_NEAR);
         align(4);
 
-        L(lb10);
+        L(labels[76]);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm1, xword[AO - 0x70]);
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(ld08, T_NEAR);
+        jle(labels[81], T_NEAR);
         sub(H, 0x8);
-        jle(lc18, T_NEAR);
+        jle(labels[79], T_NEAR);
         align(4);
 
-        L(lb3c);
+        L(labels[77]);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -899,17 +808,17 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x40);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(lb3c, T_NEAR);
+        jg(labels[77], T_NEAR);
         align(4);
 
-        L(lc18);
+        L(labels[79]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(ld08, T_NEAR);
+        jle(labels[81], T_NEAR);
         align(4);
 
-        L(lc2c);
+        L(labels[80]);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -955,13 +864,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x40);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(lc2c, T_NEAR);
+        jg(labels[80], T_NEAR);
         align(4);
 
-        L(ld08);
+        L(labels[81]);
         mov(H, K);
         test(H, 0x4);
-        je(ld64, T_NEAR);
+        je(labels[82], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -982,10 +891,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(ld64);
+        L(labels[82]);
         mov(H, K);
         test(H, 0x2);
-        je(lde0, T_NEAR);
+        je(labels[83], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -1013,10 +922,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(lde0);
+        L(labels[83]);
         mov(H, K);
         test(H, 0x1);
-        je(le70, T_NEAR);
+        je(labels[84], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -1048,7 +957,7 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(le70);
+        L(labels[84]);
         movdqu(xword[CO1], xmm8);
         xorps(xmm8, xmm8);
         movdqu(xword[CO1 + 0x10], xmm10);
@@ -1060,24 +969,24 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(lb10, T_NEAR);
+        jge(labels[76], T_NEAR);
         align(4);
 
-        L(leac);
+        L(labels[85]);
         test(I, 0x1);
-        jle(l1130, T_NEAR);
+        jle(labels[3], T_NEAR);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm1, xword[AO - 0x70]);
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1020, T_NEAR);
+        jle(labels[90], T_NEAR);
         sub(H, 0x8);
-        jle(lf7c, T_NEAR);
+        jle(labels[88], T_NEAR);
         align(4);
 
-        L(lee8);
+        L(labels[87]);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -1107,16 +1016,16 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x40);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(lee8, T_NEAR);
+        jg(labels[87], T_NEAR);
         align(4);
 
-        L(lf7c);
+        L(labels[88]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l1020, T_NEAR);
+        jle(labels[90], T_NEAR);
         align(4);
 
-        L(lf8c);
+        L(labels[89]);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -1146,13 +1055,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x40);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(lf8c, T_NEAR);
+        jg(labels[89], T_NEAR);
         align(4);
 
-        L(l1020);
+        L(labels[90]);
         mov(H, K);
         test(H, 0x4);
-        je(l1058, T_NEAR);
+        je(labels[0], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         movaps(xmm6, xmm4);
         pmaddubsw(xmm6, xmm0);
@@ -1165,10 +1074,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1058);
+        L(labels[0]);
         mov(H, K);
         test(H, 0x2);
-        je(l10b0, T_NEAR);
+        je(labels[1], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -1188,10 +1097,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l10b0);
+        L(labels[1]);
         mov(H, K);
         test(H, 0x1);
-        je(l1118, T_NEAR);
+        je(labels[2], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -1215,7 +1124,7 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(l1118);
+        L(labels[2]);
         movdqu(xword[CO1], xmm8);
         xorps(xmm8, xmm8);
         movdqu(xword[CO1 + 0x10], xmm10);
@@ -1223,13 +1132,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(l1130);
+        L(labels[3]);
         mov(A, AO);
         align(4);
 
-        L(l1134);
+        L(labels[4]);
         test(J, 0x4);
-        jle(l15a4, T_NEAR);
+        jle(labels[22], T_NEAR);
         mov(CO1, C);
         add(C, 0x10);
         mov(BO, B);
@@ -1238,21 +1147,21 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(AA, ptr[A + AA * 1 + 0x200]);
         mov(I, N);
         cmp(I, 0x2);
-        jl(l13d8, T_NEAR);
+        jl(labels[13], T_NEAR);
         align(4);
 
-        L(l116c);
+        L(labels[5]);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l12c0, T_NEAR);
+        jle(labels[9], T_NEAR);
         sub(H, 0x8);
-        jle(l1220, T_NEAR);
+        jle(labels[7], T_NEAR);
         align(4);
 
-        L(l1194);
+        L(labels[6]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1280,17 +1189,17 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x20);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1194, T_NEAR);
+        jg(labels[6], T_NEAR);
         align(4);
 
-        L(l1220);
+        L(labels[7]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(l12c0, T_NEAR);
+        jle(labels[9], T_NEAR);
         align(4);
 
-        L(l1234);
+        L(labels[8]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1318,13 +1227,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x20);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1234, T_NEAR);
+        jg(labels[8], T_NEAR);
         align(4);
 
-        L(l12c0);
+        L(labels[9]);
         mov(H, K);
         test(H, 0x4);
-        je(l12fc, T_NEAR);
+        je(labels[10], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1337,10 +1246,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(l12fc);
+        L(labels[10]);
         mov(H, K);
         test(H, 0x2);
-        je(l1350, T_NEAR);
+        je(labels[11], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -1359,10 +1268,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1350);
+        L(labels[11]);
         mov(H, K);
         test(H, 0x1);
-        je(l13b0, T_NEAR);
+        je(labels[12], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -1383,7 +1292,7 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l13b0);
+        L(labels[12]);
         movdqu(xword[CO1], xmm8);
         xorps(xmm8, xmm8);
         movdqu(xword[CO1 + LDC * 1], xmm9);
@@ -1391,23 +1300,23 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(l116c, T_NEAR);
+        jge(labels[5], T_NEAR);
         align(4);
 
-        L(l13d8);
+        L(labels[13]);
         test(I, 0x1);
-        jle(l15a0, T_NEAR);
+        jle(labels[21], T_NEAR);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l14dc, T_NEAR);
+        jle(labels[17], T_NEAR);
         sub(H, 0x8);
-        jle(l146c, T_NEAR);
+        jle(labels[15], T_NEAR);
         align(4);
 
-        L(l1408);
+        L(labels[14]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1427,16 +1336,16 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x20);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1408, T_NEAR);
+        jg(labels[14], T_NEAR);
         align(4);
 
-        L(l146c);
+        L(labels[15]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l14dc, T_NEAR);
+        jle(labels[17], T_NEAR);
         align(4);
 
-        L(l1478);
+        L(labels[16]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1456,13 +1365,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x20);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1478, T_NEAR);
+        jg(labels[16], T_NEAR);
         align(4);
 
-        L(l14dc);
+        L(labels[17]);
         mov(H, K);
         test(H, 0x4);
-        je(l1504, T_NEAR);
+        je(labels[18], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1471,10 +1380,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1504);
+        L(labels[18]);
         mov(H, K);
         test(H, 0x2);
-        je(l1544, T_NEAR);
+        je(labels[19], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -1489,10 +1398,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1544);
+        L(labels[19]);
         mov(H, K);
         test(H, 0x1);
-        je(l1590, T_NEAR);
+        je(labels[20], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -1509,19 +1418,19 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(l1590);
+        L(labels[20]);
         movdqu(xword[CO1], xmm8);
         xorps(xmm8, xmm8);
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(l15a0);
+        L(labels[21]);
         mov(A, AO);
         align(4);
 
-        L(l15a4);
+        L(labels[22]);
         test(J, 0x2);
-        jle(l1a0c, T_NEAR);
+        jle(labels[40], T_NEAR);
         mov(CO1, C);
         add(C, 0x8);
         mov(BO, B);
@@ -1530,21 +1439,21 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(AA, ptr[A + AA * 1 + 0x200]);
         mov(I, N);
         cmp(I, 0x2);
-        jl(l1844, T_NEAR);
+        jl(labels[31], T_NEAR);
         align(4);
 
-        L(l15dc);
+        L(labels[23]);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1730, T_NEAR);
+        jle(labels[27], T_NEAR);
         sub(H, 0x8);
-        jle(l1690, T_NEAR);
+        jle(labels[25], T_NEAR);
         align(4);
 
-        L(l1604);
+        L(labels[24]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1572,17 +1481,17 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x10);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1604, T_NEAR);
+        jg(labels[24], T_NEAR);
         align(4);
 
-        L(l1690);
+        L(labels[25]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(l1730, T_NEAR);
+        jle(labels[27], T_NEAR);
         align(4);
 
-        L(l16a4);
+        L(labels[26]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1610,13 +1519,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x10);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l16a4, T_NEAR);
+        jg(labels[26], T_NEAR);
         align(4);
 
-        L(l1730);
+        L(labels[27]);
         mov(H, K);
         test(H, 0x4);
-        je(l176c, T_NEAR);
+        je(labels[28], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1629,10 +1538,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(l176c);
+        L(labels[28]);
         mov(H, K);
         test(H, 0x2);
-        je(l17c0, T_NEAR);
+        je(labels[29], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -1651,10 +1560,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l17c0);
+        L(labels[29]);
         mov(H, K);
         test(H, 0x1);
-        je(l1820, T_NEAR);
+        je(labels[30], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -1675,7 +1584,7 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1820);
+        L(labels[30]);
         movlps(qword[CO1], xmm8);
         xorps(xmm8, xmm8);
         movlps(qword[CO1 + LDC * 1], xmm9);
@@ -1683,23 +1592,23 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(l15dc, T_NEAR);
+        jge(labels[23], T_NEAR);
         align(4);
 
-        L(l1844);
+        L(labels[31]);
         test(I, 0x1);
-        jle(l1a08, T_NEAR);
+        jle(labels[39], T_NEAR);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1948, T_NEAR);
+        jle(labels[35], T_NEAR);
         sub(H, 0x8);
-        jle(l18d8, T_NEAR);
+        jle(labels[33], T_NEAR);
         align(4);
 
-        L(l1874);
+        L(labels[32]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1719,16 +1628,16 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x10);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1874, T_NEAR);
+        jg(labels[32], T_NEAR);
         align(4);
 
-        L(l18d8);
+        L(labels[33]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l1948, T_NEAR);
+        jle(labels[35], T_NEAR);
         align(4);
 
-        L(l18e4);
+        L(labels[34]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1748,13 +1657,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x10);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l18e4, T_NEAR);
+        jg(labels[34], T_NEAR);
         align(4);
 
-        L(l1948);
+        L(labels[35]);
         mov(H, K);
         test(H, 0x4);
-        je(l1970, T_NEAR);
+        je(labels[36], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1763,10 +1672,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1970);
+        L(labels[36]);
         mov(H, K);
         test(H, 0x2);
-        je(l19b0, T_NEAR);
+        je(labels[37], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -1781,10 +1690,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l19b0);
+        L(labels[37]);
         mov(H, K);
         test(H, 0x1);
-        je(l19fc, T_NEAR);
+        je(labels[38], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -1801,19 +1710,19 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(l19fc);
+        L(labels[38]);
         movlps(qword[CO1], xmm8);
         xorps(xmm8, xmm8);
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(l1a08);
+        L(labels[39]);
         mov(A, AO);
         align(4);
 
-        L(l1a0c);
+        L(labels[40]);
         test(J, 0x1);
-        jle(l1e7c, T_NEAR);
+        jle(labels[58], T_NEAR);
         mov(CO1, C);
         add(C, 0x4);
         mov(BO, B);
@@ -1822,21 +1731,21 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(AA, ptr[A + AA * 1 + 0x200]);
         mov(I, N);
         cmp(I, 0x2);
-        jl(l1cb0, T_NEAR);
+        jl(labels[49], T_NEAR);
         align(4);
 
-        L(l1a44);
+        L(labels[41]);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1b98, T_NEAR);
+        jle(labels[45], T_NEAR);
         sub(H, 0x8);
-        jle(l1af8, T_NEAR);
+        jle(labels[43], T_NEAR);
         align(4);
 
-        L(l1a6c);
+        L(labels[42]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1864,17 +1773,17 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x8);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1a6c, T_NEAR);
+        jg(labels[42], T_NEAR);
         align(4);
 
-        L(l1af8);
+        L(labels[43]);
         prefetcht0(byte[CO1 + 0x3c]);
         prefetcht0(byte[CO1 + LDC * 1 + 0x3c]);
         add(H, 0x8);
-        jle(l1b98, T_NEAR);
+        jle(labels[45], T_NEAR);
         align(4);
 
-        L(l1b0c);
+        L(labels[44]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1902,13 +1811,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x8);
         add(BO, 0x10);
         sub(H, 0x1);
-        jg(l1b0c, T_NEAR);
+        jg(labels[44], T_NEAR);
         align(4);
 
-        L(l1b98);
+        L(labels[45]);
         mov(H, K);
         test(H, 0x4);
-        je(l1bd4, T_NEAR);
+        je(labels[46], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -1921,10 +1830,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x8);
         align(4);
 
-        L(l1bd4);
+        L(labels[46]);
         mov(H, K);
         test(H, 0x2);
-        je(l1c28, T_NEAR);
+        je(labels[47], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -1943,10 +1852,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1c28);
+        L(labels[47]);
         mov(H, K);
         test(H, 0x1);
-        je(l1c88, T_NEAR);
+        je(labels[48], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -1967,7 +1876,7 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1c88);
+        L(labels[48]);
         movss(dword[CO1], xmm8);
         xorps(xmm8, xmm8);
         movss(dword[CO1 + LDC * 1], xmm9);
@@ -1975,23 +1884,23 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         lea(CO1, ptr[CO1 + LDC * 2]);
         sub(I, 0x2);
         cmp(I, 0x2);
-        jge(l1a44, T_NEAR);
+        jge(labels[41], T_NEAR);
         align(4);
 
-        L(l1cb0);
+        L(labels[49]);
         test(I, 0x1);
-        jle(l1e78, T_NEAR);
+        jle(labels[57], T_NEAR);
         mov(AO, A);
         movdqu(xmm0, xword[AO - 0x80]);
         movdqu(xmm5, xword[BO - 0x80]);
         mov(H, K);
         sar(H, 0x3);
-        jle(l1db4, T_NEAR);
+        jle(labels[53], T_NEAR);
         sub(H, 0x8);
-        jle(l1d44, T_NEAR);
+        jle(labels[51], T_NEAR);
         align(4);
 
-        L(l1ce0);
+        L(labels[50]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -2011,16 +1920,16 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x8);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1ce0, T_NEAR);
+        jg(labels[50], T_NEAR);
         align(4);
 
-        L(l1d44);
+        L(labels[51]);
         prefetcht0(byte[CO1 + 0x3c]);
         add(H, 0x8);
-        jle(l1db4, T_NEAR);
+        jle(labels[53], T_NEAR);
         align(4);
 
-        L(l1d50);
+        L(labels[52]);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -2040,13 +1949,13 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(AO, 0x8);
         add(BO, 0x8);
         sub(H, 0x1);
-        jg(l1d50, T_NEAR);
+        jg(labels[52], T_NEAR);
         align(4);
 
-        L(l1db4);
+        L(labels[53]);
         mov(H, K);
         test(H, 0x4);
-        je(l1ddc, T_NEAR);
+        je(labels[54], T_NEAR);
         pshufd(xmm4, xmm5, 0x0);
         pmaddubsw(xmm4, xmm0);
         pmaddwd(xmm4, xmm7);
@@ -2055,10 +1964,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x4);
         align(4);
 
-        L(l1ddc);
+        L(labels[54]);
         mov(H, K);
         test(H, 0x2);
-        je(l1e1c, T_NEAR);
+        je(labels[55], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm0, xword[AO - 0x80]);
         movaps(xmm1, xmm0);
@@ -2073,10 +1982,10 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x2);
         align(4);
 
-        L(l1e1c);
+        L(labels[55]);
         mov(H, K);
         test(H, 0x1);
-        je(l1e68, T_NEAR);
+        je(labels[56], T_NEAR);
         xorps(xmm6, xmm6);
         movdqu(xmm3, xword[AO - 0x80]);
         pshufd(xmm0, xmm3, 0x0);
@@ -2093,17 +2002,17 @@ void jit_sse41_kernel_b0_gemm_s8u8s32_kern::generate() {
         add(BO, 0x1);
         align(4);
 
-        L(l1e68);
+        L(labels[56]);
         movss(dword[CO1], xmm8);
         xorps(xmm8, xmm8);
         lea(CO1, ptr[CO1 + LDC * 1]);
         align(4);
 
-        L(l1e78);
+        L(labels[57]);
         mov(A, AO);
         align(4);
 
-        L(l1e7c);
+        L(labels[58]);
         add(rsp, stack_alloc_size);
         postamble();
     }
