@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 * Copyright 2018 YANDEX LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,8 +16,10 @@
 *******************************************************************************/
 
 #include <assert.h>
+#include <limits>
 
 #include "common/c_types_map.hpp"
+#include "common/convolution_pd.hpp"
 #include "common/memory.hpp"
 #include "common/memory_tracking.hpp"
 #include "common/nstl.hpp"
@@ -46,9 +48,7 @@ using namespace Xbyak;
 jit_avx2_1x1_conv_kernel_f32::jit_avx2_1x1_conv_kernel_f32(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, avx2)
-    , jcp(ajcp)
-    , attr_(attr) {
+    : jit_generator(jit_name(), avx2), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -58,8 +58,8 @@ jit_avx2_1x1_conv_kernel_f32::jit_avx2_1x1_conv_kernel_f32(
         const size_t tail_size = jcp.oc_without_padding % isa_simd_width_;
 
         rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx, r13, r14,
-                preserve_gpr, preserve_vmm,
-                GET_OFF(post_ops_binary_rhs_arg_vec),
+                r15, preserve_gpr, preserve_vmm,
+                GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size,
                 use_exact_tail_scalar_bcast};
         static_params_t static_params {this->param1, rhs_arg_static_params};
@@ -71,7 +71,7 @@ jit_avx2_1x1_conv_kernel_f32::jit_avx2_1x1_conv_kernel_f32(
 }
 
 void jit_avx2_1x1_conv_kernel_f32::generate_bcast_loop(int load_loop_blk) {
-    mov(aux1_reg_bcast_data, reg_bcast_data);
+    mov(aux1_reg_bcast_data, ptr[rsp + reg_bcast_data_off]);
     mov(aux_reg_output_data, reg_output_data);
     mov(bcast_loop_iter, reg_bcast_loop_work);
 
@@ -158,27 +158,21 @@ void jit_avx2_1x1_conv_kernel_f32::apply_postops(
         if (jcp.with_binary) {
             binary_injector::rhs_arg_dynamic_params_t rhs_arg_params,
                     rhs_arg_params_tail;
-            const auto oc_off_oprnd = r12;
 
             iterate(load_loop_blk, ur, load_dim_tail,
                     [&](const bool mask_flag, const int i, const int j) {
-                        const int aux_output_offset
-                                = (i * get_output_i_offset(jcp)
-                                        + j * get_output_j_offset(jcp));
+                        const size_t aux_output_offset
+                                = (i * get_output_i_offset(jcp, true)
+                                          + j * get_output_j_offset(jcp))
+                                * sizeof(float);
                         const auto vmm_idx
                                 = vreg_accum_idx(load_loop_blk, i, j);
                         vmm_idxs.emplace(vmm_idx);
 
-                        rhs_arg_params_tail.vmm_idx_to_oc_elem_off_addr.emplace(
-                                vmm_idx, ptr[param1 + GET_OFF(oc_l_off)]);
-                        rhs_arg_params_tail.vmm_idx_to_oc_elem_off_val.emplace(
-                                vmm_idx, i * jcp.oc_block);
-                        rhs_arg_params_tail.vmm_idx_to_oc_off_oprnd.emplace(
-                                vmm_idx, oc_off_oprnd);
+                        rhs_arg_params_tail.vmm_idx_to_out_reg.emplace(
+                                vmm_idx, aux_reg_output_data);
                         rhs_arg_params_tail.vmm_idx_to_out_elem_off_val.emplace(
                                 vmm_idx, aux_output_offset);
-                        rhs_arg_params_tail.vmm_idx_to_out_off_oprnd.emplace(
-                                vmm_idx, oc_off_oprnd);
                         if (mask_flag)
                             rhs_arg_params_tail.vmm_tail_idx_.emplace(vmm_idx);
                     });
@@ -186,15 +180,17 @@ void jit_avx2_1x1_conv_kernel_f32::apply_postops(
             rhs_arg_params.vmm_tail_idx_.clear();
 
             const injector_utils::register_preserve_guard_t register_guard(
-                    this, {abi_param1, oc_off_oprnd});
+                    this, {abi_param1, aux_reg_output_data});
             const size_t reg_guard_stack_occupied
                     = register_guard.stack_space_occupied();
+            if (jcp.with_dw_conv) {
+                add(aux_reg_output_data,
+                        ptr[rsp + reg_dw_binary_output_off
+                                + reg_guard_stack_occupied]);
+            }
             mov(abi_param1,
                     ptr[rsp + reg_abi_param1_backup
                             + reg_guard_stack_occupied]);
-            mov(oc_off_oprnd,
-                    ptr[rsp + reg_binary_post_op_acc_off
-                            + reg_guard_stack_occupied]);
 
             Label postops_done;
             if (load_dim_tail) {
@@ -231,26 +227,28 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
             % jcp.load_block;
     const int reduce_dim_tail = jcp.reduce_dim % jcp.reduce_block;
 
-    auto vreg_load = [=](int i) { return Ymm(ur * load_loop_blk + i); };
+    auto vreg_load = [ur, load_loop_blk](
+                             int i) { return Ymm(ur * load_loop_blk + i); };
 
-    auto bias_ptr = [=](int i) {
+    auto bias_ptr = [this](int i) {
         return ptr[reg_bias_data + sizeof(float) * jcp.oc_block * i];
     };
 
-    auto bcast_ptr = [=](int u, int j) {
+    auto bcast_ptr = [this](int u, int j) {
         assert(j < jcp.ur);
         assert(u <= jcp.reduce_loop_unroll);
-        return ptr[aux_reg_bcast_data + get_bcast_offset(jcp, u, j)];
+        const size_t offset = get_bcast_offset(jcp, u, j);
+        return make_safe_addr(aux_reg_bcast_data, offset, reg_long_offt);
     };
 
-    auto get_load_offset_bwd_w = [=](int u, int i) {
+    auto get_load_offset_bwd_w = [this](int u, int i) {
         size_t u0 = u % jcp.reduce_loop_unroll;
         size_t u1 = u / jcp.reduce_loop_unroll;
         return u1 * jcp.reduce_loop_load_step
                 + sizeof(float) * get_load_bwd_w_offset(jcp, i, u0);
     };
 
-    auto load_ptr = [=](int u, int i) {
+    auto load_ptr = [this](int u, int i) {
         size_t offt;
         size_t u0 = u % jcp.reduce_loop_unroll;
         size_t u1 = u / jcp.reduce_loop_unroll;
@@ -268,10 +266,9 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
                 + sizeof(float) * offt];
     };
 
-    auto get_output_offset = [=](int i, int j) {
+    auto get_output_offset = [this](int i, int j) {
         switch (jcp.prop_kind) {
             case backward_weights: return sizeof(float) * jcp.oc_block * j;
-            case backward_data:
             default:
                 return (i * get_output_i_offset(jcp)
                                + j * get_output_j_offset(jcp))
@@ -279,23 +276,20 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
         }
     };
 
-    auto output_ptr = [=](int i, int j) {
+    auto output_ptr = [this, get_output_offset](int i, int j) {
         switch (jcp.prop_kind) {
             case backward_weights:
                 return ptr[aux_reg_output_data
                         + (i ? reg_output_stride * i
                              : 0) // TODO: Xbyak should allow 0 scale
                         + sizeof(float) * jcp.oc_block * j];
-            case backward_data:
             default:
-                return ptr[aux_reg_output_data
-                        + (i * get_output_i_offset(jcp)
-                                  + j * get_output_j_offset(jcp))
-                                * sizeof(float)];
+                const size_t off = get_output_offset(i, j);
+                return make_safe_addr(aux_reg_output_data, off, reg_long_offt);
         }
     };
 
-    auto init = [=]() {
+    auto init = [&]() {
         Label init_done, init_zero;
 
         if (jcp.with_bias
@@ -356,7 +350,7 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
         vbroadcastss(vreg_bcast, bcast_ptr(0, 0));
     };
 
-    auto store = [=]() {
+    auto store = [&]() {
         Label store_noadd;
 
         if (!jcp.with_sum) {
@@ -392,10 +386,11 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
         apply_postops(load_loop_blk, ur, load_dim_tail);
 
         if (jcp.prop_kind == backward_weights && load_dim_tail > 0) {
-            push(reg_bcast_data);
             push(aux_reg_bcast_data);
         }
 
+        const auto is_padding = jcp.oc_without_padding != jcp.oc;
+        if (is_padding) uni_vxorps(vtmp, vtmp, vtmp);
         for (int j = 0; j < ur; ++j)
             for (int i = 0; i < load_loop_blk; ++i) {
                 if (load_dim_tail > 0 && i == load_loop_blk - 1) {
@@ -420,10 +415,14 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
                         lea(reg_tmp,
                                 ptr[aux_reg_output_data
                                         + reg_tmp_output_stride]);
-                        store_bytes(vreg_accum(load_loop_blk, i, j), reg_tmp,
-                                get_output_offset(i, j),
-                                load_dim_tail * sizeof(float));
+                        vmovups(output_ptr(i, j),
+                                vreg_accum(load_loop_blk, i, j));
                     } else {
+                        if (is_padding && jcp.with_binary) {
+                            vmovups(ptr[aux_reg_output_data
+                                            + get_output_offset(i, j)],
+                                    vtmp);
+                        }
                         store_bytes(vreg_accum(load_loop_blk, i, j),
                                 aux_reg_output_data, get_output_offset(i, j),
                                 load_dim_tail * sizeof(float));
@@ -436,11 +435,10 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
 
         if (jcp.prop_kind == backward_weights && load_dim_tail > 0) {
             pop(aux_reg_bcast_data);
-            pop(reg_bcast_data);
         }
     };
 
-    auto fma_block = [=](bool last_block) {
+    auto fma_block = [&](bool last_block) {
         const bool is_tail = reduce_dim_tail && last_block;
         const int u_end = is_tail ? reduce_dim_tail : jcp.reduce_loop_unroll;
         for (int u = 0; u < u_end; ++u) {
@@ -496,7 +494,7 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
     L(reduce_loop);
     {
         fma_block(false);
-        add(aux_reg_bcast_data, jcp.reduce_loop_bcast_step);
+        safe_add(aux_reg_bcast_data, jcp.reduce_loop_bcast_step, reg_long_offt);
         add(aux_reg_load_data, jcp.reduce_loop_load_step);
         sub(reduce_loop_iter, jcp.reduce_loop_unroll);
         jg(reduce_loop, T_NEAR);
@@ -514,16 +512,16 @@ void jit_avx2_1x1_conv_kernel_f32::generate_diff_bias_loop(int load_loop_blk) {
     Label diff_bias_loop, diff_bias_loop_out, diff_bias_init_out;
     Label diff_bias_load;
 
-    auto diff_bias_ptr = [=](int i) {
+    auto diff_bias_ptr = [this](int i) {
         return ptr[reg_diff_bias_data + i * jcp.oc_block * sizeof(float)];
     };
 
-    auto load_ptr = [=](int u, int i) {
+    auto load_ptr = [this](int u, int i) {
         return ptr[aux_reg_load_data
                 + (i * jcp.os + u) * jcp.oc_block * sizeof(float)];
     };
 
-    auto diff_bias_reg = [=](int i) { return Ymm(i); };
+    auto diff_bias_reg = [](int i) { return Ymm(i); };
 
     mov(reg_diff_bias_data, ptr[rsp + reg_diff_bias_data_stack_offt]);
     cmp(reg_diff_bias_data, 0);
@@ -567,17 +565,19 @@ void jit_avx2_1x1_conv_kernel_f32::generate_diff_bias_loop(int load_loop_blk) {
 void jit_avx2_1x1_conv_kernel_f32::generate() {
     preamble();
 
-    if (jcp.with_binary || (jcp.with_bias && jcp.prop_kind == backward_weights))
-        sub(rsp, stack_space_needed);
+    sub(rsp, stack_space_needed);
 
     if (jcp.with_binary) {
-        const auto zeroed_reg = r15;
-        xor_(zeroed_reg, zeroed_reg);
-        mov(ptr[rsp + reg_binary_post_op_acc_off], zeroed_reg);
         mov(ptr[rsp + reg_abi_param1_backup], abi_param1);
+        if (jcp.with_dw_conv) {
+            const auto zeroed_reg = r15;
+            xor_(zeroed_reg, zeroed_reg);
+            mov(ptr[rsp + reg_dw_binary_output_off], zeroed_reg);
+        }
     }
 
     mov(reg_bcast_data, ptr[param1 + GET_OFF(bcast_data)]);
+    mov(ptr[rsp + reg_bcast_data_off], reg_bcast_data);
     mov(reg_load_data, ptr[param1 + GET_OFF(load_data)]);
     mov(reg_output_data, ptr[param1 + GET_OFF(output_data)]);
     if (jcp.with_bias) {
@@ -595,27 +595,30 @@ void jit_avx2_1x1_conv_kernel_f32::generate() {
     if (jcp.prop_kind == backward_weights)
         mov(reg_output_stride, ptr[param1 + GET_OFF(output_stride)]);
 
-    auto generate_load_loop_body = [=](int load_loop_blk) {
+    auto generate_load_loop_body = [&](int load_loop_blk) {
         generate_bcast_loop(load_loop_blk);
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
+        const size_t offst_with_dw_conv
+                = get_load_loop_output_fwd_offset(jcp, load_loop_blk);
+        const size_t offst_wo_dw_conv
+                = get_load_loop_output_fwd_offset(jcp, load_loop_blk, true);
         switch (jcp.prop_kind) {
             case forward_training:
             case forward_inference:
                 add(reg_bias_data,
                         load_loop_blk * jcp.oc_block * sizeof(float));
-                add(reg_output_data,
-                        get_load_loop_output_fwd_offset(jcp, load_loop_blk));
-                if (jcp.with_binary) {
-                    mov(aux_reg_load_data,
-                            ptr[rsp + reg_binary_post_op_acc_off]);
-                    add(aux_reg_load_data, jcp.load_block * load_loop_blk);
-                    mov(ptr[rsp + reg_binary_post_op_acc_off],
-                            aux_reg_load_data);
+                safe_add(reg_output_data, offst_with_dw_conv, reg_long_offt);
+                if (jcp.with_binary && jcp.with_dw_conv) {
+                    mov(aux_reg_load_data, ptr[rsp + reg_dw_binary_output_off]);
+                    add(aux_reg_load_data,
+                            offst_wo_dw_conv - offst_with_dw_conv);
+                    mov(ptr[rsp + reg_dw_binary_output_off], aux_reg_load_data);
                 }
                 break;
             case backward_data:
-                add(reg_output_data,
-                        get_load_loop_output_bwd_d_offset(jcp, load_loop_blk));
+                safe_add(reg_output_data,
+                        get_load_loop_output_bwd_d_offset(jcp, load_loop_blk),
+                        reg_long_offt);
                 break;
             case backward_weights:
                 for (int i = 0; i < load_loop_blk; i++)
@@ -671,18 +674,19 @@ void jit_avx2_1x1_conv_kernel_f32::generate() {
 
     L(load_loop_blk_end);
 
-    if (jcp.with_binary || (jcp.with_bias && jcp.prop_kind == backward_weights))
-        add(rsp, stack_space_needed);
+    add(rsp, stack_space_needed);
 
     postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
 status_t jit_avx2_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
         const convolution_desc_t &cd, const memory_desc_wrapper &src_d,
         const memory_desc_wrapper &weights_d, const memory_desc_wrapper &dst_d,
         const primitive_attr_t &attr) {
+    // disabling verbose dispatch messages for unsupported isa for better readability
     if (!mayiuse(avx)) return status::unimplemented;
     jcp.isa = mayiuse(avx2) ? avx2 : avx;
 
@@ -726,8 +730,8 @@ status_t jit_avx2_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
                             format_kind::undef, cd.diff_bias_desc.format_kind)
             != format_kind::undef;
 
-    jcp.os = jcp.od * jcp.oh * jcp.ow;
-    jcp.is = jcp.id * jcp.ih * jcp.iw;
+    jcp.os = static_cast<dim_t>(jcp.od) * jcp.oh * jcp.ow;
+    jcp.is = static_cast<dim_t>(jcp.id) * jcp.ih * jcp.iw;
 
     jcp.typesize_in = sizeof(prec_traits<data_type::f32>::type);
     jcp.typesize_out = sizeof(prec_traits<data_type::f32>::type);
@@ -745,7 +749,8 @@ status_t jit_avx2_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
     jcp.with_eltwise = eltwise_ind != -1;
     const int binary_ind
             = post_ops.find(primitive_kind::binary, 0, dw_conv_ind);
-    jcp.with_binary = binary_ind != -1;
+    const int prelu_ind = post_ops.find(primitive_kind::prelu, 0, dw_conv_ind);
+    jcp.with_binary = !everyone_is(-1, binary_ind, prelu_ind);
 
     if (dw_conv_ind >= 0) {
         // dw_conv and post_ops after it are handled externally, so skip them
@@ -757,8 +762,8 @@ status_t jit_avx2_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
 
     const auto dat_tag_nxc = utils::pick(ndims - 3, nwc, nhwc, ndhwc);
     const auto dat_tag_nCx8c = utils::pick(ndims - 3, nCw8c, nChw8c, nCdhw8c);
-    jcp.src_tag = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx8c);
-    jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx8c);
+    jcp.src_tag = src_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_nCx8c);
+    jcp.dst_tag = dst_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_nCx8c);
     const bool is_data_layout_nxc
             = utils::everyone_is(dat_tag_nxc, jcp.src_tag, jcp.dst_tag);
     const auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx8c;
@@ -780,18 +785,22 @@ status_t jit_avx2_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
     }
 
     if (jcp.with_eltwise || jcp.with_binary)
-        if (jcp.isa < avx2) return status::unimplemented;
+        VDISPATCH_CONV_IC(jcp.isa >= avx2, VERBOSE_UNSUPPORTED_FEATURE,
+                "eltwise and binary post-ops not implemented on isa");
 
     using namespace injector;
     static constexpr bool sum_at_pos_0_only = true;
     static constexpr bool sum_requires_scale_one = true;
-    const bool post_ops_ok_ = post_ops_ok({avx2, {eltwise, binary, sum},
-            jcp.post_ops, &dst_d, sum_at_pos_0_only, sum_requires_scale_one});
-    if (!post_ops_ok_) return status::unimplemented;
+    static constexpr bool sum_requires_zp_zero = true;
+    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(jcp.isa,
+            {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
+            sum_requires_scale_one, sum_requires_zp_zero));
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
 
     bool args_ok = true && jcp.ngroups == 1 && jcp.src_tag == dat_tag
             && jcp.wei_tag == wei_tag && jcp.dst_tag == dat_tag;
-    if (!args_ok) return status::unimplemented;
+
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_UNSUPPORTED_TAG);
 
     args_ok = true && jcp.id == jcp.od && jcp.ih == jcp.oh && jcp.iw == jcp.ow
             && IMPLICATION(!is_data_layout_nxc,
@@ -799,12 +808,12 @@ status_t jit_avx2_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
             && jcp.f_pad == 0 && jcp.t_pad == 0 && jcp.l_pad == 0
             && jcp.stride_w == 1 && jcp.stride_h == 1 && jcp.stride_d == 1
             && jcp.kd == 1 && jcp.kh == 1 && jcp.kw == 1;
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_BAD_PARAM, "");
 
     // TODO: remove this restriction
     // optimized 1x1 bwd_w does not support Intel AVX
-    if (jcp.prop_kind == backward_weights && jcp.isa != avx2)
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(!(jcp.prop_kind == backward_weights && jcp.isa != avx2),
+            VERBOSE_UNSUPPORTED_ISA);
 
     jcp.ic_block = jcp.oc_block = simd_w;
 
@@ -988,6 +997,14 @@ status_t jit_avx2_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
     jcp.nb_load = div_up(jcp.load_dim, jcp.load_block);
     jcp.nb_reduce = div_up(jcp.reduce_dim, jcp.reduce_block);
 
+    if (jcp.prop_kind == backward_weights) {
+        const auto mb_with_nb_reduce
+                = static_cast<dim_t>(jcp.mb) * jcp.nb_reduce;
+        // prevent too large argument to cpu reducer
+        VDISPATCH_CONV_IC(mb_with_nb_reduce <= std::numeric_limits<int>::max(),
+                VERBOSE_BLOCKING_FAIL, "bad argument for cpu reducer");
+    }
+
     return status::success;
 }
 
