@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -69,7 +69,7 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward(
         bias = padded_bias;
     }
 
-    parallel(0, [&](const int ithr, const int nthr) {
+    parallel(jcp.nthr, [&](const int ithr, const int nthr) {
         execute_forward_thr(ithr, nthr, src, weights, bias, weights_dw, bias_dw,
                 dst, scratchpad, post_ops_binary_rhs_arg_vec.data(),
                 post_ops_binary_rhs_arg_vec_dw.data());
@@ -86,8 +86,9 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
         const void *post_ops_binary_rhs_arg_vec_dw) const {
 
     const memory_desc_wrapper src_d(pd()->src_md());
-    const memory_desc_wrapper dst_d(pd()->dst_md());
+    const memory_desc_wrapper dst_d(pd()->dst_1x1_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
+    const memory_desc_wrapper dw_dst_d(pd()->dst_md());
     const memory_desc_wrapper dw_weights_d(
             pd()->arg_md(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS));
     const memory_desc_wrapper dw_bias_d(
@@ -130,6 +131,11 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
     std::vector<data_t *> addrs;
     jit_generator *dw_jit_ker = nullptr;
 
+    const bool is_src_layout_nxc = utils::one_of(
+            jcp.src_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
+    const bool is_dst_layout_nxc = utils::one_of(
+            jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
+
     auto step = [](int default_step, int remaining, int tail_step) {
         assert(default_step <= tail_step);
         return remaining < tail_step ? remaining : default_step;
@@ -170,15 +176,12 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
 
     auto ker_1x1 = [&](int ocb, int icb, int ocb_start, int n, int g, int od,
                            int oh, int ow, int id, int ih, int iw) {
-        const bool is_dst_layout_nxc = utils::one_of(jcp.dst_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int oc_off_idx = is_dst_layout_nxc
                 ? g * jcp.oc + ocb * jcp.oc_block
                 : g * nb_oc + ocb;
-
-        p.output_data = jcp.with_dw_conv
-                ? pbuf + (oh % jcp_dw->kh) * row_offset
-                : &dst[data_blk_off(dst_d, n, oc_off_idx, od, oh, ow)];
+        const size_t dst_off = data_blk_off(dst_d, n, oc_off_idx, od, oh, ow);
+        p.output_data = jcp.with_dw_conv ? pbuf + (oh % jcp_dw->kh) * row_offset
+                                         : &dst[dst_off];
         p.bias_data
                 = &bias[oc_off_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block)];
 
@@ -193,8 +196,6 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
                 = &weights[pd()->with_groups() ? weights_d.blk_off(g, ocb, icb)
                                                : weights_d.blk_off(ocb, icb)];
 
-        const bool is_src_layout_nxc = utils::one_of(jcp.src_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int ic_off_idx = is_src_layout_nxc
                 ? g * jcp.ic + icb * jcp.ic_block
                 : g * nb_ic + icb;
@@ -213,9 +214,8 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
         } else
             p.bcast_data = src + data_blk_off(src_d, n, ic_off_idx, id, ih, iw);
 
-        p.oc_l_off = ocb * jcp.oc_block;
         p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
-        p.dst_orig = dst;
+        p.dst_orig = static_cast<const float *>(p.output_data) - dst_off;
 
         (*kernel_)(&p);
     };
@@ -247,9 +247,10 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
         for (int i = 0; i < jcp_dw->kh; ++i)
             addrs[i] = pbuf + ((oh_1x1++) % jcp_dw->kh) * row_offset;
 
+        const ptrdiff_t wch_stride
+                = static_cast<ptrdiff_t>(is_src_layout_nxc ? 1 : jcp_dw->iw)
+                * jcp_dw->nb_ch_blocking * jcp_dw->ch_block;
         const auto ocb_end = ocb_start + load_step;
-        const auto wch_stride
-                = jcp_dw->iw * jcp_dw->nb_ch_blocking * jcp_dw->ch_block;
         const int dil_h = jcp_dw->dilate_h + 1;
         const int str_h = jcp_dw->stride_h;
         const int ch_num = jcp_dw->nb_ch_blocking;
@@ -273,7 +274,12 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
             jit_conv_call_s par_conv_dw;
 
             par_conv_dw.src = addrs.data();
-            par_conv_dw.dst = &dst[dst_d.blk_off(n, ch, dw_oh, ow)];
+
+            const size_t ch_step = is_dst_layout_nxc
+                    ? jcp_dw->ch_block
+                    : dw_dst_d.blk_off(0, 1, 0, 0);
+            par_conv_dw.dst
+                    = &dst[dw_dst_d.blk_off(n, 0, dw_oh, ow) + ch * ch_step];
 
             par_conv_dw.filt
                     = &weights_dw[dw_weights_d.blk_off(ch, 0, 0, kh, kw)];
@@ -286,7 +292,6 @@ void jit_avx2_1x1_convolution_fwd_t::execute_forward_thr(const int ithr,
             par_conv_dw.load_work = (nstl::min(ch + ch_num, jcp_dw->nb_ch) - ch)
                     * jcp_dw->ch_block;
 
-            par_conv_dw.oc_l_off = ch * jcp_dw->ch_block;
             par_conv_dw.post_ops_binary_rhs_arg_vec
                     = post_ops_binary_rhs_arg_vec_dw;
             par_conv_dw.dst_orig = dst;
@@ -573,7 +578,7 @@ void jit_avx2_1x1_convolution_bwd_weights_t::execute_backward_weights(
         return remaining < tail_step ? remaining : default_step;
     };
 
-    auto oc_ic_sp_loop = [=](int sp_start, int sp_end, bool first_image,
+    auto oc_ic_sp_loop = [&](int sp_start, int sp_end, bool first_image,
                                  data_t *store_to, size_t store_to_ld,
                                  const data_t *diff_dst, const data_t *src,
                                  int ithr) {
@@ -656,6 +661,29 @@ void jit_avx2_1x1_convolution_bwd_weights_t::execute_backward_weights(
         }
     };
 
+    auto maybe_zero_icpad = [&](const int g_start, const int g_end,
+                                    const int ocb_start, const int ocb_end) {
+        // write zeros to IC padded region.
+        const int ic_tail = jcp.ic_without_padding % jcp.ic_block;
+        if (is_ddst_layout_nxc && ic_tail != 0) {
+            for_(int g = g_start; g < g_end; ++g)
+            for (int z_ocb = ocb_start; z_ocb < ocb_end; ++z_ocb) {
+                const int z_icb = nb_ic - 1;
+                const size_t off = pd()->with_groups()
+                        ? diff_weights_d.blk_off(g, z_ocb, z_icb)
+                        : diff_weights_d.blk_off(z_ocb, z_icb);
+                data_t *z_wei = diff_weights + off + ic_tail * jcp.oc_block;
+                const int zero_work
+                        = (nb_ic * jcp.ic_block - jcp.ic_without_padding)
+                        * jcp.oc_block;
+                PRAGMA_OMP_SIMD()
+                for (int o = 0; o < zero_work; ++o) {
+                    z_wei[o] = 0;
+                }
+            }
+        }
+    };
+
     auto ker = [&](const int ithr, const int nthr) {
         assert(nthr == rw->balancer().nthr_);
 
@@ -729,6 +757,11 @@ void jit_avx2_1x1_convolution_bwd_weights_t::execute_backward_weights(
                 img += 1;
             }
 
+            if (rw->balancer().nthr_per_group_ == 1
+                    && bcast_i + 1 >= bcast_work)
+                maybe_zero_icpad(g, g + 1, oc_b,
+                        nstl::min(nb_oc, oc_b + nb_oc_blocking));
+
             nd_iterator_step(
                     g, jcp.ngroups, load_i, load_work, bcast_i, bcast_work);
         }
