@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -20,6 +20,8 @@
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 
+#include "cpu/platform.hpp"
+#include "cpu/scale_utils.hpp"
 #include "cpu/x64/injectors/jit_uni_postops_injector.hpp"
 #include "cpu/x64/jit_avx512_core_amx_1x1_conv_kernel.hpp"
 
@@ -38,21 +40,20 @@ using namespace Xbyak;
 jit_avx512_core_amx_1x1_fwd_kernel_t::jit_avx512_core_amx_1x1_fwd_kernel_t(
         const jit_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, avx512_core_amx)
-    , jcp(ajcp)
-    , attr_(attr) {
+    : jit_generator(jit_name(), avx512_core_amx), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
         using namespace binary_injector;
         const auto &rhs_addr_reg = bin_injector_helper_reg_1;
         const auto &rhs_helper_reg = bin_injector_helper_reg_2;
+        const auto &rhs_addr_cache_reg = bin_injector_helper_reg_3;
         static constexpr bool preserve_gpr = false;
         static constexpr bool preserve_vmm = false;
         const size_t tail_size = jcp.oc_without_padding % isa_simd_width_;
         static constexpr bool use_exact_tail_scalar_bcast = true;
 
         const rhs_arg_static_params_t rhs_arg_static_params {31, rhs_addr_reg,
-                rhs_helper_reg, preserve_gpr, preserve_vmm,
-                GET_OFF(post_ops_binary_rhs_arg_vec),
+                rhs_helper_reg, rhs_addr_cache_reg, preserve_gpr, preserve_vmm,
+                GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size, ktail_mask,
                 use_exact_tail_scalar_bcast};
         const static_params_t static_params {
@@ -112,7 +113,8 @@ size_t jit_avx512_core_amx_1x1_fwd_kernel_t::out_row_offset(
 }
 
 void jit_avx512_core_amx_1x1_fwd_kernel_t::update_buffer_pointers() {
-    auto buffer_offset = [=](bool shift) { return ((buf_count_ + shift) % 2); };
+    auto buffer_offset
+            = [this](bool shift) { return ((buf_count_ + shift) % 2); };
     int wsp_shift = jcp.typesize_acc * (jcp.wsp_buffer_size / 2);
 
     int postop_shift = wsp_shift * buffer_offset(true);
@@ -143,7 +145,8 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::interleave_store() {
             const injector_utils::conditional_register_preserve_guard_t
                     cond_register_guard(jcp.with_binary, this,
                             {bin_injector_helper_reg_1,
-                                    bin_injector_helper_reg_2});
+                                    bin_injector_helper_reg_2,
+                                    bin_injector_helper_reg_3});
             const int wsp_row_offset = jcp.typesize_acc
                     * (osb * jcp.nb_oc_blocking * jcp.max_width * jcp.oc_block
                             + ocb * jcp.max_width * jcp.oc_block
@@ -169,37 +172,48 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::interleave_store() {
 }
 
 Ymm jit_avx512_core_amx_1x1_fwd_kernel_t::ymm_mask(
-        const Ymm &ymm_in, bool mask_flag, bool store) {
+        const Ymm ymm_in, bool mask_flag, bool store) {
     return mask_flag ? (store ? ymm_in | ktail_mask : ymm_in | ktail_mask | T_z)
                      : ymm_in;
 }
 
 Zmm jit_avx512_core_amx_1x1_fwd_kernel_t::zmm_mask(
-        const Zmm &zmm_in, bool mask_flag, bool store) {
+        const Zmm zmm_in, bool mask_flag, bool store) {
     return mask_flag ? (store ? zmm_in | ktail_mask : zmm_in | ktail_mask | T_z)
                      : zmm_in;
 }
 
 void jit_avx512_core_amx_1x1_fwd_kernel_t::cvt2ps(data_type_t type_in,
-        const Zmm &zmm_in, const Operand &op, bool mask_flag = false) {
+        const Zmm zmm_in, const Operand &op, bool mask_flag = false) {
+    using namespace dnnl::impl::data_type;
     const Zmm zmm = zmm_mask(zmm_in, mask_flag);
     switch (type_in) {
-        case data_type::f32:
-        case data_type::s32: vmovups(zmm, op); break;
-        case data_type::s8: vpmovsxbd(zmm, op); break;
-        case data_type::u8: vpmovzxbd(zmm, op); break;
+        case bf16:
+            vpmovzxwd(zmm, op);
+            vpslld(zmm_in, zmm_in, 16);
+            break;
+        case f32:
+        case s32: vmovups(zmm, op); break;
+        case s8: vpmovsxbd(zmm, op); break;
+        case u8: vpmovzxbd(zmm, op); break;
         default: assert(!"unsupported data type");
     }
-    if (type_in != data_type::f32) vcvtdq2ps(zmm_in, zmm_in);
+    if (utils::one_of(type_in, s32, s8, u8)) vcvtdq2ps(zmm_in, zmm_in);
 }
 
-void jit_avx512_core_amx_1x1_fwd_kernel_t::apply_sum(const Zmm &zmm_out,
-        const float *p_sum_scale, const Xbyak::Address &addr,
-        const bool mask_flag) {
+void jit_avx512_core_amx_1x1_fwd_kernel_t::apply_sum(const Zmm zmm_out,
+        const float *p_sum_scale, const int32_t *p_sum_zp,
+        const Xbyak::Address &addr, const bool mask_flag) {
     if (p_sum_scale) {
         const auto p_sum_scale_val = *p_sum_scale;
-        const auto sum_injector = [&, p_sum_scale_val, mask_flag]() {
-            cvt2ps(jcp.dst_dt, zmm_prev_dst, addr, mask_flag);
+        const auto p_sum_zp_val = *p_sum_zp;
+        const auto sum_injector = [&, zmm_out, p_sum_scale_val, p_sum_zp_val,
+                                          mask_flag]() {
+            cvt2ps(jcp.sum_dt, zmm_prev_dst, addr, mask_flag);
+            if (p_sum_zp_val != 0) {
+                vcvtdq2ps(zmm_sum_zp, ptr_b[reg_ptr_sum_zp]);
+                vsubps(zmm_prev_dst, zmm_sum_zp);
+            }
             if (p_sum_scale_val == 1.f)
                 vaddps(zmm_out, zmm_prev_dst);
             else
@@ -210,33 +224,185 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::apply_sum(const Zmm &zmm_out,
     }
 }
 
-void jit_avx512_core_amx_1x1_fwd_kernel_t::apply_postops(const Zmm &zmm_out,
-        const float *p_sum_scale, const Xbyak::Address &addr,
-        const bool mask_flag, const size_t off, const int ocb) {
+void jit_avx512_core_amx_1x1_fwd_kernel_t::apply_postops(const Zmm zmm_out,
+        const float *p_sum_scale, const int32_t *p_sum_zp,
+        const Xbyak::Address &addr, const size_t off, const bool mask_flag) {
     if (jcp.with_eltwise || jcp.with_binary
             || (jcp.with_sum && p_sum_scale != nullptr)) {
-        binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
-
-        apply_sum(zmm_out, p_sum_scale, addr, mask_flag);
+        apply_sum(zmm_out, p_sum_scale, p_sum_zp, addr, mask_flag);
 
         const auto vmm_idx = zmm_out.getIdx();
         if (jcp.with_binary) {
-            const int oc_l_offset = ocb * jcp.oc_block;
-            rhs_arg_params.vmm_idx_to_oc_elem_off_addr.emplace(
-                    vmm_idx, ptr[param1 + GET_OFF(oc_l_off)]);
-            rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
-                    vmm_idx, oc_l_offset);
-            rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
-                    vmm_idx, static_cast<int>(off));
+            binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
+            rhs_arg_params.vmm_idx_to_out_reg.emplace(vmm_idx, out_ptr);
+            rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(vmm_idx, off);
             if (mask_flag) rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
-        }
 
-        postops_injector_->compute_vector(vmm_idx, rhs_arg_params);
+            postops_injector_->compute_vector(vmm_idx, rhs_arg_params);
+        } else {
+            postops_injector_->compute_vector(vmm_idx);
+        }
+    }
+}
+
+bool jit_avx512_core_amx_1x1_fwd_kernel_t::is_fast_postops(
+        const jit_conv_conf_t &jcp) {
+    const auto &p = jcp.post_ops;
+    auto is_relu = [&](int idx) { return p.entry_[idx].is_relu(); };
+    auto is_sum = [&](int idx) {
+        const bool require_scale_one = jcp.src_dt == data_type::bf16;
+        return p.entry_[idx].is_sum(require_scale_one);
+    };
+    switch (p.len()) {
+        case 0: return true;
+        case 1: return is_relu(0) || is_sum(0);
+        case 2: return is_sum(0) && is_relu(1);
+        default: return false;
+    }
+    return false;
+}
+
+inline void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_ymm_bf16(
+        const int idx, const Xbyak::Address &addr, const bool mask_flag) {
+    Ymm ymm_out = Ymm(idx);
+    vcvtneps2bf16(ymm_out, Zmm(idx));
+    vmovdqu16(addr, ymm_mask(ymm_out, mask_flag, true));
+}
+
+void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vectors_int8(
+        int ocb, int osb) {
+    const bool mask_flag
+            = last_oc_block_flag_ && ocb == (jcp.nb_oc_blocking - 1);
+    const auto &p = attr_.post_ops_;
+    const int sum_idx = p.find(primitive_kind::sum);
+    const float *p_sum_scale = nullptr;
+    const int32_t *p_sum_zp = nullptr;
+    if (sum_idx != -1) {
+        const auto &p_entry = p.entry_[sum_idx];
+        p_sum_scale = &p_entry.sum.scale;
+        p_sum_zp = &p_entry.sum.zero_point;
+    }
+    if (p_sum_scale) {
+        if (*p_sum_scale != 1.f)
+            mov(reg_ptr_sum_scale, reinterpret_cast<size_t>(p_sum_scale));
+        if (*p_sum_zp != 0)
+            mov(reg_ptr_sum_zp, reinterpret_cast<size_t>(p_sum_zp));
+    }
+
+    if (jcp.src_zero_point) {
+        const int zp_offset = sizeof(int32_t) * ocb * jcp.oc_block;
+        const Zmm zmm_zp_m = zmm_mask(zmm_zp, mask_flag);
+        vpmulld(zmm_zp_m, zmm_src_zp,
+                EVEX_compress_addr(reg_zp_compensation, zp_offset));
+        for (int j = 0; j < jcp.tile_width; j++) {
+            const Zmm zmm_r = zmm_out(j);
+            vpaddd(zmm_r, zmm_r, zmm_zp_m);
+        }
+    }
+
+    for (int j = 0; j < jcp.tile_width; j++) {
+        const Zmm zmm_r = zmm_out(j);
+        vcvtdq2ps(zmm_r, zmm_r);
+    }
+
+    mov(reg_ptr_scales, ptr[param1 + GET_OFF(scales)]);
+    for (int j = 0; j < jcp.tile_width; j++) {
+        const int scale_offset
+                = jcp.is_oc_scale * (sizeof(float) * ocb * jcp.oc_block);
+        const Zmm zmm_r = zmm_out(j);
+        const Zmm zmm_r_msk = zmm_mask(zmm_r, mask_flag);
+        vmulps(zmm_r_msk, zmm_r,
+                EVEX_compress_addr(reg_ptr_scales, scale_offset));
+    }
+
+    if (jcp.with_bias) {
+        mov(reg_bias, ptr[param1 + GET_OFF(bias)]);
+        int bias_offset = jcp.typesize_bia * ocb * jcp.oc_block;
+        auto bias_addr = EVEX_compress_addr(reg_bias, bias_offset);
+        cvt2ps(jcp.bia_dt, zmm_bias, bias_addr, mask_flag);
+        for (int j = 0; j < jcp.tile_width; j++) {
+            const Zmm zmm_r = zmm_out(j);
+            vaddps(zmm_r, zmm_r, zmm_bias);
+        }
+    }
+
+    if (p_sum_zp && *p_sum_zp != 0)
+        vcvtdq2ps(zmm_sum_zp, ptr_b[reg_ptr_sum_zp]);
+    if (jcp.with_sum && p_sum_scale != nullptr) {
+        const auto p_sum_scale_val = *p_sum_scale;
+        const auto p_sum_zp_val = *p_sum_zp;
+        for (int j = 0; j < jcp.tile_width; j++) {
+            int h = ((osb * jcp.tile_width + j) / jcp.ow);
+            int w = ((osb * jcp.tile_width + j) % jcp.ow);
+
+            const auto off = out_row_offset(h, w, ocb);
+            const auto addr = EVEX_compress_addr(out_ptr, off);
+
+            const Zmm zmm_r = zmm_out(j);
+            cvt2ps(jcp.sum_dt, zmm_prev_dst, addr, mask_flag);
+            if (p_sum_zp_val != 0) vsubps(zmm_prev_dst, zmm_sum_zp);
+            if (p_sum_scale_val == 1.f)
+                vaddps(zmm_r, zmm_prev_dst);
+            else
+                vfmadd231ps(zmm_r, zmm_prev_dst, zword_b[reg_ptr_sum_scale]);
+        }
+    }
+    if (jcp.with_eltwise) {
+        vxorps(zmm_zero, zmm_zero, zmm_zero);
+        for (int j = 0; j < jcp.tile_width; j++) {
+            const Zmm zmm_r = zmm_out(j);
+            vmaxps(zmm_r, zmm_r, zmm_zero);
+        }
+    }
+
+    if (jcp.dst_scale) {
+        mov(reg_ptr_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
+        for (int j = 0; j < jcp.tile_width; j++) {
+            const Zmm zmm_r = zmm_out(j);
+            const Zmm zmm_r_msk = zmm_mask(zmm_r, mask_flag);
+            vmulps(zmm_r_msk, zmm_r, EVEX_compress_addr(reg_ptr_dst_scale, 0));
+        }
+    }
+
+    if (jcp.dst_zero_point) {
+        for (int j = 0; j < jcp.tile_width; j++) {
+            const Zmm zmm_r = zmm_out(j);
+            vaddps(zmm_r, zmm_r, zmm_dst_zp);
+        }
+    }
+
+    // Properly saturate the accumulators for integer datatypes
+    if (one_of(jcp.dst_dt, u8, s8, s32)) {
+        init_saturate_f32(
+                zmm_zero, zmm_saturation, aux_reg_saturation, f32, jcp.dst_dt);
+        for (int j = 0; j < jcp.tile_width; j++) {
+            const Zmm zmm_r = zmm_out(j);
+            saturate_cvt_f32(zmm_r, zmm_zero, zmm_saturation, jcp.dst_dt);
+        }
+    }
+
+    for (int j = 0; j < jcp.tile_width; j++) {
+        const int h = ((osb * jcp.tile_width + j) / jcp.ow);
+        const int w = ((osb * jcp.tile_width + j) % jcp.ow);
+        const auto off = out_row_offset(h, w, ocb);
+        const auto addr = EVEX_compress_addr(out_ptr, off);
+
+        const Zmm zmm_out_store = zmm_mask(zmm_out(j), mask_flag, true);
+        switch (jcp.dst_dt) {
+            case data_type::f32:
+            case data_type::s32: vmovups(addr, zmm_out_store); break;
+            case data_type::bf16:
+                store_output_ymm_bf16(zmm_out_store.getIdx(), addr, mask_flag);
+                break;
+            case data_type::s8: vpmovsdb(addr, zmm_out_store); break;
+            case data_type::u8: vpmovusdb(addr, zmm_out_store); break;
+            default: assert(!"unknown dst_dt");
+        }
     }
 }
 
 void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vector_int8(
-        const Zmm &zmm_out, int ocb, int h, int w) {
+        const Zmm zmm_out, int ocb, int h, int w) {
 
     const auto off = out_row_offset(h, w, ocb);
     const auto addr = EVEX_compress_addr(out_ptr, off);
@@ -246,13 +412,19 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vector_int8(
     const auto &p = attr_.post_ops_;
     const int sum_idx = p.find(primitive_kind::sum);
     const float *p_sum_scale = nullptr;
+    const int32_t *p_sum_zp = nullptr;
     if (sum_idx != -1) {
         const auto &p_entry = p.entry_[sum_idx];
         p_sum_scale = &p_entry.sum.scale;
+        p_sum_zp = &p_entry.sum.zero_point;
     }
 
-    if (p_sum_scale && *p_sum_scale != 1.f)
-        mov(reg_ptr_sum_scale, (size_t)p_sum_scale);
+    if (p_sum_scale) {
+        if (*p_sum_scale != 1.f)
+            mov(reg_ptr_sum_scale, reinterpret_cast<size_t>(p_sum_scale));
+        if (*p_sum_zp != 0)
+            mov(reg_ptr_sum_zp, reinterpret_cast<size_t>(p_sum_zp));
+    }
 
     mov(reg_bias, ptr[param1 + GET_OFF(bias)]);
     mov(reg_ptr_scales, ptr[param1 + GET_OFF(scales)]);
@@ -273,35 +445,91 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vector_int8(
     /* add to zmm_accum: compensation, bias and permute */
     vcvtdq2ps(zmm_out, zmm_out);
 
-    if (jcp.with_bias) vaddps(zmm_out, zmm_out, zmm_bias);
     const Zmm zmm_out_msk = zmm_mask(zmm_out, mask_flag);
     vmulps(zmm_out_msk, zmm_out,
             EVEX_compress_addr(reg_ptr_scales, scale_offset));
 
-    apply_postops(zmm_out, p_sum_scale, addr, mask_flag, off, ocb);
+    if (jcp.with_bias) vaddps(zmm_out_msk, zmm_out, zmm_bias);
 
+    apply_postops(zmm_out, p_sum_scale, p_sum_zp, addr, off, mask_flag);
+
+    if (jcp.dst_scale) {
+        mov(reg_ptr_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
+        vmulps(zmm_out, zmm_out, EVEX_compress_addr(reg_ptr_dst_scale, 0));
+    }
     if (jcp.dst_zero_point) { vaddps(zmm_out, zmm_out, zmm_dst_zp); }
 
     // Properly saturate the accumulators for integer datatypes
     if (one_of(jcp.dst_dt, u8, s8, s32)) {
         init_saturate_f32(
                 zmm_zero, zmm_saturation, aux_reg_saturation, f32, jcp.dst_dt);
-        saturate_f32(zmm_out, zmm_zero, zmm_saturation, jcp.dst_dt);
-        vcvtps2dq(zmm_out, zmm_out);
+        saturate_cvt_f32(zmm_out, zmm_zero, zmm_saturation, jcp.dst_dt);
     }
 
     const Zmm zmm_out_store = zmm_mask(zmm_out, mask_flag, true);
     switch (jcp.dst_dt) {
         case data_type::f32:
         case data_type::s32: vmovups(addr, zmm_out_store); break;
+        case data_type::bf16:
+            store_output_ymm_bf16(zmm_out.getIdx(), addr, mask_flag);
+            break;
         case data_type::s8: vpmovsdb(addr, zmm_out_store); break;
         case data_type::u8: vpmovusdb(addr, zmm_out_store); break;
         default: assert(!"unknown dst_dt");
     }
 }
 
+void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vectors_bf16(
+        int ocb, int osb) {
+    const bool mask_flag
+            = last_oc_block_flag_ && ocb == (jcp.nb_oc_blocking - 1);
+
+    if (jcp.with_bias) {
+        mov(reg_bias, ptr[param1 + GET_OFF(bias)]);
+        const int bias_offset = jcp.typesize_bia * ocb * jcp.oc_block;
+        const auto bias_addr = EVEX_compress_addr(reg_bias, bias_offset);
+        cvt2ps(jcp.bia_dt, zmm_bias, bias_addr, mask_flag);
+        for (int j = 0; j < jcp.tile_width; j++) {
+            const Zmm zmm_r = zmm_out(j);
+            vaddps(zmm_r, zmm_r, zmm_bias);
+        }
+    }
+
+    if (jcp.with_sum) {
+        for (int j = 0; j < jcp.tile_width; j++) {
+            int h = ((osb * jcp.tile_width + j) / jcp.ow);
+            int w = ((osb * jcp.tile_width + j) % jcp.ow);
+            const auto off = out_row_offset(h, w, ocb);
+            const auto addr = EVEX_compress_addr(out_ptr, off);
+            const Zmm zmm_r = zmm_out(j);
+            cvt2ps(jcp.dst_dt, zmm_prev_dst, addr, mask_flag);
+            vaddps(zmm_r, zmm_prev_dst);
+        }
+    }
+    if (jcp.with_eltwise) {
+        vxorps(zmm_zero, zmm_zero, zmm_zero);
+        for (int j = 0; j < jcp.tile_width; j++) {
+            const Zmm zmm_r = zmm_out(j);
+            vmaxps(zmm_r, zmm_r, zmm_zero);
+        }
+    }
+
+    for (int j = 0; j < jcp.tile_width; j++) {
+        const int h = ((osb * jcp.tile_width + j) / jcp.ow);
+        const int w = ((osb * jcp.tile_width + j) % jcp.ow);
+        const auto off = out_row_offset(h, w, ocb);
+        const auto addr = EVEX_compress_addr(out_ptr, off);
+        const Zmm zmm_r = zmm_out(j);
+        if (jcp.dst_dt == data_type::bf16) {
+            store_output_ymm_bf16(zmm_r.getIdx(), addr, mask_flag);
+        } else {
+            vmovups(addr, zmm_mask(zmm_r, mask_flag, true));
+        }
+    }
+}
+
 void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vector_bf16(
-        const Zmm &zmm_out, int ocb, int h, int w) {
+        const Zmm zmm_out, int ocb, int h, int w) {
     const auto off = out_row_offset(h, w, ocb);
     const auto addr = EVEX_compress_addr(out_ptr, off);
 
@@ -335,19 +563,29 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vector_bf16(
     }
 
     static constexpr auto skip_sum_in_injection = nullptr;
-    apply_postops(zmm_out, skip_sum_in_injection, addr, mask_flag, off, ocb);
+    apply_postops(zmm_out, skip_sum_in_injection, skip_sum_in_injection, addr,
+            off, mask_flag);
 
     if (jcp.dst_dt == data_type::bf16) {
-        Ymm ymm_out = Ymm(zmm_out.getIdx());
-        vcvtneps2bf16(ymm_out, zmm_out);
-        vmovdqu16(addr, ymm_mask(ymm_out, mask_flag, true));
+        store_output_ymm_bf16(zmm_out.getIdx(), addr, mask_flag);
     } else {
         vmovups(addr, zmm_mask(zmm_out, mask_flag, true));
     }
 }
 
+// Store all rows of a tile
+void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vectors(
+        int ocb, int osb) {
+    if (is_bf16()) {
+        store_output_vectors_bf16(ocb, osb);
+    } else {
+        store_output_vectors_int8(ocb, osb);
+    }
+}
+
+// Store single row
 void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output_vector(
-        const Zmm &zmm_out, int ocb, int h, int w) {
+        const Zmm zmm_out, int ocb, int h, int w) {
     if (is_bf16()) {
         store_output_vector_bf16(zmm_out, ocb, h, w);
     } else {
@@ -364,7 +602,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::prepare_output() {
 void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output(
         bool do_store, bool has_tail) {
 
-    auto store_output_subblock = [=](int ocb, int osb) {
+    auto store_output_subblock = [&](int ocb, int osb) {
         const int wsp_offset = jcp.typesize_acc
                 * (osb * jcp.nb_oc_blocking * jcp.max_width * jcp.oc_block
                         + ocb * jcp.max_width * jcp.oc_block);
@@ -385,11 +623,12 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::store_output(
                     + wsp_offset];
             const Zmm zmm_r = zmm_out(j);
             vmovups(zmm_r, addr);
-            store_output_vector(zmm_r, ocb, oh_, ow_);
+            if (!jcp.is_fast_postops) store_output_vector(zmm_r, ocb, oh_, ow_);
         }
+        if (do_store && jcp.is_fast_postops) store_output_vectors(ocb, osb);
     };
 
-    auto store_output_block = [=](int os_b = 1) {
+    auto store_output_block = [&](int os_b = 1) {
         if (jcp.src_zero_point) {
             mov(reg_src_zero_point, ptr[param1 + GET_OFF(src_zero_point)]);
             mov(reg_zp_compensation, ptr[param1 + GET_OFF(zp_compensation)]);
@@ -427,7 +666,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::icb_loop(bool do_store) {
     enum tiles_cfg_t { cfg_tiles, cfg_tiles_tail };
     enum restore_tiles_t { write_tiles, read_tiles };
 
-    auto tdpbxxd = [=](const Tmm &x1, const Tmm &x2, const Tmm &x3) {
+    auto tdpbxxd = [this](const Tmm &x1, const Tmm &x2, const Tmm &x3) {
         if (jcp.src_dt == data_type::bf16 && jcp.wei_dt == data_type::bf16) {
             tdpbf16ps(x1, x2, x3);
         } else if (jcp.src_dt == data_type::u8 && jcp.wei_dt == data_type::u8) {
@@ -443,7 +682,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::icb_loop(bool do_store) {
         }
     };
 
-    auto tileloadd_nt = [=](const Tmm &t1, int offset) {
+    auto tileloadd_nt = [this](const Tmm &t1, int offset) {
         int ab_size = jcp.nb_os2_blocking * jcp.nb_os_blocking * jcp.tile_width
                 * (jcp.nb_ic_int * jcp.ic_block_int_np
                         + jcp.nb_oc_blocking * jcp.oc_block);
@@ -458,7 +697,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::icb_loop(bool do_store) {
             tileloadd(t1, ptr[wei_ptr + offset + stride_seq]);
     };
 
-    auto compute_block = [=](int icb, int os_b) {
+    auto compute_block = [&](int icb, int os_b) {
         for (int osb = 0; osb < os_b; osb++) {
             int ih = ((osb * jcp.tile_width) / jcp.ow) * jcp.stride_h;
             int iw = ((osb * jcp.tile_width) % jcp.ow) * jcp.stride_w;
@@ -481,7 +720,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::icb_loop(bool do_store) {
         }
     };
 
-    auto reconfig_tiles = [=](tiles_cfg_t cfg) {
+    auto reconfig_tiles = [this](tiles_cfg_t cfg) {
         tilerelease();
         if (cfg == cfg_tiles) {
             mov(reg_scratch, ptr[param1 + GET_OFF(tile_cfg)]);
@@ -491,7 +730,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::icb_loop(bool do_store) {
         ldtilecfg(ptr[reg_scratch]);
     };
 
-    auto restore_output_tiles = [=](int os_b, restore_tiles_t restore) {
+    auto restore_output_tiles = [&](int os_b, restore_tiles_t restore) {
         mov(reg_tilebuff, ptr[param1 + GET_OFF(src_prf)]);
         for (int ocb = 0; ocb < jcp.nb_oc_blocking; ocb++)
             for (int osb = 0; osb < os_b; osb++) {
@@ -508,7 +747,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::icb_loop(bool do_store) {
             }
     };
 
-    auto reset_tiles = [=](int os_b, bool tail) {
+    auto reset_tiles = [&](int os_b, bool tail) {
         if (jcp.nb_ic_int != 1) {
             restore_output_tiles(os_b, write_tiles);
             reconfig_tiles((tail) ? cfg_tiles_tail : cfg_tiles);
@@ -516,7 +755,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::icb_loop(bool do_store) {
         }
     };
 
-    auto compute_icb_loop = [=](int os_b = 1) {
+    auto compute_icb_loop = [&](int os_b = 1) {
         int shift = (get_ic_tail() && os_b == 1) ? 1 : 0;
         int nb_ic_int = jcp.nb_ic_int - shift;
 
@@ -650,16 +889,17 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::generate() {
     L(label_done);
     postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
 void jit_avx512_core_amx_1x1_fwd_kernel_t::tile_configure(char *tcfg_buff) {
 
     int tile_max_columns_in_bytes
-            = amx::get_max_column_bytes(amx::get_max_palette());
+            = amx::get_max_column_bytes(amx::get_target_palette());
     const int max_palette_size_in_bytes = 64;
 
-    auto cfg_tiles = [=](palette_config_t *buff, int Ac) {
+    auto cfg_tiles = [&](palette_config_t *buff, int Ac) {
         char *_tc = (char *)buff;
         for (int i = 0; i < max_palette_size_in_bytes; i++)
             _tc[i] = 0;
@@ -681,7 +921,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::tile_configure(char *tcfg_buff) {
                 tc_configure_tile(buff, get_out_tensor(s, i), Cr, Cc);
             }
 
-        buff->palette_id = amx::get_max_palette();
+        buff->palette_id = amx::get_target_palette();
     };
 
     int Ac = jcp.typesize_in
@@ -699,7 +939,7 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::tile_configure(char *tcfg_buff) {
 status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
         const convolution_desc_t &cd, memory_desc_t &src_md,
         memory_desc_t &weights_md, memory_desc_t &dst_md,
-        memory_desc_t &bias_md, const primitive_attr_t &attr, int nthreads) {
+        memory_desc_t &bias_md, primitive_attr_t &attr, int nthreads) {
     using namespace prop_kind;
 
     const memory_desc_wrapper src_d(&src_md);
@@ -721,16 +961,14 @@ status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
                     || src_d.data_type() == data_type::s8),
             weights_d.data_type() == data_type::s8,
             one_of(dst_d.data_type(), data_type::f32, data_type::s32,
-                    data_type::s8, data_type::u8));
+                    data_type::s8, data_type::u8, data_type::bf16));
 
-    bool supported = false
-            || (is_bf16_convolution && mayiuse(avx512_core_bf16_amx_bf16))
-            || (is_int8_convolution && mayiuse(avx512_core_bf16_amx_int8));
+    bool supported = mayiuse(avx512_core_amx)
+            && (is_bf16_convolution || is_int8_convolution);
     if (!supported) return status::unimplemented;
 
     jcp = zero<decltype(jcp)>();
-    jcp.isa = is_bf16_convolution ? avx512_core_bf16_amx_bf16
-                                  : avx512_core_bf16_amx_int8;
+    jcp.isa = avx512_core_amx;
     jcp.ndims = ndims;
     jcp.prop_kind = cd.prop_kind;
     jcp.ngroups = with_groups ? weights_d.dims()[0] : 1;
@@ -780,6 +1018,14 @@ status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
     jcp.src_dt = cd.src_desc.data_type;
     jcp.wei_dt = cd.weights_desc.data_type;
 
+    // Dispatch small shapes to VNNI for better performance
+    const auto is_small_shape = jcp.od * jcp.oh * jcp.ow <= 4 && jcp.ic <= 512
+            && jcp.mb * jcp.ngroups * jcp.ic * jcp.oc <= static_cast<int32_t>(
+                       platform::get_per_core_cache_size(1) / 2);
+    const auto is_3d_small_ic = jcp.ndims == 5 && jcp.ic * jcp.oc <= 32
+            && jcp.od >= 128 && jcp.oh >= 128 && jcp.ow >= 128;
+    if (is_small_shape || is_3d_small_ic) return status::unimplemented;
+
     const auto zp = attr.zero_points_;
     jcp.dst_zero_point = !zp.has_default_values(DNNL_ARG_DST);
     jcp.src_zero_point = !zp.has_default_values(DNNL_ARG_SRC);
@@ -820,24 +1066,6 @@ status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
         jcp.ic = rnd_up(jcp.ic, jcp.ic_block);
     }
 
-    const auto &p = attr.post_ops_;
-
-    const int sum_ind = p.find(primitive_kind::sum);
-    jcp.with_sum = sum_ind != -1;
-    const int eltwise_ind = p.find(primitive_kind::eltwise);
-    jcp.with_eltwise = eltwise_ind != -1;
-    const int binary_ind = p.find(primitive_kind::binary);
-    jcp.with_binary = binary_ind != -1;
-
-    jcp.post_ops = p;
-
-    using namespace injector;
-    const bool sum_at_pos_0_only = (jcp.src_dt == data_type::bf16);
-    const bool sum_requires_scale_one = sum_at_pos_0_only;
-    const bool post_ops_ok_ = post_ops_ok({avx512_core, {eltwise, binary, sum},
-            jcp.post_ops, &dst_d, sum_at_pos_0_only, sum_requires_scale_one});
-    if (!post_ops_ok_) return status::unimplemented;
-
     auto set_or_check_wei_format = [&]() {
         using namespace format_tag;
         using namespace memory_extra_flags;
@@ -850,7 +1078,7 @@ status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
                         OIhw16i16o4i, gOIhw16i16o4i, OIdhw16i16o4i,
                         gOIdhw16i16o4i);
         memory_desc_t want_wei_md = weights_md;
-        memory_desc_init_by_tag(want_wei_md, wei_tag);
+        CHECK_BOOL(memory_desc_init_by_tag(want_wei_md, wei_tag));
 
         if (jcp.src_zero_point) {
             want_wei_md.extra.flags |= compensation_conv_asymmetric_src;
@@ -890,6 +1118,31 @@ status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
             CHECK(memory_desc_init_by_tag(bias_md, format_tag::x));
     }
 
+    CHECK(attr.set_default_formats(&dst_md));
+
+    const auto &p = attr.post_ops_;
+
+    const int sum_ind = p.find(primitive_kind::sum);
+    jcp.with_sum = sum_ind != -1;
+    const int eltwise_ind = p.find(primitive_kind::eltwise);
+    jcp.with_eltwise = eltwise_ind != -1;
+    const int binary_ind = p.find(primitive_kind::binary);
+    const int prelu_ind = p.find(primitive_kind::prelu);
+    jcp.with_binary = !everyone_is(-1, binary_ind, prelu_ind);
+    jcp.sum_dt = p.get_sum_dt(jcp.dst_dt);
+
+    jcp.post_ops = p;
+    jcp.is_fast_postops = is_fast_postops(jcp);
+
+    using namespace injector;
+    const bool sum_at_pos_0_only = (jcp.src_dt == data_type::bf16);
+    const bool sum_requires_scale_one = sum_at_pos_0_only;
+    const bool sum_requires_zp_zero = sum_at_pos_0_only;
+    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(avx512_core,
+            {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
+            sum_requires_scale_one, sum_requires_zp_zero));
+    if (!post_ops_ok_) return status::unimplemented;
+
     jcp.typesize_in = types::data_type_size(src_d.data_type());
     jcp.typesize_out = types::data_type_size(dst_d.data_type());
     jcp.typesize_bia
@@ -900,7 +1153,9 @@ status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
     jcp.nb_oc = jcp.oc / jcp.oc_block;
     jcp.nb_ic_int = div_up(jcp.ic_without_padding, jcp.ic_block_int_np);
 
-    jcp.max_width = amx::get_max_rows(amx::get_max_palette());
+    jcp.max_width = amx::get_max_rows(amx::get_target_palette());
+    if (jcp.max_width <= 0) return status::unimplemented;
+
     const int size_treshold = 32;
     const int min_width
             = 1; // TODO: Possible optimizations: do not use small values
@@ -947,10 +1202,10 @@ status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
     jcp.nb_oc_blocking = (jcp.nb_oc % 2 == 0) ? 2 : 1;
     jcp.nb_ic_blocking = 1;
     jcp.nb_os_blocking = (os / jcp.tile_width > 2) ? 2 : 1;
-    jcp.nb_os2_blocking = (jcp.nb_os_blocking > 1)
-            ? ((jcp.nb_os_blocking * jcp.tile_width) % 2 == 0) ? 2 : 1
-            : 1;
     jcp.nb_os = os / jcp.tile_width;
+    jcp.nb_os2_blocking = (jcp.nb_os_blocking > 1)
+            ? (jcp.nb_os % jcp.nb_os_blocking == 0) ? 2 : 1
+            : 1;
 
     jcp.wsp_buffer_size = (size_t)2 * jcp.nb_os_blocking * jcp.nb_oc_blocking
             * jcp.max_width * jcp.oc_block;
@@ -961,8 +1216,11 @@ status_t jit_avx512_core_amx_1x1_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
     jcp.per_one_pstore
             = (avaliable_ops) ? ops_tile_store / avaliable_ops + 1 : 0;
     if (jcp.per_one_pstore > 12) jcp.per_one_pstore = 0;
-    const auto &oscales = attr.output_scales_;
-    jcp.is_oc_scale = oscales.mask_ == 1 << 1;
+
+    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
+    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
+    jcp.is_oc_scale = wei_scales.mask_ != 0;
+    jcp.dst_scale = !dst_scales.has_default_values();
 
     return status::success;
 }
@@ -980,6 +1238,8 @@ void jit_avx512_core_amx_1x1_fwd_kernel_t::init_scratchpad(
         scratchpad.book(key_conv_padded_bias, jcp.oc, jcp.typesize_bia);
     }
     scratchpad.book(key_conv_amx_tilecfg, 2, 64); // 2 whole cachelines
+    book_precomputed_scales(
+            scratchpad, attr.scales_, jcp.ngroups * jcp.oc_without_padding);
 }
 
 } // namespace x64
