@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -45,10 +45,8 @@ using namespace nstl;
                                     ? (md).blk_off((n), (c), (h), (w)) \
                                     : (md).blk_off((n), (c), (d), (h), (w))))
 
-template <data_type_t src_type, data_type_t wei_type, data_type_t dst_type>
-void jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
-        dst_type>::prepare_padded_bias(const char *&bias,
-        const memory_tracking::grantor_t &scratchpad) const {
+void jit_avx512_core_amx_1x1_convolution_fwd_t::prepare_padded_bias(
+        const char *&bias, const memory_tracking::grantor_t &scratchpad) const {
     if (!pd()->wants_padded_bias()) return;
 
     const size_t bia_dt_size = pd()->jcp_.typesize_bia;
@@ -61,9 +59,8 @@ void jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
     bias = padded_bias;
 }
 
-template <data_type_t src_type, data_type_t wei_type, data_type_t dst_type>
-status_t jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
-        dst_type>::execute_forward(const exec_ctx_t &ctx) const {
+status_t jit_avx512_core_amx_1x1_convolution_fwd_t::execute_forward(
+        const exec_ctx_t &ctx) const {
 
     auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
     auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
@@ -72,6 +69,17 @@ status_t jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(pd()->jcp_.post_ops, ctx);
 
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const int wei_scale_mask
+            = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const float *oscales = scale_utils::precompute_scales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales, pd()->IC(),
+            pd()->OC(), false, wei_scale_mask != 0, pd()->attr(),
+            jit_scale_precompute_.get());
+
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
 
@@ -100,8 +108,6 @@ status_t jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
             ? reinterpret_cast<const int32_t *>(&weights[offset])
             : nullptr;
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-
     const bool is_ic_tail = jcp.ic_without_padding % jcp.ic_block_int_np;
     auto wsp = ctx.get_scratchpad_grantor().template get<int32_t>(
             key_conv_amx_wsp_buffer);
@@ -126,7 +132,7 @@ status_t jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
             = (size_t)jcp.mb * jcp.ngroups * os_chunks * oc_chunks;
     kernel_->tile_configure(tcfg);
 
-    parallel(0, [&](const int ithr, const int nthr) {
+    parallel(jcp.nthr, [&](const int ithr, const int nthr) {
         size_t start {0}, end {0};
         balance211(work_amount, nthr, ithr, start, end);
 
@@ -155,6 +161,7 @@ status_t jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
             p.filt = weights + wei_dt_size * _ocb * wei_oc_shift;
             p.bias = bias_w;
             p.scales = &oscales[jcp.is_oc_scale * oc];
+            p.dst_scale = &dst_scales[0];
             p.oc_blocks = ocb;
 
             p.zp_compensation
@@ -162,7 +169,6 @@ status_t jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
             p.src_zero_point = jcp.src_zero_point ? src_zero_point : nullptr;
             p.dst_zero_point = jcp.dst_zero_point ? dst_zero_point : nullptr;
 
-            p.oc_l_off = oc;
             p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec.data();
             p.dst_orig = dst;
 
@@ -213,31 +219,12 @@ status_t jit_avx512_core_amx_1x1_convolution_fwd_t<src_type, wei_type,
             nd_iterator_step(mb, jcp.mb, g, jcp.ngroups, _osb, os_chunks, _ocb,
                     oc_chunks);
         }
+
+        amx_tile_release();
     });
     return status::success;
 }
 
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::s8,
-        data_type::s8, data_type::u8>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::u8,
-        data_type::s8, data_type::u8>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::s8,
-        data_type::s8, data_type::s8>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::u8,
-        data_type::s8, data_type::s8>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::s8,
-        data_type::s8, data_type::s32>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::u8,
-        data_type::s8, data_type::s32>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::s8,
-        data_type::s8, data_type::f32>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::u8,
-        data_type::s8, data_type::f32>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::bf16,
-        data_type::bf16, data_type::bf16>;
-template struct jit_avx512_core_amx_1x1_convolution_fwd_t<data_type::bf16,
-        data_type::bf16, data_type::f32>;
-
 } // namespace x64
 } // namespace cpu
 } // namespace impl
