@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 #include "common/utils.hpp"
 
 #include "cpu/platform.hpp"
+#include "cpu/x64/cpu_barrier.hpp"
 #include "cpu/x64/injectors/injector_utils.hpp"
 #include "cpu/x64/injectors/jit_uni_binary_injector.hpp"
 #include "cpu/x64/injectors/jit_uni_eltwise_injector.hpp"
@@ -44,9 +45,7 @@ using namespace Xbyak;
 jit_avx512_core_bf16_1x1_conv_kernel::jit_avx512_core_bf16_1x1_conv_kernel(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, ker_code_size, true, avx512_core_bf16)
-    , jcp(ajcp)
-    , attr_(attr) {
+    : jit_generator(jit_name(), avx512_core_bf16), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -56,8 +55,8 @@ jit_avx512_core_bf16_1x1_conv_kernel::jit_avx512_core_bf16_1x1_conv_kernel(
         static constexpr bool use_exact_tail_scalar_bcast = true;
 
         const rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx,
-                r14, r15, preserve_gpr, preserve_vmm,
-                GET_OFF(post_ops_binary_rhs_arg_vec),
+                r14, r15, r12, preserve_gpr, preserve_vmm,
+                GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size, k_load_dim_tail_mask,
                 use_exact_tail_scalar_bcast};
         const static_params_t static_params {
@@ -148,18 +147,16 @@ static int vreg_accum_idx(
 }
 
 Address jit_avx512_core_bf16_1x1_conv_kernel::output_ptr(
-        const int i_load, const int i_ur, const int scale) {
+        const int i_load, const int i_ur) {
     if (one_of(jcp.prop_kind, forward_training, forward_inference,
                 backward_data)) {
         const bool is_output_layout_nxc = is_out_layout_nxc();
-        assert(IMPLICATION(
-                is_output_layout_nxc, scale == 1 && !jcp.with_dw_conv));
         int i_load_shift = is_output_layout_nxc
                 ? jcp.load_block
                 : (jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim) * jcp.load_block;
         int i_ur_shift = is_output_layout_nxc ? jcp.load_dim : jcp.load_block;
         int offset = (i_load * i_load_shift + i_ur * i_ur_shift)
-                * jcp.typesize_out * scale;
+                * jcp.typesize_out;
         return EVEX_compress_addr(aux_reg_output_data, offset);
     } else
         return ptr[aux_reg_output_data
@@ -189,28 +186,22 @@ void jit_avx512_core_bf16_1x1_conv_kernel::apply_postops(
         if (jcp.with_binary) {
             binary_injector::rhs_arg_dynamic_params_t rhs_arg_params,
                     rhs_arg_params_tail;
-            const auto oc_off_oprnd = r12;
             const auto mask_tail = jcp.oc_without_padding % isa_simd_width_;
+            if (jcp.with_dw_conv) {
+                add(aux_reg_output_data,
+                        EVEX_compress_addr(rsp, reg_dw_binary_output_off));
+            }
             iterate(load_loop_blk, ur, mask_tail,
                     [&](const bool mask_flag, const int i_load,
                             const int i_ur) {
                         const int aux_output_l_off
-                                = output_ptr(i_load, i_ur).getDisp()
-                                / jcp.typesize_out;
-                        const int oc_l_offset = i_load * jcp.oc_block;
+                                = get_output_offset(i_load, i_ur, true);
                         const auto vmm_idx
                                 = vreg_accum_idx(load_loop_blk, i_load, i_ur);
                         vmm_idxs.emplace(vmm_idx);
 
-                        rhs_arg_params_tail.vmm_idx_to_oc_elem_off_addr.emplace(
-                                vmm_idx, ptr[param1 + GET_OFF(oc_l_off)]);
-                        rhs_arg_params_tail.vmm_idx_to_oc_elem_off_val.emplace(
-                                vmm_idx, oc_l_offset);
-                        rhs_arg_params_tail.vmm_idx_to_oc_off_oprnd.emplace(
-                                vmm_idx, oc_off_oprnd);
-                        rhs_arg_params_tail.vmm_idx_to_out_elem_off_addr
-                                .emplace(vmm_idx,
-                                        ptr[param1 + GET_OFF(dst_l_off)]);
+                        rhs_arg_params_tail.vmm_idx_to_out_reg.emplace(
+                                vmm_idx, aux_reg_output_data);
                         rhs_arg_params_tail.vmm_idx_to_out_elem_off_val.emplace(
                                 vmm_idx, aux_output_l_off);
                         if (mask_flag)
@@ -220,9 +211,15 @@ void jit_avx512_core_bf16_1x1_conv_kernel::apply_postops(
             rhs_arg_params.vmm_tail_idx_.clear();
 
             const injector_utils::register_preserve_guard_t register_guard(
-                    this, {oc_off_oprnd});
+                    this, {reg_tmp, aux_reg_output_data});
             const size_t reg_guard_stack_occupied
                     = register_guard.stack_space_occupied();
+            if (jcp.with_dw_conv) {
+                add(aux_reg_output_data,
+                        EVEX_compress_addr(rsp,
+                                reg_dw_binary_output_off
+                                        + reg_guard_stack_occupied));
+            }
             mov(abi_param1,
                     EVEX_compress_addr(rsp,
                             reg_abi_param1_backup + reg_guard_stack_occupied));
@@ -230,26 +227,16 @@ void jit_avx512_core_bf16_1x1_conv_kernel::apply_postops(
             Label postops_done;
             if (mask_tail) {
                 Label postops_no_tail;
-                const auto tmp_reg = oc_off_oprnd;
-                mov(tmp_reg,
+                mov(reg_tmp,
                         ptr[rsp + reg_load_loop_work_off
                                 + reg_guard_stack_occupied]);
-                cmp(tmp_reg, jcp.oc_block * load_loop_blk);
+                cmp(reg_tmp, jcp.oc_block * load_loop_blk);
                 jge(postops_no_tail, T_NEAR);
-
-                mov(oc_off_oprnd,
-                        EVEX_compress_addr(rsp,
-                                reg_binary_post_op_acc_off
-                                        + reg_guard_stack_occupied));
                 postops_injector_->compute_vector_range(
                         vmm_idxs, rhs_arg_params_tail);
                 jmp(postops_done, T_NEAR);
                 L(postops_no_tail);
             }
-            mov(oc_off_oprnd,
-                    EVEX_compress_addr(rsp,
-                            reg_binary_post_op_acc_off
-                                    + reg_guard_stack_occupied));
             postops_injector_->compute_vector_range(vmm_idxs, rhs_arg_params);
             L(postops_done);
 
@@ -271,24 +258,25 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
     const int reduce_dim_tail = jcp.reduce_dim % jcp.reduce_block;
     const int load_dim_tail = jcp.load_dim % jcp.load_block;
 
-    auto vreg_load = [=](int i_load) {
+    auto vreg_load = [ur, load_loop_blk](int i_load) {
         int idx = ur * load_loop_blk + i_load;
         assert(idx < 31);
         return Zmm(idx);
     };
-    auto ymm_store = [=]() { return Xbyak::Ymm(31); };
-    auto zmm_store = [=]() { return Xbyak::Zmm(31); };
+    auto ymm_store = []() { return Xbyak::Ymm(31); };
+    auto zmm_store = []() { return Xbyak::Zmm(31); };
 
-    const auto vreg_accum = [=](int i_load, int i_ur) {
+    const auto vreg_accum = [load_loop_blk](int i_load, int i_ur) {
         return Zmm(vreg_accum_idx(load_loop_blk, i_load, i_ur));
     };
 
-    auto bias_ptr = [=](int i_load) {
+    auto bias_ptr = [this](int i_load) {
         return EVEX_compress_addr(
                 reg_bias_data, jcp.typesize_bia * jcp.oc_block * i_load);
     };
 
-    auto bcast_ptr = [=](int i_reduce, int i_ur, bool bcast) {
+    auto bcast_ptr = [this, bcast_layout_nxc](
+                             int i_reduce, int i_ur, bool bcast) {
         assert(i_ur < jcp.ur);
         assert(i_reduce <= jcp.reduce_loop_unroll);
         int offt;
@@ -313,7 +301,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
                 aux_reg_bcast_data, jcp.typesize_in * offt, bcast);
     };
 
-    auto load_ptr = [=](int i_reduce, int i_load) {
+    auto load_ptr = [this, load_layout_nxc](int i_reduce, int i_load) {
         int u0 = i_reduce % jcp.reduce_loop_unroll;
         int u1 = i_reduce / jcp.reduce_loop_unroll;
         int lmul = jcp.load_block
@@ -326,7 +314,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
                 u1 * jcp.reduce_loop_load_step + jcp.typesize_in * offt);
     };
 
-    auto store_buffer_ptr = [=](int i_load, int i_ur) {
+    auto store_buffer_ptr = [this](int i_load, int i_ur) {
         const bool is_output_layout_nxc = is_out_layout_nxc();
         int i_load_shift
                 = jcp.load_block * (is_output_layout_nxc ? 1 : jcp.bcast_dim);
@@ -336,7 +324,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
         return EVEX_compress_addr(aux_reg_store_buf, offset);
     };
 
-    auto init = [=]() {
+    auto init = [&]() {
         for (int i_load = 0; i_load < load_loop_blk; ++i_load)
             for (int i_ur = 0; i_ur < ur; ++i_ur) {
                 auto r = vreg_accum(i_load, i_ur);
@@ -344,11 +332,11 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
             }
     };
 
-    auto store = [=]() {
+    auto store = [&]() {
         if (!isa_has_bf16(jcp.isa)) bf16_emu_->init_vcvtneps2bf16();
 
-        auto preamble = [=]() {
-            auto preamble_read = [=](bool from_buf) {
+        auto preamble = [&]() {
+            auto preamble_read = [&](bool from_buf) {
                 for (int i_ur = 0; i_ur < ur; ++i_ur) {
                     for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
                         auto r = vreg_accum(i_load, i_ur);
@@ -409,7 +397,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
             }
         };
 
-        const auto apply_bias = [=]() {
+        const auto apply_bias = [&]() {
             if (jcp.with_bias
                     && one_of(jcp.prop_kind, forward_training,
                             forward_inference)) {
@@ -434,7 +422,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
             }
         };
 
-        const auto apply_bias_and_postops = [=]() {
+        const auto apply_bias_and_postops = [&]() {
             Label store_no_post_ops;
 
             test(reg_reduce_pos_flag,
@@ -446,7 +434,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
             L(store_no_post_ops);
         };
 
-        auto store_output = [=](bool to_buf) {
+        auto store_output = [&](bool to_buf) {
             if (to_buf) {
                 for (int i_ur = 0; i_ur < ur; ++i_ur) {
                     for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
@@ -552,7 +540,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
         }
     };
 
-    auto fma_block_bwd_w = [=](bool is_tail) {
+    auto fma_block_bwd_w = [&](bool is_tail) {
         int n_reduce_tail = jcp.reduce_dim % jcp.reduce_loop_unroll;
         int n_reduce
                 = is_tail && n_reduce_tail > 0 && !jcp.uses_permw_transposition
@@ -574,13 +562,14 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
         }
 
         const int pipeline = saturate(1, 4, pipeline_length_max);
-        auto zmm_prm = [=]() { return zmm_store(); };
-        auto get_load_start_idx = [=](int bcast_count) {
-            return pipeline * jcp.uses_permw_transposition
-                    + (bcast_count % pipeline) * load_loop_blk;
-        };
-        auto pipeline_bcast_ptr = [=](int i_reduce, int i_ur, bool bcast,
-                                          int pipeline_idx) {
+        auto zmm_prm = [zmm_store]() { return zmm_store(); };
+        auto get_load_start_idx
+                = [this, pipeline, load_loop_blk](int bcast_count) {
+                      return pipeline * jcp.uses_permw_transposition
+                              + (bcast_count % pipeline) * load_loop_blk;
+                  };
+        auto pipeline_bcast_ptr = [this, bcast_ptr](int i_reduce, int i_ur,
+                                          bool bcast, int pipeline_idx) {
             if (jcp.uses_permw_transposition) {
                 int offset = 64 * pipeline_idx + jcp.typesize_in * 2 * i_ur;
                 auto p = rsp + broadcast_space + offset;
@@ -590,13 +579,13 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
             }
         };
 
-        auto get_load_mask = [=](int i_reduce) {
+        auto get_load_mask = [this](int i_reduce) {
             bool is_reduce_tail = jcp.reduce_loop_unroll % 2
                     && i_reduce + 2 >= jcp.reduce_loop_unroll;
             return is_load_layout_nxc() || is_reduce_tail ? half_mask
                                                           : full_mask;
         };
-        auto get_bcast_mask = [=](int i_reduce) {
+        auto get_bcast_mask = [this](int i_reduce) {
             bool is_reduce_tail = jcp.reduce_loop_unroll % 2
                     && i_reduce + 2 >= jcp.reduce_loop_unroll;
             return is_bcast_layout_nxc() || is_reduce_tail ? half_mask
@@ -754,7 +743,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
             mov(reg_reduce_pos_flag, EVEX_compress_addr(rsp, perm_reg_offset));
     };
 
-    auto fma_block_fwd_bwd_d = [=](bool is_tail) {
+    auto fma_block_fwd_bwd_d = [&](bool is_tail) {
         int n_reduce_tail = jcp.reduce_dim % jcp.reduce_loop_unroll;
         int n_reduce = is_tail && n_reduce_tail > 0 ? n_reduce_tail
                                                     : jcp.reduce_loop_unroll;
@@ -851,7 +840,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::reduce_loop(
         }
     };
 
-    auto fma_block = [=](bool is_tail) {
+    auto fma_block = [&](bool is_tail) {
         return (jcp.prop_kind == backward_weights)
                 ? fma_block_bwd_w(is_tail)
                 : fma_block_fwd_bwd_d(is_tail);
@@ -899,24 +888,27 @@ void jit_avx512_core_bf16_1x1_conv_kernel::compute_diff_bias(
     auto vunit = Zmm(31);
     auto vreg_prm = Zmm(30);
 
-    auto get_load_offset = [=](int i_reduce, int i_load) {
+    auto get_load_offset = [this](int i_reduce, int i_load) {
         dim_t lmul
                 = jcp.load_block * (is_load_layout_nxc() ? 1 : jcp.reduce_dim);
         dim_t rmul = (is_load_layout_nxc() ? jcp.load_dim : jcp.load_block);
         return (i_load * lmul + i_reduce * rmul) * jcp.typesize_in;
     };
-    auto load_ptr = [=](int i_reduce, int i_load, int offset = 0) {
+    auto load_ptr = [this, get_load_offset](
+                            int i_reduce, int i_load, int offset = 0) {
         return EVEX_compress_addr(
                 aux_reg_load_data, get_load_offset(i_reduce, i_load) + offset);
     };
-    auto bias_ptr = [=](int i_load) {
+    auto bias_ptr = [this](int i_load) {
         return ptr[reg_bias_data + i_load * jcp.load_block * jcp.typesize_acc];
     };
 
-    auto vreg_acc = [=](int i_load) { return Zmm(i_load); };
-    auto vreg_load = [=](int i_load) { return Zmm(load_loop_blk + i_load); };
+    auto vreg_acc = [](int i_load) { return Zmm(i_load); };
+    auto vreg_load = [load_loop_blk](int i_load) {
+        return Zmm(load_loop_blk + i_load);
+    };
 
-    auto compute_diff_bias_block = [=](bool is_tail) {
+    auto compute_diff_bias_block = [&](bool is_tail) {
         for (int i_load = 0; i_load < load_loop_blk; i_load++) {
             auto vacc = vreg_acc(i_load);
             auto vload = vreg_load(i_load);
@@ -1003,10 +995,12 @@ void jit_avx512_core_bf16_1x1_conv_kernel::generate() {
 
     sub(rsp, stack_space_needed);
     if (jcp.with_binary) {
-        const auto zeroed_reg = r15;
-        xor_(zeroed_reg, zeroed_reg);
-        mov(EVEX_compress_addr(rsp, reg_binary_post_op_acc_off), zeroed_reg);
         mov(EVEX_compress_addr(rsp, reg_abi_param1_backup), abi_param1);
+        if (jcp.with_dw_conv) {
+            const auto zeroed_reg = r15;
+            xor_(zeroed_reg, zeroed_reg);
+            mov(EVEX_compress_addr(rsp, reg_dw_binary_output_off), zeroed_reg);
+        }
     }
 
     mov(reg_bcast_data, ptr[param1 + GET_OFF(bcast_data)]);
@@ -1057,7 +1051,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::generate() {
         mov(reg_output_stride, ptr[param1 + GET_OFF(output_stride)]);
     }
 
-    auto load_loop_body = [=](int load_loop_blk) {
+    auto load_loop_body = [&](int load_loop_blk) {
         Label no_update_mask, update_mask_done;
         if (load_dim_tail) {
             cmp(reg_load_loop_work, load_loop_blk * jcp.load_loop_iter_step);
@@ -1078,29 +1072,26 @@ void jit_avx512_core_bf16_1x1_conv_kernel::generate() {
         mov(reg_load_loop_work, ptr[rsp + reg_load_loop_work_off]);
 
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
+        const size_t off_with_dw_conv = load_loop_blk * jcp.load_block
+                * jcp.typesize_out
+                * (is_out_layout_nxc()
+                                ? 1
+                                : (jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim));
+        const size_t offst_wo_dw_conv = load_loop_blk * jcp.load_block
+                * jcp.typesize_out * (is_out_layout_nxc() ? 1 : jcp.bcast_dim);
         switch (jcp.prop_kind) {
             case forward_training:
             case forward_inference:
                 add(reg_bias_data,
                         load_loop_blk * jcp.load_block * jcp.typesize_bia);
-                add(reg_output_data,
-                        load_loop_blk * jcp.load_block * jcp.typesize_out
-                                * (is_out_layout_nxc()
-                                                ? 1
-                                                : (jcp.with_dw_conv
-                                                                ? jcp.ow
-                                                                : jcp.bcast_dim)));
+                add(reg_output_data, off_with_dw_conv);
                 add(reg_store_buf,
                         load_loop_blk * jcp.load_block * jcp.typesize_acc
                                 * (is_out_layout_nxc() ? 1 : jcp.bcast_dim));
-                if (jcp.with_binary) {
-                    const auto oc_off_oprnd = rcx;
-                    mov(oc_off_oprnd,
-                            EVEX_compress_addr(
-                                    rsp, reg_binary_post_op_acc_off));
-                    add(oc_off_oprnd, jcp.load_block * load_loop_blk);
-                    mov(EVEX_compress_addr(rsp, reg_binary_post_op_acc_off),
-                            oc_off_oprnd);
+                if (jcp.with_binary && jcp.with_dw_conv) {
+                    mov(rcx, EVEX_compress_addr(rsp, reg_dw_binary_output_off));
+                    add(rcx, offst_wo_dw_conv - off_with_dw_conv);
+                    mov(EVEX_compress_addr(rsp, reg_dw_binary_output_off), rcx);
                 }
                 break;
             case backward_data:
@@ -1181,7 +1172,8 @@ void jit_avx512_core_bf16_1x1_conv_kernel::generate() {
 
     postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 
     if (jcp.prop_kind == backward_weights) {
         const uint16_t dst_prm_array[32] = {0, 16, 1, 17, 2, 18, 3, 19, 4, 20,
@@ -1198,8 +1190,8 @@ void jit_avx512_core_bf16_1x1_conv_kernel::generate() {
 status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
         jit_1x1_conv_conf_t &jcp, const convolution_desc_t &cd,
         const memory_desc_wrapper &src_d, const memory_desc_wrapper &weights_d,
-        const memory_desc_wrapper &dst_d, const primitive_attr_t &attr,
-        int nthreads, bool reduce_src) {
+        const memory_desc_wrapper &dst_d, primitive_attr_t &attr, int nthreads,
+        bool reduce_src) {
     const bool with_groups = weights_d.ndims() == src_d.ndims() + 1;
     const int simd_w = cpu_isa_traits<avx512_core>::vlen / sizeof(float);
     const int ndims = src_d.ndims();
@@ -1246,8 +1238,8 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
             : data_type::undef;
     jcp.typesize_bia = jcp.with_bias ? types::data_type_size(jcp.bia_dt) : 0;
 
-    jcp.os = jcp.od * jcp.oh * jcp.ow;
-    jcp.is = jcp.id * jcp.ih * jcp.iw;
+    jcp.os = static_cast<dim_t>(jcp.od) * jcp.oh * jcp.ow;
+    jcp.is = static_cast<dim_t>(jcp.id) * jcp.ih * jcp.iw;
 
     const auto &post_ops = attr.post_ops_;
     const int dw_conv_ind = post_ops.find(primitive_kind::convolution);
@@ -1265,7 +1257,8 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
     }
     const int binary_ind
             = post_ops.find(primitive_kind::binary, 0, dw_conv_ind);
-    jcp.with_binary = binary_ind != -1;
+    const int prelu_ind = post_ops.find(primitive_kind::prelu, 0, dw_conv_ind);
+    jcp.with_binary = !everyone_is(-1, binary_ind, prelu_ind);
 
     if (dw_conv_ind >= 0) {
         // dw_conv and post_ops after it are handled externally, so skip them
@@ -1278,15 +1271,17 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
     using namespace injector;
     static constexpr bool sum_at_pos_0_only = true;
     static constexpr bool sum_requires_scale_one = true;
-    const bool post_ops_ok_ = post_ops_ok({avx512_core, {eltwise, binary, sum},
-            jcp.post_ops, &dst_d, sum_at_pos_0_only, sum_requires_scale_one});
+    static constexpr bool sum_requires_zp_zero = true;
+    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(avx512_core,
+            {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
+            sum_requires_scale_one, sum_requires_zp_zero));
     if (!post_ops_ok_) return status::unimplemented;
 
     using namespace format_tag;
     const auto dat_tag_nxc = pick(ndims - 3, nwc, nhwc, ndhwc);
     const auto dat_tag_nCx16c = pick(ndims - 3, nCw16c, nChw16c, nCdhw16c);
-    jcp.src_tag = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
-    jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
+    jcp.src_tag = src_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_nCx16c);
+    jcp.dst_tag = dst_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_nCx16c);
     bool is_data_layout_nxc
             = utils::everyone_is(dat_tag_nxc, jcp.src_tag, jcp.dst_tag);
     auto required_dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx16c;
@@ -1436,7 +1431,7 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
             }
         }
         if (jcp.ur == 1) {
-            jcp.ur = nstl::min(max_regs, jcp.os);
+            jcp.ur = nstl::min<dim_t>(max_regs, jcp.os);
             int os_tail = jcp.os % max_regs;
             for (int i = max_regs; i >= min_regs; i -= ur_step) {
                 int i_tail = jcp.os % i;
@@ -1476,11 +1471,11 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
             if (jcp.expl_bcast) {
                 if (jcp.load_dim <= BIG_LOAD_DIM && spatial > SMALL_SPATIAL
                         && spatial < BIG_SPATIAL)
-                    reduce_blocking = nstl::min(jcp.reduce_dim, 160);
+                    reduce_blocking = nstl::min<dim_t>(jcp.reduce_dim, 160);
                 else if (spatial > SMALL_SPATIAL)
-                    reduce_blocking = nstl::min(jcp.reduce_dim, 1024);
+                    reduce_blocking = nstl::min<dim_t>(jcp.reduce_dim, 1024);
                 else
-                    reduce_blocking = nstl::min(jcp.reduce_dim, 512);
+                    reduce_blocking = nstl::min<dim_t>(jcp.reduce_dim, 512);
             } else {
                 reduce_blocking = nb_reduce;
                 if (spatial <= SMALL_SPATIAL
@@ -1526,7 +1521,7 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
         if (jcp.nthr <= 28 && jcp.mb < jcp.nthr
                 && nb_load * nb_bcast > jcp.nthr) {
             // Some heuristic here
-            float calc_koef = 0.01, best_cost = FLT_MAX;
+            float calc_koef = 0.01f, best_cost = FLT_MAX;
             int n_lgc = jcp.nthr;
             float ratio = (float)load_size / (float)bcast_size;
             int best_lgc = ratio > 1 ? n_lgc : 1;
@@ -1581,7 +1576,7 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
         bcast_blocking = div_up(jcp.mb * jcp.ngroups * nb_bcast,
                                  div_up(jcp.nthr, jcp.load_grp_count))
                 * jcp.bcast_block;
-        bcast_blocking = nstl::min(jcp.bcast_dim, bcast_blocking);
+        bcast_blocking = nstl::min<dim_t>(jcp.bcast_dim, bcast_blocking);
         bcast_blocking = rnd_up(bcast_blocking, jcp.bcast_block);
 
         int space_for_bcast = (L2_capacity - /* kernel_size - */
@@ -1611,7 +1606,7 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
                 && IMPLICATION(ndims == 5, jcp.oc <= 64);
 
         if (jcp.uses_permw_transposition) {
-            int rdim = nstl::min(256, jcp.reduce_dim);
+            int rdim = nstl::min<dim_t>(256, jcp.reduce_dim);
             jcp.reduce_block = best_divider(jcp.reduce_dim, 7, rdim, true, 2);
         } else
             jcp.reduce_block = best_divider(jcp.reduce_dim, 8, 16, true, 2);
@@ -1684,7 +1679,7 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_conf(
 
         // for reduction balance
         int max_reduce_blocking
-                = nstl::min(L1_capacity / jcp.ur, jcp.reduce_dim);
+                = nstl::min<dim_t>(L1_capacity / jcp.ur, jcp.reduce_dim);
         int min_reduce_blocking
                 = nstl::min(L1_capacity / jcp.ur, nstl::max(jcp.iw, jcp.ih));
         reduce_blocking = best_divider(
@@ -1775,6 +1770,10 @@ status_t jit_avx512_core_bf16_1x1_conv_kernel::init_scratchpad(
         const size_t wei_bia_size
                 = wei_size * n_wei_buffers + bias_size * n_bias_buffers;
         scratchpad.book(key_conv_wei_reduction, wei_bia_size, jcp.typesize_acc);
+        if (dnnl_thr_syncable() && jcp.nthr_mb > 1) {
+            scratchpad.book<simple_barrier::ctx_t>(
+                    key_conv_wei_reduction_bctx, 1);
+        }
 
         if (!jcp.uses_permw_transposition) {
             const size_t dst_diff_tr_size_per_thr
@@ -1839,7 +1838,7 @@ void jit_avx512_core_bf16_1x1_conv_kernel::balance(
     jcp.nthr_g = jcp.ngroups;
     const int nthr = nthreads / jcp.nthr_g;
 
-    auto calc_mem_cost = [=](int nthr_mb, int nthr_oc_b, int nthr_ic_b) {
+    auto calc_mem_cost = [&](int nthr_mb, int nthr_oc_b, int nthr_ic_b) {
         /* calculate per thread memory cost (read/write). high level
         * optimizer tries to minimize memory consumption. few notes: (n1)
         * unclear why, but that essentially helps first convolution...
