@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -38,15 +38,35 @@ using namespace nstl;
     (pd()->with_groups() ? (d).blk_off((g), __VA_ARGS__) \
                          : (d).blk_off(__VA_ARGS__))
 
-template <data_type_t src_type, data_type_t dst_type>
-status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
-        dst_type>::execute_forward_1d(const exec_ctx_t &ctx) const {
+const float *jit_avx512_core_x8s8s32x_convolution_fwd_t::adjust_oscales(
+        const memory_tracking::grantor_t &scratchpad, const float *src_scales,
+        const float *wei_scales) const {
+    auto loc_scales = scratchpad.template get<float>(key_conv_adjusted_scales);
+    const float src_scale = src_scales[0];
+    const int wei_mask = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    float factor = (pd()->jcp_.signed_input && (!pd()->jcp_.has_vnni))
+            ? 1.f / pd()->jcp_.wei_adj_scale
+            : 1.f;
+    switch (wei_mask) {
+        case 0:
+            utils::array_set(loc_scales, src_scale * wei_scales[0] * factor,
+                    pd()->jcp_.simd_w);
+            break;
+        default:
+            for (dim_t c = 0; c < pd()->OC(); c++)
+                loc_scales[c] = src_scale * wei_scales[c] * factor;
+    }
+    return loc_scales;
+}
+
+status_t jit_avx512_core_x8s8s32x_convolution_fwd_t::execute_forward_1d(
+        const exec_ctx_t &ctx) const {
     const auto &jcp = pd()->jcp_;
 
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
+    auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
@@ -58,33 +78,25 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const size_t bia_dt_size = pd()->with_bias()
-            ? types::data_type_size(pd()->desc()->bias_desc.data_type)
-            : 0;
+    const size_t bia_dt_size
+            = pd()->with_bias() ? types::data_type_size(bias_d.data_type()) : 0;
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
 
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        auto local_scales = ctx.get_scratchpad_grantor().template get<float>(
-                key_conv_adjusted_scales);
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(local_scales, oscales[0] * factor, 16);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = oscales[c] * factor;
-        }
-        oscales = local_scales;
-    }
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = adjust_oscales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales);
 
     size_t extra_data_offset
             = weights_d.size() - weights_d.additional_buffer_size();
     size_t ch_offset = jcp.is_depthwise ? jcp.nb_ch * jcp.ch_block
                                         : jcp.ngroups * jcp.oc;
-    auto w = const_cast<wei_data_t *>(weights);
+    auto w = const_cast<char *>(weights);
     int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(&w[extra_data_offset])
             : nullptr;
@@ -140,7 +152,8 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                     = jcp.src_zero_point ? zp_compensation + g_oc : nullptr;
             p.src_zero_point = jcp.src_zero_point ? src_zero_point : nullptr;
             p.dst_zero_point = jcp.dst_zero_point ? dst_zero_point : nullptr;
-            p.dst = dst + dst_d.blk_off(n, g_oc, ow_s);
+            p.dst_scale = dst_scales;
+            p.dst = dst + dst_dt_size * dst_d.blk_off(n, g_oc, ow_s);
             p.src = src + src_d.blk_off(n, g_ic, iw_s);
             p.filt = weights + wht_blk_off(weights_d, gb, ocb, 0);
             p.scales = &oscales[jcp.is_oc_scale * g_oc];
@@ -150,7 +163,6 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
             p.b_overflow = 0;
             p.owb = owb;
 
-            p.oc_l_off = (g * jcp.nb_oc + ocb) * jcp.oc_block;
             p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec.data();
             p.dst_orig = dst;
             (*kernel_)(&p);
@@ -180,51 +192,42 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
     return status::success;
 }
 
-template <data_type_t src_type, data_type_t dst_type>
-status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
-        dst_type>::execute_forward_2d(const exec_ctx_t &ctx) const {
+status_t jit_avx512_core_x8s8s32x_convolution_fwd_t::execute_forward_2d(
+        const exec_ctx_t &ctx) const {
     const auto &jcp = pd()->jcp_;
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
+    auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
 
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
     const memory_desc_wrapper src_d(pd()->src_md());
     const memory_desc_wrapper dst_d(pd()->dst_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const size_t bia_dt_size = pd()->with_bias()
-            ? types::data_type_size(pd()->desc()->bias_desc.data_type)
-            : 0;
+    const size_t bia_dt_size
+            = pd()->with_bias() ? types::data_type_size(bias_d.data_type()) : 0;
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
 
     assert(jcp.ch_block == 1);
     assert(jcp.nb_ch_blocking == 1);
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        auto local_scales = ctx.get_scratchpad_grantor().template get<float>(
-                key_conv_adjusted_scales);
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(local_scales, oscales[0] * factor, 16);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = oscales[c] * factor;
-        }
-        oscales = local_scales;
-    }
+    const float *oscales = adjust_oscales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales);
 
     size_t offset = weights_d.size() - weights_d.additional_buffer_size();
-    auto w = const_cast<wei_data_t *>(weights);
+    auto w = const_cast<char *>(weights);
     int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(&w[offset])
             : nullptr;
@@ -284,7 +287,8 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                 int32_t *compensation_w
                         = (jcp.signed_input) ? compensation + g_oc : nullptr;
 
-                auto dst_w = dst + dst_d.blk_off(n, g_oc, oh_s, ow_s);
+                auto dst_w = dst
+                        + dst_dt_size * dst_d.blk_off(n, g_oc, oh_s, ow_s);
                 auto src_w = src + src_d.blk_off(n, g_ic, ih_s, iw_s);
                 auto wht_w = weights + wht_blk_off(weights_d, g, ocb, 0);
 
@@ -318,6 +322,7 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                             = jcp.src_zero_point ? src_zero_point : nullptr;
                     p.dst_zero_point
                             = jcp.dst_zero_point ? dst_zero_point : nullptr;
+                    p.dst_scale = dst_scales;
                     p.oc_blocks = ocb;
                     p.kh_padding = kh_padding;
                     p.scales = scales;
@@ -325,14 +330,13 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                     p.b_overflow = i_b_overflow;
                     p.owb = owb;
 
-                    p.oc_l_off = (g * jcp.nb_oc + ocb) * jcp.oc_block;
                     p.post_ops_binary_rhs_arg_vec
                             = post_ops_binary_rhs_arg_vec.data();
                     p.dst_orig = dst;
                     (*kernel_)(&p);
 
                     src_w += src_h_stride * jcp.stride_h;
-                    dst_w += dst_h_stride;
+                    dst_w += dst_dt_size * dst_h_stride;
                 }
             }
             switch (jcp.loop_order) {
@@ -356,28 +360,31 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
     return status::success;
 }
 
-template <data_type_t src_type, data_type_t dst_type>
-status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
-        dst_type>::execute_forward_2d_dw(const exec_ctx_t &ctx) const {
+status_t jit_avx512_core_x8s8s32x_convolution_fwd_t::execute_forward_2d_dw(
+        const exec_ctx_t &ctx) const {
     const auto &jcp = pd()->jcp_;
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
+    auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
 
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
     const memory_desc_wrapper src_d(pd()->src_md());
     const memory_desc_wrapper dst_d(pd()->dst_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const size_t bia_dt_size = pd()->with_bias()
-            ? types::data_type_size(pd()->desc()->bias_desc.data_type)
-            : 0;
+    const size_t bia_dt_size
+            = pd()->with_bias() ? types::data_type_size(bias_d.data_type()) : 0;
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
 
     assert(jcp.ic_block == 1);
     assert(jcp.oc_block == 1);
@@ -386,23 +393,11 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
     assert(jcp.nb_oc_blocking == 1);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        auto local_scales = ctx.get_scratchpad_grantor().template get<float>(
-                key_conv_adjusted_scales);
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(local_scales, oscales[0] * factor, 16);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = oscales[c] * factor;
-        }
-        oscales = local_scales;
-    }
+    const float *oscales = adjust_oscales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales);
 
     size_t offset = weights_d.size() - weights_d.additional_buffer_size();
-    auto w = const_cast<wei_data_t *>(weights);
+    auto w = const_cast<char *>(weights);
     int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(&w[offset])
             : nullptr;
@@ -414,7 +409,7 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
     int group_block = jcp.ch_block;
 
     parallel_nd(jcp.mb, jcp.oh, jcp.nb_ow, nb_groups,
-            [&](int n, int oh_s, int owb, int gg) {
+            [&](dim_t n, dim_t oh_s, dim_t owb, dim_t gg) {
                 auto p = jit_conv_call_s();
 
                 size_t src_h_stride = src_d.blk_off(0, 0, 1);
@@ -432,7 +427,8 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                 int32_t *compensation_w
                         = jcp.signed_input ? compensation + g : nullptr;
 
-                auto dst_w = dst + dst_d.blk_off(n, g, oh_s, ow_s);
+                auto dst_w
+                        = dst + dst_dt_size * dst_d.blk_off(n, g, oh_s, ow_s);
                 auto src_w = src + src_d.blk_off(n, g, ih_s, iw_s);
                 auto wht_w = weights + wht_blk_off(weights_d, gb, 0);
 
@@ -463,6 +459,7 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                         = jcp.src_zero_point ? src_zero_point : nullptr;
                 p.dst_zero_point
                         = jcp.dst_zero_point ? dst_zero_point : nullptr;
+                p.dst_scale = dst_scales;
                 p.oc_blocks = gb;
                 p.kh_padding = kh_padding;
                 p.scales = scales;
@@ -470,7 +467,6 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                 p.b_overflow = i_b_overflow;
                 p.owb = owb;
 
-                p.oc_l_off = g * jcp.oc;
                 p.post_ops_binary_rhs_arg_vec
                         = post_ops_binary_rhs_arg_vec.data();
                 p.dst_orig = dst;
@@ -480,14 +476,13 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
     return status::success;
 }
 
-template <data_type_t src_type, data_type_t dst_type>
-status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
-        dst_type>::execute_forward_3d(const exec_ctx_t &ctx) const {
+status_t jit_avx512_core_x8s8s32x_convolution_fwd_t::execute_forward_3d(
+        const exec_ctx_t &ctx) const {
     const auto &jcp = pd()->jcp_;
-    auto src = CTX_IN_MEM(const src_data_t *, DNNL_ARG_SRC);
-    auto weights = CTX_IN_MEM(const wei_data_t *, DNNL_ARG_WEIGHTS);
+    auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
+    auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
     auto bias = CTX_IN_MEM(const char *, DNNL_ARG_BIAS);
-    auto dst = CTX_OUT_MEM(dst_data_t *, DNNL_ARG_DST);
+    auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(jcp.post_ops, ctx);
 
@@ -499,32 +494,24 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    const size_t bia_dt_size = pd()->with_bias()
-            ? types::data_type_size(pd()->desc()->bias_desc.data_type)
-            : 0;
+    const size_t bia_dt_size
+            = pd()->with_bias() ? types::data_type_size(bias_d.data_type()) : 0;
+    const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
 
     assert(jcp.ch_block == 1);
     assert(jcp.nb_ch_blocking == 1);
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
     assert(jcp.nb_ch % jcp.nb_ch_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        auto local_scales = ctx.get_scratchpad_grantor().template get<float>(
-                key_conv_adjusted_scales);
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(local_scales, oscales[0] * factor, 16);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = oscales[c] * factor;
-        }
-        oscales = local_scales;
-    }
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = adjust_oscales(
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales);
 
     size_t offset = weights_d.size() - weights_d.additional_buffer_size();
-    auto w = const_cast<wei_data_t *>(weights);
+    auto w = const_cast<char *>(weights);
     int32_t *compensation = (jcp.signed_input)
             ? reinterpret_cast<int32_t *>(&w[offset])
             : nullptr;
@@ -598,7 +585,9 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                 int32_t *compensation_w
                         = (jcp.signed_input) ? compensation + g_oc : nullptr;
 
-                auto dst_w = dst + dst_d.blk_off(n, g_oc, od_s, oh_s, ow_s);
+                auto dst_w = dst
+                        + dst_dt_size
+                                * dst_d.blk_off(n, g_oc, od_s, oh_s, ow_s);
                 auto src_w = src + src_d.blk_off(n, g_ic, id_s, ih_s, iw_s)
                         + d_f_overflow * dilate_d * src_d_stride;
                 auto wht_w = weights + wht_blk_off(weights_d, g, ocb, 0)
@@ -637,6 +626,7 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                             = jcp.src_zero_point ? src_zero_point : nullptr;
                     p.dst_zero_point
                             = jcp.dst_zero_point ? dst_zero_point : nullptr;
+                    p.dst_scale = dst_scales;
                     p.oc_blocks = ocb;
                     p.kh_padding = kh_padding;
                     p.kd_padding = kd_padding;
@@ -647,14 +637,13 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
                     p.back_overflow = d_back_overflow;
                     p.owb = owb;
 
-                    p.oc_l_off = (g * jcp.nb_oc + ocb) * jcp.oc_block;
                     p.post_ops_binary_rhs_arg_vec
                             = post_ops_binary_rhs_arg_vec.data();
                     p.dst_orig = dst;
                     (*kernel_)(&p);
 
                     src_w += src_h_stride * jcp.stride_h;
-                    dst_w += dst_h_stride;
+                    dst_w += dst_dt_size * dst_h_stride;
                 }
             }
             switch (jcp.loop_order) {
@@ -680,22 +669,6 @@ status_t jit_avx512_core_x8s8s32x_convolution_fwd_t<src_type,
     return status::success;
 }
 
-template struct jit_avx512_core_x8s8s32x_convolution_fwd_t<data_type::s8,
-        data_type::u8>;
-template struct jit_avx512_core_x8s8s32x_convolution_fwd_t<data_type::u8,
-        data_type::u8>;
-template struct jit_avx512_core_x8s8s32x_convolution_fwd_t<data_type::s8,
-        data_type::s8>;
-template struct jit_avx512_core_x8s8s32x_convolution_fwd_t<data_type::u8,
-        data_type::s8>;
-template struct jit_avx512_core_x8s8s32x_convolution_fwd_t<data_type::s8,
-        data_type::s32>;
-template struct jit_avx512_core_x8s8s32x_convolution_fwd_t<data_type::u8,
-        data_type::s32>;
-template struct jit_avx512_core_x8s8s32x_convolution_fwd_t<data_type::s8,
-        data_type::f32>;
-template struct jit_avx512_core_x8s8s32x_convolution_fwd_t<data_type::u8,
-        data_type::f32>;
 } // namespace x64
 } // namespace cpu
 } // namespace impl
