@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
 *******************************************************************************/
 
 #include "common/c_types_map.hpp"
+#include "common/convolution_pd.hpp"
 #include "common/memory.hpp"
 #include "common/nstl.hpp"
 #include "common/type_helpers.hpp"
@@ -41,9 +42,7 @@ using namespace Xbyak;
 jit_sse41_1x1_conv_kernel_f32::jit_sse41_1x1_conv_kernel_f32(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, sse41)
-    , jcp(ajcp)
-    , attr_(attr) {
+    : jit_generator(jit_name(), sse41), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
         static constexpr bool preserve_gpr = true;
         static constexpr bool preserve_vmm = false;
@@ -52,8 +51,8 @@ jit_sse41_1x1_conv_kernel_f32::jit_sse41_1x1_conv_kernel_f32(
         static constexpr bool use_exact_tail_scalar_bcast = false;
 
         const binary_injector::rhs_arg_static_params_t rhs_arg_static_params {
-                helper_vmm_idx, r13, r14, preserve_gpr, preserve_vmm,
-                GET_OFF(post_ops_binary_rhs_arg_vec),
+                helper_vmm_idx, r13, r14, r15, preserve_gpr, preserve_vmm,
+                GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size,
                 use_exact_tail_scalar_bcast};
         const binary_injector::static_params_t static_params {
@@ -112,8 +111,9 @@ void jit_sse41_1x1_conv_kernel_f32::generate_bcast_loop(int load_loop_blk) {
 }
 
 size_t jit_sse41_1x1_conv_kernel_f32::get_fwd_output_ptr_l_off(
-        int i, int j, int n) {
-    return i * get_output_i_offset(jcp) + j * get_output_j_offset(jcp) + n * 4;
+        int i, int j, int n, bool ignore_dw_conv = false) const {
+    return i * get_output_i_offset(jcp, ignore_dw_conv)
+            + j * get_output_j_offset(jcp) + n * 4;
 }
 
 static int reg_accum_idx(
@@ -133,32 +133,30 @@ void jit_sse41_1x1_conv_kernel_f32::apply_postops(
     injector_utils::vmm_index_set_t vmm_idxs;
     if (jcp.with_binary) {
         binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
-        const auto oc_off_oprnd = r12;
         iterate(load_loop_blk, ur, [&](const int i, const int j, const int n) {
             const bool mask_flag = (2 * i + n) == load_loop_blk - 1;
-            const int aux_output_offset = get_fwd_output_ptr_l_off(i, j, n);
+            const size_t aux_output_offset
+                    = get_fwd_output_ptr_l_off(i, j, n, true) * sizeof(float);
             const auto vmm_idx = reg_accum_idx(load_loop_blk, i, j, n);
             vmm_idxs.emplace(vmm_idx);
 
-            rhs_arg_params.vmm_idx_to_oc_elem_off_addr.emplace(
-                    vmm_idx, ptr[param1 + GET_OFF(oc_l_off)]);
-            rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
-                    vmm_idx, (2 * i + n) * jcp.load_block / 2);
-            rhs_arg_params.vmm_idx_to_oc_off_oprnd.emplace(
-                    vmm_idx, oc_off_oprnd);
+            rhs_arg_params.vmm_idx_to_out_reg.emplace(
+                    vmm_idx, aux_reg_output_data);
             rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
                     vmm_idx, aux_output_offset);
             if (mask_flag) rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
         });
         const injector_utils::register_preserve_guard_t register_guard(
-                this, {abi_param1, oc_off_oprnd});
+                this, {abi_param1, aux_reg_output_data});
         const size_t reg_guard_stack_occupied
                 = register_guard.stack_space_occupied();
+        if (jcp.with_dw_conv) {
+            add(aux_reg_output_data,
+                    ptr[rsp + reg_dw_binary_output_off
+                            + reg_guard_stack_occupied]);
+        }
         mov(abi_param1,
                 ptr[rsp + reg_abi_param1_backup + reg_guard_stack_occupied]);
-        mov(oc_off_oprnd,
-                ptr[rsp + reg_binary_post_op_acc_off
-                        + reg_guard_stack_occupied]);
 
         postops_injector_->compute_vector_range(vmm_idxs, rhs_arg_params);
     } else {
@@ -171,20 +169,20 @@ void jit_sse41_1x1_conv_kernel_f32::apply_postops(
 
 void jit_sse41_1x1_conv_kernel_f32::generate_reduce_loop(
         int load_loop_blk, int ur) {
-    auto reg_load = [=](int i, int n) {
+    auto reg_load = [ur, load_loop_blk](int i, int n) {
         return Xmm(2 * ur * load_loop_blk + 2 * i + n + 1);
     };
 
-    auto reg_accum = [=](int i, int j, int n) {
+    auto reg_accum = [load_loop_blk](int i, int j, int n) {
         return Xmm(reg_accum_idx(load_loop_blk, i, j, n));
     };
 
-    auto bias_ptr = [=](int i, int n) {
+    auto bias_ptr = [this](int i, int n) {
         return ptr[reg_bias_data + sizeof(float) * jcp.oc_block * i
                 + n * 4 * sizeof(float)];
     };
 
-    auto bcast_ptr = [=](int u, int j) {
+    auto bcast_ptr = [&](int u, int j) {
         assert(j < jcp.ur);
         assert(u <= jcp.reduce_loop_unroll);
         size_t offt;
@@ -197,7 +195,7 @@ void jit_sse41_1x1_conv_kernel_f32::generate_reduce_loop(
         return ptr[aux_reg_bcast_data + offt];
     };
 
-    auto load_ptr = [=](int u, int i, int n) {
+    auto load_ptr = [&](int u, int i, int n) {
         size_t offt;
         size_t u0 = u % jcp.reduce_loop_unroll;
         size_t u1 = u / jcp.reduce_loop_unroll;
@@ -214,7 +212,7 @@ void jit_sse41_1x1_conv_kernel_f32::generate_reduce_loop(
                 + sizeof(float) * offt + n * 4 * sizeof(float)];
     };
 
-    auto output_ptr = [=](int i, int j, int n) {
+    auto output_ptr = [this](int i, int j, int n) {
         switch (jcp.prop_kind) {
             case backward_data:
                 return ptr[aux_reg_output_data
@@ -232,7 +230,7 @@ void jit_sse41_1x1_conv_kernel_f32::generate_reduce_loop(
         }
     };
 
-    auto init = [=]() {
+    auto init = [&]() {
         Label init_done;
         Label init_zero;
 
@@ -270,7 +268,7 @@ void jit_sse41_1x1_conv_kernel_f32::generate_reduce_loop(
         shufps(reg_bcast, reg_bcast, 0);
     }; // init()
 
-    auto store = [=]() {
+    auto store = [&]() {
         Label store_noadd;
 
         if (!jcp.with_sum) {
@@ -307,7 +305,7 @@ void jit_sse41_1x1_conv_kernel_f32::generate_reduce_loop(
             }
     };
 
-    auto fma_block = [=](bool last_block) {
+    auto fma_block = [&](bool last_block) {
         for (int u = 0; u < jcp.reduce_loop_unroll; ++u) {
             for (int j = 0; j < ur; ++j) {
                 for (int i = 0; i < load_loop_blk; ++i) {
@@ -368,18 +366,18 @@ void jit_sse41_1x1_conv_kernel_f32::generate_diff_bias_loop(int load_loop_blk) {
     Label diff_bias_loop, diff_bias_loop_out, diff_bias_init_out;
     Label diff_bias_load;
 
-    auto diff_bias_ptr = [=](int i, int n) {
+    auto diff_bias_ptr = [this](int i, int n) {
         return ptr[reg_diff_bias_data + i * jcp.oc_block * sizeof(float)
                 + 4 * n * sizeof(float)];
     };
 
-    auto load_ptr = [=](int u, int i, int n) {
+    auto load_ptr = [this](int u, int i, int n) {
         return ptr[aux_reg_load_data
                 + (i * jcp.os + u) * jcp.oc_block * sizeof(float)
                 + 4 * n * sizeof(float)];
     };
 
-    auto diff_bias_reg = [=](int i, int n) { return Xmm(2 * i + n + 1); };
+    auto diff_bias_reg = [](int i, int n) { return Xmm(2 * i + n + 1); };
 
     mov(reg_diff_bias_data, ptr[rsp + reg_diff_bias_data_stack_offt]);
     cmp(reg_diff_bias_data, 0);
@@ -437,10 +435,12 @@ void jit_sse41_1x1_conv_kernel_f32::generate() {
         // backup abi_param1 for usage in post_ops processing
         mov(ptr[rsp + reg_abi_param1_backup], abi_param1);
 
-        // zero initialize binary post_ops offset accumulator (store on stack)
-        const auto zeroed_reg = r15;
-        xor_(zeroed_reg, zeroed_reg);
-        mov(ptr[rsp + reg_binary_post_op_acc_off], zeroed_reg);
+        if (jcp.with_dw_conv) {
+            // zero initialize dw binary output offset (store on stack)
+            const auto zeroed_reg = r15;
+            xor_(zeroed_reg, zeroed_reg);
+            mov(ptr[rsp + reg_dw_binary_output_off], zeroed_reg);
+        }
     }
 
     mov(reg_bcast_data, ptr[param1 + GET_OFF(bcast_data)]);
@@ -461,7 +461,11 @@ void jit_sse41_1x1_conv_kernel_f32::generate() {
     if (jcp.prop_kind == backward_weights)
         mov(reg_output_stride, ptr[param1 + GET_OFF(output_stride)]);
 
-    auto generate_load_loop_body = [=](int load_loop_blk) {
+    auto generate_load_loop_body = [&](int load_loop_blk) {
+        const size_t offst_with_dw_conv
+                = get_load_loop_output_fwd_offset(jcp, load_loop_blk);
+        const size_t offst_wo_dw_conv
+                = get_load_loop_output_fwd_offset(jcp, load_loop_blk, true);
         generate_bcast_loop(load_loop_blk);
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
         switch (jcp.prop_kind) {
@@ -469,15 +473,12 @@ void jit_sse41_1x1_conv_kernel_f32::generate() {
             case forward_inference:
                 add(reg_bias_data,
                         load_loop_blk * jcp.oc_block * sizeof(float));
-                add(reg_output_data,
-                        get_load_loop_output_fwd_offset(jcp, load_loop_blk));
-                if (jcp.with_binary) {
-                    mov(aux_reg_load_data,
-                            EVEX_compress_addr(
-                                    rsp, reg_binary_post_op_acc_off));
-                    add(aux_reg_load_data, jcp.load_block * load_loop_blk);
-                    mov(EVEX_compress_addr(rsp, reg_binary_post_op_acc_off),
-                            aux_reg_load_data);
+                add(reg_output_data, offst_with_dw_conv);
+                if (jcp.with_binary && jcp.with_dw_conv) {
+                    mov(aux_reg_load_data, ptr[rsp + reg_dw_binary_output_off]);
+                    add(aux_reg_load_data,
+                            offst_wo_dw_conv - offst_with_dw_conv);
+                    mov(ptr[rsp + reg_dw_binary_output_off], aux_reg_load_data);
                 }
                 break;
             case backward_data:
@@ -542,13 +543,15 @@ void jit_sse41_1x1_conv_kernel_f32::generate() {
 
     postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
 status_t jit_sse41_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
         const convolution_desc_t &cd, const memory_desc_wrapper &src_d,
         const memory_desc_wrapper &weights_d, const memory_desc_wrapper &dst_d,
         const primitive_attr_t &attr, int nthreads) {
+    // disabling verbose dispatch messages for unsupported isa for better readability
     if (!mayiuse(sse41)) return status::unimplemented;
 
     // TODO (Roma): this code is duplicated from the generic kernel; maybe the
@@ -600,7 +603,8 @@ status_t jit_sse41_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
     jcp.with_eltwise = eltwise_ind != -1;
     const int binary_ind
             = post_ops.find(primitive_kind::binary, 0, dw_conv_ind);
-    jcp.with_binary = binary_ind != -1;
+    const int prelu_ind = post_ops.find(primitive_kind::prelu, 0, dw_conv_ind);
+    jcp.with_binary = !everyone_is(-1, binary_ind, prelu_ind);
 
     if (dw_conv_ind >= 0) {
         // dw_conv and post_ops after it are handled externally, so skip them
@@ -613,14 +617,16 @@ status_t jit_sse41_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
     using namespace injector;
     static constexpr bool sum_at_pos_0_only = true;
     static constexpr bool sum_requires_scale_one = true;
-    const bool post_ops_ok_ = post_ops_ok({sse41, {eltwise, binary, sum},
-            jcp.post_ops, &dst_d, sum_at_pos_0_only, sum_requires_scale_one});
-    if (!post_ops_ok_) return status::unimplemented;
+    static constexpr bool sum_requires_zp_zero = true;
+    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(sse41,
+            {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
+            sum_requires_scale_one, sum_requires_zp_zero));
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
 
     const auto dat_tag_nxc = utils::pick(ndims - 3, nwc, nhwc);
     const auto dat_tag_blocked = utils::pick(ndims - 3, nCw8c, nChw8c);
-    jcp.src_tag = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_blocked);
-    jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag_nxc, dat_tag_blocked);
+    jcp.src_tag = src_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_blocked);
+    jcp.dst_tag = dst_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_blocked);
     const bool is_data_layout_nxc
             = utils::everyone_is(dat_tag_nxc, jcp.src_tag, jcp.dst_tag);
     const auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_blocked;
@@ -635,7 +641,7 @@ status_t jit_sse41_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
 
     bool args_ok = true && jcp.ngroups == 1 && jcp.src_tag == dat_tag
             && jcp.wei_tag == wei_tag && jcp.dst_tag == dat_tag;
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_UNSUPPORTED_TAG);
 
     const int simd_w = 4;
 
@@ -646,7 +652,8 @@ status_t jit_sse41_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
             && jcp.stride_h == 1 // TODO: support some strides
             && jcp.ow == jcp.iw && jcp.oh == jcp.ih // enforce rpad=0
             && jcp.kh == 1 && jcp.kw == 1;
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            args_ok, VERBOSE_BLOCKING_FAIL, "bad blocking parameters");
 
     jcp.ur = 1;
     if (jcp.with_dw_conv) jcp.ur = nstl::min(jcp.ow, jcp.ur);
@@ -683,11 +690,14 @@ status_t jit_sse41_1x1_conv_kernel_f32::init_conf(jit_1x1_conv_conf_t &jcp,
         jcp.load_loop_load_step = jcp.ic * jcp.oc_block * sizeof(float);
         jcp.load_loop_iter_step = jcp.oc_block;
 
-        load_blocking = 120; // assumes the kernel is jcp.ur x 3
-        load_blocking_max = 144;
+        load_blocking = is_data_layout_nxc
+                ? jcp.load_dim
+                : 120; // assumes the kernel is jcp.ur x 3
+        load_blocking_max = is_data_layout_nxc ? jcp.load_dim : 144;
         bcast_blocking = 128; // affects load balancing across threads
         bcast_blocking_max = 192;
-        reduce_blocking = 128; // affects L1$ utilization
+        reduce_blocking = is_data_layout_nxc ? jcp.reduce_dim
+                                             : 128; // affects L1$ utilization
     } else if (jcp.prop_kind == backward_data) {
         jcp.reduce_dim = jcp.oc;
         jcp.reduce_block = jcp.oc_block;
