@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2020 Intel Corporation
+* Copyright 2017-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
 *******************************************************************************/
 
 #include "common/c_types_map.hpp"
+#include "common/convolution_pd.hpp"
 #include "common/memory.hpp"
 #include "common/nstl.hpp"
 #include "common/type_helpers.hpp"
@@ -40,7 +41,7 @@ using namespace Xbyak;
 jit_sse41_conv_fwd_kernel_f32::jit_sse41_conv_fwd_kernel_f32(
         const jit_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, sse41), jcp(ajcp), attr_(attr) {
+    : jit_generator(jit_name(), sse41), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
         static constexpr bool preserve_gpr = true;
         static constexpr bool preserve_vmm = false;
@@ -49,8 +50,8 @@ jit_sse41_conv_fwd_kernel_f32::jit_sse41_conv_fwd_kernel_f32(
         static constexpr bool use_exact_tail_scalar_bcast = false;
 
         const binary_injector::rhs_arg_static_params_t rhs_arg_static_params {
-                helper_vmm_idx, r14, r15, preserve_gpr, preserve_vmm,
-                GET_OFF(post_ops_binary_rhs_arg_vec),
+                helper_vmm_idx, r14, r15, r12, preserve_gpr, preserve_vmm,
+                GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size,
                 use_exact_tail_scalar_bcast};
         const binary_injector::static_params_t static_params {
@@ -159,27 +160,20 @@ void jit_sse41_conv_fwd_kernel_f32::apply_postops(
     injector_utils::vmm_index_set_t vmm_idxs;
     if (jcp.with_binary) {
         binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
-        const auto temp_offset_reg = this->r13;
         iterate(oc_blocks, ur_w,
                 [&](const bool mask_flag, const int i, const int j) {
-                    const int o_off = get_output_offset(i, j) * sizeof(float);
+                    const size_t o_off = get_output_offset(i, j);
                     const auto vmm_idx = get_xmm_idx(ur_w, i, j);
                     vmm_idxs.emplace(vmm_idx);
 
-                    rhs_arg_params.vmm_idx_to_oc_elem_off_addr.emplace(
-                            vmm_idx, ptr[param1 + GET_OFF(oc_l_off)]);
-                    rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
-                            vmm_idx, i * jcp.oc_block);
+                    rhs_arg_params.vmm_idx_to_out_reg.emplace(
+                            vmm_idx, reg_output);
                     rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
                             vmm_idx, o_off);
-                    rhs_arg_params.vmm_idx_to_oc_off_oprnd.emplace(
-                            vmm_idx, temp_offset_reg);
                     if (mask_flag)
                         rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
                 });
-        const injector_utils::register_preserve_guard_t register_guard(
-                this, {temp_offset_reg});
-        imul(temp_offset_reg, simd_iter, simd_w_);
+
         postops_injector_->compute_vector_range(vmm_idxs, rhs_arg_params);
     } else {
         iterate(oc_blocks, ur_w, [&](const bool, const int i, const int j) {
@@ -381,13 +375,15 @@ void jit_sse41_conv_fwd_kernel_f32::generate() {
 
     this->postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
 status_t jit_sse41_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
         const convolution_desc_t &cd, const memory_desc_wrapper &src_d,
         const memory_desc_wrapper &weights_d, const memory_desc_wrapper &dst_d,
         const primitive_attr_t &attr, int nthreads) {
+    // disabling verbose dispatch messages for unsupported isa for better readability
     if (!mayiuse(sse41)) return status::unimplemented;
 
     jcp.nthr = nthreads;
@@ -430,7 +426,8 @@ status_t jit_sse41_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     bool kernel_outside_src = false || ext_kw <= jcp.l_pad
             || ext_kw <= jcp.r_pad || ext_kh <= jcp.t_pad
             || ext_kh <= jcp.b_pad;
-    if (kernel_outside_src) return status::unimplemented;
+    VDISPATCH_CONV_IC(!kernel_outside_src, VERBOSE_UNSUPPORTED_PAD_FEATURE,
+            "weights and src size mismatch");
 
     const auto dat_tag_nxc = (ndims == 3 ? nwc : nhwc);
     const auto dat_tag_ncx = (ndims == 3 ? ncw : nchw);
@@ -441,10 +438,10 @@ status_t jit_sse41_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     const auto wei_tag_Oxio = with_groups ? pick(ndims - 3, gOwi8o, gOhwi8o)
                                           : pick(ndims - 3, Owi8o, Ohwi8o);
 
-    jcp.src_tag
-            = src_d.matches_one_of_tag(dat_tag_ncx, dat_tag_nxc, dat_tag_nCx8c);
+    jcp.src_tag = src_d.mb_stride_relaxed_match(
+            dat_tag_ncx, dat_tag_nxc, dat_tag_nCx8c);
     jcp.wei_tag = weights_d.matches_one_of_tag(wei_tag_OIxio, wei_tag_Oxio);
-    jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx8c);
+    jcp.dst_tag = dst_d.mb_stride_relaxed_match(dat_tag_nxc, dat_tag_nCx8c);
 
     const bool is_data_layout_nxc
             = utils::everyone_is(dat_tag_nxc, jcp.src_tag, jcp.dst_tag);
@@ -457,21 +454,24 @@ status_t jit_sse41_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     jcp.with_eltwise = eltwise_ind != -1;
 
     const int binary_ind = post_ops.find(primitive_kind::binary);
-    jcp.with_binary = binary_ind != -1;
+    const int prelu_ind = post_ops.find(primitive_kind::prelu);
+    jcp.with_binary = !everyone_is(-1, binary_ind, prelu_ind);
 
     jcp.post_ops = post_ops;
 
     using namespace injector;
     static constexpr bool sum_at_pos_0_only = true;
     static constexpr bool sum_requires_scale_one = true;
-    const bool post_ops_ok_ = post_ops_ok({sse41, {eltwise, binary, sum},
-            jcp.post_ops, &dst_d, sum_at_pos_0_only, sum_requires_scale_one});
-    if (!post_ops_ok_) return status::unimplemented;
+    static constexpr bool sum_requires_zp_zero = true;
+    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(sse41,
+            {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
+            sum_requires_scale_one, sum_requires_zp_zero));
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
 
     const bool flat = jcp.ic == 3;
     const bool mimo = !flat;
 
-    bool args_ok = true
+    const bool tag_ok = true
             && IMPLICATION(flat,
                     jcp.wei_tag == wei_tag_Oxio
                             && ((jcp.src_tag == dat_tag_ncx
@@ -483,10 +483,13 @@ status_t jit_sse41_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
                             && ((jcp.src_tag == dat_tag_nCx8c
                                         && jcp.dst_tag == dat_tag_nCx8c)
                                     || (jcp.src_tag == dat_tag_nxc
-                                            && jcp.dst_tag == dat_tag_nxc)))
-            && jcp.ic <= src_d.padded_dims()[1]
+                                            && jcp.dst_tag == dat_tag_nxc)));
+    VDISPATCH_CONV_IC(tag_ok, VERBOSE_UNSUPPORTED_TAG);
+
+    const bool channel_pad_ok = true && jcp.ic <= src_d.padded_dims()[1]
             && jcp.oc <= dst_d.padded_dims()[1];
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(channel_pad_ok, VERBOSE_UNSUPPORTED_PAD_FEATURE,
+            "i/o and padded channel size mismatch");
 
     const int simd_w = 8; // 2 SSE vectors processing at once
 
@@ -498,12 +501,12 @@ status_t jit_sse41_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
     jcp.nb_oc_blocking
             = is_data_layout_nxc ? 1 : 4; /* the optimal value for the kernel */
 
-    args_ok = true && jcp.oc % simd_w == 0 && jcp.l_pad <= jcp.ur_w
+    const bool args_ok = true && jcp.oc % simd_w == 0 && jcp.l_pad <= jcp.ur_w
             && IMPLICATION(jcp.kw > 7,
                     (jcp.t_pad == 0 && jcp.l_pad == 0)
                             || (jcp.stride_w == 1 && jcp.stride_h == 1))
             && IMPLICATION(mimo, jcp.ic % simd_w == 0);
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_BLOCKING_FAIL, "bad parameters");
 
     int r_pad_no_tail = nstl::max(0,
             calculate_end_padding(jcp.l_pad, jcp.ow - jcp.ur_w_tail, jcp.iw,
@@ -522,8 +525,9 @@ status_t jit_sse41_conv_fwd_kernel_f32::init_conf(jit_conv_conf_t &jcp,
                 calculate_end_padding(jcp.l_pad, jcp.ow - jcp.ur_w_tail, jcp.iw,
                         jcp.stride_w, ext_kw));
 
-        if (jcp.ur_w < nstl::max(jcp.l_pad, r_pad_no_tail))
-            return status::unimplemented;
+        VDISPATCH_CONV_IC(jcp.ur_w >= nstl::max(jcp.l_pad, r_pad_no_tail),
+                VERBOSE_UNSUPPORTED_PAD_FEATURE,
+                "width unroll exceeds padding size");
     }
     assert(jcp.nb_oc_blocking > 0);
     assert(jcp.ur_w * (jcp.nb_oc_blocking + 1) <= num_avail_regs);
