@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -30,7 +30,7 @@ namespace x64 {
 
 using namespace Xbyak;
 
-struct jit_args_t {
+struct jit_args_int8_t {
     const void *from;
     const void *for_comparison;
     const void *to;
@@ -38,18 +38,20 @@ struct jit_args_t {
 };
 
 struct jit_uni_eltwise_int_kernel : public jit_generator {
-    jit_uni_eltwise_int_kernel(const eltwise_desc_t &desc) : desc_(desc) {}
+    jit_uni_eltwise_int_kernel(
+            const eltwise_pd_t *pd, const cpu_isa_t isa, const char *name)
+        : jit_generator(name, isa), pd_(pd) {}
 
-    void operator()(jit_args_t *p) { jit_generator::operator()(p); }
+    void operator()(jit_args_int8_t *p) { jit_generator::operator()(p); }
 
 protected:
-    data_type_t data_type() const { return desc_.data_desc.data_type; }
+    data_type_t data_type() const { return pd_->src_md()->data_type; }
     int dtype_size() const { return types::data_type_size(data_type()); }
 
-    const eltwise_desc_t &desc() const { return desc_; }
+    const eltwise_desc_t &desc() const { return *pd_->desc(); }
 
 private:
-    const eltwise_desc_t &desc_;
+    const eltwise_pd_t *pd_;
 };
 
 /* jit kernels */
@@ -60,15 +62,15 @@ template <cpu_isa_t isa>
 struct jit_uni_subkernel_int_t : public jit_uni_eltwise_int_kernel {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_uni_subkernel_int)
 
-    jit_uni_subkernel_int_t(const eltwise_desc_t &desc)
-        : jit_uni_eltwise_int_kernel(desc) {
+    jit_uni_subkernel_int_t(const eltwise_pd_t *pd)
+        : jit_uni_eltwise_int_kernel(pd, isa, jit_name()) {
         using namespace data_type;
 
         // Relu and linear for int types: s32, s8, u8; Only forward direction
-        assert(utils::one_of(desc.alg_kind, alg_kind::eltwise_relu,
-                alg_kind::eltwise_linear));
+        assert(utils::one_of(desc().alg_kind, alg_kind::eltwise_relu,
+                alg_kind::eltwise_linear, alg_kind::eltwise_clip));
         assert(utils::one_of(data_type(), s32, s8, u8));
-        assert(utils::one_of(isa, sse41, avx2, avx512_common));
+        assert(utils::one_of(isa, sse41, avx2, avx512_core));
     }
 
     void generate() override {
@@ -83,7 +85,7 @@ struct jit_uni_subkernel_int_t : public jit_uni_eltwise_int_kernel {
 
         preamble();
 
-#define GET_OFF(field) offsetof(jit_args_t, field)
+#define GET_OFF(field) offsetof(jit_args_int8_t, field)
         mov(reg_from, ptr[param + GET_OFF(from)]);
         mov(reg_to, ptr[param + GET_OFF(to)]);
         mov(reg_work_amount, ptr[param + GET_OFF(work_amount)]);
@@ -99,7 +101,7 @@ struct jit_uni_subkernel_int_t : public jit_uni_eltwise_int_kernel {
 
         uni_vpxor(vmm_zero, vmm_zero, vmm_zero);
         xor_(reg_int8, reg_int8);
-        if (isa == avx512_common) {
+        if (isa == avx512_core) {
             mov(reg_int8.cvt8(), 0x01);
             kmovw(k_mask_int8, reg_int8.cvt32());
         }
@@ -138,11 +140,11 @@ private:
     Xmm xmm_alpha = Xmm(13);
     Xmm xmm_beta = Xmm(14);
 
-    Vmm vmm_tmp = Vmm(isa == avx512_common ? 26 : 11);
-    Vmm vmm_alpha = Vmm(isa == avx512_common ? 27 : 13);
-    Vmm vmm_beta = Vmm(isa == avx512_common ? 28 : 14);
-    Vmm vmm_zero = Vmm(isa == avx512_common ? 29 : 15);
-    Vmm vmm_mask = Vmm(isa == avx512_common ? 30 : 12);
+    Vmm vmm_tmp = Vmm(isa == avx512_core ? 26 : 11);
+    Vmm vmm_alpha = Vmm(isa == avx512_core ? 27 : 13);
+    Vmm vmm_beta = Vmm(isa == avx512_core ? 28 : 14);
+    Vmm vmm_zero = Vmm(isa == avx512_core ? 29 : 15);
+    Vmm vmm_mask = Vmm(isa == avx512_core ? 30 : 12);
 
     opmask_t k_mask = k1;
     opmask_t k_mask_int8 = k2; // Mask for store 1 byte in case of AVX512
@@ -199,6 +201,7 @@ private:
     // Processing
     void process_linear(const Vmm &vr_to, const Vmm &vr_from);
     void process_relu(const Vmm &vr_to, const Vmm &vr_from);
+    void process_clip(const Vmm &vr_to, const Vmm &vr_from);
 
     // Store s32 for any isa
     void store_32bit(
@@ -245,6 +248,10 @@ private:
                 for (size_t i = 0; i < uf; i++)
                     process_relu(vreg_to(i), vreg_from(i));
                 break;
+            case alg_kind::eltwise_clip:
+                for (size_t i = 0; i < uf; i++)
+                    process_clip(vreg_to(i), vreg_from(i));
+                break;
             default: assert(!"unsupported alg");
         }
 
@@ -266,9 +273,7 @@ void jit_uni_subkernel_int_t<isa>::process_linear(
     uni_vpxor(vmm_zero, vmm_zero, vmm_zero);
     init_saturate_f32(vmm_zero, vmm_saturation_ubound, reg_tmp, data_type::f32,
             data_type());
-    saturate_f32(vr_to, vmm_zero, vmm_saturation_ubound, data_type());
-
-    uni_vcvtps2dq(vr_to, vr_to);
+    saturate_cvt_f32(vr_to, vmm_zero, vmm_saturation_ubound, data_type());
 }
 
 template <cpu_isa_t isa>
@@ -304,7 +309,7 @@ void jit_uni_subkernel_int_t<avx2>::process_relu(
 }
 
 template <>
-void jit_uni_subkernel_int_t<avx512_common>::process_relu(
+void jit_uni_subkernel_int_t<avx512_core>::process_relu(
         const Vmm &vr_to, const Vmm &vr_from) {
 
     vcvtdq2ps(vr_from, vr_from);
@@ -314,6 +319,43 @@ void jit_uni_subkernel_int_t<avx512_common>::process_relu(
     vcvtps2dq(vr_to, vr_to);
 }
 
+template <cpu_isa_t isa>
+void jit_uni_subkernel_int_t<isa>::process_clip(
+        const Vmm &vr_to, const Vmm &vr_from) {
+    assert(!"unsupported isa");
+}
+
+template <>
+void jit_uni_subkernel_int_t<sse41>::process_clip(
+        const Vmm &vr_to, const Vmm &vr_from) {
+
+    cvtdq2ps(vr_from, vr_from);
+    movups(vr_to, vr_from);
+    maxps(vr_to, vmm_alpha);
+    minps(vr_to, vmm_beta);
+    cvtps2dq(vr_to, vr_to);
+}
+
+template <>
+void jit_uni_subkernel_int_t<avx2>::process_clip(
+        const Vmm &vr_to, const Vmm &vr_from) {
+
+    vcvtdq2ps(vr_from, vr_from);
+    vmaxps(vr_to, vr_from, vmm_alpha);
+    vminps(vr_to, vr_to, vmm_beta);
+    vcvtps2dq(vr_to, vr_to);
+}
+
+template <>
+void jit_uni_subkernel_int_t<avx512_core>::process_clip(
+        const Vmm &vr_to, const Vmm &vr_from) {
+
+    vcvtdq2ps(vr_from, vr_from);
+    vmaxps(vr_to, vr_from, vmm_alpha);
+    vminps(vr_to, vr_to, vmm_beta);
+    vcvtps2dq(vr_to, vr_to);
+}
+
 template <cpu_isa_t isa>
 void jit_uni_subkernel_int_t<isa>::store_8bit(const bool vectorize,
         const Address &mem_to, const Vmm &vr_to, bool is_signed) {
@@ -377,7 +419,7 @@ void jit_uni_subkernel_int_t<avx2>::store_8bit(const bool vectorize,
 }
 
 template <>
-void jit_uni_subkernel_int_t<avx512_common>::store_8bit(const bool vectorize,
+void jit_uni_subkernel_int_t<avx512_core>::store_8bit(const bool vectorize,
         const Address &mem_to, const Vmm &vr_to, bool is_signed) {
     if (vectorize) {
         // store full Vmm size
@@ -399,16 +441,27 @@ void jit_uni_subkernel_int_t<avx512_common>::store_8bit(const bool vectorize,
 
 template <cpu_isa_t isa, data_type_t d_type>
 status_t jit_uni_eltwise_int_fwd_t<isa, d_type>::pd_t::init(engine_t *engine) {
-    bool ok = mayiuse(isa)
-            && desc()->data_desc.data_type == d_type
-            // only relu and linear so far
-            && utils::one_of(desc()->alg_kind, alg_kind::eltwise_relu,
-                    alg_kind::eltwise_linear)
-            && !has_zero_dim_memory()
-            && memory_desc_wrapper(data_md()).is_dense(true)
-            && attr()->has_default_values();
+    // disabling verbose dispatch messages for unsupported isa for better readability
+    if (!mayiuse(isa)) return status::unimplemented;
 
-    return ok ? status::success : status::unimplemented;
+    VDISPATCH_ELTWISE(is_fwd(), VERBOSE_BAD_PROPKIND);
+    VDISPATCH_ELTWISE(utils::everyone_is(
+                              d_type, src_md()->data_type, dst_md()->data_type),
+            VERBOSE_UNSUPPORTED_DT);
+    // only relu and linear so far
+    VDISPATCH_ELTWISE(utils::one_of(desc()->alg_kind, alg_kind::eltwise_relu,
+                              alg_kind::eltwise_linear, alg_kind::eltwise_clip),
+            VERBOSE_BAD_ALGORITHM);
+    VDISPATCH_ELTWISE(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "");
+    VDISPATCH_ELTWISE(memory_desc_wrapper(src_md()).is_dense(true),
+            VERBOSE_UNSUPPORTED_SPARSE_CFG);
+    VDISPATCH_ELTWISE(attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+    VDISPATCH_ELTWISE(set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
+    VDISPATCH_ELTWISE(
+            memory_desc_wrapper(src_md()) == memory_desc_wrapper(dst_md()),
+            VERBOSE_INCONSISTENT_MDS, "src", "dst");
+
+    return status::success;
 }
 
 template <cpu_isa_t isa, data_type_t d_type>
@@ -418,8 +471,7 @@ jit_uni_eltwise_int_fwd_t<isa, d_type>::jit_uni_eltwise_int_fwd_t(
 
 template <cpu_isa_t isa, data_type_t d_type>
 status_t jit_uni_eltwise_int_fwd_t<isa, d_type>::init(engine_t *engine) {
-    const auto &desc = *pd()->desc();
-    CHECK(safe_ptr_assign(kernel_, new jit_uni_subkernel_int_t<isa>(desc)));
+    CHECK(safe_ptr_assign(kernel_, new jit_uni_subkernel_int_t<isa>(pd())));
     return kernel_->create_kernel();
 }
 
@@ -431,19 +483,17 @@ jit_uni_eltwise_int_fwd_t<isa, d_type>::~jit_uni_eltwise_int_fwd_t() {
 template <cpu_isa_t isa, impl::data_type_t d_type>
 status_t jit_uni_eltwise_int_fwd_t<isa, d_type>::execute_forward(
         const exec_ctx_t &ctx) const {
-    status_t status = status::success;
     auto src = CTX_IN_MEM(const data_t *, DNNL_ARG_SRC);
-    auto dst = CTX_OUT_CLEAN_MEM(data_t *, DNNL_ARG_DST, status);
-    CHECK(status);
+    auto dst = CTX_OUT_MEM(data_t *, DNNL_ARG_DST);
 
-    const memory_desc_wrapper data_d(pd()->data_md());
+    const memory_desc_wrapper src_d(pd()->src_md());
 
-    const size_t nelems = data_d.nelems(true);
+    const size_t nelems = src_d.nelems(true);
 
-    src += data_d.offset0();
-    dst += data_d.offset0();
+    src += src_d.offset0();
+    dst += src_d.offset0();
 
-    const int cache_line = 64 / data_d.data_type_size();
+    const int cache_line = 64 / src_d.data_type_size();
     parallel(0, [&](const int ithr, const int nthr) {
         size_t start {0}, end {0};
 
@@ -451,7 +501,7 @@ status_t jit_uni_eltwise_int_fwd_t<isa, d_type>::execute_forward(
         start = nstl::min(nelems, start * cache_line);
         end = nstl::min(nelems, end * cache_line);
 
-        auto arg = jit_args_t();
+        auto arg = jit_args_int8_t();
         arg.from = (const void *)&src[start];
         arg.for_comparison = (const void *)&src[start];
         arg.to = (const void *)&dst[start];
@@ -465,15 +515,15 @@ using namespace data_type;
 
 template struct jit_uni_eltwise_int_fwd_t<sse41, s32>;
 template struct jit_uni_eltwise_int_fwd_t<avx2, s32>;
-template struct jit_uni_eltwise_int_fwd_t<avx512_common, s32>;
+template struct jit_uni_eltwise_int_fwd_t<avx512_core, s32>;
 
 template struct jit_uni_eltwise_int_fwd_t<sse41, s8>;
 template struct jit_uni_eltwise_int_fwd_t<avx2, s8>;
-template struct jit_uni_eltwise_int_fwd_t<avx512_common, s8>;
+template struct jit_uni_eltwise_int_fwd_t<avx512_core, s8>;
 
 template struct jit_uni_eltwise_int_fwd_t<sse41, u8>;
 template struct jit_uni_eltwise_int_fwd_t<avx2, u8>;
-template struct jit_uni_eltwise_int_fwd_t<avx512_common, u8>;
+template struct jit_uni_eltwise_int_fwd_t<avx512_core, u8>;
 
 } // namespace x64
 } // namespace cpu
