@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2021 Intel Corporation
+* Copyright 2017-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #include <math.h>
 
 #include "common/dnnl_thread.hpp"
+#include "common/math_utils.hpp"
 #include "common/utils.hpp"
 
 #include "cpu/x64/injectors/jit_uni_postops_injector.hpp"
@@ -49,11 +50,12 @@ using namespace dnnl::impl::utils;
 using namespace dnnl::impl::types;
 using namespace alg_kind;
 
-#define GET_OFF(field) offsetof(call_params_t, field)
+#define GET_OFF(field) offsetof(jit_uni_i8i8_pool_call_params_t, field)
 
-struct call_params_t {
+struct jit_uni_i8i8_pool_call_params_t {
     const char *src_i8;
     const char *dst_i8;
+    const char *dst_orig;
     const void *post_ops_binary_rhs_arg_vec;
     size_t kd_range;
     size_t kh_range;
@@ -164,6 +166,24 @@ struct jit_uni_i8i8_pooling_fwd_ker_t : public jit_generator {
         mmx_msk_base_reg = 3
     };
 
+    inline size_t get_offset_dst(int jj, int ll) const {
+        size_t offset = 0;
+        switch (jpp.alg) {
+            case pooling_max: {
+                offset = jj * jpp.c_block * sizeof_dst_dt();
+                break;
+            }
+            case pooling_avg_include_padding:
+            case pooling_avg_exclude_padding: {
+                offset = (ll * (jpp.c_block / max_num_ll) + jj * jpp.c_block)
+                        * sizeof_dst_dt();
+                break;
+            }
+            default: assert(!"unsupported pooling algorithm");
+        }
+        return offset;
+    }
+
     Vmm vreg_src_s32(int jj, int ll) {
         return avg_base_vr(3 * max_num_ll * jj + ll + 0 * max_num_ll);
     } // ll: 0..4 [0..3]
@@ -212,7 +232,7 @@ struct jit_uni_i8i8_pooling_fwd_ker_t : public jit_generator {
 
     jit_uni_i8i8_pooling_fwd_ker_t(
             const jit_pool_conf_t &jpp_, const memory_desc_t *dst_md)
-        : jit_generator(nullptr, MAX_CODE_SIZE, true, isa)
+        : jit_generator(jit_name(), isa)
         , jpp(jpp_)
         , postops_injector_(nullptr) {
 
@@ -236,8 +256,8 @@ struct jit_uni_i8i8_pooling_fwd_ker_t : public jit_generator {
             static constexpr std::size_t tmp_vmm_injector = 0u;
 
             const binary_injector::rhs_arg_static_params_t rhs_sp {
-                    tmp_vmm_injector, rax, r14, preserve_gpr, preserve_vmm,
-                    GET_OFF(post_ops_binary_rhs_arg_vec),
+                    tmp_vmm_injector, r14, r15, r13, preserve_gpr, preserve_vmm,
+                    GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                     memory_desc_wrapper(*dst_md), c_tail_elems,
                     mask(post_op_tail_opmask_idx_),
                     use_exact_tail_scalar_bcast};
@@ -272,7 +292,7 @@ void jit_uni_i8i8_pooling_fwd_ker_t<sse41>::load_src_max_op(
 
     if (masked) {
         if (jpp.src_dt == s32)
-            for (int64_t i = 0; i < jpp.c_tail; i++)
+            for (size_t i = 0; i < static_cast<size_t>(jpp.c_tail); i++)
                 pinsrd(vreg_src(jj),
                         ptr[aux_reg_src_w + offset + i * data_type_size(s32)],
                         i);
@@ -362,7 +382,7 @@ void jit_uni_i8i8_pooling_fwd_ker_t<sse41>::load_src_avg_op(
 
     if (jpp.src_dt == s32) {
         if (masked)
-            for (int64_t i = 0; i < jpp.c_tail; i++)
+            for (size_t i = 0; i < static_cast<size_t>(jpp.c_tail); i++)
                 pinsrd(vr_src,
                         ptr[aux_reg_src_w + offset + i * data_type_size(s32)],
                         i);
@@ -527,7 +547,7 @@ void jit_uni_i8i8_pooling_fwd_ker_t<sse41>::store_dst_max_op(
 
     if (masked) {
         if (jpp.src_dt == s32)
-            for (int i = 0; i < jpp.c_tail; i++)
+            for (size_t i = 0; i < static_cast<size_t>(jpp.c_tail); i++)
                 pextrd(ptr[reg_ptr_dst_i8 + offset + i * data_type_size(s32)],
                         vreg_dst(jj), i);
         else if (utils::one_of(jpp.src_dt, u8, s8))
@@ -643,7 +663,7 @@ void jit_uni_i8i8_pooling_fwd_ker_t<sse41>::store_dst_avg_op(
 
     if (jpp.src_dt == s32) {
         if (masked)
-            for (int i = 0; i < jpp.c_tail; i++)
+            for (size_t i = 0; i < static_cast<size_t>(jpp.c_tail); i++)
                 pextrd(ptr[reg_ptr_dst_i8 + offset + i * data_type_size(s32)],
                         vr_dst, i);
         else
@@ -969,18 +989,6 @@ void jit_uni_i8i8_pooling_fwd_ker_t<isa>::compute_avg_step(
         jl(l_kd, T_NEAR);
     }
 
-    static constexpr int vlen_size_elem
-            = cpu_isa_traits<isa>::vlen / sizeof(float);
-    const auto reg_tmp_postops = r15;
-    const injector_utils::register_preserve_guard_t reg_guard(this,
-            jpp.with_binary
-                    ? std::initializer_list<Xbyak::Reg64> {reg_tmp_postops}
-                    : std::initializer_list<Xbyak::Reg64> {},
-            {});
-    if (jpp.with_binary) {
-        imul(reg_tmp_postops, c_iter, ur_c * num_ll * vlen_size_elem);
-    }
-
     for (int jj = 0; jj < ur_c; jj++) {
         for (int ll = 0; ll < num_ll; ll++) {
             const bool masked = jj == ur_c - 1 && c_tail;
@@ -994,16 +1002,10 @@ void jit_uni_i8i8_pooling_fwd_ker_t<isa>::compute_avg_step(
                 if (jpp.with_postops) {
                     binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
                     if (jpp.with_binary) {
-                        rhs_arg_params.vmm_idx_to_oc_off_oprnd.emplace(
-                                reg_dst_f32.getIdx(), reg_tmp_postops);
-                        rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
-                                reg_dst_f32.getIdx(),
-                                ll * vlen_size_elem + jj * vlen_size_elem);
-                        rhs_arg_params.vmm_idx_to_oc_off_oprnd.emplace(
-                                reg_dst_f32.getIdx(), reg_tmp_postops);
-                        rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
-                                reg_dst_f32.getIdx(),
-                                ll * vlen_size_elem + jj * vlen_size_elem);
+                        rhs_arg_params.vmm_idx_to_out_reg.emplace(
+                                reg_dst_f32.getIdx(), reg_ptr_dst_i8);
+                        rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
+                                reg_dst_f32.getIdx(), get_offset_dst(jj, ll));
                         const bool tail = ll == post_op_tail_opmask_idx_;
                         if (tail && masked)
                             rhs_arg_params.vmm_tail_idx_.emplace(
@@ -1193,7 +1195,10 @@ void jit_uni_i8i8_pooling_fwd_ker_t<isa>::init_tmp_reg() {
     switch (jpp.alg) {
         case pooling_avg_include_padding:
         case pooling_avg_exclude_padding:
-            mov(reg_tmp, ptr[reg_param + offsetof(call_params_t, idivider)]);
+            mov(reg_tmp,
+                    ptr[reg_param
+                            + offsetof(jit_uni_i8i8_pool_call_params_t,
+                                    idivider)]);
             uni_vmovq(xmm_tmp, reg_tmp);
             uni_vpbroadcastd(vreg_tmp, xmm_tmp);
             break;
@@ -1234,7 +1239,7 @@ void jit_uni_i8i8_pooling_fwd_ker_t<isa>::generate() {
 #endif
 
 #define READ_PARAM(reg, field) \
-    mov(reg, ptr[reg_param + offsetof(call_params_t, field)])
+    mov(reg, ptr[reg_param + offsetof(jit_uni_i8i8_pool_call_params_t, field)])
     READ_PARAM(reg_ptr_src_i8, src_i8);
     READ_PARAM(reg_ptr_dst_i8, dst_i8);
     READ_PARAM(reg_kd, kd_range);
@@ -1257,12 +1262,14 @@ void jit_uni_i8i8_pooling_fwd_ker_t<isa>::generate() {
     postamble();
 
     if (jpp.with_eltwise && postops_injector_)
-        postops_injector_->prepare_table();
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
 template <cpu_isa_t isa>
 status_t jit_uni_i8i8_pooling_fwd_ker_t<isa>::init_conf(
         jit_pool_conf_t &jpp, const pooling_pd_t *ppd) {
+    // disabling verbose dispatch messages for unsupported isa for
+    // better readability
     if (!mayiuse(isa)) return status::unimplemented;
 
     const auto &pd = *ppd->desc();
@@ -1302,10 +1309,11 @@ status_t jit_uni_i8i8_pooling_fwd_ker_t<isa>::init_conf(
     int right_pad = calculate_end_padding(
             jpp.l_pad, jpp.ow, jpp.iw, jpp.stride_w, jpp.kw);
 
-    if (jpp.f_pad >= jpp.kd || jpp.t_pad >= jpp.kh || jpp.l_pad >= jpp.kw
-            || back_pad >= jpp.kd || bottom_pad >= jpp.kh
-            || right_pad >= jpp.kw)
-        return status::unimplemented;
+    VDISPATCH_POOLING_IC(
+            !(jpp.f_pad >= jpp.kd || jpp.t_pad >= jpp.kh || jpp.l_pad >= jpp.kw
+                    || back_pad >= jpp.kd || bottom_pad >= jpp.kh
+                    || right_pad >= jpp.kw),
+            VERBOSE_UNSUPPORTED_PAD_FEATURE, "");
 
     jpp.alg = pd.alg_kind;
 
@@ -1321,12 +1329,12 @@ status_t jit_uni_i8i8_pooling_fwd_ker_t<isa>::init_conf(
     /* Verify that vlen-sized memory access happens within the tensor's
      * size, otherwise load/store will always spill outside the memory
      * boundary.*/
-    bool safe_load_n_store = IMPLICATION(utils::one_of(isa, avx2, sse41),
+    const bool safe_load_n_store = IMPLICATION(utils::one_of(isa, avx2, sse41),
             jpp.mb * jpp.c * nstl::min(jpp.id, jpp.od)
                             * nstl::min(jpp.ih, jpp.oh)
                             * nstl::min(jpp.iw, jpp.ow)
                     >= simd_w);
-    if (!safe_load_n_store) return status::unimplemented;
+    VDISPATCH_POOLING_IC(safe_load_n_store, "safe load-and-store not possible");
 
     jpp.c_block = simd_w;
     jpp.c_tail = jpp.c % jpp.c_block;
@@ -1365,7 +1373,8 @@ status_t jit_uni_i8i8_pooling_fwd_ker_t<isa>::init_conf(
         default: return status::unimplemented;
     }
 
-    if (!post_ops_ok(jpp, *ppd->attr(), dst_d)) return status::unimplemented;
+    VDISPATCH_POOLING_IC(
+            post_ops_ok(jpp, *ppd->attr(), dst_d), VERBOSE_UNSUPPORTED_POSTOP);
 
     return status::success;
 }
@@ -1384,7 +1393,8 @@ bool jit_uni_i8i8_pooling_fwd_ker_t<isa>::post_ops_ok(jit_pool_conf_t &jpp,
     for (const auto &entry : entries) {
         if (entry.is_eltwise()) {
             const auto alg = entry.eltwise.alg;
-            jpp.with_eltwise = eltwise_injector::is_supported(isa, alg);
+            jpp.with_eltwise
+                    = eltwise_injector::is_supported(isa, alg, data_type::f32);
         } else if (entry.is_binary()) {
             if (isa != avx512_core
                     && entry.binary.src1_desc.data_type == data_type::bf16)
@@ -1448,33 +1458,34 @@ status_t jit_uni_i8i8_pooling_fwd_t<isa>::execute_forward(
             reinterpret_cast<ptrdiff_t>(dst_i8 + dst_d.size() - 1)
             - (cpu_isa_traits<isa>::vlen - 1));
 
-    parallel_nd(
-            jpp.mb, jpp.od, jpp.oh, jpp.ow, [&](int n, int od, int oh, int ow) {
-                const int id = nstl::max(od * jpp.stride_d - jpp.f_pad, 0);
-                const int ih = nstl::max(oh * jpp.stride_h - jpp.t_pad, 0);
-                const int iw = nstl::max(ow * jpp.stride_w - jpp.l_pad, 0);
+    parallel_nd(jpp.mb, jpp.od, jpp.oh, jpp.ow,
+            [&](dim_t n, dim_t od, dim_t oh, dim_t ow) {
+                dim_t id = nstl::max(od * jpp.stride_d - jpp.f_pad, dim_t(0));
+                dim_t ih = nstl::max(oh * jpp.stride_h - jpp.t_pad, dim_t(0));
+                dim_t iw = nstl::max(ow * jpp.stride_w - jpp.l_pad, dim_t(0));
 
-                const int kd_start
-                        = nstl::max(0, jpp.f_pad - od * jpp.stride_d);
-                const int kd_end = nstl::min(
-                        jpp.kd, jpp.id + jpp.f_pad - od * jpp.stride_d);
-                const int kh_start
-                        = nstl::max(0, jpp.t_pad - oh * jpp.stride_h);
-                const int kh_end = nstl::min(
-                        jpp.kh, jpp.ih + jpp.t_pad - oh * jpp.stride_h);
-                const int kw_start
-                        = nstl::max(0, jpp.l_pad - ow * jpp.stride_w);
-                const int kw_end = nstl::min(
-                        jpp.kw, jpp.iw + jpp.l_pad - ow * jpp.stride_w);
+                dim_t kd_start
+                        = nstl::max(dim_t(0), jpp.f_pad - od * jpp.stride_d);
+                dim_t kd_end = nstl::min(
+                        dim_t(jpp.kd), jpp.id + jpp.f_pad - od * jpp.stride_d);
+                dim_t kh_start
+                        = nstl::max(dim_t(0), jpp.t_pad - oh * jpp.stride_h);
+                dim_t kh_end = nstl::min(
+                        dim_t(jpp.kh), jpp.ih + jpp.t_pad - oh * jpp.stride_h);
+                dim_t kw_start
+                        = nstl::max(dim_t(0), jpp.l_pad - ow * jpp.stride_w);
+                dim_t kw_end = nstl::min(
+                        dim_t(jpp.kw), jpp.iw + jpp.l_pad - ow * jpp.stride_w);
 
-                auto p = call_params_t();
+                auto p = jit_uni_i8i8_pool_call_params_t();
                 p.src_i8 = &src_i8[get_offset(src_d, n, 0, id, ih, iw)
                         * src_d.data_type_size()];
                 p.dst_i8 = &dst_i8[get_offset(dst_d, n, 0, od, oh, ow)
                         * dst_d.data_type_size()];
-                p.kd_range = (size_t)(kd_end - kd_start);
-                p.kh_range = (size_t)(kh_end - kh_start);
-                p.kw_range = (size_t)(kw_end - kw_start);
+                p.dst_orig = dst_i8;
+                p.kd_range = kd_end - kd_start;
+                p.kh_range = kh_end - kh_start;
+                p.kw_range = kw_end - kw_start;
                 p.idivider = 1.0f
                         / ((jpp.alg == pooling_avg_exclude_padding)
                                         ? p.kd_range * p.kh_range * p.kw_range
