@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017 - 2021 Intel Corporation
+* Copyright 2017 - 2024 Intel Corporation
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
@@ -53,7 +53,8 @@ struct trans_wrapper_t {
             prb.ndims = 2;
             prb.ioff = 0;
             prb.ooff = 0;
-            prb.scale_type = scale_type_t::NONE;
+            prb.src_scale_type = scale_type_t::NONE;
+            prb.dst_scale_type = scale_type_t::NONE;
             prb.beta = 0;
             prb.nodes[0].ss = prb.nodes[1].ss = 1;
 
@@ -69,10 +70,6 @@ struct trans_wrapper_t {
             prb.nodes[1].os = x_out_str;
 
             prb.full_ndims = prb.ndims;
-            prb.ip_tail = 0;
-            prb.op_tail = 0;
-            prb.iblock = 1;
-            prb.oblock = 1;
 
             kernel_t::desc_init(desc, prb, 2);
             return kernel_t::create(desc);
@@ -103,7 +100,8 @@ struct trans_wrapper_t {
         auto call_ker = [&](tr::kernel_t &ker, dim_t inp_y, dim_t inp_x,
                                 dim_t out_y, dim_t out_x) {
             tr::call_param_t cp;
-            cp.scale = nullptr;
+            cp.src_scales = nullptr;
+            cp.dst_scales = nullptr;
 
             dim_t inp_off = (inp_y * inp_str_ + inp_x) * inp_dt_size_;
             dim_t out_off = (out_y * out_str_ + out_x) * out_dt_size_;
@@ -147,6 +145,8 @@ struct trans_context_t {
     std::unique_ptr<trans_wrapper_t> ind_tail_trans_ = nullptr;
     std::unique_ptr<trans_wrapper_t> dst_trans_ = nullptr;
     std::unique_ptr<trans_wrapper_t> dst_tail_trans_ = nullptr;
+
+    // NOLINTNEXTLINE(readability-make-member-function-const)
     status_t create_kernel() {
         if (src_trans_) CHECK(src_trans_->create_kernel());
         if (src_tail_trans_) CHECK(src_tail_trans_->create_kernel());
@@ -583,7 +583,6 @@ void jit_uni_pooling_fwd_t<isa, d_type>::execute_forward(const data_t *src,
                 = ((jpp.tag_kind == jit_memory_tag_kind_t::nspc) ? jpp.c_block
                                                                  : 1)
                 * b_c;
-        const int c_elem_off = jpp.c_block * b_c;
 
         if (trans_src)
             arg.src = transpose_facade.get_src_addr(ithr, ih, jpp);
@@ -591,11 +590,22 @@ void jit_uni_pooling_fwd_t<isa, d_type>::execute_forward(const data_t *src,
             arg.src = static_cast<const void *>(
                     &src[src_d.blk_off(n, c_off, ih)]);
 
-        if (trans_dst)
+        arg.dst_orig = dst;
+        if (trans_dst) {
             arg.dst = transpose_facade.get_dst_addr(ithr, oh, jpp);
-        else
+            if (!types::is_zero_md(&jpp.tmp_md)) {
+                const memory_desc_wrapper tmp_d
+                        = memory_desc_wrapper(jpp.tmp_md);
+                // offset needs to be f32
+                const int dt_scale
+                        = sizeof(float) / types::data_type_size(d_type);
+                const auto blk_off = tmp_d.blk_off(n, c_off, oh) * dt_scale;
+                arg.dst_po_helper = static_cast<const void *>(&dst[blk_off]);
+            }
+        } else {
             arg.dst = static_cast<const void *>(
                     &dst[dst_d.blk_off(n, c_off, oh)]);
+        }
 
         if (indices) {
             if (trans_dst)
@@ -614,26 +624,27 @@ void jit_uni_pooling_fwd_t<isa, d_type>::execute_forward(const data_t *src,
         arg.ur_bc = ur_bc;
         arg.b_c = b_c;
         arg.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec.data();
-        arg.c_elem_off = c_elem_off;
         (*kernel_)(&arg);
     };
 
+    const int nthr = jpp.nthr;
+
     if (jpp.tag_kind == jit_memory_tag_kind_t::nspc) {
         const auto nb2_c = utils::div_up(jpp.nb_c, jpp.ur_bc);
-        parallel_nd(jpp.mb, jpp.oh, nb2_c, [&](int n, int oh, int b2_c) {
+        parallel_nd(jpp.mb, jpp.oh, nb2_c, [&](dim_t n, dim_t oh, dim_t b2_c) {
             const auto b_c = b2_c * jpp.ur_bc;
-            const auto ur_bc = nstl::min(jpp.ur_bc, jpp.nb_c - b_c);
+            const auto ur_bc = nstl::min(dim_t(jpp.ur_bc), jpp.nb_c - b_c);
             ker(0, n, b_c, oh, ur_bc);
         });
     } else {
         if (trans_src || trans_dst) {
             // ncsp format
-            parallel_nd_ext(0, jpp.mb, jpp.nb_c,
-                    [&](int ithr, int nthr, int n, int b_c) {
+            parallel_nd_ext(nthr, jpp.mb, jpp.nb_c,
+                    [&](dim_t ithr, dim_t nthr, dim_t n, dim_t b_c) {
                         if (trans_src)
                             transpose_facade.execute_transpose_input(
                                     ithr, n, b_c);
-                        for (int oh = 0; oh < jpp.oh; ++oh)
+                        for (dim_t oh = 0; oh < jpp.oh; ++oh)
                             ker(ithr, n, b_c, oh, 1);
                         if (trans_dst)
                             transpose_facade.execute_transpose_output(
@@ -641,19 +652,19 @@ void jit_uni_pooling_fwd_t<isa, d_type>::execute_forward(const data_t *src,
                     });
         } else {
             // nChw16c, nChw8c format
-            parallel(0, [&](std::size_t ithr, std::size_t nthr) {
-                const std::size_t work_amount
-                        = static_cast<std::size_t>(jpp.mb) * jpp.nb_c * jpp.oh;
+            parallel(nthr, [&](dim_t ithr, dim_t nthr) {
+                dim_t work_amount
+                        = static_cast<dim_t>(jpp.mb) * jpp.nb_c * jpp.oh;
                 if (ithr >= work_amount) return;
 
-                std::size_t start {0}, end {0};
-                int n {0}, b_c {0}, oh {0};
+                dim_t start {0}, end {0};
+                dim_t n {0}, b_c {0}, oh {0};
 
                 balance211(work_amount, nthr, ithr, start, end);
                 utils::nd_iterator_init(
                         start, n, jpp.mb, b_c, jpp.nb_c, oh, jpp.oh);
 
-                for (std::size_t iwork = start; iwork < end; ++iwork) {
+                for (dim_t iwork = start; iwork < end; ++iwork) {
                     ker(ithr, n, b_c, oh, 1);
                     utils::nd_iterator_step(
                             n, jpp.mb, b_c, jpp.nb_c, oh, jpp.oh);
@@ -708,10 +719,21 @@ void jit_uni_pooling_fwd_t<isa, d_type>::execute_forward_3d(const data_t *src,
         else
             arg.src = &src[src_d.blk_off(n, c_off, id, ih)];
 
-        if (trans_dst)
+        arg.dst_orig = dst;
+        if (trans_dst) {
             arg.dst = transpose_facade.get_dst_addr_3d(ithr, od, oh, jpp);
-        else
+            if (!types::is_zero_md(&jpp.tmp_md)) {
+                const memory_desc_wrapper tmp_d
+                        = memory_desc_wrapper(jpp.tmp_md);
+                // offset needs to be f32
+                const int dt_scale
+                        = sizeof(float) / types::data_type_size(d_type);
+                const auto blk_off = tmp_d.blk_off(n, c_off, od, oh) * dt_scale;
+                arg.dst_po_helper = static_cast<const void *>(&dst[blk_off]);
+            }
+        } else {
             arg.dst = &dst[dst_d.blk_off(n, c_off, od, oh)];
+        }
 
         if (indices) {
             if (trans_dst) {
@@ -741,30 +763,32 @@ void jit_uni_pooling_fwd_t<isa, d_type>::execute_forward_3d(const data_t *src,
         arg.ur_bc = ur_bc;
         arg.b_c = b_c;
         arg.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec.data();
-        arg.c_elem_off = jpp.c_block * b_c;
         (*kernel_)(&arg);
     };
 
+    const int nthr = jpp.nthr;
+
     if (jpp.tag_kind == jit_memory_tag_kind_t::nspc) {
         const auto nb2_c = utils::div_up(jpp.nb_c, jpp.ur_bc);
-        parallel_nd(jpp.mb, jpp.od, nb2_c, [&](int n, int od, int b2_c) {
-            const auto b_c = b2_c * jpp.ur_bc;
-            const auto ur_bc = nstl::min(jpp.ur_bc, jpp.nb_c - b_c);
+        parallel_nd(jpp.mb, jpp.od, nb2_c, [&](dim_t n, dim_t od, dim_t b2_c) {
+            const dim_t b_c = b2_c * jpp.ur_bc;
+            const dim_t ur_bc = nstl::min(dim_t(jpp.ur_bc), jpp.nb_c - b_c);
 
-            const int ik = od * jpp.stride_d;
-            const int d_t_overflow = nstl::max(0, jpp.f_pad - ik);
-            const int d_b_overflow
-                    = nstl::max(jpp.id, ik + jpp.kd - jpp.f_pad) - jpp.id;
-            const int id = nstl::max(ik - jpp.f_pad, 0);
-            for (int oh = 0; oh < jpp.oh; ++oh) {
+            const dim_t ik = od * jpp.stride_d;
+            const dim_t d_t_overflow = nstl::max(dim_t(0), jpp.f_pad - ik);
+            const dim_t d_b_overflow
+                    = nstl::max(dim_t(jpp.id), ik + jpp.kd - jpp.f_pad)
+                    - jpp.id;
+            const dim_t id = nstl::max(ik - jpp.f_pad, dim_t(0));
+            for (dim_t oh = 0; oh < jpp.oh; ++oh) {
                 ker(n, b_c, od, oh, id, d_t_overflow, d_b_overflow, ur_bc,
                         first_ithr);
             }
         });
     } else {
         if (trans_src || trans_dst) {
-            parallel_nd_ext(0, jpp.mb, jpp.nb_c,
-                    [&](int ithr, int nthr, int n, int b_c) {
+            parallel_nd_ext(nthr, jpp.mb, jpp.nb_c,
+                    [&](dim_t ithr, dim_t nthr, dim_t n, dim_t b_c) {
                         if (trans_src)
                             transpose_facade.execute_transpose_input(
                                     ithr, n, b_c);
@@ -788,27 +812,26 @@ void jit_uni_pooling_fwd_t<isa, d_type>::execute_forward_3d(const data_t *src,
                                     ithr, n, b_c);
                     });
         } else {
-            parallel_nd(jpp.mb, jpp.nb_c, jpp.od, [&](int n, int b_c, int od) {
-                const int ik = od * jpp.stride_d;
-                const int d_t_overflow = nstl::max(0, jpp.f_pad - ik);
-                const int d_b_overflow
-                        = nstl::max(jpp.id, ik + jpp.kd - jpp.f_pad) - jpp.id;
-                const int id = nstl::max(ik - jpp.f_pad, 0);
-                for (int oh = 0; oh < jpp.oh; ++oh) {
-                    ker(n, b_c, od, oh, id, d_t_overflow, d_b_overflow, 1,
-                            first_ithr);
-                }
-            });
+            parallel_nd(jpp.mb, jpp.nb_c, jpp.od,
+                    [&](dim_t n, dim_t b_c, dim_t od) {
+                        const int ik = od * jpp.stride_d;
+                        const int d_t_overflow = nstl::max(0, jpp.f_pad - ik);
+                        const int d_b_overflow
+                                = nstl::max(jpp.id, ik + jpp.kd - jpp.f_pad)
+                                - jpp.id;
+                        const int id = nstl::max(ik - jpp.f_pad, 0);
+                        for (int oh = 0; oh < jpp.oh; ++oh) {
+                            ker(n, b_c, od, oh, id, d_t_overflow, d_b_overflow,
+                                    1, first_ithr);
+                        }
+                    });
         }
     }
 }
 
 template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_pooling_bwd_t<isa, d_type>::jit_uni_pooling_bwd_t(const pd_t *apd)
-    : primitive_t(apd)
-    , kernel_(utils::make_unique<jit_uni_pool_kernel<isa>>(
-              pd()->jpp_, pd()->invariant_dst_md()))
-    , trans_ctx_(nullptr) {}
+    : primitive_t(apd), kernel_(nullptr), trans_ctx_(nullptr) {}
 
 template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_pooling_bwd_t<isa, d_type>::~jit_uni_pooling_bwd_t() = default;
@@ -855,6 +878,9 @@ status_t jit_uni_pooling_bwd_t<isa, d_type>::init_ncsp_trans_ctx() {
 
 template <cpu_isa_t isa, data_type_t d_type>
 status_t jit_uni_pooling_bwd_t<isa, d_type>::init(engine_t *engine) {
+    CHECK(safe_ptr_assign(kernel_,
+            new jit_uni_pool_kernel<isa>(
+                    pd()->jpp_, pd()->invariant_dst_md())));
     if (pd()->jpp_.tag_kind == jit_memory_tag_kind_t::ncsp)
         CHECK(init_ncsp_trans_ctx());
     return kernel_->create_kernel();
@@ -954,7 +980,9 @@ void jit_uni_pooling_bwd_t<isa, d_type>::execute_backward(
             transpose_facade.execute_transpose_output(ithr, n, b_c);
     };
 
-    parallel(0, [&](int ithr, int nthr) {
+    const int nthr = jpp.nthr;
+
+    parallel(nthr, [&](int ithr, int nthr) {
         const auto nb2_c = utils::div_up(jpp.nb_c, jpp.ur_bc);
         const std::size_t work_amount
                 = static_cast<std::size_t>(jpp.mb) * nb2_c;
@@ -1104,19 +1132,23 @@ void jit_uni_pooling_bwd_t<isa, d_type>::execute_backward_3d(
         }
     };
 
+    const int nthr = jpp.nthr;
+
     if (jpp.simple_alg) {
         if (jpp.tag_kind == jit_memory_tag_kind_t::nspc) {
-            const auto nb2_c = utils::div_up(jpp.nb_c, jpp.ur_bc);
-            parallel_nd(jpp.mb, jpp.od, nb2_c, [&](int n, int od, int b2_c) {
-                const auto b_c = b2_c * jpp.ur_bc;
-                const auto ur_bc = nstl::min(jpp.ur_bc, jpp.nb_c - b_c);
-                process_simple(n, b_c, od, ur_bc, first_ithr);
-            });
+            const dim_t nb2_c = utils::div_up(jpp.nb_c, jpp.ur_bc);
+            parallel_nd(
+                    jpp.mb, jpp.od, nb2_c, [&](dim_t n, dim_t od, dim_t b2_c) {
+                        const dim_t b_c = b2_c * jpp.ur_bc;
+                        const dim_t ur_bc
+                                = nstl::min(dim_t(jpp.ur_bc), jpp.nb_c - b_c);
+                        process_simple(n, b_c, od, ur_bc, first_ithr);
+                    });
         } else {
             assert(jpp.ur_bc == 1);
             if (trans_src || trans_dst) {
-                parallel_nd_ext(0, jpp.mb, jpp.nb_c,
-                        [&](int ithr, int nthr, int n, int b_c) {
+                parallel_nd_ext(nthr, jpp.mb, jpp.nb_c,
+                        [&](dim_t ithr, dim_t nthr, dim_t n, dim_t b_c) {
                             if (trans_src)
                                 transpose_facade.execute_transpose_input(
                                         ithr, n, b_c);
@@ -1128,8 +1160,8 @@ void jit_uni_pooling_bwd_t<isa, d_type>::execute_backward_3d(
                                         ithr, n, b_c);
                         });
             } else {
-                parallel_nd(
-                        jpp.mb, jpp.nb_c, jpp.od, [&](int n, int b_c, int od) {
+                parallel_nd(jpp.mb, jpp.nb_c, jpp.od,
+                        [&](dim_t n, dim_t b_c, dim_t od) {
                             process_simple(n, b_c, od, 1, first_ithr);
                         });
             }
@@ -1138,7 +1170,7 @@ void jit_uni_pooling_bwd_t<isa, d_type>::execute_backward_3d(
         const data_t zero_val = 0;
         if (jpp.tag_kind == jit_memory_tag_kind_t::nspc) {
             const size_t chunk_size = (size_t)jpp.ih * jpp.iw * jpp.c;
-            parallel_nd(jpp.mb, jpp.id, [&](int n, int id) {
+            parallel_nd(jpp.mb, jpp.id, [&](dim_t n, dim_t id) {
                 const size_t offset = ((size_t)n * jpp.id + id) * chunk_size;
                 PRAGMA_OMP_SIMD()
                 for (size_t idx = 0; idx < chunk_size; ++idx)
@@ -1148,8 +1180,8 @@ void jit_uni_pooling_bwd_t<isa, d_type>::execute_backward_3d(
             if (!trans_src) {
                 const size_t chunk_size
                         = (size_t)jpp.id * jpp.ih * jpp.iw * jpp.c_block;
-                parallel_nd_ext(0, jpp.mb, jpp.nb_c,
-                        [&](int ithr, int nthr, int n, int b_c) {
+                parallel_nd_ext(nthr, jpp.mb, jpp.nb_c,
+                        [&](dim_t ithr, dim_t nthr, dim_t n, dim_t b_c) {
                             const size_t offset
                                     = ((size_t)n * jpp.nb_c + b_c) * chunk_size;
                             PRAGMA_OMP_SIMD()
@@ -1161,9 +1193,9 @@ void jit_uni_pooling_bwd_t<isa, d_type>::execute_backward_3d(
 
         const auto nb2_c = utils::div_up(jpp.nb_c, jpp.ur_bc);
         if (trans_src || trans_dst) {
-            parallel_nd_ext(
-                    0, jpp.mb, nb2_c, [&](int ithr, int nthr, int n, int b2_c) {
-                        const auto b_c = b2_c * jpp.ur_bc;
+            parallel_nd_ext(nthr, jpp.mb, nb2_c,
+                    [&](dim_t ithr, dim_t nthr, dim_t n, dim_t b2_c) {
+                        const dim_t b_c = b2_c * jpp.ur_bc;
 
                         if (trans_dst) {
                             transpose_facade.execute_transpose_input(
@@ -1177,21 +1209,23 @@ void jit_uni_pooling_bwd_t<isa, d_type>::execute_backward_3d(
                             std::memset((void *)src, zero_val, block_size);
                         }
 
-                        for (int kd = 0; kd < jpp.kd; ++kd) {
-                            const auto ur_bc
-                                    = nstl::min(jpp.ur_bc, jpp.nb_c - b_c);
+                        for (dim_t kd = 0; kd < jpp.kd; ++kd) {
+                            const dim_t ur_bc = nstl::min(
+                                    dim_t(jpp.ur_bc), jpp.nb_c - b_c);
                             for (int od = 0; od < jpp.od; ++od) {
-                                const int ik = od * jpp.stride_d;
-                                const int d_t_overflow
-                                        = nstl::max(0, jpp.f_pad - ik);
-                                const int d_b_overflow
-                                        = nstl::max(jpp.id,
+                                const dim_t ik
+                                        = static_cast<dim_t>(od) * jpp.stride_d;
+                                const dim_t d_t_overflow
+                                        = nstl::max(dim_t(0), jpp.f_pad - ik);
+                                const dim_t d_b_overflow
+                                        = nstl::max(dim_t(jpp.id),
                                                   ik + jpp.kd - jpp.f_pad)
                                         - jpp.id;
                                 if (kd >= jpp.kd - d_t_overflow - d_b_overflow)
                                     continue;
-                                const int id = nstl::max(ik - jpp.f_pad, 0);
-                                for (int oh = 0; oh < jpp.oh; ++oh) {
+                                const dim_t id
+                                        = nstl::max(ik - jpp.f_pad, dim_t(0));
+                                for (dim_t oh = 0; oh < jpp.oh; ++oh) {
                                     ker(n, b_c, od, oh, id, d_t_overflow,
                                             d_b_overflow, false, kd, ur_bc,
                                             ithr);
@@ -1204,20 +1238,23 @@ void jit_uni_pooling_bwd_t<isa, d_type>::execute_backward_3d(
                                     ithr, n, b_c);
                     });
         } else {
-            for (int kd = 0; kd < jpp.kd; ++kd) {
-                parallel_nd(jpp.mb, nb2_c, [&](int n, int b2_c) {
-                    const auto b_c = b2_c * jpp.ur_bc;
-                    const auto ur_bc = nstl::min(jpp.ur_bc, jpp.nb_c - b_c);
+            for (dim_t kd = 0; kd < jpp.kd; ++kd) {
+                parallel_nd(jpp.mb, nb2_c, [&](dim_t n, dim_t b2_c) {
+                    const dim_t b_c = b2_c * jpp.ur_bc;
+                    const dim_t ur_bc
+                            = nstl::min(dim_t(jpp.ur_bc), jpp.nb_c - b_c);
                     for (int od = 0; od < jpp.od; ++od) {
-                        const int ik = od * jpp.stride_d;
-                        const int d_t_overflow = nstl::max(0, jpp.f_pad - ik);
-                        const int d_b_overflow
-                                = nstl::max(jpp.id, ik + jpp.kd - jpp.f_pad)
+                        const dim_t ik = static_cast<dim_t>(od) * jpp.stride_d;
+                        const dim_t d_t_overflow
+                                = nstl::max(dim_t(0), jpp.f_pad - ik);
+                        const dim_t d_b_overflow
+                                = nstl::max(dim_t(jpp.id),
+                                          ik + jpp.kd - jpp.f_pad)
                                 - jpp.id;
                         if (kd >= jpp.kd - d_t_overflow - d_b_overflow)
                             continue;
-                        const int id = nstl::max(ik - jpp.f_pad, 0);
-                        for (int oh = 0; oh < jpp.oh; ++oh) {
+                        const dim_t id = nstl::max(ik - jpp.f_pad, dim_t(0));
+                        for (dim_t oh = 0; oh < jpp.oh; ++oh) {
                             ker(n, b_c, od, oh, id, d_t_overflow, d_b_overflow,
                                     false, kd, ur_bc, first_ithr);
                         }
@@ -1233,13 +1270,18 @@ template struct jit_uni_pooling_bwd_t<sse41, data_type::f32>;
 template struct jit_uni_pooling_fwd_t<avx, data_type::f32>;
 template struct jit_uni_pooling_bwd_t<avx, data_type::f32>;
 template struct jit_uni_pooling_fwd_t<avx2, data_type::f32>;
+template struct jit_uni_pooling_fwd_t<avx2_vnni_2, data_type::bf16>;
+template struct jit_uni_pooling_fwd_t<avx2_vnni_2, data_type::f16>;
 template struct jit_uni_pooling_bwd_t<avx2, data_type::f32>;
-template struct jit_uni_pooling_fwd_t<avx512_common, data_type::f32>;
-template struct jit_uni_pooling_bwd_t<avx512_common, data_type::f32>;
 template struct jit_uni_pooling_fwd_t<avx512_core, data_type::f32>;
 template struct jit_uni_pooling_bwd_t<avx512_core, data_type::f32>;
 template struct jit_uni_pooling_fwd_t<avx512_core, data_type::bf16>;
 template struct jit_uni_pooling_bwd_t<avx512_core, data_type::bf16>;
+template struct jit_uni_pooling_fwd_t<avx512_core_fp16, data_type::f16>;
+template struct jit_uni_pooling_bwd_t<avx512_core_fp16, data_type::f16>;
+
+template struct jit_uni_pooling_fwd_t<avx512_core_fp16, data_type::f8_e5m2>;
+template struct jit_uni_pooling_fwd_t<avx512_core_fp16, data_type::f8_e4m3>;
 
 } // namespace x64
 } // namespace cpu
