@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,12 +15,14 @@
 *******************************************************************************/
 
 #include "common/c_types_map.hpp"
+#include "common/convolution_pd.hpp"
 #include "common/memory.hpp"
 #include "common/memory_tracking.hpp"
 #include "common/nstl.hpp"
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 
+#include "cpu/platform.hpp"
 #include "cpu/x64/injectors/injector_utils.hpp"
 #include "cpu/x64/injectors/jit_uni_binary_injector.hpp"
 #include "cpu/x64/injectors/jit_uni_eltwise_injector.hpp"
@@ -55,20 +57,23 @@ template <cpu_isa_t isa, typename Vmm>
 _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::_jit_uni_x8s8s32x_fwd_kernel(
         const jit_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, isa), jcp(ajcp), attr_(attr) {
+    : jit_generator(jit_name(), isa), jcp(ajcp), attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
         static constexpr bool preserve_vmm = false;
         static constexpr size_t helper_vmm_idx = 15;
-        const size_t oc_block_tail = jcp.oc_block % isa_simd_width_;
-        const size_t tail_size = oc_block_tail
-                ? oc_block_tail
-                : jcp.oc_without_padding % isa_simd_width_;
+        const size_t block_tail
+                = (jcp.is_depthwise ? jcp.ch_block : jcp.oc_block)
+                % isa_simd_width_;
+        const size_t tail_size = block_tail
+                ? block_tail
+                : (jcp.is_depthwise ? jcp.ngroups : jcp.oc_without_padding)
+                        % isa_simd_width_;
 
         const rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx,
-                r13, r14, preserve_gpr, preserve_vmm,
-                GET_OFF(post_ops_binary_rhs_arg_vec),
+                r13, r14, r15, preserve_gpr, preserve_vmm,
+                GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size, true};
         const static_params_t static_params {
                 this->param1, rhs_arg_static_params};
@@ -130,18 +135,25 @@ void iterate(const int nb_oc_block, const int ur_w, const F &f) {
 template <cpu_isa_t isa, typename Vmm>
 void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::apply_sum(const int nb_oc_block,
         const int ur_w, const bool last_oc_block_flag, const int oc_block,
-        const float *p_sum_scale) {
+        const float *p_sum_scale, const int32_t *p_sum_zp) {
     if (jcp.with_sum) {
-        assert(p_sum_scale != nullptr && "p_sum_scale = nullptr");
+        assert(!utils::any_null(p_sum_scale, p_sum_zp)
+                && "p_sum_scale or p_sum_zp = nullptr");
         const float sum_scale = *p_sum_scale;
-        const auto sum_injector_lam = [this, oc_block, sum_scale](
+        const int32_t sum_zp = *p_sum_zp;
+        const auto sum_injector_lam = [this, oc_block, sum_scale, sum_zp](
                                               const bool mask_flag, const int k,
                                               const int j) {
             const int aux_output_offset = jcp.typesize_out
                     * (k * oc_block + j * jcp.oc_without_padding * jcp.ngroups);
-            cvt2ps(jcp.dst_dt, vmm_prev_dst, reg_out, aux_output_offset,
+            cvt2ps(jcp.sum_dt, vmm_prev_dst, reg_out, aux_output_offset,
                     mask_flag ? get_tail_size() : get_blocking_size());
             const Vmm vmm = vmm_out(j, k);
+            if (sum_zp != 0) {
+                uni_vbroadcastss(vmm_tmp, ptr[reg_ptr_sum_zp]);
+                uni_vcvtdq2ps(vmm_tmp, vmm_tmp);
+                uni_vsubps(vmm_prev_dst, vmm_prev_dst, vmm_tmp);
+            }
             if (sum_scale == 1.f)
                 uni_vaddps(vmm, vmm, vmm_prev_dst);
             else {
@@ -149,10 +161,16 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::apply_sum(const int nb_oc_block,
                 uni_vfmadd231ps(vmm, vmm_prev_dst, vmm_tmp);
             }
         };
-        const auto sum_injector = [=]() {
+        // Capture by value has to be applied since this lambda is called from
+        // a different context when stack values are unavailable.
+        const auto sum_injector = [nb_oc_block, ur_w, last_oc_block_flag,
+                                          sum_injector_lam]() {
             iterate(nb_oc_block, ur_w, last_oc_block_flag, sum_injector_lam);
         };
-        if (*p_sum_scale != 1.f) mov(reg_ptr_sum_scale, (size_t)p_sum_scale);
+        if (*p_sum_scale != 1.f)
+            mov(reg_ptr_sum_scale, reinterpret_cast<size_t>(p_sum_scale));
+        if (*p_sum_zp != 0)
+            mov(reg_ptr_sum_zp, reinterpret_cast<size_t>(p_sum_zp));
         postops_injector_->set_lambda_injector(
                 primitive_kind::sum, sum_injector);
     }
@@ -161,40 +179,44 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::apply_sum(const int nb_oc_block,
 template <cpu_isa_t isa, typename Vmm>
 void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::apply_postops(
         const int nb_oc_block, const int ur_w, const bool last_oc_block_flag,
-        const int oc_block, const float *p_sum_scale) {
+        const int oc_block, const float *p_sum_scale, const int32_t *p_sum_zp) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
-        apply_sum(nb_oc_block, ur_w, last_oc_block_flag, oc_block, p_sum_scale);
+        if (jcp.with_sum && *p_sum_zp != 0) push(reg_ptr_sum_zp);
+        apply_sum(nb_oc_block, ur_w, last_oc_block_flag, oc_block, p_sum_scale,
+                p_sum_zp);
 
         vmm_index_set_t vmm_idxs;
-        binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
         if (jcp.with_binary) {
+            binary_injector::rhs_arg_dynamic_params_t rhs_arg_params;
             const bool oc_blk_is_smaller_than_vmm = oc_block < isa_simd_width_;
             iterate(nb_oc_block, ur_w, last_oc_block_flag,
                     oc_blk_is_smaller_than_vmm,
                     [&](const bool mask_flag, const int k, const int j) {
-                        const int aux_output_offset = jcp.typesize_out
+                        const size_t aux_output_offset = jcp.typesize_out
                                 * (k * oc_block
                                         + j * jcp.oc_without_padding
                                                 * jcp.ngroups);
                         const auto vmm_idx = vmm_out_idx(j, k);
                         vmm_idxs.emplace(vmm_idx);
 
-                        rhs_arg_params.vmm_idx_to_oc_elem_off_addr.emplace(
-                                vmm_idx, ptr[param1 + GET_OFF(oc_l_off)]);
-                        rhs_arg_params.vmm_idx_to_oc_elem_off_val.emplace(
-                                vmm_idx, k * oc_block);
+                        rhs_arg_params.vmm_idx_to_out_reg.emplace(
+                                vmm_idx, reg_out);
                         rhs_arg_params.vmm_idx_to_out_elem_off_val.emplace(
                                 vmm_idx, aux_output_offset);
+
                         if (mask_flag)
                             rhs_arg_params.vmm_tail_idx_.emplace(vmm_idx);
                     });
-        } else
+
+            postops_injector_->compute_vector_range(vmm_idxs, rhs_arg_params);
+        } else {
             iterate(nb_oc_block, ur_w,
                     [&](const bool, const int k, const int j) {
                         vmm_idxs.emplace(vmm_out_idx(j, k));
                     });
-
-        postops_injector_->compute_vector_range(vmm_idxs, rhs_arg_params);
+            postops_injector_->compute_vector_range(vmm_idxs);
+        }
+        if (jcp.with_sum && *p_sum_zp != 0) pop(reg_ptr_sum_zp);
     }
 }
 
@@ -219,15 +241,11 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::store_output(
     const auto &p = attr_.post_ops_;
     const int sum_idx = p.find(primitive_kind::sum);
     const float *p_sum_scale = nullptr;
+    const int32_t *p_sum_zp = nullptr;
     if (sum_idx != -1) {
         const auto &p_entry = p.entry_[sum_idx];
         p_sum_scale = &p_entry.sum.scale;
-    }
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        /* put 'wei_adj_scale = 0.5' for bias calculation */
-        mov(reg_bias_alpha, float2int(jcp.wei_adj_scale));
-        uni_vmovq(xmm_bias_alpha(), reg_bias_alpha);
-        uni_vbroadcastss(vmm_bias_alpha(), xmm_bias_alpha());
+        p_sum_zp = &p_entry.sum.zero_point;
     }
 
     for (int k = 0; k < nb_oc_block; ++k) {
@@ -235,11 +253,8 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::store_output(
         const int load_size = mask_flag ? get_tail_size() : get_blocking_size();
         int scale_offset = jcp.is_oc_scale * (sizeof(float) * k * oc_block);
         if (jcp.with_bias) {
-
             int bias_offset = jcp.typesize_bia * k * oc_block;
             cvt2ps(jcp.bia_dt, vmm_bias, reg_bias, bias_offset, load_size);
-            if (jcp.signed_input && jcp.ver != ver_vnni) /* bias *= 0.5 */
-                uni_vmulps(vmm_bias, vmm_bias, vmm_bias_alpha());
         }
         if (jcp.signed_input) {
             const int comp_offset = sizeof(int32_t) * k * oc_block;
@@ -254,7 +269,6 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::store_output(
         }
         /* add to ymm_accum: compensation, zero_point, bias and permute */
         if (mask_flag) {
-            uni_vpxor(vmm_scale, vmm_scale, vmm_scale);
             load_data(data_type::s32, vmm_scale, reg_ptr_scales, scale_offset,
                     get_tail_size());
         } else {
@@ -271,13 +285,27 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::store_output(
             if (jcp.src_zero_point) uni_vpaddd(vmm, vmm, vmm_zp_comp);
             uni_vcvtdq2ps(vmm, vmm);
 
-            if (jcp.with_bias) uni_vaddps(vmm, vmm, vmm_bias);
-
             uni_vmulps(vmm, vmm, vmm_scale);
+
+            if (jcp.with_bias) uni_vaddps(vmm, vmm, vmm_bias);
         }
     }
 
-    apply_postops(nb_oc_block, ur_w, last_oc_block_flag, oc_block, p_sum_scale);
+    apply_postops(nb_oc_block, ur_w, last_oc_block_flag, oc_block, p_sum_scale,
+            p_sum_zp);
+
+    if (jcp.dst_scale) {
+        mov(reg_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
+        uni_vmovups(vmm_dst_scale, ptr[reg_dst_scale]);
+
+        /* Apply dst scale to accumulator */
+        for (int k = 0; k < nb_oc_block; k++) {
+            for (int j = 0; j < ur_w; j++) {
+                const Vmm vmm = vmm_out(j, k);
+                uni_vmulps(vmm, vmm, vmm_dst_scale);
+            }
+        }
+    }
 
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
@@ -361,11 +389,11 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::compute_ker_dw(int ur_w, int pad_l,
         uni_vpbroadcastd(vmm_zp, ptr[reg_src_zero_point]);
     }
 
-    auto input_spatial_index = [=](int oi, int ki) {
+    auto input_spatial_index = [this, pad_l](int oi, int ki) {
         return (ki * (jcp.dilate_w + 1) + oi * jcp.stride_w - pad_l);
     };
 
-    auto input_offset2 = [=](int ii, int ci) {
+    auto input_offset2 = [this](int ii, int ci) {
         if (jcp.is_fused_conv)
             return jcp.typesize_in
                     * (ii * jcp.dw_conv_buffer_oc + ci * jcp.ch_block);
@@ -373,16 +401,17 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::compute_ker_dw(int ur_w, int pad_l,
             return jcp.typesize_in * (ii * jcp.ngroups + ci * jcp.ch_block);
     };
 
-    auto input_offset3 = [=](int oi, int ci, int ki) {
+    auto input_offset3 = [this, input_offset2, input_spatial_index](
+                                 int oi, int ci, int ki) {
         return jcp.typesize_in * input_offset2(input_spatial_index(oi, ki), ci);
     };
 
-    auto kernel_offset = [=](int ci, int ki) {
+    auto kernel_offset = [this](int ci, int ki) {
         return jcp.typesize_in * ((ci * jcp.kh * jcp.kw + ki) * jcp.ch_block);
     };
 
-    auto compute = [=](Vmm vreg_acc, Vmm vreg_wei, Vmm vreg_src) {
-        if (jcp.ver == ver_vnni) {
+    auto compute = [this](Vmm vreg_acc, Vmm vreg_wei, Vmm vreg_src) {
+        if (jcp.has_vnni) {
             vpdpbusd(vreg_acc, vreg_src, vreg_wei, VexEncoding);
         } else {
             // okay for depthwise since src is zero-extended
@@ -417,7 +446,6 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::compute_ker_dw(int ur_w, int pad_l,
                 int aux_input_offset = input_offset2(ii, ci);
                 const Vmm vmm_inp_tmp = vmm_inp(ii, jcp.nb_ch_blocking);
 
-                uni_vpxor(vmm_inp_tmp, vmm_inp_tmp, vmm_inp_tmp);
                 load_data(data_type::u8, vmm_inp_tmp, aux_reg_inp,
                         aux_input_offset,
                         mask_flag ? get_tail_size() : get_blocking_size());
@@ -508,20 +536,21 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::compute_ker(int ur_w, int pad_l,
         mov(reg_src_zero_point, ptr[param1 + GET_OFF(src_zero_point)]);
     }
 
-    auto input_offset = [=](int oi, int ic, int ki) {
+    auto input_offset = [this, stride_w, pad_l](int oi, int ic, int ki) {
         return jcp.typesize_in
                 * ((ki * (jcp.dilate_w + 1) + oi * stride_w - pad_l)
                                 * jcp.ic_without_padding * jcp.ngroups
                         + ic_sub_step * ic);
     };
-    auto kernel_offset = [=](int ii, int ic, int ki) {
-        return jcp.typesize_in
-                * ((ii * jcp.nb_ic * jcp.kd * jcp.kh * jcp.kw + ki)
-                                * ch_block_all
-                        + ic_sub_step * ic * oc_block);
-    };
-    auto compute = [=](Vmm vreg_acc, Vmm vreg_wei, Vmm vreg_src) {
-        if (jcp.ver == ver_vnni) {
+    auto kernel_offset
+            = [this, ch_block_all, oc_block](int ii, int ic, int ki) {
+                  return jcp.typesize_in
+                          * ((ii * jcp.nb_ic * jcp.kd * jcp.kh * jcp.kw + ki)
+                                          * ch_block_all
+                                  + ic_sub_step * ic * oc_block);
+              };
+    auto compute = [this](Vmm vreg_acc, Vmm vreg_wei, Vmm vreg_src) {
+        if (jcp.has_vnni) {
             vpdpbusd(vreg_acc, vreg_src, vreg_wei, VexEncoding);
         } else {
             uni_vpmaddubsw(vmm_tmp, vreg_src, vreg_wei);
@@ -608,7 +637,7 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::compute_ker(int ur_w, int pad_l,
                         for (int ic = 0; ic < icb; ic++) {
                             const int aux_kernel_offset
                                     = kernel_offset(ii, ic, ki);
-                            if (jcp.ver == ver_vnni) {
+                            if (jcp.has_vnni) {
                                 vpdpbusd(vmm_tmp, vmm_zp_one,
                                         ptr[aux_reg_ker + aux_kernel_offset],
                                         VexEncoding);
@@ -878,7 +907,7 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::generate() {
         const bool is_zero_point = jcp.src_zero_point || jcp.dst_zero_point;
         int idx = ker_max_reg + 1 - jcp.max_regs_ur - 2 * is_zero_point;
         if (!jcp.is_resrc_depthwise) vmm_dw_src = Vmm(--idx);
-        if (jcp.ver != ver_vnni) vmm_dw_tmp = Vmm(--idx);
+        if (!jcp.has_vnni) vmm_dw_tmp = Vmm(--idx);
         if (jcp.signed_input) {
             --idx; // due to extra register used for compensations
         }
@@ -886,7 +915,7 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::generate() {
                 !is_zero_point, idx == ker_max_reg - ker_dw_reg_base_idx));
     }
 
-    if (!jcp.is_depthwise && jcp.ver != ver_vnni) {
+    if (!jcp.is_depthwise && (!jcp.has_vnni)) {
         auto vmm_one_128 = Xbyak::Xmm(vmm_one.getIdx());
         mov(reg_scratch, 0x10001);
         uni_vmovq(vmm_one_128, reg_scratch);
@@ -1086,7 +1115,8 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::generate() {
             if (jcp.nb_ow > 1 && cur_n_oi == 0) {
                 // cur_n_oi == 0 signifies beginning of new ow_block
                 // (or end of previous block)
-                const dim_t inp_lpad_region_shift = -label_cntr * jcp.ow_block
+                const dim_t inp_lpad_region_shift
+                        = static_cast<dim_t>(-label_cntr) * jcp.ow_block
                         * jcp.stride_w * in_ic_shift;
                 L(ow_block_jmp_table[label_cntr++]);
                 // harness passes shifted src pointer that does not take
@@ -1201,14 +1231,15 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::generate() {
 
     postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
 template <cpu_isa_t isa>
 status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
         const convolution_desc_t &cd, memory_desc_t &src_md,
         memory_desc_t &weights_md, memory_desc_t &dst_md,
-        memory_desc_t &bias_md, const primitive_attr_t &attr, int nthreads) {
+        memory_desc_t &bias_md, primitive_attr_t &attr, int nthreads) {
     using namespace prop_kind;
 
     const memory_desc_wrapper src_d(&src_md);
@@ -1224,12 +1255,14 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
     const bool is_avx2 = isa == avx2;
     assert(is_1d || is_2d || is_3d);
 
-    if (!(mayiuse(isa)
-                && one_of(src_d.data_type(), data_type::u8, data_type::s8)
-                && weights_d.data_type() == data_type::s8
-                && one_of(dst_d.data_type(), data_type::f32, data_type::s32,
-                        data_type::s8, data_type::u8)))
-        return status::unimplemented;
+    // disabling verbose dispatch messages for unsupported isa for better readability
+    if (!mayiuse(isa)) return status::unimplemented;
+
+    const bool dt_ok = one_of(src_d.data_type(), data_type::u8, data_type::s8)
+            && weights_d.data_type() == data_type::s8
+            && one_of(dst_d.data_type(), data_type::f32, data_type::s32,
+                    data_type::s8, data_type::u8);
+    VDISPATCH_CONV_IC(dt_ok, VERBOSE_UNSUPPORTED_DT_CFG);
 
     jcp = zero<decltype(jcp)>();
     jcp.nthr = nthreads;
@@ -1274,11 +1307,17 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
     jcp.back_pad = calculate_end_padding(
             jcp.f_pad, jcp.od, jcp.id, jcp.stride_d, ext_kd);
 
-    jcp.ver = mayiuse(avx2_vnni) ? ver_vnni : ver_unused;
+    jcp.has_vnni = mayiuse(avx2_vnni);
 
     jcp.signed_input = src_d.data_type() == data_type::s8;
     jcp.is_depthwise = true && with_groups && everyone_is(1, jcp.ic, jcp.oc);
 
+    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
+    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
+
+    jcp.is_oc_scale = wei_scales.mask_ != 0;
+    jcp.dst_scale = !dst_scales.has_default_values();
+
     const auto zp = attr.zero_points_;
     jcp.dst_zero_point = !zp.has_default_values(DNNL_ARG_DST);
     jcp.src_zero_point = !zp.has_default_values(DNNL_ARG_SRC);
@@ -1286,10 +1325,13 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
             = zp.common(DNNL_ARG_SRC); // otherwise, it's per-channel
     assert(IMPLICATION(jcp.src_zero_point, jcp.zp_src_is_common));
 
-    if ((jcp.dst_zero_point || jcp.src_zero_point) && jcp.is_fused_conv)
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            !((jcp.dst_zero_point || jcp.src_zero_point) && jcp.is_fused_conv),
+            VERBOSE_UNSUPPORTED_FEATURE,
+            "fused depthwise convolution does not support zero-point");
 
-    if (is_3d && jcp.is_depthwise) return status::unimplemented;
+    VDISPATCH_CONV_IC(!(is_3d && jcp.is_depthwise), VERBOSE_UNSUPPORTED_FEATURE,
+            "unsupported depthwise implementation for 3D convolution");
 
     if (jcp.is_depthwise) {
         jcp.ch_block = is_avx2 ? 8 : 4;
@@ -1313,34 +1355,19 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
              * - Otherwise return unimplemented */
             jcp.oc_block = jcp.ic_block = 4;
         }
-        if (jcp.ic % jcp.ic_block != 0 || jcp.oc % jcp.oc_block != 0)
-            return status::unimplemented;
+        VDISPATCH_CONV_IC(
+                !(jcp.ic % jcp.ic_block != 0 || jcp.oc % jcp.oc_block != 0),
+                VERBOSE_BLOCKING_FAIL, "bad blocking parameters");
     }
 
-    const auto &post_ops = attr.post_ops_;
-
-    const int eltwise_ind = post_ops.find(primitive_kind::eltwise);
-    jcp.with_eltwise = eltwise_ind != -1;
-    const int binary_ind = post_ops.find(primitive_kind::binary);
-    jcp.with_binary = binary_ind != -1;
-    const int sum_ind = post_ops.find(primitive_kind::sum);
-    jcp.with_sum = sum_ind != -1;
-
-    jcp.post_ops = post_ops;
-
-    using namespace injector;
-    const bool post_ops_ok_
-            = post_ops_ok({isa, {eltwise, binary, sum}, jcp.post_ops, &dst_d});
-    if (!post_ops_ok_) return status::unimplemented;
-
     jcp.is_resrc_depthwise = true && jcp.is_depthwise && jcp.stride_w < jcp.kw
             && jcp.kw < 4 && jcp.dilate_w == 0;
 
     if (jcp.is_depthwise) {
         jcp.max_regs_ur = 14 - !jcp.is_resrc_depthwise - jcp.signed_input
-                + (jcp.ver == ver_vnni);
+                + (jcp.has_vnni);
     } else {
-        jcp.max_regs_ur = jcp.ver == ver_vnni ? 15 - jcp.signed_input : 12;
+        jcp.max_regs_ur = jcp.has_vnni ? 15 - jcp.signed_input : 12;
     }
 
     if (jcp.src_zero_point || jcp.dst_zero_point) jcp.max_regs_ur = 9;
@@ -1379,7 +1406,7 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
             }
         }
         memory_desc_t want_wei_md = weights_md;
-        memory_desc_init_by_tag(want_wei_md, wei_tag);
+        CHECK_BOOL(memory_desc_init_by_tag(want_wei_md, wei_tag));
 
         if (jcp.signed_input) {
             want_wei_md.extra.flags = 0
@@ -1387,7 +1414,7 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
                     | memory_extra_flags::scale_adjust;
             want_wei_md.extra.compensation_mask
                     = (with_groups && !jcp.is_depthwise) ? g_mask : c_mask;
-            want_wei_md.extra.scale_adjust = (jcp.ver == ver_vnni) ? 1.f : 0.5f;
+            want_wei_md.extra.scale_adjust = (jcp.has_vnni) ? 1.f : 0.5f;
         }
         if (jcp.src_zero_point) {
             want_wei_md.extra.flags |= compensation_conv_asymmetric_src;
@@ -1403,7 +1430,8 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
         return weights_md == want_wei_md;
     };
 
-    if (!set_or_check_wei_format()) return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            set_or_check_wei_format(), VERBOSE_UNSUPPORTED_TAG_S, "weights");
     format_tag_t dat_tag = utils::pick(
             ndims - 3, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
 
@@ -1413,7 +1441,7 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
     } else {
         jcp.src_tag = src_d.matches_one_of_tag(dat_tag);
     }
-    if (jcp.src_tag != dat_tag) return status::unimplemented;
+    VDISPATCH_CONV_IC(jcp.src_tag == dat_tag, VERBOSE_UNSUPPORTED_TAG_S, "src");
 
     if (dst_d.format_kind() == format_kind::any) {
         CHECK(memory_desc_init_by_tag(dst_md, dat_tag));
@@ -1421,7 +1449,7 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
     } else {
         jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag);
     }
-    if (jcp.dst_tag != dat_tag) return status::unimplemented;
+    VDISPATCH_CONV_IC(jcp.dst_tag == dat_tag, VERBOSE_UNSUPPORTED_TAG_S, "dst");
 
     if (jcp.with_bias) {
         if (bias_d.format_kind() == format_kind::any)
@@ -1430,6 +1458,27 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
     jcp.bia_dt = jcp.with_bias ? cd.bias_desc.data_type : data_type::undef;
     jcp.dst_dt = cd.dst_desc.data_type;
 
+    CHECK(attr.set_default_formats(&dst_md));
+
+    const auto &post_ops = attr.post_ops_;
+
+    const int eltwise_ind = post_ops.find(primitive_kind::eltwise);
+    jcp.with_eltwise = eltwise_ind != -1;
+    const int binary_ind = post_ops.find(primitive_kind::binary);
+    const int prelu_ind = post_ops.find(primitive_kind::prelu);
+    jcp.with_binary = !everyone_is(-1, binary_ind, prelu_ind);
+    const int sum_ind = post_ops.find(primitive_kind::sum);
+    jcp.with_sum = sum_ind != -1;
+    jcp.sum_dt = post_ops.get_sum_dt(jcp.dst_dt);
+
+    jcp.post_ops = post_ops;
+
+    using namespace injector;
+
+    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(isa,
+            {eltwise, binary, sum}, jcp.post_ops, &dst_d, false, false, false));
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
+
     jcp.typesize_in = types::data_type_size(src_d.data_type());
     jcp.typesize_out = types::data_type_size(dst_d.data_type());
     jcp.typesize_bia
@@ -1464,14 +1513,14 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
         if (is_oc_blocking_ok(jcp.nb_oc_blocking_thr_chunk)) break;
     }
 
-    auto get_thr_eff = [=](int nb_ow, int nthr) {
+    auto get_thr_eff = [&](int nb_ow, int nthr) {
         int base_work_amount = jcp.mb * jcp.nb_ch * jcp.od * jcp.oh
                 * (jcp.nb_oc / jcp.nb_oc_blocking_thr_chunk);
         auto work_amount = base_work_amount * nb_ow;
         return float(work_amount) / rnd_up(work_amount, nthr);
     };
 
-    auto get_ow_block = [=](int ur_w, int nthr) {
+    auto get_ow_block = [&jcp, get_thr_eff](int ur_w, int nthr) {
         int res_ow_block = jcp.ow;
         float best_thr_eff = get_thr_eff(1, nthr);
         float thr_eff;
@@ -1545,19 +1594,13 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
 
     bool args_ok = true && jcp.oc % jcp.oc_block == 0
             && IMPLICATION(!jcp.is_1stconv, jcp.ic % jcp.ic_block == 0);
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            args_ok, VERBOSE_BLOCKING_FAIL, "bad blocking dimensions");
 
     pick_loop_order(jcp);
 
     jcp.nb_ic_L2 = jcp.nb_ic;
 
-    const auto &oscales = attr.output_scales_;
-    jcp.is_oc_scale = oscales.mask_ == 1 << 1;
-
-    // only common and per-oc-channel scales are supported
-    const bool oscales_ok = one_of(oscales.mask_, 0, 1 << 1);
-    if (!oscales_ok) return status::unimplemented;
-
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
             ? weights_d.extra().scale_adjust
@@ -1571,12 +1614,11 @@ void jit_uni_x8s8s32x_fwd_kernel<isa>::init_scratchpad(
         memory_tracking::registrar_t &scratchpad, const jit_conv_conf_t &jcp,
         const primitive_attr_t &attr) {
 
-    if (jcp.signed_input && jcp.ver != ver_vnni) {
-        dim_t count = attr.output_scales_.count_ == 1
-                ? (dim_t)8
-                : attr.output_scales_.count_;
-        scratchpad.book<float>(key_conv_adjusted_scales, count);
-    }
+    const int mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const dim_t scales_count
+            = mask == 0 ? 1 : static_cast<dim_t>(jcp.oc) * jcp.ngroups;
+    dim_t count = scales_count == 1 ? (dim_t)8 : scales_count;
+    scratchpad.book<float>(key_conv_adjusted_scales, count);
 }
 
 template struct _jit_uni_x8s8s32x_fwd_kernel<avx2, Ymm>;
