@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -30,43 +30,45 @@ namespace x64 {
 using namespace dnnl::impl::format_tag;
 
 #define IRB_LOOP(statement) \
-    if (1 == reg_block) { \
-        const int irb_off = 0; \
-        const int irb = this->reg_block_idx_ % vsum.size(); \
+    for (int irb = 0; irb < reg_block; irb++) { \
+        const int irb_off = irb * this->single_pixel_offset_; \
         statement; \
         MAYBE_UNUSED(irb_off); \
-    } else { \
-        for (int irb = 0; irb < reg_block; irb++) { \
-            const int irb_off = irb * this->single_pixel_offset_; \
-            statement; \
-            MAYBE_UNUSED(irb_off); \
-        } \
     }
 
 using namespace Xbyak;
 
-template <template <cpu_isa_t isa, data_type_t d_type> class Derived,
-        cpu_isa_t isa, data_type_t d_type>
-jit_uni_lrn_kernel_t<Derived<isa, d_type>>::jit_uni_lrn_kernel_t(
-        void *code_ptr, size_t code_size)
-    : jit_generator(code_ptr, code_size, true, isa)
-    , emulate_bfloat_(isa == avx512_common
-              && d_type == dnnl::impl::data_type::bf16
-              && !mayiuse(avx512_core_bf16))
-    , bf16_emu_(
-              emulate_bfloat_ ? utils::make_unique<bf16_emulation_t>(this,
-                      bf16_emu_reserv_1_, bf16_emu_reserv_2_,
-                      bf16_emu_reserv_3_, bf16_emu_scratch_, bf16_emu_reserv_4_)
-                              : nullptr) {
-
-    if (bf16_emu_) bf16_emu_->init_vcvtneps2bf16();
+cpu_isa_t get_io_isa(cpu_isa_t isa, data_type_t d_type) {
+    // re-using avx512_core instantiation for bf16
+    return isa == avx512_core && d_type == data_type::bf16
+                    && mayiuse(avx512_core_bf16)
+            ? avx512_core_bf16
+            : isa;
 }
 
 template <template <cpu_isa_t isa, data_type_t d_type> class Derived,
         cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_kernel_t<Derived<isa, d_type>>::jit_uni_lrn_kernel_t(
-        const within_config_t &config, void *code_ptr, size_t code_size)
-    : jit_uni_lrn_kernel_t(code_ptr, code_size) {
+        const char *name)
+    : jit_generator(name, isa)
+    , emulate_bfloat_(d_type == data_type::bf16 && !mayiuse(avx512_core_bf16)
+              && is_superset(isa, avx512_core))
+    , bf16_emu_(
+              emulate_bfloat_ ? utils::make_unique<bf16_emulation_t>(this,
+                      bf16_emu_reserv_1_, bf16_emu_reserv_2_,
+                      bf16_emu_reserv_3_, bf16_emu_scratch_, bf16_emu_reserv_4_)
+                              : nullptr)
+    , io_(this, get_io_isa(isa, d_type), {d_type}, {},
+              io::io_tail_conf_t {simd_w_, 0, this->k1, 0, this->reg_tmp_},
+              io::io_emu_bf16_conf_t {bf16_emu_reserv_1_, bf16_emu_reserv_2_,
+                      bf16_emu_reserv_3_, bf16_emu_scratch_,
+                      bf16_emu_reserv_4_}) {}
+
+template <template <cpu_isa_t isa, data_type_t d_type> class Derived,
+        cpu_isa_t isa, data_type_t d_type>
+jit_uni_lrn_kernel_t<Derived<isa, d_type>>::jit_uni_lrn_kernel_t(
+        const within_config_t &config, const char *name)
+    : jit_uni_lrn_kernel_t(name) {
     if (config.dat_tag == nhwc)
         single_pixel_offset_
                 = config.C * sizeof(typename prec_traits<d_type>::type);
@@ -176,65 +178,6 @@ void jit_uni_lrn_kernel_t<Derived<isa, d_type>>::within_body_reg_blocked(
     }
 }
 
-template <template <cpu_isa_t isa, data_type_t d_type> class Derived,
-        cpu_isa_t isa, data_type_t d_type>
-void jit_uni_lrn_kernel_t<Derived<isa, d_type>>::load_data(
-        const Vmm &reg, const Xbyak::Address &p) {
-    this->uni_vmovups(reg, p);
-}
-
-template <typename Gen, typename Reg, typename Addr>
-void load_bf16_data(Gen generator, const Reg &reg, const Addr &p) {
-    generator->vpmovzxwd(reg, p);
-    generator->vpslld(reg, reg, 0x10);
-}
-
-template <>
-void jit_uni_lrn_kernel_t<jit_uni_lrn_fwd_kernel_t<avx512_common,
-        dnnl::impl::data_type::bf16>>::load_data(const Vmm &reg,
-        const Xbyak::Address &p) {
-    load_bf16_data(this, reg, p);
-}
-
-template <>
-void jit_uni_lrn_kernel_t<jit_uni_lrn_bwd_kernel_t<avx512_common,
-        dnnl::impl::data_type::bf16>>::load_data(const Vmm &reg,
-        const Xbyak::Address &p) {
-    load_bf16_data(this, reg, p);
-}
-
-template <template <cpu_isa_t isa, data_type_t d_type> class Derived,
-        cpu_isa_t isa, data_type_t d_type>
-void jit_uni_lrn_kernel_t<Derived<isa, d_type>>::store_data(
-        const Xbyak::Address &addr, const Vmm &reg) {
-    this->uni_vmovups(addr, reg);
-}
-
-template <typename Gen, typename Bf16Emu>
-void store_bf16_data(
-        Gen generator, Bf16Emu emu, const Xbyak::Address &addr, const Zmm &zr) {
-    const Ymm yr = Ymm(zr.getIdx());
-    if (mayiuse(avx512_core_bf16))
-        generator->vcvtneps2bf16(yr, zr);
-    else
-        emu->vcvtneps2bf16(yr, zr);
-    generator->vmovdqu16(addr, yr);
-}
-
-template <>
-void jit_uni_lrn_kernel_t<jit_uni_lrn_fwd_kernel_t<avx512_common,
-        dnnl::impl::data_type::bf16>>::store_data(const Xbyak::Address &addr,
-        const Zmm &zr) {
-    store_bf16_data(this, bf16_emu_.get(), addr, zr);
-}
-
-template <>
-void jit_uni_lrn_kernel_t<jit_uni_lrn_bwd_kernel_t<avx512_common,
-        dnnl::impl::data_type::bf16>>::store_data(const Xbyak::Address &addr,
-        const Zmm &zr) {
-    store_bf16_data(this, bf16_emu_.get(), addr, zr);
-}
-
 template <template <cpu_isa_t isa, data_type_t d_type> class Derived,
         cpu_isa_t isa, data_type_t d_type>
 void jit_uni_lrn_kernel_t<Derived<isa, d_type>>::load_constant(
@@ -246,8 +189,8 @@ void jit_uni_lrn_kernel_t<Derived<isa, d_type>>::load_constant(
 
 template <>
 void jit_uni_lrn_kernel_t<jit_uni_lrn_fwd_kernel_t<sse41,
-        dnnl::impl::data_type::f32>>::load_constant(float constant,
-        const Vmm &v_constant, const Xbyak::Xmm &x_constant) {
+        data_type::f32>>::load_constant(float constant, const Vmm &v_constant,
+        const Xbyak::Xmm &x_constant) {
     this->mov(this->imm_addr64_, float2int(constant));
     this->uni_vmovq(x_constant, this->imm_addr64_);
     this->shufps(x_constant, x_constant, 0);
@@ -260,39 +203,47 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::within_body(int hoff, int Hoff,
         int woff, int Woff, int stride, prop_kind_t pk, const int reg_block,
         int pixel_offset) {
 
-    static const std::array<Vmm, 3> vsum {{Vmm(2), Vmm(11), Vmm(20)}};
-    static const std::array<Vmm, 3> vsum2 {{Vmm(3), Vmm(12), Vmm(21)}};
-    static const std::array<Vmm, 3> vdst {{Vmm(4), Vmm(13), Vmm(22)}};
-    static const std::array<std::array<Vmm, 6u>, 3u> vtmp {
-            {{{Vmm(5), Vmm(6), Vmm(7), Vmm(8), Vmm(9), Vmm(14)}},
-                    {{Vmm(18), Vmm(15), Vmm(16), Vmm(17), Vmm(29), Vmm(30)}},
-                    {{Vmm(23), Vmm(24), Vmm(25), Vmm(26), Vmm(28), Vmm(31)}}}};
-    static const std::array<Vmm, 3> vscratch = {{Vmm(10), Vmm(19), Vmm(27)}};
-    static const std::size_t used_tmp_regs
-            = this->emulate_bfloat_ ? vtmp[0].size() - 2 : vtmp[0].size();
+    static const std::array<Vmm, 5> vsum {
+            {Vmm(2), Vmm(7), Vmm(12), Vmm(17), Vmm(22)}};
+    static const std::array<Vmm, 5> vsum2 {
+            {Vmm(3), Vmm(8), Vmm(13), Vmm(18), Vmm(23)}};
+    static const std::array<Vmm, 5> vdst {
+            {Vmm(4), Vmm(9), Vmm(14), Vmm(19), Vmm(24)}};
+    static const std::array<Vmm, 5> vtmp {
+            {Vmm(5), Vmm(10), Vmm(15), Vmm(20), Vmm(25)}};
+    static const std::array<Vmm, 5> vscratch {
+            {Vmm(6), Vmm(11), Vmm(16), Vmm(21), Vmm(26)}};
+    static const std::array<Vmm, 2> vtmp2 {{Vmm(12), Vmm(13)}};
+    static const Vmm vaux = Vmm(14);
+    const bool has_ne_convert_xf16_support = isa == avx2_vnni_2
+            && utils::one_of(d_type, data_type::bf16, data_type::f16);
 
     IRB_LOOP(this->uni_vxorps(vsum[irb], vsum[irb], vsum[irb]));
     for (int i = hoff; i <= Hoff; ++i) {
         for (int j = woff; j <= Woff; ++j) {
-            if (i == 0 && j == 0) {
-                IRB_LOOP(this->load_data(
-                        vdst[irb], this->ptr[src_ + pixel_offset + irb_off]));
-                IRB_LOOP(this->vfmadd231ps(vsum[irb], vdst[irb], vdst[irb]));
+            const auto p_off = pixel_offset
+                    + (i * stride + j) * this->single_pixel_offset_;
+            const bool can_load_two_simdw = has_ne_convert_xf16_support
+                    && Woff - j >= 2 && !(i == 0 && (j == 0 || j + 1 == 0));
+            if (can_load_two_simdw) {
+                IRB_LOOP(this->io_.at(d_type)->load_two_simdw_xf16(
+                        this->ptr[src_ + p_off + irb_off], vtmp[irb],
+                        vtmp2[irb]));
+                IRB_LOOP(this->io_.at(d_type)->merge_interleaved_to_plain(
+                        vtmp[irb], vtmp2[irb], vaux));
+                IRB_LOOP(this->vfmadd231ps(vsum[irb], vtmp[irb], vtmp[irb]));
+                IRB_LOOP(this->vfmadd231ps(vsum[irb], vtmp2[irb], vtmp2[irb]));
+                ++j;
             } else {
-                const auto idx = this->tempIdx_ % used_tmp_regs;
-                IRB_LOOP(this->load_data(vtmp[irb][idx],
-                        this->ptr[(src_ + pixel_offset + irb_off)
-                                + (i * stride + j)
-                                        * this->single_pixel_offset_]));
-                IRB_LOOP(this->vfmadd231ps(
-                        vsum[irb], vtmp[irb][idx], vtmp[irb][idx]));
-                ++(this->tempIdx_);
+                const auto vdata = (i == 0 && j == 0) ? vdst : vtmp;
+                IRB_LOOP(this->io_.at(d_type)->load(
+                        this->ptr[(src_ + p_off + irb_off)], vdata[irb],
+                        false));
+                IRB_LOOP(this->vfmadd231ps(vsum[irb], vdata[irb], vdata[irb]));
             }
         }
     }
 
-    this->tempIdx_ = this->tempIdx_ % used_tmp_regs;
-
     IRB_LOOP(this->vfmadd132ps(
             vsum[irb], vk_, valpha_)); // ysum <- ysum*valpha_+yk_
     IRB_LOOP(this->vmovaps(vscratch[irb], vsum[irb]));
@@ -307,25 +258,22 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::within_body(int hoff, int Hoff,
             vdst[irb], vdst[irb], vsum[irb])); // ydst <- ydst / ysum
 
     if (pk_ != prop_kind::forward_inference) {
-        IRB_LOOP(this->store_data(
-                this->ptr[scratch_ + pixel_offset + irb_off], vsum[irb]));
+        IRB_LOOP(this->io_.at(d_type)->store(vsum[irb],
+                this->ptr[scratch_ + pixel_offset + irb_off], false));
         IRB_LOOP(this->vdivps(vscratch[irb], vdst[irb], vscratch[irb]));
-        IRB_LOOP(this->store_data(
+        IRB_LOOP(this->io_.at(d_type)->store(vscratch[irb],
                 this->ptr[bwd_intermediate_res_ + pixel_offset + irb_off],
-                vscratch[irb]));
+                false));
     }
 
-    IRB_LOOP(this->store_data(
-            this->ptr[dst_ + pixel_offset + irb_off], vdst[irb]));
-
-    if (isa == avx512_common)
-        this->reg_block_idx_ = (this->reg_block_idx_ % vsum.size()) + 1;
+    IRB_LOOP(this->io_.at(d_type)->store(
+            vdst[irb], this->ptr[dst_ + pixel_offset + irb_off], false));
 }
 
 template <>
-void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::within_body(
-        int hoff, int Hoff, int woff, int Woff, int stride, prop_kind_t pk,
-        int reg_block, int pixel_offset) {
+void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::within_body(int hoff,
+        int Hoff, int woff, int Woff, int stride, prop_kind_t pk, int reg_block,
+        int pixel_offset) {
 
     const Xbyak::Xmm &xtmp_lo = this->xmm2;
     const Xbyak::Xmm &xtmp_hi = this->xmm3;
@@ -409,9 +357,8 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::move_data_pointers(
 
 template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_fwd_kernel_t<isa, d_type>::jit_uni_lrn_fwd_kernel_t(
-        const within_config_t &config, float A, float K, prop_kind_t pk,
-        void *code_ptr, size_t code_size)
-    : Base(config, code_ptr, code_size)
+        const within_config_t &config, float A, float K, prop_kind_t pk)
+    : Base(config, jit_name())
     , config_(lrn_config_t::within_config)
     , within_config_(config)
     , alpha_(A)
@@ -422,6 +369,7 @@ template <cpu_isa_t isa, data_type_t d_type>
 void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(
         const within_config_t &config) {
     this->preamble();
+    if (this->emulate_bfloat_) this->io_.init_bf16();
 
 #define GET_OFF(field) offsetof(jit_args_fwd_t, field)
     this->mov(src_, this->ptr[this->param1 + GET_OFF(src)]);
@@ -436,7 +384,9 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(
     this->load_constant(alpha_, valpha_, xalpha_);
     this->load_constant(k_, vk_, xk_);
 
-    static const int max_reg_blocks = isa == avx512_common ? 3 : 1;
+    static const int max_reg_blocks = is_superset(isa, avx512_core) ? 5
+            : is_superset(isa, avx2)                                ? 2
+                                                                    : 1;
     this->within_loop(config, max_reg_blocks, pk_);
 
     this->postamble();
@@ -444,9 +394,8 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(
 
 template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_fwd_kernel_t<isa, d_type>::jit_uni_lrn_fwd_kernel_t(
-        const struct nchw8c_across_t &J, float A, float K, prop_kind_t pk,
-        void *code_ptr, size_t code_size)
-    : Base(code_ptr, code_size)
+        const struct nchw8c_across_t &J, float A, float K, prop_kind_t pk)
+    : Base(jit_name())
     , config_(lrn_config_t::nchw8c_across)
     , nchw8c_across_(J)
     , alpha_(A)
@@ -471,6 +420,7 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(const nchw8c_across_t &J) {
     const Xbyak::Ymm &ybase = this->ymm12;
 
     this->preamble();
+    if (this->emulate_bfloat_) this->io_.init_bf16();
 
     this->mov(src_, this->ptr[this->param1 + 0]);
     this->mov(dst_, this->ptr[this->param1 + 8]);
@@ -542,10 +492,9 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(const nchw8c_across_t &J) {
 }
 
 template <>
-jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::
-        jit_uni_lrn_fwd_kernel_t(const struct nchw8c_across_t &J, float A,
-                float K, prop_kind_t pk, void *code_ptr, size_t code_size)
-    : Base(code_ptr, code_size)
+jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::jit_uni_lrn_fwd_kernel_t(
+        const struct nchw8c_across_t &J, float A, float K, prop_kind_t pk)
+    : Base(jit_name())
     , config_(lrn_config_t::nchw8c_across)
     , nchw8c_across_(J)
     , alpha_(A)
@@ -553,7 +502,7 @@ jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::
     , pk_(pk) {}
 
 template <>
-void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
+void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::generate(
         const nchw8c_across_t &J) {
 
     const Xbyak::Reg64 &t = this->rsp;
@@ -578,6 +527,7 @@ void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
     const Xbyak::Xmm &xbase_hi = this->xmm15;
 
     this->preamble();
+    if (this->emulate_bfloat_) this->io_.init_bf16();
 
     this->mov(src_, this->ptr[this->param1 + 0]);
     this->mov(dst_, this->ptr[this->param1 + 8]);
@@ -683,9 +633,8 @@ void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
 
 template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_fwd_kernel_t<isa, d_type>::jit_uni_lrn_fwd_kernel_t(
-        const struct nhwc_across_t &J, float A, float K, prop_kind_t pk,
-        void *code_ptr, size_t code_size)
-    : Base(code_ptr, code_size)
+        const struct nhwc_across_t &J, float A, float K, prop_kind_t pk)
+    : Base(jit_name())
     , config_(lrn_config_t::nhwc_across)
     , nhwc_across_(J)
     , alpha_(A)
@@ -709,17 +658,18 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(const nhwc_across_t &J) {
     const Xbyak::Ymm &ymask = this->ymm10;
 
     this->preamble();
+    if (this->emulate_bfloat_) this->io_.init_bf16();
 
     this->mov(src_, this->ptr[this->param1 + 0]);
     this->mov(dst_, this->ptr[this->param1 + 8]);
     if (pk_ != prop_kind::forward_inference)
         this->mov(scratch_, this->ptr[this->param1 + 16]);
     this->mov(this->imm_addr64_, float2int(this->alpha_));
-    this->movq(xalpha_, this->imm_addr64_);
+    this->vmovq(xalpha_, this->imm_addr64_);
     this->vbroadcastss(valpha_, xalpha_);
 
     this->mov(this->imm_addr64_, float2int(this->k_));
-    this->movq(xk_, this->imm_addr64_);
+    this->vmovq(xk_, this->imm_addr64_);
     this->vbroadcastss(yk_, xk_);
 
     this->vxorps(ysum, ysum, ysum);
@@ -805,10 +755,9 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(const nhwc_across_t &J) {
 }
 
 template <>
-jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::
-        jit_uni_lrn_fwd_kernel_t(const struct nhwc_across_t &J, float A,
-                float K, prop_kind_t pk, void *code_ptr, size_t code_size)
-    : Base(code_ptr, code_size)
+jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::jit_uni_lrn_fwd_kernel_t(
+        const struct nhwc_across_t &J, float A, float K, prop_kind_t pk)
+    : Base(jit_name())
     , config_(lrn_config_t::nhwc_across)
     , nhwc_across_(J)
     , alpha_(A)
@@ -816,7 +765,7 @@ jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::
     , pk_(pk) {}
 
 template <>
-void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
+void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::generate(
         const nhwc_across_t &J) {
     static uint32_t store[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
     const Xbyak::Reg64 c = this->r9;
@@ -840,6 +789,7 @@ void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
     const Xbyak::Xmm &xbase_hi = this->xmm15;
 
     this->preamble();
+    if (this->emulate_bfloat_) this->io_.init_bf16();
 
     this->mov(src_, this->ptr[this->param1 + 0]);
     this->mov(dst_, this->ptr[this->param1 + 8]);
@@ -1032,10 +982,9 @@ void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
 }
 
 template <>
-void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::nchw_body(
-        int tail, int HW, prop_kind_t pk, Xbyak::Ymm ymask, Xbyak::Ymm ya,
-        Xbyak::Ymm yb, Xbyak::Ymm yc, Xbyak::Ymm yd, Xbyak::Ymm ye,
-        Xbyak::Ymm ysum) {}
+void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::nchw_body(int tail,
+        int HW, prop_kind_t pk, Xbyak::Ymm ymask, Xbyak::Ymm ya, Xbyak::Ymm yb,
+        Xbyak::Ymm yc, Xbyak::Ymm yd, Xbyak::Ymm ye, Xbyak::Ymm ysum) {}
 
 template <cpu_isa_t isa, data_type_t d_type>
 void jit_uni_lrn_fwd_kernel_t<isa, d_type>::nchw_body(int tail, int HW,
@@ -1079,8 +1028,7 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::nchw_tail_sse41(int tail,
         Xbyak::Reg64 reg_dst, Xbyak::Xmm xtail_lo, Xbyak::Xmm xtail_hi) {}
 
 template <>
-void jit_uni_lrn_fwd_kernel_t<sse41,
-        dnnl::impl::data_type::f32>::nchw_tail_sse41(int tail,
+void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::nchw_tail_sse41(int tail,
         Xbyak::Reg64 reg_dst, Xbyak::Xmm xtail_lo, Xbyak::Xmm xtail_hi) {
     Xbyak::Xmm xmm_tmp = xmm10;
     this->movaps(xmm_tmp, xtail_hi);
@@ -1105,10 +1053,9 @@ void jit_uni_lrn_fwd_kernel_t<sse41,
 }
 
 template <>
-void jit_uni_lrn_fwd_kernel_t<sse41,
-        dnnl::impl::data_type::f32>::nchw_body_sse41(int tail, int HW,
-        prop_kind_t pk, Xbyak::Xmm xe_lo, Xbyak::Xmm xe_hi, Xbyak::Xmm xsum_lo,
-        Xbyak::Xmm xsum_hi) {
+void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::nchw_body_sse41(int tail,
+        int HW, prop_kind_t pk, Xbyak::Xmm xe_lo, Xbyak::Xmm xe_hi,
+        Xbyak::Xmm xsum_lo, Xbyak::Xmm xsum_hi) {
     const Xbyak::Xmm &xdst_lo = this->xmm0;
     const Xbyak::Xmm &xdst_hi = this->xmm1;
     const Xbyak::Xmm &xbase_lo = this->xmm6;
@@ -1212,9 +1159,8 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::nchw_body_sse41(int tail, int HW,
 
 template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_fwd_kernel_t<isa, d_type>::jit_uni_lrn_fwd_kernel_t(
-        const nchw_across_t &J, float A, float K, prop_kind_t pk,
-        void *code_ptr, size_t code_size)
-    : Base(code_ptr, code_size)
+        const nchw_across_t &J, float A, float K, prop_kind_t pk)
+    : Base(jit_name())
     , config_(lrn_config_t::nchw_across)
     , nchw_across_(J)
     , alpha_(A)
@@ -1236,6 +1182,7 @@ void jit_uni_lrn_fwd_kernel_t<isa, d_type>::generate(const nchw_across_t &J) {
     const Xbyak::Ymm &ysum = this->ymm8;
 
     this->preamble();
+    if (this->emulate_bfloat_) this->io_.init_bf16();
 
     if (J.tail != 0) {
         this->mov(
@@ -1304,10 +1251,9 @@ template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_fwd_kernel_t<isa, d_type>::~jit_uni_lrn_fwd_kernel_t() = default;
 
 template <>
-jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::
-        jit_uni_lrn_fwd_kernel_t(const nchw_across_t &J, float A, float K,
-                prop_kind_t pk, void *code_ptr, size_t code_size)
-    : Base(code_ptr, code_size)
+jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::jit_uni_lrn_fwd_kernel_t(
+        const nchw_across_t &J, float A, float K, prop_kind_t pk)
+    : Base(jit_name())
     , config_(lrn_config_t::nchw_across)
     , nchw_across_(J)
     , alpha_(A)
@@ -1315,7 +1261,7 @@ jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::
     , pk_(pk) {}
 
 template <>
-void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
+void jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>::generate(
         const nchw_across_t &J) {
 
     /* Load from within the memory boundary of 'src_' and apply a zero-mask to
@@ -1371,6 +1317,7 @@ void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
     size_t l_shift = 0;
 
     this->preamble();
+    if (this->emulate_bfloat_) this->io_.init_bf16();
 
     this->mov(src_, this->ptr[this->param1 + 0]);
     this->mov(dst_, this->ptr[this->param1 + 8]);
@@ -1484,9 +1431,8 @@ void jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>::generate(
 // backward kernel
 template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_bwd_kernel_t<isa, d_type>::jit_uni_lrn_bwd_kernel_t(
-        const nchw8c_across_t &J, float A, float B, int use_h_parallel,
-        void *code_ptr, size_t code_size)
-    : Base(code_ptr, code_size)
+        const nchw8c_across_t &J, float A, float B, int use_h_parallel)
+    : Base(jit_name())
     , config_(lrn_config_t::nchw8c_across)
     , nchw8c_across_(J)
     , nalphabeta_(-2 * A * B)
@@ -1515,6 +1461,7 @@ void jit_uni_lrn_bwd_kernel_t<isa, d_type>::generate(const nchw8c_across_t &J) {
     const Xbyak::Ymm &ydiffsrc = this->ymm15;
 
     this->preamble();
+    if (this->bf16_emu_) this->bf16_emu_->init_vcvtneps2bf16();
 
 #define GET_OFF(field) offsetof(jit_args_bwd_t, field)
     this->mov(src_, this->ptr[this->param1 + GET_OFF(src)]);
@@ -1621,18 +1568,19 @@ void jit_uni_lrn_bwd_kernel_t<isa, d_type>::generate(const nchw8c_across_t &J) {
 
 template <cpu_isa_t isa, data_type_t d_type>
 jit_uni_lrn_bwd_kernel_t<isa, d_type>::jit_uni_lrn_bwd_kernel_t(
-        const within_config_t &config, float A, float B, void *code_ptr,
-        size_t code_size)
-    : Base(config, code_ptr, code_size)
+        const within_config_t &config, float A, float B)
+    : Base(config, jit_name())
     , config_(lrn_config_t::within_config)
     , within_config_(config)
-    , nalphabeta_(-2.0f * A * B) {}
+    , nalphabeta_(-2.0f * A * B)
+    , use_h_parallelizm_(0) {}
 
 template <cpu_isa_t isa, data_type_t d_type>
 void jit_uni_lrn_bwd_kernel_t<isa, d_type>::generate(
         const within_config_t &config) {
 
     this->preamble();
+    if (this->bf16_emu_) this->bf16_emu_->init_vcvtneps2bf16();
 
 #define GET_OFF(field) offsetof(jit_args_bwd_t, field)
     this->mov(src_, this->ptr[this->param1 + GET_OFF(src)]);
@@ -1644,7 +1592,7 @@ void jit_uni_lrn_bwd_kernel_t<isa, d_type>::generate(
 #undef GET_OFF
     this->load_constant(nalphabeta_, vnalphabeta_, xnalphabeta_);
 
-    static const int max_reg_blocks = isa == avx512_common ? 3 : 1;
+    static const int max_reg_blocks = is_superset(isa, avx512_core) ? 4 : 2;
     this->within_loop(config, max_reg_blocks, prop_kind::backward);
 
     this->postamble();
@@ -1655,60 +1603,45 @@ void jit_uni_lrn_bwd_kernel_t<isa, d_type>::within_body(int hoff, int Hoff,
         int woff, int Woff, int stride, prop_kind_t pk, const int reg_block,
         int pixel_offset) {
 
-    static const std::array<Vmm, 3> vsum {{Vmm(1), Vmm(9), Vmm(18)}};
-    static const std::array<std::array<Vmm, 3>, 3> diff_dst {{
-            {{Vmm(2), Vmm(3), Vmm(6)}},
-            {{Vmm(10), Vmm(11), Vmm(23)}},
-            {{Vmm(19), Vmm(20), Vmm(26)}},
-    }};
-    static const std::array<std::array<Vmm, 3>, 3> ws1 {{
-            {{Vmm(4), Vmm(5), Vmm(15)}},
-            {{Vmm(12), Vmm(13), Vmm(27)}},
-            {{Vmm(21), Vmm(22), Vmm(28)}},
-    }};
-    static const std::array<Vmm, 3> ws0 = !this->emulate_bfloat_
-            ? std::array<Vmm, 3> {{Vmm(29), Vmm(30), Vmm(31)}}
-            : std::array<Vmm, 3> {{Vmm(6), Vmm(15), Vmm(23)}};
-    static const std::array<Vmm, 3> src {{Vmm(7), Vmm(16), Vmm(24)}};
-    static const std::array<Vmm, 3> a {{Vmm(8), Vmm(17), Vmm(25)}};
-
-    static const std::size_t used_tmp_regs
-            = this->emulate_bfloat_ ? ws1[0].size() - 1 : ws1[0].size();
+    static const std::array<Vmm, 4> vsum {{Vmm(1), Vmm(7), Vmm(13), Vmm(19)}};
+    static const std::array<Vmm, 4> diff_dst {
+            {Vmm(2), Vmm(8), Vmm(14), Vmm(20)}};
+    static const std::array<Vmm, 4> ws1 {{Vmm(3), Vmm(9), Vmm(15), Vmm(21)}};
+    static const std::array<Vmm, 4> ws0 {{Vmm(4), Vmm(10), Vmm(16), Vmm(22)}};
+    static const std::array<Vmm, 4> src {{Vmm(5), Vmm(11), Vmm(17), Vmm(23)}};
+    static const std::array<Vmm, 4> a {{Vmm(6), Vmm(12), Vmm(18), Vmm(24)}};
 
     IRB_LOOP(this->uni_vxorps(vsum[irb], vsum[irb], vsum[irb]));
     for (int i = hoff; i <= Hoff; ++i) {
         for (int j = woff; j <= Woff; ++j) {
-            const auto idx = this->tempIdx_ % used_tmp_regs;
-            IRB_LOOP(this->load_data(diff_dst[irb][idx],
+            IRB_LOOP(this->io_.at(d_type)->load(
                     this->ptr[(diffdst_ + pixel_offset + irb_off)
-                            + (i * stride + j) * this->single_pixel_offset_]));
-            IRB_LOOP(this->load_data(ws1[irb][idx],
+                            + (i * stride + j) * this->single_pixel_offset_],
+                    diff_dst[irb], false));
+            IRB_LOOP(this->io_.at(d_type)->load(
                     this->ptr[(bwd_intermediate_res_ + pixel_offset + irb_off)
-                            + (i * stride + j) * this->single_pixel_offset_]));
+                            + (i * stride + j) * this->single_pixel_offset_],
+                    ws1[irb], false));
 
             if (i == 0 && j == 0) {
-                if (d_type == dnnl::impl::data_type::bf16) {
-                    IRB_LOOP(this->load_data(ws0[irb],
-                            this->ptr[(scratch_ + pixel_offset + irb_off)]));
-                    IRB_LOOP(
-                            this->vdivps(a[irb], diff_dst[irb][idx], ws0[irb]));
+                if (utils::one_of(d_type, data_type::bf16, data_type::f16)) {
+                    IRB_LOOP(this->io_.at(d_type)->load(
+                            this->ptr[(scratch_ + pixel_offset + irb_off)],
+                            ws0[irb], false));
+                    IRB_LOOP(this->vdivps(a[irb], diff_dst[irb], ws0[irb]));
                 } else {
-                    IRB_LOOP(this->vdivps(a[irb], diff_dst[irb][idx],
+                    IRB_LOOP(this->vdivps(a[irb], diff_dst[irb],
                             this->ptr[(scratch_ + pixel_offset + irb_off)]));
                 }
             }
 
-            IRB_LOOP(this->vfmadd231ps(
-                    vsum[irb], ws1[irb][idx], diff_dst[irb][idx]));
-            ++(this->tempIdx_);
+            IRB_LOOP(this->vfmadd231ps(vsum[irb], ws1[irb], diff_dst[irb]));
         }
     }
 
-    this->tempIdx_ = this->tempIdx_ % used_tmp_regs;
-
-    if (d_type == dnnl::impl::data_type::bf16) {
-        IRB_LOOP(this->load_data(
-                src[irb], this->ptr[(src_ + pixel_offset + irb_off)]));
+    if (utils::one_of(d_type, data_type::bf16, data_type::f16)) {
+        IRB_LOOP(this->io_.at(d_type)->load(
+                this->ptr[(src_ + pixel_offset + irb_off)], src[irb], false));
         IRB_LOOP(this->vmulps(src[irb], this->vnalphabeta_, src[irb]));
     } else {
         IRB_LOOP(this->vmulps(src[irb], this->vnalphabeta_,
@@ -1717,11 +1650,8 @@ void jit_uni_lrn_bwd_kernel_t<isa, d_type>::within_body(int hoff, int Hoff,
 
     IRB_LOOP(this->vfmadd231ps(a[irb], src[irb], vsum[irb]));
 
-    IRB_LOOP(this->store_data(
-            this->ptr[diffsrc_ + pixel_offset + irb_off], a[irb]));
-
-    if (isa == avx512_common)
-        this->reg_block_idx_ = (this->reg_block_idx_ % vsum.size()) + 1;
+    IRB_LOOP(this->io_.at(d_type)->store(
+            a[irb], this->ptr[diffsrc_ + pixel_offset + irb_off], false));
 }
 
 template <cpu_isa_t isa, data_type_t d_type>
@@ -1735,34 +1665,42 @@ void jit_uni_lrn_bwd_kernel_t<isa, d_type>::move_data_pointers(
     this->add(bwd_intermediate_res_, pixel_offset);
 }
 
-template class jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>;
-template class jit_uni_lrn_fwd_kernel_t<avx2, dnnl::impl::data_type::f32>;
-template class jit_uni_lrn_fwd_kernel_t<avx512_common,
-        dnnl::impl::data_type::f32>;
-template class jit_uni_lrn_fwd_kernel_t<avx512_common,
-        dnnl::impl::data_type::bf16>;
+template class jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>;
+template class jit_uni_lrn_fwd_kernel_t<avx2, data_type::f32>;
+template class jit_uni_lrn_fwd_kernel_t<avx2_vnni_2, data_type::bf16>;
+template class jit_uni_lrn_fwd_kernel_t<avx2_vnni_2, data_type::f16>;
+template class jit_uni_lrn_fwd_kernel_t<avx512_core, data_type::f32>;
+template class jit_uni_lrn_fwd_kernel_t<avx512_core, data_type::bf16>;
+template class jit_uni_lrn_fwd_kernel_t<avx512_core_fp16, data_type::f16>;
 
 template class jit_uni_lrn_kernel_t<
-        jit_uni_lrn_fwd_kernel_t<sse41, dnnl::impl::data_type::f32>>;
+        jit_uni_lrn_fwd_kernel_t<sse41, data_type::f32>>;
 template class jit_uni_lrn_kernel_t<
-        jit_uni_lrn_fwd_kernel_t<avx2, dnnl::impl::data_type::f32>>;
+        jit_uni_lrn_fwd_kernel_t<avx2, data_type::f32>>;
 template class jit_uni_lrn_kernel_t<
-        jit_uni_lrn_fwd_kernel_t<avx512_common, dnnl::impl::data_type::f32>>;
+        jit_uni_lrn_fwd_kernel_t<avx2_vnni_2, data_type::bf16>>;
 template class jit_uni_lrn_kernel_t<
-        jit_uni_lrn_fwd_kernel_t<avx512_common, dnnl::impl::data_type::bf16>>;
+        jit_uni_lrn_fwd_kernel_t<avx2_vnni_2, data_type::f16>>;
+template class jit_uni_lrn_kernel_t<
+        jit_uni_lrn_fwd_kernel_t<avx512_core, data_type::f32>>;
+template class jit_uni_lrn_kernel_t<
+        jit_uni_lrn_fwd_kernel_t<avx512_core, data_type::bf16>>;
+template class jit_uni_lrn_kernel_t<
+        jit_uni_lrn_fwd_kernel_t<avx512_core_fp16, data_type::f16>>;
 
-template class jit_uni_lrn_bwd_kernel_t<avx512_common,
-        dnnl::impl::data_type::f32>;
-template class jit_uni_lrn_bwd_kernel_t<avx512_common,
-        dnnl::impl::data_type::bf16>;
-template class jit_uni_lrn_bwd_kernel_t<avx2, dnnl::impl::data_type::f32>;
+template class jit_uni_lrn_bwd_kernel_t<avx512_core_fp16, data_type::f16>;
+template class jit_uni_lrn_bwd_kernel_t<avx512_core, data_type::f32>;
+template class jit_uni_lrn_bwd_kernel_t<avx512_core, data_type::bf16>;
+template class jit_uni_lrn_bwd_kernel_t<avx2, data_type::f32>;
 
 template class jit_uni_lrn_kernel_t<
-        jit_uni_lrn_bwd_kernel_t<avx2, dnnl::impl::data_type::f32>>;
+        jit_uni_lrn_bwd_kernel_t<avx2, data_type::f32>>;
 template class jit_uni_lrn_kernel_t<
-        jit_uni_lrn_bwd_kernel_t<avx512_common, dnnl::impl::data_type::f32>>;
+        jit_uni_lrn_bwd_kernel_t<avx512_core, data_type::f32>>;
 template class jit_uni_lrn_kernel_t<
-        jit_uni_lrn_bwd_kernel_t<avx512_common, dnnl::impl::data_type::bf16>>;
+        jit_uni_lrn_bwd_kernel_t<avx512_core, data_type::bf16>>;
+template class jit_uni_lrn_kernel_t<
+        jit_uni_lrn_bwd_kernel_t<avx512_core_fp16, data_type::f16>>;
 
 } // namespace x64
 } // namespace cpu
