@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@
 #include "common/bfloat16.hpp"
 #include "common/c_types_map.hpp"
 #include "common/dnnl_thread.hpp"
+#include "common/math_utils.hpp"
 #include "common/type_helpers.hpp"
 
 #include "cpu/x64/jit_generator.hpp"
@@ -29,35 +30,55 @@ namespace impl {
 namespace cpu {
 namespace x64 {
 
+static bool impl_supports_datatype(cpu_isa_t isa, data_type_t data_type) {
+    switch (data_type) {
+        case data_type::bf16: return is_superset(isa, avx512_core);
+        case data_type::f16: return is_superset(isa, avx512_core_fp16);
+        case data_type::f32:
+        case data_type::s32:
+        case data_type::s8:
+        case data_type::u8: return true;
+        default: return false;
+    }
+}
+
 template <cpu_isa_t isa>
 status_t jit_uni_shuffle_t<isa>::pd_t::init(engine_t *engine) {
     using namespace format_tag;
     using namespace data_type;
 
-    conf_.data_type = data_md()->data_type;
+    const memory_desc_wrapper src_d(is_fwd() ? src_md() : diff_src_md());
+    const memory_desc_wrapper dst_d(is_fwd() ? dst_md() : diff_dst_md());
 
-    const bool ok = mayiuse(isa)
-            && utils::one_of(conf_.data_type, f32, s32, bf16)
-            && platform::has_data_type_support(conf_.data_type)
-            && attr()->has_default_values() && axis() == 1
-            && IMPLICATION(!is_fwd(), set_default_formats_common());
+    conf_.data_type = src_d.data_type();
 
-    if (!ok || (conf_.data_type == bf16 && isa != sse41))
-        return status::unimplemented;
+    // disabling verbose dispatch messages for unsupported isa for better readability
+    if (!mayiuse(isa)) return status::unimplemented;
 
-    if (isa != avx512_common)
-        conf_.isa = mayiuse(avx2) ? avx2 : isa;
-    else
-        conf_.isa = isa;
+    VDISPATCH_SHUFFLE(utils::one_of(conf_.data_type, f32, s32, bf16),
+            VERBOSE_UNSUPPORTED_DT);
+    VDISPATCH_SHUFFLE(src_d.data_type() == dst_d.data_type(),
+            VERBOSE_INCONSISTENT_DT, "src", "dst");
+    VDISPATCH_SHUFFLE(impl_supports_datatype(isa, conf_.data_type),
+            VERBOSE_ISA_DT_MISMATCH);
+    VDISPATCH_SHUFFLE(attr()->has_default_values(), VERBOSE_UNSUPPORTED_ATTR);
+    VDISPATCH_SHUFFLE(axis() == 1, VERBOSE_BAD_AXIS);
+    VDISPATCH_SHUFFLE(set_default_formats_common(), VERBOSE_UNSUPPORTED_TAG);
+    VDISPATCH_SHUFFLE(src_d == dst_d, VERBOSE_INCONSISTENT_MDS, "src", "dst");
+
+    conf_.isa = isa;
+    if (isa == avx) conf_.isa = mayiuse(avx2) ? avx2 : avx;
+    if (conf_.data_type == bf16)
+        conf_.isa = mayiuse(avx512_core_bf16) ? avx512_core_bf16 : avx512_core;
 
     const format_tag_t blocked_format
-            = memory_desc_matches_one_of_tag(*data_md(), nCw16c, nChw16c,
+            = memory_desc_matches_one_of_tag(*src_d.md_, nCw16c, nChw16c,
                     nCdhw16c, nCw8c, nChw8c, nCdhw8c, nCw4c, nChw4c, nCdhw4c);
 
-    if (blocked_format == format_tag::undef) return status::unimplemented;
+    VDISPATCH_SHUFFLE(
+            blocked_format != format_tag::undef, VERBOSE_UNSUPPORTED_TAG);
 
-    const memory_desc_wrapper data_d(data_md());
-    conf_.blk_size = data_d.blocking_desc().strides[ndims() - 1];
+    conf_.blk_size = src_d.blocking_desc().strides[ndims() - 1];
     conf_.simd_w = cpu_isa_traits<isa>::vlen / sizeof(float);
 
     const bool has_spatial = utils::one_of(ndims(), 3, 4, 5);
@@ -69,8 +90,9 @@ status_t jit_uni_shuffle_t<isa>::pd_t::init(engine_t *engine) {
         conf_.simd_tail = C() % conf_.simd_w;
         conf_.c_split_size = conf_.blk_size;
         if (C() < std::sqrt(conf_.sp))
-            conf_.sp_split_size
-                    = conf_.sp / math::gcd(conf_.sp, dnnl_get_max_threads());
+            conf_.sp_split_size = conf_.sp
+                    / math::gcd(conf_.sp,
+                            static_cast<dim_t>(dnnl_get_max_threads()));
         else
             conf_.sp_split_size = conf_.sp;
     } else
@@ -84,7 +106,7 @@ status_t jit_uni_shuffle_t<isa>::pd_t::init(engine_t *engine) {
     conf_.w = W();
 
     conf_.dt_size = types::data_type_size(conf_.data_type);
-    conf_.stride_mb = data_d.blocking_desc().strides[0];
+    conf_.stride_mb = src_d.blocking_desc().strides[0];
     conf_.group_size = group_size();
     conf_.axis = axis();
     conf_.axis_size = axis_size();
@@ -105,7 +127,7 @@ status_t jit_uni_shuffle_t<isa>::precompute_offsets() {
     std::vector<int> rev_transposed_(axis_size);
 
     // Precompute transposed axis helper array
-    parallel_nd(transpose_col, transpose_row, [&](int i, int j) {
+    parallel_nd(transpose_col, transpose_row, [&](dim_t i, dim_t j) {
         rev_transposed_[j * transpose_col + i] = i * transpose_row + j;
     });
 
@@ -120,7 +142,7 @@ status_t jit_uni_shuffle_t<isa>::precompute_offsets() {
         const dim_t SP = conf.sp;
 
         // Precompute input offsets using transposed axis
-        parallel_nd(CB, [&](int cb) {
+        parallel_nd(CB, [&](dim_t cb) {
             const int blk_end = nstl::min(blk_size, C - cb * blk_size);
             PRAGMA_OMP_SIMD()
             for (int cc = 0; cc < blk_end; ++cc) {
@@ -162,13 +184,10 @@ status_t jit_uni_shuffle_t<isa>::execute(const exec_ctx_t &ctx) const {
     using namespace prop_kind;
     using namespace utils;
 
-    status_t status = status::success;
-
     const auto i_arg = pd()->is_fwd() ? DNNL_ARG_SRC : DNNL_ARG_DIFF_DST;
     const auto o_arg = pd()->is_fwd() ? DNNL_ARG_DST : DNNL_ARG_DIFF_SRC;
     auto input = CTX_IN_MEM(const uint8_t *, i_arg);
-    auto output = CTX_OUT_CLEAN_MEM(uint8_t *, o_arg, status);
-    CHECK(status);
+    auto output = CTX_OUT_MEM(uint8_t *, o_arg);
 
     const auto conf = pd()->get_conf();
 
@@ -194,6 +213,7 @@ status_t jit_uni_shuffle_t<isa>::execute(const exec_ctx_t &ctx) const {
             args.dst = output + (off + SP * c_curr) * data_type_size;
 
             args.cb_loop_size = c_work;
+            args.is_padded_block = cb + 1 == CB;
 
             args.input_off_ptr = this->input_off_ + c_curr;
             (*kernel_)(&args);
@@ -208,7 +228,7 @@ status_t jit_uni_shuffle_t<isa>::execute(const exec_ctx_t &ctx) const {
 
 template struct jit_uni_shuffle_t<sse41>;
 template struct jit_uni_shuffle_t<avx>;
-template struct jit_uni_shuffle_t<avx512_common>;
+template struct jit_uni_shuffle_t<avx512_core>;
 
 } // namespace x64
 } // namespace cpu
