@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2021 Intel Corporation
+* Copyright 2017-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,16 +18,20 @@
 #include <limits.h>
 #include <stdint.h>
 
+#include <algorithm>
+#include <cctype>
 #include <fstream>
 #include <functional>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "dnnl.h"
+#include "oneapi/dnnl/dnnl.h"
 
 #include "common.hpp"
 
+#include "utils/parallel.hpp"
+
 // BENCHDNN_MEMORY_CHECK macro enables guarding mechanism for memory allocation:
 // memory block is allocated on a page boundary and the page after the block is
 // protected to catch possible invalid accesses.
@@ -43,112 +47,6 @@
 #include <sys/mman.h>
 #endif
 
-const char *bench_mode2str(bench_mode_t mode) {
-    const char *modes[] = {"MODE_UNDEF", "CORR", "PERF", "CORR+PERF", "LIST"};
-    assert((int)mode < sizeof(modes) / sizeof(*modes));
-    return modes[(int)mode];
-}
-
-bench_mode_t str2bench_mode(const char *str) {
-    bench_mode_t mode = MODE_UNDEF;
-    if (strchr(str, 'c') || strchr(str, 'C'))
-        mode = (bench_mode_t)((int)mode | (int)CORR);
-    if (strchr(str, 'p') || strchr(str, 'P'))
-        mode = (bench_mode_t)((int)mode | (int)PERF);
-    if (strchr(str, 'l') || strchr(str, 'L')) {
-        // list mode is exclusive
-        assert(mode == MODE_UNDEF);
-        mode = (bench_mode_t)((int)mode | (int)LIST);
-    }
-    if (mode == MODE_UNDEF)
-        []() {
-            SAFE(FAIL, CRIT);
-            return 0;
-        }();
-    return mode;
-}
-
-/* perf */
-#include <chrono>
-
-static inline double ms_now() {
-    auto timePointTmp
-            = std::chrono::high_resolution_clock::now().time_since_epoch();
-    return std::chrono::duration<double, std::milli>(timePointTmp).count();
-}
-
-#if !defined(BENCHDNN_USE_RDPMC) || defined(_WIN32)
-unsigned long long ticks_now() {
-    return (unsigned long long)0;
-}
-#else
-unsigned long long ticks_now() {
-    unsigned eax, edx, ecx;
-
-    ecx = (1 << 30) + 1;
-    __asm__ volatile("rdpmc" : "=a"(eax), "=d"(edx) : "c"(ecx));
-
-    return (unsigned long long)eax | (unsigned long long)edx << 32;
-}
-#endif
-
-void benchdnn_timer_t::reset() {
-    times_ = 0;
-    for (int i = 0; i < n_modes; ++i)
-        ticks_[i] = 0;
-    ticks_start_ = 0;
-    for (int i = 0; i < n_modes; ++i)
-        ms_[i] = 0;
-    ms_start_ = 0;
-
-    start();
-}
-
-void benchdnn_timer_t::start() {
-    ticks_start_ = ticks_now();
-    ms_start_ = ms_now();
-}
-
-void benchdnn_timer_t::stop(int add_times) {
-    if (add_times == 0) return;
-
-    unsigned long long d_ticks = ticks_now() - ticks_start_;
-    double d_ms = ms_now() - ms_start_;
-
-    ticks_start_ += d_ticks;
-    ms_start_ += d_ms;
-
-    ms_[benchdnn_timer_t::avg] += d_ms;
-    ticks_[benchdnn_timer_t::avg] += d_ticks;
-
-    d_ticks /= add_times;
-    d_ms /= add_times;
-
-    ms_[benchdnn_timer_t::min]
-            = times_ ? MIN2(ms_[benchdnn_timer_t::min], d_ms) : d_ms;
-    ms_[benchdnn_timer_t::max]
-            = times_ ? MAX2(ms_[benchdnn_timer_t::max], d_ms) : d_ms;
-
-    ticks_[benchdnn_timer_t::min]
-            = times_ ? MIN2(ticks_[benchdnn_timer_t::min], d_ticks) : d_ticks;
-    ticks_[benchdnn_timer_t::max]
-            = times_ ? MAX2(ticks_[benchdnn_timer_t::max], d_ticks) : d_ticks;
-
-    times_ += add_times;
-}
-
-benchdnn_timer_t &benchdnn_timer_t::operator=(const benchdnn_timer_t &rhs) {
-    if (this == &rhs) return *this;
-    times_ = rhs.times_;
-    for (int i = 0; i < n_modes; ++i)
-        ticks_[i] = rhs.ticks_[i];
-    ticks_start_ = rhs.ticks_start_;
-    for (int i = 0; i < n_modes; ++i)
-        ms_[i] = rhs.ms_[i];
-    ms_start_ = rhs.ms_start_;
-    return *this;
-}
-
 /* result structure */
 const char *state2str(res_state_t state) {
     if (state == UNTESTED) return "UNTESTED_FAILED"; // for easier fail search
@@ -159,93 +57,129 @@ const char *state2str(res_state_t state) {
     CASE(SKIPPED);
     CASE(MISTRUSTED);
     CASE(UNIMPLEMENTED);
+    CASE(INVALID_ARGUMENTS);
     CASE(FAILED);
     CASE(LISTED);
+    CASE(EXECUTED);
+    CASE(INITIALIZED);
 #undef CASE
     assert(!"unknown res state");
     return "STATE_UNDEF";
 }
 
-const char *skip_reason2str(skip_reason_t skip_reason) {
+namespace skip_reason {
+std::string case_not_supported("Case not supported");
+std::string data_type_not_supported("Data type not supported");
+std::string invalid_case("Invalid case");
+std::string not_enough_ram("Not enough RAM");
+std::string skip_impl_hit("Skip-impl option hit");
+std::string skip_start("Skip-start option hit");
+} // namespace skip_reason
+
+dir_t str2dir(const char *str) {
 #define CASE(x) \
-    if (skip_reason == (x)) return STRINGIFY(x)
-    CASE(CASE_NOT_SUPPORTED);
-    CASE(DATA_TYPE_NOT_SUPPORTED);
-    CASE(INVALID_CASE);
-    CASE(KNOWN_LIMITATION);
-    CASE(NOT_ENOUGH_RAM);
-    CASE(SKIP_IMPL_HIT);
-    CASE(SKIP_START);
+    if (!strcasecmp(STRINGIFY(x), str)) return x
+    CASE(FWD_D);
+    CASE(FWD_I);
+    CASE(FWD_B);
+    CASE(BWD_D);
+    CASE(BWD_W);
+    CASE(BWD_WB);
+    CASE(BWD_DW);
 #undef CASE
-    return "SKIP_UNKNOWN";
+    assert(!"unknown dir");
+    return DIR_UNDEF;
 }
 
-void parse_result(
-        res_t &res, bool &want_perf_report, int status, const char *pstr) {
+void parse_result(res_t &res, const char *pstr) {
     auto &bs = benchdnn_stat;
     const char *state = state2str(res.state);
 
     switch (res.state) {
         case UNTESTED:
-            if (!(bench_mode & CORR)) {
-                want_perf_report = true;
-                if (status == FAIL)
-                    bs.failed++;
-                else
-                    bs.passed++;
-                break;
-            }
-        case FAILED:
-            assert(status == FAIL);
+            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
             bs.failed++;
-            BENCHDNN_PRINT(0, "%d:%s (errors:%lu total:%lu) __REPRO: %s\n",
-                    bs.tests, state, (unsigned long)res.errors,
-                    (unsigned long)res.total, pstr);
             break;
+        case EXECUTED:
+            if (bench_mode == bench_mode_t::exec)
+                BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            bs.passed++;
+            break;
+        case FAILED: {
+            bs.failed++;
+            std::string error_stat;
+            if (res.errors > 0) {
+                error_stat = " (errors:" + std::to_string(res.errors)
+                        + " total:" + std::to_string(res.total) + ")";
+            }
+
+            std::string reason;
+            if (!res.reason.empty()) { reason = " (" + res.reason + ")"; }
+            BENCHDNN_PRINT(0, "%d:%s%s%s __REPRO: %s\n", bs.tests, state,
+                    reason.c_str(), error_stat.c_str(), pstr);
+        } break;
         case SKIPPED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s (%s) __REPRO: %s\n", bs.tests, state,
-                    skip_reason2str(res.reason), pstr);
+                    res.reason.c_str(), pstr);
             bs.skipped++;
             break;
         case UNIMPLEMENTED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
             bs.unimplemented++;
             bs.failed++;
             break;
-        case MISTRUSTED:
-            assert(status == OK);
-            bs.mistrusted++;
+        case INVALID_ARGUMENTS:
             BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            // bs.failed++; /* temporal workaround for some tests */
+            bs.invalid_arguments++;
+            bs.failed++;
+            break;
+        case MISTRUSTED:
+            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            bs.mistrusted++;
             break;
         case PASSED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            want_perf_report = true;
             bs.passed++;
             break;
         case LISTED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            want_perf_report = false;
             bs.listed++;
             break;
-        default:
-            assert(!"unknown state");
-            {
-                []() {
-                    SAFE(FAIL, CRIT);
-                    return 0;
-                }();
+        case INITIALIZED:
+            // TODO: workaround for failed fill functions.
+            if (bench_mode != bench_mode_t::init) {
+                state = "FAILED";
+                bs.failed++;
+            } else {
+                bs.passed++;
             }
+
+            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            break;
+        default: assert(!"unknown state"); SAFE_V(FAIL);
     }
 
-    if (bench_mode & PERF) {
-        using bt = benchdnn_timer_t;
+    bs.tests++;
+    assert(bs.tests
+            == bs.passed + bs.skipped + bs.mistrusted + bs.failed + bs.listed);
+
+    using bt = timer::timer_t;
+    using namespace timer::names;
+
+    if (has_bench_mode_bit(mode_bit_t::perf)) {
+        const auto &t = res.timer_map.perf_timer();
         for (int mode = 0; mode < (int)bt::n_modes; ++mode)
-            bs.ms[mode] += res.timer.ms((bt::mode_t)mode);
+            bs.ms[perf_timer][mode] += t.ms((bt::mode_t)mode);
+    }
+
+    for (const auto &e : timer::get_global_service_timers()) {
+        const auto &supported_mode_bit = std::get<1>(e);
+        if (!has_bench_mode_bit(supported_mode_bit)) continue;
+
+        const auto &t_name = std::get<2>(e);
+        const auto &t = res.timer_map.get_timer(t_name);
+        // Only summary time is populated to the highest level report.
+        bs.ms[t_name][bt::mode_t::sum] += t.sec(bt::mode_t::sum);
     }
 }
 
@@ -255,8 +189,8 @@ void parse_result(
 static void *zmalloc_protect(size_t size) {
     const size_t page_sz = getpagesize();
 
-    const size_t block_sz = size + 2 * sizeof(void *);
-    const size_t total_sz = div_up(block_sz, page_sz) * page_sz + page_sz;
+    const size_t block_sz = size + 3 * sizeof(void *);
+    const size_t total_sz = rnd_up(block_sz, page_sz) + page_sz;
 
     void *mem_ptr;
     int rc = ::posix_memalign(&mem_ptr, page_sz, total_sz);
@@ -282,29 +216,36 @@ static void *zmalloc_protect(size_t size) {
         return nullptr;
     }
 
+    // Align down `ptr` on 8 bytes before storing addresses to make behavior
+    // defined.
+    ptrdiff_t to_align = reinterpret_cast<ptrdiff_t>(ptr) % sizeof(void *);
+    void *ptr_aligned_8 = ptr - to_align;
     // Save pointers for zfree_protect
-    ((void **)ptr)[-2] = ptr_start;
-    ((void **)ptr)[-1] = ptr_protect;
+    ((void **)ptr_aligned_8)[-2] = ptr_start;
+    ((void **)ptr_aligned_8)[-1] = ptr_protect;
 
     return ptr;
 }
 
 static void zfree_protect(void *ptr) {
-    const size_t page_sz = getpagesize();
+    // Get aligned ptr before obtaining addresses
+    ptrdiff_t to_align = reinterpret_cast<ptrdiff_t>(ptr) % sizeof(void *);
+    void *ptr_aligned_8 = reinterpret_cast<uint8_t *>(ptr) - to_align;
 
     // Restore read-write access for the protected region
-    void *ptr_protect = ((void **)ptr)[-1];
+    void *ptr_protect = ((void **)ptr_aligned_8)[-1];
+    const size_t page_sz = getpagesize();
     mprotect(ptr_protect, page_sz, PROT_READ | PROT_WRITE);
 
     // Deallocate the whole region
-    void *ptr_start = ((void **)ptr)[-2];
+    void *ptr_start = ((void **)ptr_aligned_8)[-2];
     ::free(ptr_start);
 }
 #endif
 
 void *zmalloc(size_t size, size_t align) {
 #ifdef BENCHDNN_MEMORY_CHECK
-    if (bench_mode & CORR) { return zmalloc_protect(size); }
+    if (has_bench_mode_bit(mode_bit_t::corr)) { return zmalloc_protect(size); }
 #endif
 
     void *ptr;
@@ -319,15 +260,17 @@ void *zmalloc(size_t size, size_t align) {
 
     // TODO. Heuristics: Increasing the size to alignment increases
     // the stability of performance results.
-    if ((bench_mode & PERF) && (size < align)) size = align;
+    if (has_bench_mode_bit(mode_bit_t::perf) && (size < align)) size = align;
     int rc = ::posix_memalign(&ptr, align, size);
 #endif /* _WIN32 */
     return rc == 0 ? ptr : nullptr;
 }
 
+// zfree behavior is aligned with UNIX free().
 void zfree(void *ptr) {
+    if (!ptr) return;
 #ifdef BENCHDNN_MEMORY_CHECK
-    if (bench_mode & CORR) {
+    if (has_bench_mode_bit(mode_bit_t::corr)) {
         zfree_protect(ptr);
         return;
     }
@@ -379,31 +322,16 @@ bool match_regex(const char *str, const char *pattern) {
 }
 #endif /* _WIN32 */
 
-bool maybe_skip(const std::string &impl_str) {
-    if (skip_impl.empty()) return false;
-
-    size_t start_pos = 0, end_pos = 0;
-    if (skip_impl[0] == '"' || skip_impl[0] == '\'') start_pos++;
-
-    do {
-        const size_t delim_pos = skip_impl.find_first_of(':', start_pos);
-        // rows below to identify quotes at the end and deal with them
-        end_pos = MIN2(skip_impl.size(), delim_pos);
-        size_t len = end_pos - start_pos;
-        if (skip_impl[end_pos - 1] == '"' || skip_impl[end_pos - 1] == '\'')
-            len--;
-        std::string sub_skip_impl = skip_impl.substr(start_pos, len);
-        // even incomplete match leads to skipping
-        if (!sub_skip_impl.empty()
-                && impl_str.find(sub_skip_impl) != std::string::npos)
-            return true;
-        start_pos = end_pos + 1;
-    } while (end_pos < skip_impl.size());
-
+bool skip_start(res_t *res, int idx) {
+    if (idx < test_start) {
+        res->state = SKIPPED;
+        res->reason = skip_reason::skip_start;
+        return true;
+    }
     return false;
 }
 
-#if defined(_WIN32) && !defined(__GNUC__)
+#if defined(_WIN32)
 #include <windows.h>
 #define PATH_MAX MAX_PATH
 static char *dirname(char *path) {
@@ -433,10 +361,10 @@ int readlink(const char *path, char *buf, size_t buf_max) {
 #include <unistd.h>
 #endif /* _WIN32 */
 
-std::string locate_batch_file(const std::string &fname) {
+std::string locate_file(const std::string &fname) {
     SAFE_V(fname.length() < PATH_MAX ? OK : FAIL);
 
-    const int max_paths = 4;
+    const int max_paths = 30;
 
     static int n_paths = 0;
     static std::string search_paths[max_paths];
@@ -456,20 +384,26 @@ std::string locate_batch_file(const std::string &fname) {
             break;
         }
     if (!dir_found) {
-        SAFE_V(n_paths < max_paths ? OK : FAIL);
-        search_paths[n_paths++] = std::move(fdir);
+        if (n_paths >= max_paths) {
+            BENCHDNN_PRINT(0, "%s%d\n",
+                    "Warning: Number of searched paths exceeded ", max_paths);
+        } else {
+            search_paths[n_paths++] = std::move(fdir);
+        }
     }
 
     std::ifstream ifs(fname);
     if (ifs.is_open()) return fname;
 
     for (int n = 0; n < n_paths; ++n) {
-        const std::string fullname = search_paths[n] + "/" + fname;
+        std::string fullname = search_paths[n] + "/" + fname;
         ifs.open(fullname);
         if (ifs.is_open()) {
-            BENCHDNN_PRINT(50, "batch file used: %s\n", fullname.c_str());
+            BENCHDNN_PRINT(50, "file used: %s\n", fullname.c_str());
+            ifs.close();
             return fullname;
         }
+        ifs.close();
     }
 
     // Search in default inputs directory
@@ -478,21 +412,23 @@ std::string locate_batch_file(const std::string &fname) {
     fdir.resize(PATH_MAX);
     int length = readlink("/proc/self/exe", &fdir[0], PATH_MAX);
     if (length) {
-        std::string s_fdir = std::string(dirname(&fdir[0]));
+        std::string s_fdir = dirname(&fdir[0]);
         for (int i_try = 0; i_try < 2; ++i_try) {
-            fdir = s_fdir + std::string("/inputs/") + std::string(driver_name);
-            if (i_try == 1) { // Windows has different folder structure
-                fdir = s_fdir + std::string("/../inputs/")
-                        + std::string(driver_name);
-            }
-            // NOLINTNEXTLINE(performance-inefficient-string-concatenation)
-            const std::string fullname = fdir + "/" + fname;
+            fdir = s_fdir;
+            fdir.append(i_try == 1 ? "/../inputs/" : "/inputs/");
+            assert(!driver_name.empty());
+            fdir.append(driver_name);
+            std::string fullname = fdir + "/";
+            fullname += fname;
             ifs.open(fullname);
             if (ifs.is_open()) {
-                search_paths[n_paths++] = std::move(fdir);
-                BENCHDNN_PRINT(50, "batch file used: %s\n", fullname.c_str());
+                if (n_paths < max_paths)
+                    search_paths[n_paths++] = std::move(fdir);
+                BENCHDNN_PRINT(50, "file used: %s\n", fullname.c_str());
+                ifs.close();
                 return fullname;
             }
+            ifs.close();
         }
     }
 
@@ -501,8 +437,8 @@ std::string locate_batch_file(const std::string &fname) {
 }
 
 int batch(const char *fname, bench_f bench) {
-    std::ifstream ifs(locate_batch_file(std::string(fname)));
-    SAFE_V(ifs.is_open() ? OK : FAIL);
+    std::ifstream ifs(locate_file(std::string(fname)));
+    SAFE(ifs.is_open() ? OK : FAIL, CRIT);
 
     std::vector<std::string> opts;
     std::string str;
@@ -512,12 +448,15 @@ int batch(const char *fname, bench_f bench) {
 
         // shell style comments
         if (str.front() == '#') {
-            std::getline(ifs, str); // take whole commented line out
+            std::string dummy;
+            std::getline(ifs, dummy); // take whole commented line out
             continue;
         }
 
         // shell style line break
         if (continued_line) {
+            if (opts.empty()) SAFE_V(FAIL);
+            if (opts.back().size() + str.size() >= str.max_size()) SAFE_V(FAIL);
             // NOLINTNEXTLINE(performance-inefficient-string-concatenation)
             str = opts.back() + str; // update current line with previous
             opts.pop_back(); // take previous line out
@@ -554,6 +493,21 @@ int64_t div_up(const int64_t a, const int64_t b) {
     return (a + b - 1) / b;
 }
 
+size_t div_up(const size_t a, const size_t b) {
+    SAFE_V(b != 0 ? OK : FAIL);
+    return (a + b - 1) / b;
+}
+
+int64_t rnd_up(const int64_t a, const int64_t b) {
+    SAFE_V(b != 0 ? OK : FAIL);
+    return div_up(a, b) * b;
+}
+
+size_t rnd_up(const size_t a, const size_t b) {
+    SAFE_V(b != 0 ? OK : FAIL);
+    return div_up(a, b) * b;
+}
+
 int64_t next_pow2(int64_t a) {
     assert(a > 0 && a <= ((int64_t)1 << 62));
     if (a > 1) a--;
@@ -589,6 +543,7 @@ void gemm(const char *layout, const char *transa, const char *transb, int64_t m,
         int64_t n, int64_t k, const float alpha, const float *a,
         const int64_t lda, const float *b, const int64_t ldb, const float beta,
         float *c, const int64_t ldc) {
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
     if (*layout == 'C') {
         dnnl_sgemm(
                 *transa, *transb, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
@@ -596,12 +551,37 @@ void gemm(const char *layout, const char *transa, const char *transb, int64_t m,
         dnnl_sgemm(
                 *transb, *transa, n, m, k, alpha, b, ldb, a, lda, beta, c, ldc);
     }
+#else
+    if (std::toupper(*layout) != 'C') {
+        gemm("C", transb, transa, n, m, k, alpha, b, ldb, a, lda, beta, c, ldc);
+        return;
+    }
+
+    auto a_accessor = [&](int64_t i, int64_t j) {
+        if (std::toupper(*transa) == 'N') return a[i * lda + j];
+        return a[j * lda + i];
+    };
+
+    auto b_accessor = [&](int64_t i, int64_t j) {
+        if (std::toupper(*transb) == 'N') return b[i * ldb + j];
+        return b[j * ldb + i];
+    };
+
+    benchdnn_parallel_nd(m, n, [&](int64_t i, int64_t j) {
+        float ab = 0.0f;
+        for (int64_t _k = 0; _k < k; ++_k)
+            ab += a_accessor(i, _k) * b_accessor(_k, j);
+        float cij = (beta == 0) ? 0.0f : beta * c[i * ldc + j];
+        c[i * ldc + j] = alpha * ab + cij;
+    });
+#endif
 }
 
 int sanitize_desc(int &ndims, std::vector<std::reference_wrapper<int64_t>> d,
         std::vector<std::reference_wrapper<int64_t>> h,
         std::vector<std::reference_wrapper<int64_t>> w,
-        const std::vector<int64_t> &def_values, bool must_have_spatial) {
+        const std::vector<int64_t> &def_values, const char *str,
+        bool must_have_spatial) {
     size_t N = d.size();
     assert(h.size() == N && w.size() == N && def_values.size() == N);
 
@@ -615,7 +595,13 @@ int sanitize_desc(int &ndims, std::vector<std::reference_wrapper<int64_t>> d,
     if (no_d) ndims--;
     if (no_d && no_h) ndims--;
     if (no_d && no_h && no_w) ndims--;
-    if (must_have_spatial && ndims <= 2) return FAIL;
+    if (must_have_spatial && ndims <= 2) {
+        BENCHDNN_PRINT(0,
+                "ERROR: the problem must have at least one spatial dimension "
+                "specified. Full descriptor input: `%s`.\n",
+                str);
+        return FAIL;
+    }
 
     if (ndims == 5) {
         if (no_h && no_w) {
@@ -626,7 +612,11 @@ int sanitize_desc(int &ndims, std::vector<std::reference_wrapper<int64_t>> d,
         } else if (!no_h && !no_w) {
             // User specified them all, good to go.
         } else {
-            // Problem is not cubic and one of h or w dimension is missing.
+            BENCHDNN_PRINT(0,
+                    "ERROR: the problem requires either all `h` and `w` "
+                    "dimensions specified or none of them. Full descriptor "
+                    "input: `%s`.\n",
+                    str);
             return FAIL;
         }
     } else if (ndims == 4 && no_w) {
@@ -662,3 +652,63 @@ void print_dhw(bool &print_d, bool &print_h, bool &print_w, int ndims,
     print_w = ndims == 3 || (ndims == 5 && (!cubic_shape || canonical))
             || (ndims == 4 && (!square_shape || canonical));
 }
+
+// Copied from utils::getenv.
+// An underlined unified implementation for getting env var value.
+int getenv(const char *name, char *buffer, int buffer_size) {
+    if (name == nullptr || buffer_size < 0
+            || (buffer == nullptr && buffer_size > 0))
+        return INT_MIN;
+
+    int result = 0;
+    int term_zero_idx = 0;
+    size_t value_length = 0;
+
+#ifdef _WIN32
+    value_length = GetEnvironmentVariable(name, buffer, buffer_size);
+#else
+    const char *value = ::getenv(name);
+    value_length = value == nullptr ? 0 : strlen(value);
+#endif
+
+    if (value_length > INT_MAX)
+        result = INT_MIN;
+    else {
+        int int_value_length = (int)value_length;
+        if (int_value_length >= buffer_size) {
+            result = -int_value_length;
+        } else {
+            term_zero_idx = int_value_length;
+            result = int_value_length;
+#ifndef _WIN32
+            if (value) strncpy(buffer, value, buffer_size - 1);
+#endif
+        }
+    }
+
+    if (buffer != nullptr) buffer[term_zero_idx] = '\0';
+    return result;
+}
+
+// Copied from utils::getenv_int_user.
+// Collects an integer value from an env var.
+int benchdnn_getenv_int(const char *name, int default_value) {
+    int value = default_value;
+    // # of digits in the longest 32-bit signed int + sign + terminating null
+    const int len = 12;
+    char value_str[len];
+    if (getenv(name, value_str, len) > 0) { value = atoi(value_str); }
+    return value;
+}
+
+// Copied from utils::getenv_string_user.
+// Collects a string lower case value from an env var.
+std::string benchdnn_getenv_string(const char *name) {
+    // Random number to fit possible string input.
+    std::string value;
+    const int len = 128;
+    char value_str[len];
+    if (getenv(name, value_str, len) > 0) { value = value_str; }
+    std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+    return value;
+}
