@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -40,40 +40,48 @@ typedef enum {
 
 typedef enum { action_copy = 0, action_sum, action_concat } rnn_action_t;
 
-dnnl_status_t init_rnn_fwd_desc(dnnl_rnn_desc_t *rd, const prb_t &prb,
-        dnnl_prop_kind_t prop_kind, const dnnl_memory_desc_t *src_layer_d,
-        const dnnl_memory_desc_t *src_iter_d,
-        const dnnl_memory_desc_t *src_iter_c_d,
-        const dnnl_memory_desc_t *weights_layer_d,
-        const dnnl_memory_desc_t *weights_iter_d,
-        const dnnl_memory_desc_t *weights_peephole_d,
-        const dnnl_memory_desc_t *weights_projection_d,
-        const dnnl_memory_desc_t *bias_d, const dnnl_memory_desc_t *dst_layer_d,
-        const dnnl_memory_desc_t *dst_iter_d,
-        const dnnl_memory_desc_t *dst_iter_c_d);
+dnnl_status_t init_rnn_fwd_pd(dnnl_primitive_desc_t *pd, dnnl_engine_t engine,
+        const prb_t &prb, dnnl_prop_kind_t prop_kind,
+        const_dnnl_memory_desc_t src_layer_d,
+        const_dnnl_memory_desc_t src_iter_d,
+        const_dnnl_memory_desc_t src_iter_c_d,
+        const_dnnl_memory_desc_t attention_d,
+        const_dnnl_memory_desc_t weights_layer_d,
+        const_dnnl_memory_desc_t weights_iter_d,
+        const_dnnl_memory_desc_t weights_peephole_d,
+        const_dnnl_memory_desc_t weights_projection_d,
+        const_dnnl_memory_desc_t bias_d, const_dnnl_memory_desc_t dst_layer_d,
+        const_dnnl_memory_desc_t dst_iter_d,
+        const_dnnl_memory_desc_t dst_iter_c_d, dnnl_primitive_attr_t attr,
+        res_t *res);
 
-dnnl_status_t init_rnn_bwd_desc(dnnl_rnn_desc_t *rd, const prb_t &prb,
-        dnnl_prop_kind_t prop_kind, const dnnl_memory_desc_t *src_layer_d,
-        const dnnl_memory_desc_t *src_iter_d,
-        const dnnl_memory_desc_t *src_iter_c_d,
-        const dnnl_memory_desc_t *weights_layer_d,
-        const dnnl_memory_desc_t *weights_iter_d,
-        const dnnl_memory_desc_t *weights_peephole_d,
-        const dnnl_memory_desc_t *weights_projection_d,
-        const dnnl_memory_desc_t *bias_d, const dnnl_memory_desc_t *dst_layer_d,
-        const dnnl_memory_desc_t *dst_iter_d,
-        const dnnl_memory_desc_t *dst_iter_c_d,
-        const dnnl_memory_desc_t *diff_src_layer_d,
-        const dnnl_memory_desc_t *diff_src_iter_d,
-        const dnnl_memory_desc_t *diff_src_iter_c_d,
-        const dnnl_memory_desc_t *diff_weights_layer_d,
-        const dnnl_memory_desc_t *diff_weights_iter_d,
-        const dnnl_memory_desc_t *diff_weights_peephole_d,
-        const dnnl_memory_desc_t *diff_weights_projection_d,
-        const dnnl_memory_desc_t *diff_bias_d,
-        const dnnl_memory_desc_t *diff_dst_layer_d,
-        const dnnl_memory_desc_t *diff_dst_iter_d,
-        const dnnl_memory_desc_t *diff_dst_iter_c_d);
+dnnl_status_t init_rnn_bwd_pd(dnnl_primitive_desc_t *pd, dnnl_engine_t engine,
+        const prb_t &prb, dnnl_prop_kind_t prop_kind,
+        const_dnnl_memory_desc_t src_layer_d,
+        const_dnnl_memory_desc_t src_iter_d,
+        const_dnnl_memory_desc_t src_iter_c_d,
+        const_dnnl_memory_desc_t attention_d,
+        const_dnnl_memory_desc_t weights_layer_d,
+        const_dnnl_memory_desc_t weights_iter_d,
+        const_dnnl_memory_desc_t weights_peephole_d,
+        const_dnnl_memory_desc_t weights_projection_d,
+        const_dnnl_memory_desc_t bias_d, const_dnnl_memory_desc_t dst_layer_d,
+        const_dnnl_memory_desc_t dst_iter_d,
+        const_dnnl_memory_desc_t dst_iter_c_d,
+        const_dnnl_memory_desc_t diff_src_layer_d,
+        const_dnnl_memory_desc_t diff_src_iter_d,
+        const_dnnl_memory_desc_t diff_src_iter_c_d,
+        const_dnnl_memory_desc_t diff_attention_d,
+        const_dnnl_memory_desc_t diff_weights_layer_d,
+        const_dnnl_memory_desc_t diff_weights_iter_d,
+        const_dnnl_memory_desc_t diff_weights_peephole_d,
+        const_dnnl_memory_desc_t diff_weights_projection_d,
+        const_dnnl_memory_desc_t diff_bias_d,
+        const_dnnl_memory_desc_t diff_dst_layer_d,
+        const_dnnl_memory_desc_t diff_dst_iter_d,
+        const_dnnl_memory_desc_t diff_dst_iter_c_d,
+        const_dnnl_primitive_desc_t hint, dnnl_primitive_attr_t attr,
+        res_t *res);
 
 void init_buffer(float *buf, int64_t size, float value);
 
@@ -103,8 +111,7 @@ void data_deq10n(int64_t dimc, int64_t dimr, int64_t ld_src, float *src_,
 void gates_reduction(
         const prb_t &prb, const float *b_gates_, float *diff_bias_);
 
-int compare_dat(const prb_t &prb, data_kind_t kind, dnn_mem_t &mem_dt,
-        dnn_mem_t &mem_fp, res_t *res, bool final_compare);
+rnn_data_kind_t data_kind2rnn_data_kind(data_kind_t data_kind);
 
 }; // namespace rnn
 
