@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,10 @@
 #ifndef DNNL_TEST_COMMON_HPP
 #define DNNL_TEST_COMMON_HPP
 
+#ifdef _WIN32
+#include <windows.h> // GetEnvironmentVariable
+#endif
+
 #include <cmath>
 #include <limits>
 #include <numeric>
@@ -54,6 +58,7 @@
 #include "src/common/float16.hpp"
 #include "src/common/memory_desc_wrapper.hpp"
 #include "src/common/nstl.hpp"
+#include "src/common/primitive_cache.hpp"
 #include "tests/gtests/test_malloc.hpp"
 #include "tests/test_thread.hpp"
 
@@ -62,7 +67,9 @@
 #define for_ for
 
 using dnnl::impl::bfloat16_t;
-using dnnl::impl::f16_support::float16_t;
+using dnnl::impl::float16_t;
+using dnnl::impl::float8_e4m3_t;
+using dnnl::impl::float8_e5m2_t;
 
 #ifdef DNNL_ENABLE_MEM_DEBUG
 #define DNNL_CHECK(f) \
@@ -94,40 +101,71 @@ dnnl::engine::kind get_test_engine_kind();
 dnnl::engine get_test_engine();
 #endif
 
-inline int get_vendor_id(const std::string &vendor) {
+inline uint32_t get_vendor_id(const std::string &vendor) {
     if (vendor == "nvidia") {
         return 0x10DE;
+    } else if (vendor == "amd") {
+        return 0x1002;
     } else if (vendor == "intel") {
         return 0x8086;
     } else {
-        return -1;
+        return 0x0;
     }
 }
 
 inline bool is_nvidia_gpu(const dnnl::engine &eng) {
 #ifdef DNNL_WITH_SYCL
-    const int nvidia_vendor_id = get_vendor_id("nvidia");
+    if (eng.get_kind() != dnnl::engine::kind::gpu) return false;
+    const uint32_t nvidia_vendor_id = get_vendor_id("nvidia");
     const auto device = dnnl::sycl_interop::get_device(eng);
     const auto eng_vendor_id
-            = device.get_info<cl::sycl::info::device::vendor_id>();
+            = device.get_info<::sycl::info::device::vendor_id>();
     return eng_vendor_id == nvidia_vendor_id;
 #endif
     return false;
 }
 
-inline bool unsupported_data_type(memory::data_type dt, dnnl::engine eng) {
-    dnnl::engine::kind kind = eng.get_kind();
+inline bool is_amd_gpu(const dnnl::engine &eng) {
+#ifdef DNNL_WITH_SYCL
+    if (eng.get_kind() != dnnl::engine::kind::gpu) return false;
+    const uint32_t amd_vendor_id = get_vendor_id("amd");
+    const auto device = dnnl::sycl_interop::get_device(eng);
+    const auto eng_vendor_id
+            = device.get_info<::sycl::info::device::vendor_id>();
+    return eng_vendor_id == amd_vendor_id;
+#endif
+    return false;
+}
 
+inline bool is_sycl_engine(dnnl::engine::kind eng_kind) {
+#if DNNL_CPU_RUNTIME == DNNL_RUNTIME_SYCL
+    if (eng_kind == dnnl::engine::kind::cpu) return true;
+#endif
+
+#if DNNL_GPU_RUNTIME == DNNL_RUNTIME_SYCL
+    if (eng_kind == dnnl::engine::kind::gpu) return true;
+#endif
+    return false;
+}
+
+inline bool unsupported_data_type(
+        memory::data_type dt, const dnnl::engine &eng) {
     bool supported = true; // optimism
+
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
+    dnnl::engine::kind kind = eng.get_kind();
     if (kind == dnnl::engine::kind::cpu)
         supported = dnnl::impl::cpu::platform::has_data_type_support(
                 memory::convert_to_c(dt));
-#ifdef DNNL_SYCL_CUDA
-    if (is_nvidia_gpu(eng)) {
+#endif
+
+#if defined(DNNL_SYCL_CUDA) || defined(DNNL_SYCL_HIP)
+    if (is_nvidia_gpu(eng) || is_amd_gpu(eng)) {
         switch (dt) {
             case memory::data_type::f32: return false;
             case memory::data_type::f16: return false;
             case memory::data_type::s8: return false;
+            case memory::data_type::undef: return false;
             default: return true;
         }
     }
@@ -139,6 +177,41 @@ inline bool unsupported_data_type(memory::data_type dt, dnnl::engine eng) {
 inline bool unsupported_data_type(memory::data_type dt) {
     return unsupported_data_type(dt, get_test_engine());
 }
+
+template <typename... Rest>
+inline bool unsupported_data_type(
+        memory::data_type first_dt, Rest... rest_dts) {
+    bool rval = unsupported_data_type(first_dt, get_test_engine());
+    if (rval) return rval;
+    return unsupported_data_type(rest_dts...);
+}
+#endif
+
+inline bool unsupported_prop_kind(
+        dnnl::prop_kind pk, memory::data_type dt, const dnnl::engine &eng) {
+    bool supported = !unsupported_data_type(dt, eng);
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
+    dnnl::engine::kind kind = eng.get_kind();
+    if (supported && kind == dnnl::engine::kind::cpu)
+        supported = IMPLICATION(pk != dnnl::prop_kind::forward_inference,
+                dnnl::impl::cpu::platform::has_training_support(
+                        memory::convert_to_c(dt)));
+#endif
+    return !supported;
+}
+
+#ifdef DNNL_TEST_WITH_ENGINE_PARAM
+inline bool unsupported_prop_kind(dnnl::prop_kind pk, memory::data_type dt) {
+    return unsupported_prop_kind(pk, dt, get_test_engine());
+}
+
+template <typename... Rest>
+inline bool unsupported_prop_kind(
+        dnnl::prop_kind pk, memory::data_type first_dt, Rest... rest_dts) {
+    bool rval = unsupported_prop_kind(pk, first_dt, get_test_engine());
+    if (rval) return rval;
+    return unsupported_prop_kind(pk, rest_dts...);
+}
 #endif
 
 template <typename data_t>
@@ -258,6 +331,7 @@ struct mapped_ptr_t {
 
     operator T *() { return ptr_; }
     operator const T *() const { return ptr_; }
+    operator bool() const { return ptr_ != nullptr; }
 
 private:
     const memory *mem_;
@@ -276,10 +350,10 @@ void check_zero_tail(int set_zero_flag, const memory &src) {
     auto src_data = map_memory<data_t>(src);
 
     const memory::desc src_d = src.get_desc();
-    const int ndims = src_d.data.ndims;
-    const auto *dims = src_d.data.dims;
-    const auto *pdims = src_d.data.padded_dims;
-    const dnnl::impl::memory_desc_wrapper mdw(src_d.data);
+    const int ndims = src_d.get_ndims();
+    const auto dims = src_d.get_dims();
+    const auto pdims = src_d.get_padded_dims();
+    const dnnl::impl::memory_desc_wrapper mdw(src_d.get());
 
     memory::dim idx[DNNL_MAX_NDIMS] = {}, str[DNNL_MAX_NDIMS] = {};
     memory::dim nelems = 1;
@@ -347,25 +421,58 @@ static inline data_t set_value(
 }
 
 template <typename data_t>
-static void fill_data(const memory::dim size, data_t *data, data_t mean,
+static void fill_data(const memory::dim nelems, data_t *data, data_t mean,
         data_t deviation, double sparsity = 1.) {
-    dnnl::impl::parallel_nd(size, [&](memory::dim n) {
+    dnnl::impl::parallel_nd(nelems, [&](memory::dim n) {
         data[n] = set_value<data_t>(n, mean, deviation, sparsity);
     });
 }
 
 template <typename data_t>
-static void fill_data(const memory::dim size, const memory &mem, data_t mean,
+static void fill_data(const memory::dim nelems, const memory &mem, data_t mean,
         data_t deviation, double sparsity = 1.) {
     auto data_ptr = map_memory<data_t>(mem);
-    fill_data<data_t>(size, data_ptr, mean, deviation, sparsity);
+    fill_data<data_t>(nelems, data_ptr, mean, deviation, sparsity);
+}
+
+inline void fill_data(memory::data_type dt, const memory &mem, float mean,
+        float deviation, double sparsity = 1.) {
+    const auto dt_size = memory::data_type_size(dt);
+    if (dt_size <= 0) {
+        assert(!"unexpected data type");
+        return;
+    }
+
+    memory::dim nelems
+            = static_cast<memory::dim>(mem.get_desc().get_size() / dt_size);
+    switch (dt) {
+        case memory::data_type::f32:
+            fill_data<float>(nelems, mem, mean, deviation, sparsity);
+            break;
+        case memory::data_type::bf16:
+            fill_data<bfloat16_t>(nelems, mem, mean, deviation, sparsity);
+            break;
+        case memory::data_type::f16:
+            fill_data<float16_t>(nelems, mem, mean, deviation, sparsity);
+            break;
+        case memory::data_type::s32:
+            fill_data<int>(nelems, mem, mean, deviation, sparsity);
+            break;
+        case memory::data_type::s8:
+            fill_data<int8_t>(nelems, mem, mean, deviation, sparsity);
+            break;
+        case memory::data_type::u8:
+            fill_data<uint8_t>(nelems, mem, mean, deviation, sparsity);
+            break;
+        default: assert(!"unsupported data type"); break;
+    }
 }
 
 template <typename data_t>
-static void fill_data(const memory::dim size, data_t *data,
+static void fill_data(const memory::dim nelems, data_t *data,
         double sparsity = 1., bool init_negs = false) {
-    dnnl::impl::parallel_nd(size, [&](memory::dim n) {
-        data[n] = set_value<data_t>(n, data_t(1), data_t(2e-1), sparsity);
+    dnnl::impl::parallel_nd(nelems, [&](memory::dim n) {
+        data[n] = set_value<data_t>(n, data_t(1), data_t(0.2f), sparsity);
 
         if (init_negs && n % 4 == 0)
             data[n] = static_cast<data_t>(
@@ -374,10 +481,19 @@ static void fill_data(const memory::dim size, data_t *data,
 }
 
 template <typename data_t>
-static void fill_data(const memory::dim size, const memory &mem,
+static void fill_data(const memory::dim nelems, const memory &mem,
         double sparsity = 1., bool init_negs = false) {
     auto data_ptr = map_memory<data_t>(mem);
-    fill_data<data_t>(size, data_ptr, sparsity, init_negs);
+    fill_data<data_t>(nelems, data_ptr, sparsity, init_negs);
+}
+
+template <typename data_t>
+static void remove_zeroes(const memory &mem) {
+    size_t nelems = mem.get_desc().get_size() / sizeof(data_t);
+    auto data_ptr = map_memory<data_t>(mem);
+    dnnl::impl::parallel_nd(nelems, [&](memory::dim n) {
+        if (data_ptr[n] == data_t(0)) data_ptr[n] += data_t(1);
+    });
 }
 
 template <typename data_t>
@@ -394,18 +510,18 @@ static void compare_data(
     /* Note: size_t incompatible with MSVC++ */
     auto ref_desc = ref.get_desc();
     auto dst_desc = dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper mdw_ref(ref_desc.data);
-    const dnnl::impl::memory_desc_wrapper mdw_dst(dst_desc.data);
+    const dnnl::impl::memory_desc_wrapper mdw_ref(ref_desc.get());
+    const dnnl::impl::memory_desc_wrapper mdw_dst(dst_desc.get());
 
-    ASSERT_TRUE(ref_desc.data.ndims == dst_desc.data.ndims);
+    ASSERT_TRUE(ref_desc.get_ndims() == dst_desc.get_ndims());
 
-    auto ndims = ref_desc.data.ndims;
+    auto ndims = ref_desc.get_ndims();
 
     for (auto d = 0; d < ndims; ++d) {
-        ASSERT_TRUE(ref_desc.data.dims[d] == dst_desc.data.dims[d]);
+        ASSERT_TRUE(ref_desc.get_dims()[d] == dst_desc.get_dims()[d]);
     }
 
-    auto dims = ref_desc.data.dims;
+    auto dims = ref_desc.get_dims();
 
     memory::dim num = 1;
     for (auto d = 0; d < ndims; ++d) {
@@ -499,23 +615,31 @@ struct test_convolution_attr_t {
     };
 
     void dnnl_attr_recreate() {
-        mkl_attr = dnnl::primitive_attr();
-        if (oscale.is_def()) {
-            const memory::dim count = 1;
+        dnnl_attr = dnnl::primitive_attr();
+        if (src_scale.is_def()) {
             const int mask = 0;
-            std::vector<float> s(count, oscale.scale);
-            mkl_attr.set_output_scales(mask, s);
+            dnnl_attr.set_scales_mask(DNNL_ARG_SRC, mask);
+        }
+        if (wei_scale.is_def()) {
+            const int mask = 0;
+            dnnl_attr.set_scales_mask(DNNL_ARG_WEIGHTS, mask);
+        }
+        if (dst_scale.is_def()) {
+            const int mask = 0;
+            dnnl_attr.set_scales_mask(DNNL_ARG_DST, mask);
         }
     }
 
     test_convolution_attr_t(
             float s, scale_t::policy_t p = scale_t::policy_t::NONE)
-        : oscale(s, p), mkl_attr() {}
+        : src_scale(s, p), wei_scale(s, p), dst_scale(s, p), dnnl_attr() {}
 
     test_convolution_attr_t() : test_convolution_attr_t(1.f) {}
 
-    scale_t oscale;
-    dnnl::primitive_attr mkl_attr;
+    scale_t src_scale;
+    scale_t wei_scale;
+    scale_t dst_scale;
+    dnnl::primitive_attr dnnl_attr;
 };
 
 struct test_convolution_formats_t {
@@ -548,7 +672,8 @@ struct test_convolution_eltwise_params_t {
 
 template <typename F>
 bool catch_expected_failures(const F &f, bool expect_to_fail,
-        dnnl_status_t expected_status, bool ignore_unimplemented = false) {
+        dnnl_status_t expected_status, bool ignore_unimplemented = false,
+        const char *filename = __FILE__, int64_t line_num = __LINE__) {
     try {
         f();
     } catch (const dnnl::error &e) {
@@ -558,7 +683,8 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
             // Ignore unimplemented
             if (ignore_unimplemented && (e.status == dnnl_unimplemented)) {
                 // Print unimplemented but do not treat as error
-                std::cout << "[  UNIMPL  ] "
+                std::cout << "(" << filename << ":" << line_num << ") "
+                          << "[  UNIMPL  ] "
                           << "Implementation not found" << std::endl;
                 reset_failed_malloc_counter();
                 return true;
@@ -577,7 +703,8 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
                         expected_status, ignore_unimplemented);
             } else {
                 if (expect_to_fail && (e.status != expected_status))
-                    std::cout << "expect failure status mismatch: expect("
+                    std::cout << "(" << filename << ":" << line_num << ") "
+                              << "Expect failure status mismatch: expect("
                               << dnnl_status2str(expected_status) << ") get("
                               << dnnl_status2str(e.status)
                               << "). Re-throwing...\n";
@@ -594,7 +721,8 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
 
     // Throw an exception if the failure is expected but did not happen
     if (expect_to_fail) {
-        std::cout << "expect failure with status("
+        std::cout << "(" << filename << ":" << line_num << ") "
+                  << "Expect failure with status("
                   << dnnl_status2str(expected_status) << "), "
                   << "but operation succeed. Throwing an exception...\n";
         throw std::exception();
@@ -608,16 +736,36 @@ bool catch_expected_failures(const F &f, bool expect_to_fail,
 namespace test {
 inline dnnl::memory make_memory(
         const dnnl::memory::desc &md, const dnnl::engine &eng) {
-#if defined(TEST_DNNL_DPCPP_BUFFER)
+
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_SYCL
+    if (eng.get_kind() == dnnl::engine::kind::cpu) {
+        return dnnl::memory(md, eng);
+    }
+#endif
+
+#if defined(TEST_DNNL_OCL_USM)
+    return dnnl::ocl_interop::make_memory(
+            md, eng, dnnl::ocl_interop::memory_kind::usm);
+#elif defined(TEST_DNNL_DPCPP_BUFFER)
     return dnnl::sycl_interop::make_memory(
             md, eng, dnnl::sycl_interop::memory_kind::buffer);
 #else
     return dnnl::memory(md, eng);
 #endif
 }
+
 inline dnnl::memory make_memory(
         const dnnl::memory::desc &md, const dnnl::engine &eng, void *handle) {
-#if defined(TEST_DNNL_DPCPP_BUFFER)
+#if DNNL_CPU_RUNTIME != DNNL_RUNTIME_SYCL
+    if (eng.get_kind() == dnnl::engine::kind::cpu) {
+        return dnnl::memory(md, eng, handle);
+    }
+#endif
+
+#if defined(TEST_DNNL_OCL_USM)
+    return dnnl::ocl_interop::make_memory(
+            md, eng, dnnl::ocl_interop::memory_kind::usm, handle);
+#elif defined(TEST_DNNL_DPCPP_BUFFER)
     return dnnl::sycl_interop::make_memory(
             md, eng, dnnl::sycl_interop::memory_kind::buffer, handle);
 #else
@@ -665,23 +813,9 @@ public:
         }
         // Fill with a magic number to catch possible uninitialized access
         mapped_ptr_t<char> ptr(&mem_);
-        memset(ptr, 0xFF, size_);
+        if (ptr) memset(ptr, 0xFF, size_);
     }
 
-#if 0
-    template <typename malloc_t, typename free_t>
-    test_memory(const memory::desc &d, const dnnl::engine &e,
-            const malloc_t &f_malloc, const free_t &f_free) {
-        size_ = d.get_size();
-        data_.reset(static_cast<char *>(f_malloc(size_)), f_free);
-        mem_ = test::make_memory(d, e, data_.get());
-
-        // Fill with a magic number to catch possible uninitialized access
-        mapped_ptr_t<char> ptr(&mem_);
-        memset(ptr, 0xFF, size_);
-    }
-#endif
-
     size_t get_size() const { return size_; }
     const memory &get() const { return mem_; }
 
@@ -724,62 +858,51 @@ inline std::string to_string(dnnl_stream_flags_t stream_flags) {
 
 // testing all available C++ primitive descriptor constructors
 struct allows_attr_t {
-    bool oscale;
     bool po_sum;
     bool po_eltwise;
     bool po_binary;
+    bool po_prelu;
     bool zp;
     bool scales;
 };
 
 using engine = dnnl::engine;
 // forward
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_attr(const op_desc_t &op_desc, const engine &eng) {
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_attr(const engine &eng, const prim_params_t &...prim_params) {
     dnnl::primitive_attr attr;
-    EXPECT_NO_THROW(pd_t pd(op_desc, attr, eng));
+    EXPECT_NO_THROW(pd_t pd(eng, prim_params..., attr));
 }
 
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_attr_oscale(
-        const op_desc_t &op_desc, const engine &eng, bool supports_oscale) {
-    dnnl::primitive_attr attr_oscale;
-    attr_oscale.set_output_scales(0, {2.f});
-    if (supports_oscale)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_oscale, eng));
-    else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_oscale, eng));
-}
-
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_attr_po_sum(
-        const op_desc_t &op_desc, const engine &eng, bool supports_po_sum) {
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_attr_po_sum(const engine &eng, bool supports_po_sum,
+        const prim_params_t &...prim_params) {
     dnnl::post_ops ops_sum;
     ops_sum.append_sum(1.1f);
     dnnl::primitive_attr attr_po_sum;
     attr_po_sum.set_post_ops(ops_sum);
     if (supports_po_sum)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_sum, eng));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., attr_po_sum));
     else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_sum, eng));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., attr_po_sum));
 }
 
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_attr_po_eltwise(
-        const op_desc_t &op_desc, const engine &eng, bool supports_po_eltwise) {
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_attr_po_eltwise(const engine &eng, bool supports_po_eltwise,
+        const prim_params_t &...prim_params) {
     dnnl::post_ops ops_eltwise;
-    ops_eltwise.append_eltwise(1.0f, dnnl::algorithm::eltwise_relu, 0.f, 0.f);
+    ops_eltwise.append_eltwise(dnnl::algorithm::eltwise_relu, 0.f, 0.f);
     dnnl::primitive_attr attr_po_eltwise;
     attr_po_eltwise.set_post_ops(ops_eltwise);
     if (supports_po_eltwise)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_eltwise, eng));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., attr_po_eltwise));
     else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_eltwise, eng));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., attr_po_eltwise));
 }
 
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_attr_po_binary(
-        const op_desc_t &op_desc, const engine &eng, bool supports_po_binary) {
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_attr_po_binary(const engine &eng, bool supports_po_binary,
+        const prim_params_t &...prim_params) {
     dnnl::post_ops ops_binary;
     dnnl::memory::desc src1_desc(
             {16}, memory::data_type::s8, memory::format_tag::x);
@@ -787,120 +910,119 @@ void test_fwd_pd_attr_po_binary(
     dnnl::primitive_attr attr_po_binary;
     attr_po_binary.set_post_ops(ops_binary);
     if (supports_po_binary)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_binary, eng));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., attr_po_binary));
     else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_binary, eng));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., attr_po_binary));
 }
 
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_attr_zp(
-        const op_desc_t &op_desc, const engine &eng, bool supports_zero_point) {
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_attr_po_prelu(const engine &eng, bool supports_po_prelu,
+        const prim_params_t &...prim_params) {
+    dnnl::post_ops ops_prelu;
+    ops_prelu.append_prelu(0);
+    dnnl::primitive_attr attr_po_prelu;
+    attr_po_prelu.set_post_ops(ops_prelu);
+    if (supports_po_prelu)
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., attr_po_prelu));
+    else
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., attr_po_prelu));
+}
+
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_attr_zp(const engine &eng, bool supports_zero_point,
+        const prim_params_t &...prim_params) {
     dnnl::primitive_attr attr_zp;
-    attr_zp.set_zero_points(DNNL_ARG_SRC, 0, {1});
+    attr_zp.set_zero_points_mask(DNNL_ARG_SRC, 0);
     if (supports_zero_point)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_zp, eng));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., attr_zp));
     else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_zp, eng));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., attr_zp));
 }
 
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_attr_scales(
-        const op_desc_t &op_desc, const engine &eng, bool supports_scales) {
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_attr_scales(const engine &eng, bool supports_scales,
+        const prim_params_t &...prim_params) {
     dnnl::primitive_attr attr_scales;
-    attr_scales.set_scales(DNNL_ARG_SRC, 0, {2.f});
+    attr_scales.set_scales_mask(DNNL_ARG_SRC, 0);
 
     if (supports_scales) { // Currently only used with binary ops
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_scales, eng));
-
-        // Check oscale and scales don't work together
-        dnnl::primitive_attr attr_oscale_scales;
-        attr_oscale_scales.set_output_scales(0, {2.f});
-        EXPECT_ANY_THROW(attr_oscale_scales.set_scales(DNNL_ARG_SRC, 0, {2.f}));
-
-        dnnl::primitive_attr attr_scales_oscale;
-        attr_scales_oscale.set_scales(DNNL_ARG_SRC, 0, {2.f});
-        EXPECT_ANY_THROW(attr_scales_oscale.set_output_scales(0, {2.f}));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., attr_scales));
     } else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_scales, eng));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., attr_scales));
 }
 
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_allow_empty(const pd_t &pd) {
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_allow_empty(
+        const pd_t &pd, const prim_params_t &...prim_params) {
     bool allow_empty = true;
     pd_t new_pd {};
-    ASSERT_NO_THROW(new_pd = pd_t(op_desc_t(), pd.get_engine(), allow_empty));
+    dnnl::primitive_attr unsupported_attr;
+    // Assumption is that mask "10" is a legit mask for scales
+    // from API perspective.
+    unsupported_attr.set_scales_mask(DNNL_ARG_SRC, 10);
+    ASSERT_NO_THROW(new_pd = pd_t(pd.get_engine(), prim_params...,
+                            unsupported_attr, allow_empty));
     ASSERT_FALSE(new_pd);
 }
 
 // Note: requires a valid primitive descriptor!
-template <typename op_desc_t, typename pd_t>
-void test_fwd_pd_constructors(
-        const op_desc_t &op_desc, const pd_t &pd, const allows_attr_t &aa) {
+template <typename pd_t, typename... prim_params_t>
+void test_fwd_pd_constructors(const pd_t &pd, const allows_attr_t &aa,
+        const prim_params_t &...prim_params) {
     auto test_pd = pd_t();
     auto eng = pd.get_engine();
     // ctor from C pd, should not throw
     ASSERT_NO_THROW(test_pd = pd_t(pd.get()));
     // ctor w/ empty attr, should not throw
-    test_fwd_pd_attr<op_desc_t, pd_t>(op_desc, eng);
+    test_fwd_pd_attr<pd_t>(eng, prim_params...);
     // following ctors w/ attrs may throw based on pd support
-    test_fwd_pd_attr_oscale<op_desc_t, pd_t>(op_desc, eng, aa.oscale);
-    test_fwd_pd_attr_po_sum<op_desc_t, pd_t>(op_desc, eng, aa.po_sum);
-    test_fwd_pd_attr_po_eltwise<op_desc_t, pd_t>(op_desc, eng, aa.po_eltwise);
-    test_fwd_pd_attr_po_binary<op_desc_t, pd_t>(op_desc, eng, aa.po_binary);
-    test_fwd_pd_attr_zp<op_desc_t, pd_t>(op_desc, eng, aa.zp);
-    test_fwd_pd_attr_scales<op_desc_t, pd_t>(op_desc, eng, aa.scales);
+    test_fwd_pd_attr_po_sum<pd_t>(eng, aa.po_sum, prim_params...);
+    test_fwd_pd_attr_po_eltwise<pd_t>(eng, aa.po_eltwise, prim_params...);
+    test_fwd_pd_attr_po_binary<pd_t>(eng, aa.po_binary, prim_params...);
+    test_fwd_pd_attr_po_prelu<pd_t>(eng, aa.po_prelu, prim_params...);
+    test_fwd_pd_attr_zp<pd_t>(eng, aa.zp, prim_params...);
+    test_fwd_pd_attr_scales<pd_t>(eng, aa.scales, prim_params...);
     // check allow empty, should not throw
-    test_fwd_pd_allow_empty<op_desc_t, pd_t>(test_pd);
+    test_fwd_pd_allow_empty<pd_t>(test_pd, prim_params...);
 }
 
 // backward: has hint
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_attr(
-        const op_desc_t &op_desc, const engine &eng, const hint_pd_t &hint) {
+template <typename pd_t, typename hint_pd_t, typename... prim_params_t>
+void test_bwd_pd_attr(const engine &eng, const hint_pd_t &hint,
+        const prim_params_t &...prim_params) {
     dnnl::primitive_attr attr;
-    EXPECT_NO_THROW(pd_t pd(op_desc, attr, eng, hint));
+    EXPECT_NO_THROW(pd_t pd(eng, prim_params..., hint, attr));
 }
 
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_attr_oscale(const op_desc_t &op_desc, const engine &eng,
-        const hint_pd_t &hint, bool supports_oscale) {
-    dnnl::primitive_attr attr_oscale;
-    attr_oscale.set_output_scales(0, {2.f});
-    if (supports_oscale)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_oscale, eng, hint));
-    else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_oscale, eng, hint));
-}
-
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_attr_po_sum(const op_desc_t &op_desc, const engine &eng,
-        const hint_pd_t &hint, bool supports_po_sum) {
+template <typename pd_t, typename hint_pd_t, typename... prim_params_t>
+void test_bwd_pd_attr_po_sum(const engine &eng, const hint_pd_t &hint,
+        bool supports_po_sum, const prim_params_t &...prim_params) {
     dnnl::post_ops ops_sum;
     ops_sum.append_sum(1.1f);
     dnnl::primitive_attr attr_po_sum;
     attr_po_sum.set_post_ops(ops_sum);
     if (supports_po_sum)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_sum, eng, hint));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., hint, attr_po_sum));
     else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_sum, eng, hint));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., hint, attr_po_sum));
 }
 
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_attr_po_eltwise(const op_desc_t &op_desc, const engine &eng,
-        const hint_pd_t &hint, bool supports_po_eltwise) {
+template <typename pd_t, typename hint_pd_t, typename... prim_params_t>
+void test_bwd_pd_attr_po_eltwise(const engine &eng, const hint_pd_t &hint,
+        bool supports_po_eltwise, const prim_params_t &...prim_params) {
     dnnl::post_ops ops_eltwise;
-    ops_eltwise.append_eltwise(1.f, dnnl::algorithm::eltwise_relu, 0.f, 0.f);
+    ops_eltwise.append_eltwise(dnnl::algorithm::eltwise_relu, 0.f, 0.f);
     dnnl::primitive_attr attr_po_eltwise;
     attr_po_eltwise.set_post_ops(ops_eltwise);
     if (supports_po_eltwise)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_eltwise, eng, hint));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., hint, attr_po_eltwise));
     else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_eltwise, eng, hint));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., hint, attr_po_eltwise));
 }
 
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_attr_po_binary(const op_desc_t &op_desc, const engine &eng,
-        const hint_pd_t &hint, bool supports_po_binary) {
+template <typename pd_t, typename hint_pd_t, typename... prim_params_t>
+void test_bwd_pd_attr_po_binary(const engine &eng, const hint_pd_t &hint,
+        bool supports_po_binary, const prim_params_t &...prim_params) {
     dnnl::post_ops ops_binary;
     dnnl::memory::desc src1_desc(
             {16}, memory::data_type::s8, memory::format_tag::x);
@@ -908,61 +1030,65 @@ void test_bwd_pd_attr_po_binary(const op_desc_t &op_desc, const engine &eng,
     dnnl::primitive_attr attr_po_binary;
     attr_po_binary.set_post_ops(ops_binary);
     if (supports_po_binary)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_po_binary, eng, hint));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., hint, attr_po_binary));
     else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_po_binary, eng, hint));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., hint, attr_po_binary));
 }
 
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_attr_zp(const op_desc_t &op_desc, const engine &eng,
-        const hint_pd_t &hint, bool supports_zero_point) {
+template <typename pd_t, typename hint_pd_t, typename... prim_params_t>
+void test_bwd_pd_attr_zp(const engine &eng, const hint_pd_t &hint,
+        bool supports_zero_point, const prim_params_t &...prim_params) {
     dnnl::primitive_attr attr_zp;
-    attr_zp.set_zero_points(DNNL_ARG_SRC, 0, {1});
+    attr_zp.set_zero_points_mask(DNNL_ARG_SRC, 0);
     if (supports_zero_point)
-        EXPECT_NO_THROW(pd_t pd(op_desc, attr_zp, eng, hint));
+        EXPECT_NO_THROW(pd_t pd(eng, prim_params..., hint, attr_zp));
     else
-        EXPECT_ANY_THROW(pd_t pd(op_desc, attr_zp, eng, hint));
+        EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., hint, attr_zp));
 }
 
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_attr_scales(const op_desc_t &op_desc, const engine &eng,
-        const hint_pd_t &hint, bool supports_scales) {
+template <typename pd_t, typename hint_pd_t, typename... prim_params_t>
+void test_bwd_pd_attr_scales(const engine &eng, const hint_pd_t &hint,
+        bool supports_scales, const prim_params_t &...prim_params) {
     dnnl::primitive_attr attr_scales;
-    attr_scales.set_scales(DNNL_ARG_SRC, 0, {2.f});
-    EXPECT_ANY_THROW(pd_t pd(op_desc, attr_scales, eng, hint));
+    attr_scales.set_scales_mask(DNNL_ARG_SRC, 0);
+    EXPECT_ANY_THROW(pd_t pd(eng, prim_params..., hint, attr_scales));
 }
 
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_allow_empty(const pd_t &pd, const hint_pd_t &hint) {
+template <typename pd_t, typename hint_pd_t, typename... prim_params_t>
+void test_bwd_pd_allow_empty(const pd_t &pd, const hint_pd_t &hint,
+        const prim_params_t &...prim_params) {
     bool allow_empty = true;
     pd_t new_pd {};
-    ASSERT_NO_THROW(
-            new_pd = pd_t(op_desc_t(), pd.get_engine(), hint, allow_empty));
+    dnnl::primitive_attr unsupported_attr;
+    // Assumption is that mask "10" is a legit mask for scales
+    // from API perspective.
+    unsupported_attr.set_scales_mask(DNNL_ARG_SRC, 10);
+    ASSERT_NO_THROW(new_pd = pd_t(pd.get_engine(), prim_params..., hint,
+                            unsupported_attr, allow_empty));
     ASSERT_FALSE(new_pd);
 }
 
 // Note: requires a valid primitive descriptor!
-template <typename op_desc_t, typename pd_t, typename hint_pd_t>
-void test_bwd_pd_constructors(const op_desc_t &op_desc, const pd_t &pd,
-        const hint_pd_t &hint, const allows_attr_t &aa) {
+template <typename pd_t, typename hint_pd_t, typename... prim_params_t>
+void test_bwd_pd_constructors(const pd_t &pd, const hint_pd_t &hint,
+        const allows_attr_t &aa, const prim_params_t &...prim_params) {
     auto test_pd = pd_t();
     auto hint_pd = hint;
     auto eng = pd.get_engine();
     // ctor from C pd, should not throw
     ASSERT_NO_THROW(test_pd = pd_t(pd.get()));
     // ctor w/ empty attr, should not throw
-    test_bwd_pd_attr<op_desc_t, pd_t>(op_desc, eng, hint_pd);
+    test_bwd_pd_attr<pd_t>(eng, hint_pd, prim_params...);
     // following ctors w/ attrs may throw based on pd support
-    test_bwd_pd_attr_oscale<op_desc_t, pd_t>(op_desc, eng, hint_pd, aa.oscale);
-    test_bwd_pd_attr_po_sum<op_desc_t, pd_t>(op_desc, eng, hint_pd, aa.po_sum);
-    test_bwd_pd_attr_po_eltwise<op_desc_t, pd_t>(
-            op_desc, eng, hint_pd, aa.po_eltwise);
-    test_bwd_pd_attr_po_binary<op_desc_t, pd_t>(
-            op_desc, eng, hint_pd, aa.po_binary);
-    test_bwd_pd_attr_zp<op_desc_t, pd_t>(op_desc, eng, hint_pd, aa.zp);
-    test_bwd_pd_attr_scales<op_desc_t, pd_t>(op_desc, eng, hint_pd, aa.scales);
+    test_bwd_pd_attr_po_sum<pd_t>(eng, hint_pd, aa.po_sum, prim_params...);
+    test_bwd_pd_attr_po_eltwise<pd_t>(
+            eng, hint_pd, aa.po_eltwise, prim_params...);
+    test_bwd_pd_attr_po_binary<pd_t>(
+            eng, hint_pd, aa.po_binary, prim_params...);
+    test_bwd_pd_attr_zp<pd_t>(eng, hint_pd, aa.zp, prim_params...);
+    test_bwd_pd_attr_scales<pd_t>(eng, hint_pd, aa.scales, prim_params...);
     // check allow empty, should not throw
-    test_bwd_pd_allow_empty<op_desc_t, pd_t>(test_pd, hint_pd);
+    test_bwd_pd_allow_empty<pd_t>(test_pd, hint_pd, prim_params...);
 }
 
 inline dnnl::stream make_stream(dnnl::engine engine,
@@ -975,4 +1101,48 @@ inline dnnl::stream make_stream(dnnl::engine engine,
     return dnnl::stream(engine, flags);
 }
 
+inline int get_primitive_cache_size() {
+    int result = 0;
+    auto status = dnnl::impl::get_primitive_cache_size(&result);
+    if (status != dnnl::impl::status::success) return -1;
+    return result;
+}
+
+// This is a local copy of dnnl::impl::getenv.
+// Copying to avoid exposure of internal symbol from the library.
+inline int gtest_getenv(const char *name, char *buffer, int buffer_size) {
+    if (name == nullptr || buffer_size < 0
+            || (buffer == nullptr && buffer_size > 0))
+        return INT_MIN;
+
+    int result = 0;
+    int term_zero_idx = 0;
+    size_t value_length = 0;
+
+#ifdef _WIN32
+    value_length = GetEnvironmentVariable(name, buffer, buffer_size);
+#else
+    const char *value = ::getenv(name);
+    value_length = value == nullptr ? 0 : strlen(value);
+#endif
+
+    if (value_length > INT_MAX)
+        result = INT_MIN;
+    else {
+        int int_value_length = (int)value_length;
+        if (int_value_length >= buffer_size) {
+            result = -int_value_length;
+        } else {
+            term_zero_idx = int_value_length;
+            result = int_value_length;
+#ifndef _WIN32
+            if (value) strncpy(buffer, value, buffer_size - 1);
+#endif
+        }
+    }
+
+    if (buffer != nullptr) buffer[term_zero_idx] = '\0';
+    return result;
+}
+
 #endif
