@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -29,7 +29,7 @@
     do { \
         if (cond) { \
             std::cout << "[  SKIPPED ] " << (msg) << std::endl; \
-            return; \
+            GTEST_SKIP(); \
         } \
     } while (0)
 
@@ -54,6 +54,36 @@
 #define SKIP_FOR_LOOP_CUDA(cond, message)
 #endif
 
+#ifdef DNNL_SYCL_HIP
+#define SKIP_IF_HIP(cond, message) \
+    do { \
+        SKIP_IF(get_test_engine_kind() == engine::kind::gpu && (cond), \
+                (message)); \
+    } while (0)
+
+#define SKIP_FOR_LOOP_HIP(cond, message) \
+    SKIP_FOR_LOOP( \
+            get_test_engine_kind() == engine::kind::gpu && (cond), (message));
+#else
+#define SKIP_IF_HIP(cond, message)
+#define SKIP_FOR_LOOP_HIP(cond, message)
+#endif
+
+#ifdef DNNL_SYCL_GENERIC
+#define SKIP_IF_GENERIC(cond, message) \
+    do { \
+        SKIP_IF(get_test_engine_kind() == engine::kind::gpu && (cond), \
+                (message)); \
+    } while (0)
+
+#define SKIP_FOR_LOOP_GENERIC(cond, message) \
+    SKIP_FOR_LOOP( \
+            get_test_engine_kind() == engine::kind::gpu && (cond), (message));
+#else
+#define SKIP_IF_GENERIC(cond, message)
+#define SKIP_FOR_LOOP_GENERIC(cond, message)
+#endif
+
 #define TEST_F_(test_fixture, test_name) TEST_F(test_fixture, test_name)
 
 #define CPU_TEST_F(test_fixture, test_name) \
