@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -37,19 +37,19 @@ class concat_test_t : public ::testing::TestWithParam<concat_test_params_t> {
             int concat_dim) {
         auto dst_data = map_memory<const data_t>(dst);
         const auto &dst_d = dst.get_desc();
-        const auto dst_dims = dst_d.data.dims;
-        const auto dst_pdims = dst_d.data.padded_dims;
-        const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.data);
+        const auto dst_dims = dst_d.get_dims();
+        const auto dst_pdims = dst_d.get_padded_dims();
+        const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.get());
 
         memory::dim acc_concat_dim = 0;
-        const auto ndims = dst_d.data.ndims;
+        const auto ndims = dst_d.get_ndims();
 
         for (size_t num = 0; num < srcs.size(); num++) {
             const auto src_data = map_memory<data_t>(srcs[num]);
             const auto &src_d = srcs[num].get_desc();
-            const auto src_dims = src_d.data.dims;
-            const auto src_pdims = src_d.data.padded_dims;
-            const dnnl::impl::memory_desc_wrapper src_mdw(src_d.data);
+            const auto src_dims = src_d.get_dims();
+            const auto src_pdims = src_d.get_padded_dims();
+            const dnnl::impl::memory_desc_wrapper src_mdw(src_d.get());
 
             auto N = src_dims[0];
             auto C = src_dims[1];
@@ -99,8 +99,17 @@ protected:
                 dnnl_aBcde4b);
     }
 
+    bool generic_supported_format_tag(memory::format_tag tag) {
+        return impl::utils::one_of(tag, dnnl_a, dnnl_ab, dnnl_abc, dnnl_abcd,
+                dnnl_abcde, dnnl_abcdef, dnnl_abdec, dnnl_acb, dnnl_acbde,
+                dnnl_acbdef, dnnl_acdb, dnnl_acdeb, dnnl_ba, dnnl_bac,
+                dnnl_bacd, dnnl_bca, dnnl_bcda, dnnl_bcdea, dnnl_cba, dnnl_cdba,
+                dnnl_cdeba, dnnl_decab, dnnl_defcab);
+    }
+
     void SetUp() override {
         auto data_type = data_traits<data_t>::data_type;
+        SKIP_IF_HIP(true, "Concat operator is not supported");
         SKIP_IF(unsupported_data_type(data_type),
                 "Engine does not support this data type.");
         concat_test_params_t p
@@ -108,18 +117,25 @@ protected:
         for (size_t i = 0; i < p.srcs_cds.size(); i++) {
             SKIP_IF_CUDA(!cuda_supported_format_tag(p.srcs_format[i]),
                     "Unsupported format tag");
+            SKIP_IF_GENERIC(!generic_supported_format_tag(p.srcs_format[i]),
+                    "Unsupported format tag");
         }
 
         SKIP_IF_CUDA(!cuda_supported_format_tag(p.dst_format),
                 "Unsupported format tag");
+        SKIP_IF_GENERIC(!generic_supported_format_tag(p.dst_format),
+                "Unsupported format tag");
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status, false);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status, false);
     }
 
     virtual void Test() {
         concat_test_params_t p
                 = ::testing::TestWithParam<concat_test_params_t>::GetParam();
 
+        // concat specific types and values
+        using pd_t = concat::primitive_desc;
+
         int src_dim_sum = 0;
         for (size_t i = 0; i < p.srcs_cds.size(); i++) {
             for (size_t dim = 0; dim < p.dst_cds.size(); dim++) {
@@ -143,33 +159,51 @@ protected:
         std::vector<memory> srcs;
         for (size_t i = 0; i < p.srcs_cds.size(); i++) {
             auto md = memory::desc(p.srcs_cds[i], data_type, p.srcs_format[i]);
-            auto src_memory = test::make_memory(md, eng);
-            const size_t sz = src_memory.get_desc().get_size() / sizeof(data_t);
-            fill_data<data_t>(sz, src_memory);
-            check_zero_tail<data_t>(1, src_memory);
             srcs_md.push_back(md);
-            srcs.push_back(src_memory);
         }
 
         auto dst_desc = memory::desc(p.dst_cds, data_type, p.dst_format);
-        auto concat_pd = concat::primitive_desc(
-                dst_desc, static_cast<int>(p.concat_dimension), srcs_md, eng);
+        auto concat_pd = pd_t(
+                eng, dst_desc, static_cast<int>(p.concat_dimension), srcs_md);
         // test construction from a C pd
-        concat_pd = concat::primitive_desc(concat_pd.get());
+        concat_pd = pd_t(concat_pd.get());
+
+        auto aa = allows_attr_t {false};
+        aa.scales = true;
+
+        test_fwd_pd_constructors<pd_t>(concat_pd, aa, dst_desc,
+                static_cast<int>(p.concat_dimension), srcs_md);
 
         ASSERT_TRUE(concat_pd.query_md(query::exec_arg_md, DNNL_ARG_DST)
                 == concat_pd.dst_desc());
-        for (int i = 0; i < (int)srcs.size(); i++)
-            ASSERT_TRUE(concat_pd.query_md(
-                                query::exec_arg_md, DNNL_ARG_MULTIPLE_SRC + i)
-                    == concat_pd.src_desc(i));
+
+        for (int i = 0; i < (int)srcs.size(); i++) {
+            if (p.srcs_format[i] != memory::format_tag::any) {
+                ASSERT_TRUE(srcs_md[i] == concat_pd.src_desc(i));
+            }
+        }
 
         auto dst = test::make_memory(concat_pd.dst_desc(), eng);
         fill_data<data_t>(dst.get_desc().get_size() / sizeof(data_t), dst);
         check_zero_tail<data_t>(1, dst);
 
-        ASSERT_EQ(concat_pd.dst_desc().data.ndims, dst_desc.data.ndims);
+        ASSERT_EQ(concat_pd.dst_desc().get_ndims(), dst_desc.get_ndims());
 
+        for (size_t i = 0; i < p.srcs_cds.size(); i++) {
+            auto md = concat_pd.src_desc((int)i);
+            auto src_memory = test::make_memory(md, eng);
+            const size_t sz = src_memory.get_desc().get_size() / sizeof(data_t);
+            fill_data<data_t>(sz, src_memory);
+            check_zero_tail<data_t>(1, src_memory);
+            srcs.push_back(src_memory);
+        }
+
+        for (int i = 0; i < (int)srcs.size(); i++)
+            ASSERT_TRUE(concat_pd.query_md(
+                                query::exec_arg_md, DNNL_ARG_MULTIPLE_SRC + i)
+                    == concat_pd.src_desc(i));
+
+        EXPECT_ANY_THROW(concat(concat_pd, {}));
         concat c(concat_pd);
         std::unordered_map<int, memory> args = {{DNNL_ARG_DST, dst}};
         for (int i = 0; i < (int)srcs.size(); i++) {
@@ -191,6 +225,7 @@ using concat_test_bf16 = concat_test_t<bfloat16_t>;
 TEST_P(concat_test_float, TestsConcat) {}
 TEST_P(concat_test_s8, TestsConcat) {}
 TEST_P(concat_test_bf16, TestsConcat) {}
+TEST_P(concat_test_float16, TestConcat) {}
 
 using fmt = memory::format_tag;
 
@@ -220,6 +255,8 @@ static auto case_ZeroDim = []() {
 INSTANTIATE_TEST_SUITE_P(TestConcat_ZeroDim, concat_test_float, case_ZeroDim());
 CPU_INSTANTIATE_TEST_SUITE_P(
         TestConcat_ZeroDim_bf16, concat_test_bf16, case_ZeroDim());
+CPU_INSTANTIATE_TEST_SUITE_P(
+        TestConcat_ZeroDim_f16, concat_test_float16, case_ZeroDim());
 
 static auto cases_EF = []() {
     return ::testing::Values(
@@ -249,10 +286,18 @@ static auto cases_EF = []() {
                     dnnl_invalid_arguments},
             concat_test_params_t {1, {fmt::nchw, fmt::nchw}, fmt::nchw,
                     {{1, 4, 5, 5}, {1, 2, 5, 5}}, {1, 6, 6, 5}, true,
+                    dnnl_invalid_arguments},
+            concat_test_params_t {1, {fmt::any, fmt::nchw}, fmt::nchw,
+                    {{2, 16, 1, 1}, {2, 16, 1, 1}}, {2, 32, 1, 1}, true,
+                    dnnl_invalid_arguments},
+            concat_test_params_t {1, {fmt::nchw, fmt::any}, fmt::nchw,
+                    {{2, 16, 1, 1}, {2, 16, 1, 1}}, {2, 32, 1, 1}, true,
                     dnnl_invalid_arguments});
 };
 INSTANTIATE_TEST_SUITE_P(TestConcat_EF, concat_test_float, cases_EF());
 CPU_INSTANTIATE_TEST_SUITE_P(TestConcat_EF_bf16, concat_test_bf16, cases_EF());
+CPU_INSTANTIATE_TEST_SUITE_P(
+        TestConcat_EF_f16, concat_test_float16, cases_EF());
 
 static auto cases_padded = []() {
     return ::testing::Values(
@@ -296,6 +341,8 @@ static auto cases_padded = []() {
 INSTANTIATE_TEST_SUITE_P(TestConcat_padded, concat_test_float, cases_padded());
 CPU_INSTANTIATE_TEST_SUITE_P(
         TestConcat_padded_bf16, concat_test_bf16, cases_padded());
+CPU_INSTANTIATE_TEST_SUITE_P(
+        TestConcat_padded_f16, concat_test_float16, cases_padded());
 
 static auto cases_3D = []() {
     return ::testing::Values(
@@ -326,6 +373,7 @@ static auto cases_3D = []() {
 };
 INSTANTIATE_TEST_SUITE_P(TestConcat3D, concat_test_float, cases_3D());
 CPU_INSTANTIATE_TEST_SUITE_P(TestConcat3D_bf16, concat_test_bf16, cases_3D());
+CPU_INSTANTIATE_TEST_SUITE_P(TestConcat3D_f16, concat_test_float16, cases_3D());
 
 static auto cases_concat = []() {
     return ::testing::Values(
@@ -357,6 +405,8 @@ static auto cases_concat = []() {
 };
 INSTANTIATE_TEST_SUITE_P(TestConcat, concat_test_float, cases_concat());
 CPU_INSTANTIATE_TEST_SUITE_P(TestConcat_bf16, concat_test_bf16, cases_concat());
+CPU_INSTANTIATE_TEST_SUITE_P(
+        TestConcat_f16, concat_test_float16, cases_concat());
 
 INSTANTIATE_TEST_SUITE_P(TestConcat, concat_test_s8,
         ::testing::Values(
