@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2020 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,6 +14,8 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <memory>
+
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
@@ -37,8 +39,8 @@ void compute_bias_fwd(const test_convolution_sizes_t &c,
 
     const memory::desc bias_d = bias.get_desc();
     const memory::desc dst_d = dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper bias_mdw(bias_d.data);
-    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.data);
+    const dnnl::impl::memory_desc_wrapper bias_mdw(bias_d.get());
+    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.get());
 
     dnnl::impl::parallel_nd(c.mb, c.ng, c.oc / c.ng, c.oh, c.ow,
             [&](memory::dim n, memory::dim g, memory::dim oc, memory::dim oh,
@@ -60,8 +62,8 @@ void compute_bias_bwd(const test_convolution_sizes_t &c,
 
     const memory::desc bias_d = bias.get_desc();
     const memory::desc dst_d = dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper bias_mdw(bias_d.data);
-    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.data);
+    const dnnl::impl::memory_desc_wrapper bias_mdw(bias_d.get());
+    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.get());
 
     dnnl::impl::parallel_nd(
             c.ng, c.oc / c.ng, [&](memory::dim g, memory::dim oc) {
@@ -85,10 +87,10 @@ void transpose_wei(const test_convolution_sizes_t &c,
 
     auto weights_data = map_memory<data_t>(weights);
     const memory::desc weights_d = weights.get_desc();
-    const dnnl::impl::memory_desc_wrapper weights_mdw(weights_d.data);
+    const dnnl::impl::memory_desc_wrapper weights_mdw(weights_d.get());
     auto weights_tr_data = map_memory<data_t>(weights_tr);
     const memory::desc weights_tr_d = weights_tr.get_desc();
-    const dnnl::impl::memory_desc_wrapper weights_tr_mdw(weights_tr_d.data);
+    const dnnl::impl::memory_desc_wrapper weights_tr_mdw(weights_tr_d.get());
 
     dnnl::impl::parallel_nd(c.ng, c.oc / c.ng, c.ic / c.ng, c.kh, c.kw,
             [&](memory::dim g, memory::dim oc, memory::dim ic, memory::dim kh,
@@ -127,14 +129,91 @@ private:
     engine eng;
     stream strm;
     bool with_bias;
+    memory::dims padL;
     memory::dims padR;
+    memory::dims strides;
 
 protected:
     void SetUp() override {
+        memory::data_type data_type = data_traits<data_t>::data_type;
+        SKIP_IF(unsupported_data_type(data_type),
+                "Engine does not support this data type.");
+
         auto p = ::testing::TestWithParam<
                 deconvolution_test_params_t>::GetParam();
+
+        SKIP_IF_CUDA(
+                !(cuda_check_format_tags(p.formats.src_format, data_type)
+                        && cuda_check_format_tags(
+                                p.formats.dst_format, data_type)
+                        && cuda_check_src_wei_format_tags(p.formats.src_format,
+                                p.formats.weights_format, p.sizes.ng > 1)),
+                "Format is not supported.");
+
+        SKIP_IF_HIP(
+                !(hip_check_format_tags(p.formats.src_format)
+                        && hip_check_format_tags(p.formats.dst_format)
+                        && hip_check_src_wei_dst_format_tags(
+                                p.formats.src_format, p.formats.weights_format,
+                                p.formats.dst_format, p.sizes.ng > 1)),
+                "Format is not supported.");
+
+        SKIP_IF_GENERIC(
+                !(generic_check_format_tags(p.formats.src_format)
+                        && generic_check_format_tags(p.formats.dst_format)),
+                "Format is not supported.");
+
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status);
+    }
+
+    bool cuda_check_format_tags(memory::format_tag tag, memory::data_type dt) {
+        return ((impl::utils::one_of(tag, memory::format_tag::ab,
+                        memory::format_tag::abc, memory::format_tag::abcd,
+                        memory::format_tag::abcde, memory::format_tag::abcdef,
+                        memory::format_tag::acb, memory::format_tag::acdb,
+                        memory::format_tag::acdeb))
+                || (dt == memory::data_type::s8
+                        && impl::utils::one_of(tag, memory::format_tag::aBcd4b,
+                                memory::format_tag::aBcde4b)));
+    }
+
+    bool hip_check_format_tags(memory::format_tag tag) {
+        return impl::utils::one_of(tag, memory::format_tag::ab,
+                memory::format_tag::abc, memory::format_tag::abcd,
+                memory::format_tag::abcde, memory::format_tag::abcdef);
+    }
+
+    bool cuda_check_src_wei_format_tags(
+            memory::format_tag src, memory::format_tag wei, bool is_grouped) {
+        if (src == memory::format_tag::abcd) return true;
+        if (src == memory::format_tag::acdb)
+            return wei
+                    != (is_grouped ? memory::format_tag::abcde
+                                   : memory::format_tag::abcd);
+        return false;
+    }
+
+    bool hip_check_src_wei_dst_format_tags(memory::format_tag src,
+            memory::format_tag wei, memory::format_tag dst, bool is_grouped) {
+        if (src == memory::format_tag::abcd) {
+            return (src == dst)
+                    && (is_grouped ? (wei == memory::format_tag::acbde)
+                                   : (wei == memory::format_tag::bacd));
+        }
+        if (src == memory::format_tag::acdb) {
+            return (src == dst)
+                    && (is_grouped ? (wei == memory::format_tag::acdeb)
+                                   : (wei == memory::format_tag::bcda));
+        }
+        return false;
+    }
+
+    bool generic_check_format_tags(memory::format_tag tag) {
+        return impl::utils::one_of(tag, memory::format_tag::ab,
+                memory::format_tag::abc, memory::format_tag::abcd,
+                memory::format_tag::abcde, memory::format_tag::abcdef,
+                memory::format_tag::any);
     }
 
     void Test() {
@@ -167,37 +246,48 @@ protected:
         else
             bias_dims = {};
 
-        dec_src_desc.reset(
-                new memory::desc(src_dims, data_type, p.formats.src_format));
-        dec_dst_desc.reset(
-                new memory::desc(dst_dims, data_type, p.formats.src_format));
-        dec_weights_desc.reset(new memory::desc(
-                weights_dims, data_type, p.formats.weights_format));
-        dec_bias_desc.reset(
-                new memory::desc(bias_dims, data_type, p.formats.bias_format));
+        dec_src_desc = std::make_shared<memory::desc>(
+                src_dims, data_type, p.formats.src_format);
+        dec_dst_desc = std::make_shared<memory::desc>(
+                dst_dims, data_type, p.formats.src_format);
+        dec_weights_desc = std::make_shared<memory::desc>(
+                weights_dims, data_type, p.formats.weights_format);
+        dec_bias_desc = std::make_shared<memory::desc>(
+                bias_dims, data_type, p.formats.bias_format);
 
-        con_src_desc.reset(
-                new memory::desc(dst_dims, data_type, p.formats.src_format));
-        con_dst_desc.reset(
-                new memory::desc(src_dims, data_type, p.formats.src_format));
-        con_weights_desc.reset(new memory::desc(
-                c_weights_dims, data_type, p.formats.weights_format));
+        con_src_desc = std::make_shared<memory::desc>(
+                dst_dims, data_type, p.formats.src_format);
+        con_dst_desc = std::make_shared<memory::desc>(
+                src_dims, data_type, p.formats.src_format);
+        con_weights_desc = std::make_shared<memory::desc>(
+                c_weights_dims, data_type, p.formats.weights_format);
 
-        src.reset(new test_memory(*dec_src_desc, eng));
-        weights.reset(new test_memory(*dec_weights_desc, eng));
-        bias.reset(new test_memory(*dec_bias_desc, eng));
-        dst.reset(new test_memory(*dec_dst_desc, eng));
+        src = std::make_shared<test_memory>(*dec_src_desc, eng);
+        weights = std::make_shared<test_memory>(*dec_weights_desc, eng);
+        bias = std::make_shared<test_memory>(*dec_bias_desc, eng);
+        dst = std::make_shared<test_memory>(*dec_dst_desc, eng);
 
+        strides = {dd.strh, dd.strw};
+        padL = {dd.padh, dd.padw};
         padR = {right_padding(dd.oh, dd.ih, dd.kh, dd.padh, dd.strh, dd.dilh),
                 right_padding(dd.ow, dd.iw, dd.kw, dd.padw, dd.strw, dd.dilw)};
+        SKIP_IF_CUDA(p.sizes.padh < padR[0] || p.sizes.padw < padR[1],
+                "Padding not supported");
+        SKIP_IF_HIP(p.sizes.padh < padR[0] || p.sizes.padw < padR[1],
+                "Padding not supported");
         Forward();
         BackwardData();
         BackwardWeights();
     }
+
     void Forward() {
         auto aprop_kind = prop_kind::forward;
         deconvolution_test_params_t p = ::testing::TestWithParam<
                 deconvolution_test_params_t>::GetParam();
+
+        // deconvolution specific types and values
+        using pd_t = deconvolution_forward::primitive_desc;
+
         auto conv_src = test_memory(*con_src_desc, eng);
         auto conv_dst = src;
         test_convolution_sizes_t dd = p.sizes;
@@ -211,20 +301,45 @@ protected:
 
         auto weights_tr = test::make_memory(*con_weights_desc, eng);
         transpose_wei<data_t>(dd, weights->get(), weights_tr);
-        auto deconv_desc = with_bias
-                ? deconvolution_forward::desc(aprop_kind,
-                        algorithm::deconvolution_direct, *dec_src_desc,
-                        *dec_weights_desc, *dec_bias_desc, *dec_dst_desc,
-                        {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR)
-                : deconvolution_forward::desc(aprop_kind,
-                        algorithm::deconvolution_direct, *dec_src_desc,
-                        *dec_weights_desc, *dec_dst_desc, {dd.strh, dd.strw},
-                        {dd.padh, dd.padw}, padR);
+        auto deconv_primitive_desc = with_bias
+                ? pd_t(eng, aprop_kind, algorithm::deconvolution_direct,
+                        *dec_src_desc, *dec_weights_desc, *dec_bias_desc,
+                        *dec_dst_desc, strides, padL, padR)
+                : pd_t(eng, aprop_kind, algorithm::deconvolution_direct,
+                        *dec_src_desc, *dec_weights_desc, *dec_dst_desc,
+                        strides, padL, padR);
 
-        auto deconv_primitive_desc
-                = deconvolution_forward::primitive_desc(deconv_desc, eng);
-        deconv_primitive_desc = deconvolution_forward::primitive_desc(
-                deconv_primitive_desc.get()); // test construction from a C pd
+        auto aa = allows_attr_t {false};
+
+#ifndef DNNL_SYCL_GENERIC
+        aa.po_binary = !is_amd_gpu(eng);
+        aa.po_eltwise = !is_amd_gpu(eng);
+        aa.po_prelu = !is_amd_gpu(eng);
+        aa.po_sum = !is_amd_gpu(eng);
+#else
+        aa.po_eltwise = true;
+        aa.po_sum = true;
+        if (eng.get_kind() == dnnl::engine::kind::cpu) {
+            aa.po_binary = true;
+            aa.po_prelu = true;
+        }
+#endif
+
+        bool is_int8 = impl::utils::one_of(dec_src_desc->get_data_type(),
+                memory::data_type::s8, memory::data_type::u8);
+        if (is_int8) {
+            aa.scales = true;
+            aa.zp = true;
+        }
+        if (with_bias)
+            test_fwd_pd_constructors<pd_t>(deconv_primitive_desc, aa,
+                    aprop_kind, algorithm::deconvolution_direct, *dec_src_desc,
+                    *dec_weights_desc, *dec_bias_desc, *dec_dst_desc, strides,
+                    padL, padR);
+        else
+            test_fwd_pd_constructors<pd_t>(deconv_primitive_desc, aa,
+                    aprop_kind, algorithm::deconvolution_direct, *dec_src_desc,
+                    *dec_weights_desc, *dec_dst_desc, strides, padL, padR);
 
         ASSERT_TRUE(
                 deconv_primitive_desc.query_md(query::exec_arg_md, DNNL_ARG_SRC)
@@ -239,6 +354,14 @@ protected:
                             query::exec_arg_md, DNNL_ARG_BIAS)
                 == deconv_primitive_desc.bias_desc());
 
+        ASSERT_EQ(deconv_primitive_desc.get_algorithm(),
+                algorithm::deconvolution_direct);
+        ASSERT_EQ(deconv_primitive_desc.get_prop_kind(), aprop_kind);
+        ASSERT_EQ(deconv_primitive_desc.get_strides(), strides);
+        ASSERT_EQ(deconv_primitive_desc.get_padding_l(), padL);
+        ASSERT_EQ(deconv_primitive_desc.get_padding_r(), padR);
+
+        EXPECT_ANY_THROW(deconvolution_forward(deconv_primitive_desc, {}));
         deconvolution_forward(deconv_primitive_desc)
                 .execute(strm,
                         {{DNNL_ARG_SRC, src->get()},
@@ -247,20 +370,16 @@ protected:
                                 {DNNL_ARG_DST, dst->get()}});
         strm.wait();
 
-        auto conv_desc = convolution_forward::desc(prop_kind::forward_training,
-                algorithm::convolution_direct, *con_src_desc, *con_weights_desc,
-                *con_dst_desc, {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR);
-
-        auto conv_primitive_desc
-                = convolution_forward::primitive_desc(conv_desc, eng);
-
-        auto conv_bwd_data_desc = convolution_backward_data::desc(
-                algorithm::convolution_direct, *con_src_desc, *con_weights_desc,
-                *con_dst_desc, {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR);
+        auto conv_primitive_desc = convolution_forward::primitive_desc(eng,
+                prop_kind::forward_training, algorithm::convolution_direct,
+                *con_src_desc, *con_weights_desc, *con_dst_desc, strides, padL,
+                padR);
 
         auto conv_bwd_data_primitive_desc
-                = convolution_backward_data::primitive_desc(
-                        conv_bwd_data_desc, eng, conv_primitive_desc);
+                = convolution_backward_data::primitive_desc(eng,
+                        algorithm::convolution_direct, *con_src_desc,
+                        *con_weights_desc, *con_dst_desc, strides, padL, padR,
+                        conv_primitive_desc);
 
         convolution_backward_data(conv_bwd_data_primitive_desc)
                 .execute(strm,
@@ -277,6 +396,10 @@ protected:
     void BackwardData() {
         auto p = ::testing::TestWithParam<
                 deconvolution_test_params_t>::GetParam();
+        // deconv specific types and values
+        using pd_t = deconvolution_backward_data::primitive_desc;
+        using hint_pd_t = deconvolution_forward::primitive_desc;
+
         auto conv_src = dst;
         auto conv_dst = test_memory(*con_dst_desc, eng);
         test_convolution_sizes_t dd = p.sizes;
@@ -288,25 +411,20 @@ protected:
         auto weights_tr = test::make_memory(*con_weights_desc, eng);
         transpose_wei<data_t>(dd, weights->get(), weights_tr);
 
-        auto deconv_desc = deconvolution_forward::desc(
-                prop_kind::forward_training, algorithm::deconvolution_direct,
-                *dec_src_desc, *dec_weights_desc, *dec_dst_desc,
-                {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR);
-
-        auto deconv_primitive_desc
-                = deconvolution_forward::primitive_desc(deconv_desc, eng);
-
-        auto deconv_bwd_data_desc = deconvolution_backward_data::desc(
+        auto deconv_primitive_desc = hint_pd_t(eng, prop_kind::forward_training,
                 algorithm::deconvolution_direct, *dec_src_desc,
-                *dec_weights_desc, *dec_dst_desc, {dd.strh, dd.strw},
-                {dd.padh, dd.padw}, padR);
+                *dec_weights_desc, *dec_dst_desc, strides, padL, padR);
+
         auto deconv_bwd_data_primitive_desc
-                = deconvolution_backward_data::primitive_desc(
-                        deconv_bwd_data_desc, eng, deconv_primitive_desc);
-        deconv_bwd_data_primitive_desc
-                = deconvolution_backward_data::primitive_desc(
-                        deconv_bwd_data_primitive_desc
-                                .get()); // test construction from a C pd
+                = pd_t(eng, algorithm::deconvolution_direct, *dec_src_desc,
+                        *dec_weights_desc, *dec_dst_desc, strides, padL, padR,
+                        deconv_primitive_desc);
+
+        auto aa = allows_attr_t {false};
+        test_bwd_pd_constructors<pd_t, hint_pd_t>(
+                deconv_bwd_data_primitive_desc, deconv_primitive_desc, aa,
+                algorithm::deconvolution_direct, *dec_src_desc,
+                *dec_weights_desc, *dec_dst_desc, strides, padL, padR);
 
         ASSERT_TRUE(deconv_bwd_data_primitive_desc.query_md(
                             query::exec_arg_md, DNNL_ARG_DIFF_SRC)
@@ -318,6 +436,14 @@ protected:
                             query::exec_arg_md, DNNL_ARG_WEIGHTS)
                 == deconv_bwd_data_primitive_desc.weights_desc());
 
+        ASSERT_EQ(deconv_bwd_data_primitive_desc.get_algorithm(),
+                algorithm::deconvolution_direct);
+        ASSERT_EQ(deconv_bwd_data_primitive_desc.get_prop_kind(),
+                prop_kind::backward_data);
+        ASSERT_EQ(deconv_bwd_data_primitive_desc.get_strides(), strides);
+        ASSERT_EQ(deconv_bwd_data_primitive_desc.get_padding_l(), padL);
+        ASSERT_EQ(deconv_bwd_data_primitive_desc.get_padding_r(), padR);
+
         deconvolution_backward_data(deconv_bwd_data_primitive_desc)
                 .execute(strm,
                         {{DNNL_ARG_DIFF_DST, dst->get()},
@@ -325,12 +451,10 @@ protected:
                                 {DNNL_ARG_DIFF_SRC, src->get()}});
         strm.wait();
 
-        auto conv_desc = convolution_forward::desc(prop_kind::forward_training,
-                algorithm::convolution_direct, *con_src_desc, *con_weights_desc,
-                *con_dst_desc, {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR);
-
-        auto conv_primitive_desc
-                = convolution_forward::primitive_desc(conv_desc, eng);
+        auto conv_primitive_desc = convolution_forward::primitive_desc(eng,
+                prop_kind::forward_training, algorithm::convolution_direct,
+                *con_src_desc, *con_weights_desc, *con_dst_desc, strides, padL,
+                padR);
 
         convolution_forward(conv_primitive_desc)
                 .execute(strm,
@@ -345,6 +469,11 @@ protected:
     void BackwardWeights() {
         auto p = ::testing::TestWithParam<
                 deconvolution_test_params_t>::GetParam();
+
+        // deconv specific types and values
+        using pd_t = deconvolution_backward_weights::primitive_desc;
+        using hint_pd_t = deconvolution_forward::primitive_desc;
+
         auto conv_src = dst;
         auto conv_dst = src;
         auto conv_weights = test::make_memory(*con_weights_desc, eng);
@@ -354,21 +483,22 @@ protected:
 
         fill_data<data_t>(dst->get_size() / sizeof(data_t), dst->get());
 
-        auto deconv_desc = deconvolution_forward::desc(
-                prop_kind::forward_training, algorithm::deconvolution_direct,
-                *dec_src_desc, *dec_weights_desc, *dec_bias_desc, *dec_dst_desc,
-                {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR);
-
-        auto deconv_primitive_desc
-                = deconvolution_forward::primitive_desc(deconv_desc, eng);
-
-        auto deconv_bwd_weights_desc = deconvolution_backward_weights::desc(
+        auto deconv_primitive_desc = hint_pd_t(eng, prop_kind::forward_training,
                 algorithm::deconvolution_direct, *dec_src_desc,
                 *dec_weights_desc, *dec_bias_desc, *dec_dst_desc,
                 {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR);
+
         auto deconv_bwd_weights_primitive_desc
-                = deconvolution_backward_weights::primitive_desc(
-                        deconv_bwd_weights_desc, eng, deconv_primitive_desc);
+                = pd_t(eng, algorithm::deconvolution_direct, *dec_src_desc,
+                        *dec_weights_desc, *dec_bias_desc, *dec_dst_desc,
+                        strides, padL, padR, deconv_primitive_desc);
+
+        auto aa = allows_attr_t {false};
+        test_bwd_pd_constructors<pd_t, hint_pd_t>(
+                deconv_bwd_weights_primitive_desc, deconv_primitive_desc, aa,
+                algorithm::deconvolution_direct, *dec_src_desc,
+                *dec_weights_desc, *dec_bias_desc, *dec_dst_desc, strides, padL,
+                padR);
 
         ASSERT_TRUE(deconv_bwd_weights_primitive_desc.query_md(
                             query::exec_arg_md, DNNL_ARG_SRC)
@@ -383,6 +513,14 @@ protected:
                             query::exec_arg_md, DNNL_ARG_DIFF_BIAS)
                 == deconv_bwd_weights_primitive_desc.diff_bias_desc());
 
+        ASSERT_EQ(deconv_bwd_weights_primitive_desc.get_algorithm(),
+                algorithm::deconvolution_direct);
+        ASSERT_EQ(deconv_bwd_weights_primitive_desc.get_prop_kind(),
+                prop_kind::backward_weights);
+        ASSERT_EQ(deconv_bwd_weights_primitive_desc.get_strides(), strides);
+        ASSERT_EQ(deconv_bwd_weights_primitive_desc.get_padding_l(), padL);
+        ASSERT_EQ(deconv_bwd_weights_primitive_desc.get_padding_r(), padR);
+
         deconvolution_backward_weights(deconv_bwd_weights_primitive_desc)
                 .execute(strm,
                         {{DNNL_ARG_DIFF_DST, dst->get()},
@@ -391,24 +529,20 @@ protected:
                                 {DNNL_ARG_DIFF_BIAS, bias->get()}});
         strm.wait();
 
-        auto conv_desc = convolution_forward::desc(prop_kind::forward_training,
-                algorithm::convolution_direct, *con_src_desc, *con_weights_desc,
-                *con_dst_desc, {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR);
+        auto conv_primitive_desc = convolution_forward::primitive_desc(eng,
+                prop_kind::forward_training, algorithm::convolution_direct,
+                *con_src_desc, *con_weights_desc, *con_dst_desc, strides, padL,
+                padR);
 
-        auto conv_primitive_desc
-                = convolution_forward::primitive_desc(conv_desc, eng);
-
-        auto conv_bwd_weights_desc = convolution_backward_weights::desc(
-                algorithm::convolution_direct, *con_src_desc, *con_weights_desc,
-                *con_dst_desc, {dd.strh, dd.strw}, {dd.padh, dd.padw}, padR);
         deconv_bwd_weights_primitive_desc
-                = deconvolution_backward_weights::primitive_desc(
-                        deconv_bwd_weights_primitive_desc
+                = pd_t(deconv_bwd_weights_primitive_desc
                                 .get()); // test construction from a C pd
 
         auto conv_bwd_weights_primitive_desc
-                = convolution_backward_weights::primitive_desc(
-                        conv_bwd_weights_desc, eng, conv_primitive_desc);
+                = convolution_backward_weights::primitive_desc(eng,
+                        algorithm::convolution_direct, *con_src_desc,
+                        *con_weights_desc, *con_dst_desc, strides, padL, padR,
+                        conv_primitive_desc);
 
         convolution_backward_weights(conv_bwd_weights_primitive_desc)
                 .execute(strm,
@@ -522,8 +656,13 @@ GPU_INST_TEST_CASE(SimpleSmall_NCHW,
         PARAMS(nhwc, hwio, x, nhwc, 2, 1, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1),
         PARAMS(nhwc, hwio, x, nhwc, 2, 1, 6, 2, 2, 4, 4, 4, 3, 3, 0, 0, 1, 1),
         PARAMS(nhwc, goihw, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 0, 0, 1, 1),
-        PARAMS(nhwc, hwigo, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1)
-
-);
+        PARAMS(nhwc, hwigo, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1));
 
+GPU_INST_TEST_CASE(SimpleSmall_NHWC,
+        PARAMS(nchw, oihw, x, nhwc, 2, 1, 6, 2, 2, 4, 4, 4, 3, 3, 0, 0, 1, 1),
+        PARAMS(nhwc, ohwi, x, nhwc, 2, 1, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1),
+        PARAMS(nhwc, ohwi, x, nhwc, 2, 1, 6, 2, 2, 4, 4, 4, 3, 3, 0, 0, 1, 1),
+        PARAMS(nchw, goihw, x, nchw, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1),
+        PARAMS(nchw, goihw, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1),
+        PARAMS(nhwc, gohwi, x, nhwc, 2, 2, 6, 4, 4, 4, 4, 4, 3, 3, 1, 1, 1, 1));
 } // namespace dnnl
