@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -31,7 +31,7 @@ struct test_inner_product_descr_t {
 template <typename data_t>
 void compute_ref_inner_product_fwd(test_inner_product_descr_t ipd, memory &src,
         memory &weights, memory &bias, memory &dst) {
-    const bool w_bias = bias.get_desc().data.ndims != 0;
+    const bool w_bias = bias.get_desc().get_ndims() != 0;
     auto src_data = map_memory<data_t>(src);
     auto weights_data = map_memory<data_t>(weights);
     auto bias_data = w_bias ? map_memory<data_t>(bias) : nullptr;
@@ -41,10 +41,10 @@ void compute_ref_inner_product_fwd(test_inner_product_descr_t ipd, memory &src,
     const memory::desc weights_d = weights.get_desc();
     const memory::desc bias_d = bias.get_desc();
     const memory::desc dst_d = dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.data);
-    const dnnl::impl::memory_desc_wrapper weights_mdw(weights_d.data);
-    const dnnl::impl::memory_desc_wrapper bias_mdw(bias_d.data);
-    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.data);
+    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.get());
+    const dnnl::impl::memory_desc_wrapper weights_mdw(weights_d.get());
+    const dnnl::impl::memory_desc_wrapper bias_mdw(bias_d.get());
+    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.get());
 
     auto padded_ic = src_mdw.padded_dims()[1];
 
@@ -88,13 +88,71 @@ class inner_product_test_t
 protected:
     void SetUp() override {
         auto p = ::testing::TestWithParam<inprod_test_params_t>::GetParam();
+        SKIP_IF_CUDA(!cuda_generic_check_format_tags(p.src_format,
+                             p.weights_format, p.bias_format, p.dst_format),
+                "Unsupported format tag");
+        SKIP_IF_GENERIC(!cuda_generic_check_format_tags(p.src_format,
+                                p.weights_format, p.bias_format, p.dst_format),
+                "Unsupported format tag");
+        SKIP_IF_CUDA(p.ndims > 5, "Unsupported number of dimensions");
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status);
+    }
+
+    bool cuda_generic_check_format_tags(memory::format_tag src_format,
+            memory::format_tag wei_format, memory::format_tag bia_format,
+            memory::format_tag dst_format) {
+        bool src_ok = src_format == memory::format_tag::ncdhw
+                || src_format == memory::format_tag::ndhwc
+                || src_format == memory::format_tag::nchw
+                || src_format == memory::format_tag::nhwc
+                || src_format == memory::format_tag::ncw
+                || src_format == memory::format_tag::nwc
+                || src_format == memory::format_tag::nc
+                || src_format == memory::format_tag::any;
+        bool wei_ok = wei_format == memory::format_tag::oidhw
+                || wei_format == memory::format_tag::odhwi
+                || wei_format == memory::format_tag::dhwio
+                || wei_format == memory::format_tag::oihw
+                || wei_format == memory::format_tag::ohwi
+                || wei_format == memory::format_tag::hwio
+                || wei_format == memory::format_tag::oiw
+                || wei_format == memory::format_tag::owi
+                || wei_format == memory::format_tag::wio
+                || wei_format == memory::format_tag::io
+                || wei_format == memory::format_tag::oi
+                || wei_format == memory::format_tag::any;
+        bool bia_ok = bia_format == memory::format_tag::undef
+                || bia_format == memory::format_tag::any
+                || bia_format == memory::format_tag::a
+                || bia_format == memory::format_tag::x;
+        bool dst_ok = dst_format == memory::format_tag::any
+                || dst_format == memory::format_tag::nc;
+
+        return src_ok && wei_ok && bia_ok && dst_ok;
+    }
+
+    std::vector<int> get_dim_order(const memory::dims &strides) {
+        size_t ndims = strides.size();
+        std::vector<int> order(ndims);
+        for (size_t i = 0; i < ndims; ++i) {
+            order[i] = i;
+        }
+
+        std::sort(order.begin(), order.end(), [&strides](size_t i, size_t j) {
+            return strides[i] < strides[j];
+        });
+
+        return order;
     }
 
     void Test() {
         auto p = ::testing::TestWithParam<inprod_test_params_t>::GetParam();
         test_inner_product_descr_t ipd = p.test_ipd;
+
+        // ip specific types and values
+        using pd_t = inner_product_forward::primitive_desc;
+
         bool has_spatial = ipd.kh > 1 || ipd.kw > 1;
         if (p.ndims == 5) has_spatial = has_spatial || ipd.kd > 1;
         bool with_bias = p.bias_format != memory::format_tag::undef;
@@ -127,16 +185,28 @@ protected:
                 : create_md({}, data_type, p.bias_format);
         auto ip_dst_desc = create_md({ipd.mb, ipd.oc}, data_type, p.dst_format);
 
-        auto ip_desc = with_bias
-                ? inner_product_forward::desc(p.aprop_kind, ip_src_desc,
-                        ip_weights_desc, ip_bias_desc, ip_dst_desc)
-                : inner_product_forward::desc(p.aprop_kind, ip_src_desc,
-                        ip_weights_desc, ip_dst_desc);
+        SKIP_IF_GENERIC(get_dim_order(ip_src_desc.get_strides())
+                        != get_dim_order(ip_weights_desc.get_strides()),
+                "Unsupported case for generic");
 
-        auto ip_primitive_desc
-                = inner_product_forward::primitive_desc(ip_desc, eng);
-        ip_primitive_desc = inner_product_forward::primitive_desc(
-                ip_primitive_desc.get()); // test construction from a C pd
+        auto ip_primitive_desc = with_bias
+                ? pd_t(eng, p.aprop_kind, ip_src_desc, ip_weights_desc,
+                        ip_bias_desc, ip_dst_desc)
+                : pd_t(eng, p.aprop_kind, ip_src_desc, ip_weights_desc,
+                        ip_dst_desc);
+
+        auto aa = allows_attr_t {false};
+        aa.po_eltwise = true;
+        aa.po_sum = true;
+#ifdef DNNL_SYCL_GENERIC
+        aa.po_binary = true;
+        aa.po_prelu = true;
+#else
+        aa.po_binary = !is_nvidia_gpu(eng) && !is_amd_gpu(eng);
+        aa.po_prelu = !is_nvidia_gpu(eng) && !is_amd_gpu(eng);
+#endif
+        test_fwd_pd_constructors<pd_t>(ip_primitive_desc, aa, p.aprop_kind,
+                ip_src_desc, ip_weights_desc, ip_bias_desc, ip_dst_desc);
 
         auto ip_src = test::make_memory(ip_primitive_desc.src_desc(), eng);
         auto ip_weights
@@ -167,6 +237,9 @@ protected:
                 ip_primitive_desc.query_md(query::exec_arg_md, DNNL_ARG_BIAS)
                 == ip_primitive_desc.bias_desc());
 
+        ASSERT_EQ(ip_primitive_desc.get_prop_kind(), p.aprop_kind);
+
+        EXPECT_ANY_THROW(inner_product_forward(ip_primitive_desc, {}));
         inner_product_forward(ip_primitive_desc)
                 .execute(strm,
                         {{DNNL_ARG_SRC, ip_src}, {DNNL_ARG_WEIGHTS, ip_weights},
@@ -240,7 +313,11 @@ INSTANTIATE_TEST_SUITE_P(TestInnerProductForwardNoBias_padded,
                 inprod_test_params_float {prop_kind::forward,
                         memory::format_tag::nChw8c, memory::format_tag::aBcd8b,
                         memory::format_tag::undef, memory::format_tag::nc,
-                        EXPAND_SIZES_2D(4, 10, 5, 5, 5)}));
+                        EXPAND_SIZES_2D(4, 10, 5, 5, 5)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::nChw4c, memory::format_tag::aBcd4b,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_2D(4, 16, 5, 5, 5)}));
 
 GPU_INSTANTIATE_TEST_SUITE_P(TestInnerProductForward_padded,
         inner_product_test_float,
@@ -274,18 +351,26 @@ INSTANTIATE_TEST_SUITE_P(TestInnerProductForwardNoBias,
                         memory::format_tag::any, memory::format_tag::any,
                         memory::format_tag::undef, memory::format_tag::any,
                         EXPAND_SIZES_2D(2, 512, 48, 2, 2)},
-                inprod_test_params_float {prop_kind::forward,
-                        memory::format_tag::nhwc, memory::format_tag::hwio,
-                        memory::format_tag::undef, memory::format_tag::nc,
-                        EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
                 inprod_test_params_float {prop_kind::forward,
                         memory::format_tag::nwc, memory::format_tag::wio,
                         memory::format_tag::undef, memory::format_tag::nc,
                         EXPAND_SIZES_1D(2, 32, 48, 5)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::nwc, memory::format_tag::owi,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_1D(2, 32, 48, 5)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::nwc, memory::format_tag::oiw,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_1D(2, 32, 48, 5)},
                 inprod_test_params_float {prop_kind::forward,
                         memory::format_tag::ncw, memory::format_tag::oiw,
                         memory::format_tag::undef, memory::format_tag::nc,
                         EXPAND_SIZES_1D(2, 32, 48, 5)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::ncw, memory::format_tag::owi,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_1D(2, 32, 48, 5)},
                 inprod_test_params_float {prop_kind::forward,
                         memory::format_tag::ncw, memory::format_tag::wio,
                         memory::format_tag::undef, memory::format_tag::nc,
@@ -294,6 +379,10 @@ INSTANTIATE_TEST_SUITE_P(TestInnerProductForwardNoBias,
                         memory::format_tag::nhwc, memory::format_tag::hwio,
                         memory::format_tag::undef, memory::format_tag::nc,
                         EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::nhwc, memory::format_tag::ohwi,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
                 inprod_test_params_float {prop_kind::forward,
                         memory::format_tag::nhwc, memory::format_tag::oihw,
                         memory::format_tag::undef, memory::format_tag::nc,
@@ -302,6 +391,14 @@ INSTANTIATE_TEST_SUITE_P(TestInnerProductForwardNoBias,
                         memory::format_tag::nchw, memory::format_tag::oihw,
                         memory::format_tag::undef, memory::format_tag::nc,
                         EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::nchw, memory::format_tag::hwio,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::nchw, memory::format_tag::ohwi,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_2D(2, 32, 48, 6, 6)},
                 inprod_test_params_float {prop_kind::forward,
                         memory::format_tag::nChw8c, memory::format_tag::aBcd8b,
                         memory::format_tag::undef, memory::format_tag::nc,
@@ -343,6 +440,14 @@ INSTANTIATE_TEST_SUITE_P(TestInnerProductForward3D, inner_product_test_float,
                         memory::format_tag::any, memory::format_tag::any,
                         memory::format_tag::undef, memory::format_tag::any,
                         EXPAND_SIZES_3D(2, 32, 48, 6, 6, 6)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::ncdhw, memory::format_tag::dhwio,
+                        memory::format_tag::x, memory::format_tag::nc,
+                        EXPAND_SIZES_3D(2, 32, 48, 3, 5, 7)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::ncdhw, memory::format_tag::odhwi,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_3D(2, 32, 48, 2, 4, 6)},
                 inprod_test_params_float {prop_kind::forward,
                         memory::format_tag::ncdhw, memory::format_tag::oidhw,
                         memory::format_tag::undef, memory::format_tag::nc,
@@ -360,7 +465,15 @@ INSTANTIATE_TEST_SUITE_P(TestInnerProductForward3D, inner_product_test_float,
                 inprod_test_params_float {prop_kind::forward,
                         memory::format_tag::ndhwc, memory::format_tag::dhwio,
                         memory::format_tag::undef, memory::format_tag::nc,
-                        EXPAND_SIZES_3D(2, 16, 48, 3, 3, 3)}));
+                        EXPAND_SIZES_3D(2, 16, 48, 3, 3, 3)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::ndhwc, memory::format_tag::odhwi,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_3D(2, 16, 48, 3, 4, 5)},
+                inprod_test_params_float {prop_kind::forward,
+                        memory::format_tag::ndhwc, memory::format_tag::oidhw,
+                        memory::format_tag::undef, memory::format_tag::nc,
+                        EXPAND_SIZES_3D(2, 16, 48, 3, 5, 4)}));
 
 INSTANTIATE_TEST_SUITE_P(TestInnerProductForward, inner_product_test_float,
         ::testing::Values(
