@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
+* Copyright 2022-2023 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -53,205 +54,17 @@ bool cuda_check_format_tags(memory::format_tag format) {
     return format_ok;
 }
 
-template <typename data_t>
-void check_pool_fwd(
-        const pool_bwd_test_params_t &p, const memory &src, const memory &dst) {
-    auto src_data = map_memory<data_t>(src);
-    auto dst_data = map_memory<data_t>(dst);
-
-    const memory::desc src_d = src.get_desc();
-    const memory::desc dst_d = dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.data);
-    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.data);
-
-    auto pd = p.test_pd;
-    auto padded_c = src_d.data.padded_dims[1];
-
-    dnnl::impl::parallel_nd(pd.mb, pd.c, pd.od, pd.oh, pd.ow,
-            [&](memory::dim n, memory::dim c, memory::dim od, memory::dim oh,
-                    memory::dim ow) {
-                if (is_current_test_failed()) return;
-
-                memory::dim oidx = n * padded_c * pd.od * pd.oh * pd.ow
-                        + c * pd.od * pd.oh * pd.ow + od * pd.oh * pd.ow
-                        + oh * pd.ow + ow;
-                data_t out = dst_data[dst_mdw.off_l(oidx, true)];
-
-                // match implementation for pooling_max: padding
-                // is done with lowest value and not zero, it
-                // affects the case when kernel slips into
-                // the padding area entirely
-                data_t out_ref = (p.aalgorithm == algorithm::pooling_max)
-                        ? std::numeric_limits<data_t>::lowest()
-                        : data_t(0);
-                bool is_initialized = false;
-                int num_summands = 0;
-
-                for_(memory::dim kd = 0; kd < pd.kd; ++kd)
-                for_(memory::dim kh = 0; kh < pd.kh; ++kh)
-                for (memory::dim kw = 0; kw < pd.kw; ++kw) {
-                    const memory::dim id
-                            = od * pd.strd - pd.padf + kd * (pd.dd + 1);
-                    const memory::dim ih
-                            = oh * pd.strh - pd.padt + kh * (pd.dh + 1);
-                    const memory::dim iw
-                            = ow * pd.strw - pd.padl + kw * (pd.dw + 1);
-
-                    if (id < 0 || id >= pd.id) continue;
-                    if (ih < 0 || ih >= pd.ih) continue;
-                    if (iw < 0 || iw >= pd.iw) continue;
-
-                    size_t iidx = (size_t)n * padded_c * pd.id * pd.ih * pd.iw
-                            + (size_t)c * pd.id * pd.ih * pd.iw
-                            + (size_t)id * pd.ih * pd.iw + (size_t)ih * pd.iw
-                            + iw;
-
-                    data_t d = src_data[src_mdw.off_l(iidx, true)];
-                    if (p.aalgorithm == algorithm::pooling_max) {
-                        if (!is_initialized) {
-                            out_ref = d;
-                            is_initialized = true;
-                        } else {
-                            if (out_ref < d) out_ref = d;
-                        }
-                    } else if (p.aalgorithm
-                                    == algorithm::pooling_avg_include_padding
-                            || p.aalgorithm
-                                    == algorithm::pooling_avg_exclude_padding) {
-                        out_ref += d;
-                        num_summands++;
-                    }
-                }
-                if (p.aalgorithm == algorithm::pooling_avg_include_padding)
-                    num_summands = pd.kd * pd.kh * pd.kw;
-                if ((p.aalgorithm == algorithm::pooling_avg_include_padding
-                            || p.aalgorithm
-                                    == algorithm::pooling_avg_exclude_padding)
-                        && num_summands) {
-                    out_ref /= num_summands;
-                }
-                ASSERT_NEAR(out, out_ref, 1e-6f);
-            });
+bool hip_check_format_tags(memory::format_tag format) {
+    bool format_ok = format == memory::format_tag::nchw
+            || format == memory::format_tag::ncdhw;
+    return format_ok;
 }
 
-template <typename data_t>
-void check_pool_bwd(const pool_bwd_test_params_t &p, const memory &diff_src,
-        const memory &diff_dst, const memory &ws) {
-    auto diff_src_data = map_memory<data_t>(diff_src);
-    auto diff_dst_data = map_memory<data_t>(diff_dst);
-
-    auto ws_data_ptr = map_memory<unsigned char>(ws);
-
-    auto ws_data = [&](size_t idx) -> int {
-        auto w = (const unsigned char *)ws_data_ptr;
-        if (w == nullptr) return -1;
-        if (ws.get_desc().data.data_type == dnnl_u8)
-            return (int)w[idx];
-        else
-            return ((const int *)w)[idx];
-    };
-
-    const memory::desc diff_src_d = diff_src.get_desc();
-    const memory::desc diff_dst_d = diff_dst.get_desc();
-    const memory::desc ws_d = ws.get_desc();
-
-    const dnnl::impl::memory_desc_wrapper diff_src_mdw(diff_src_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.data);
-    const dnnl::impl::memory_desc_wrapper ws_mdw(ws_d.data);
-
-    auto pd = p.test_pd;
-    if (pd.mb * pd.c * pd.id * pd.ih * pd.iw == 0) return;
-
-    std::vector<data_t> ref_diff_src_vec(pd.mb * pd.c * pd.id * pd.ih * pd.iw);
-    data_t *ref_diff_src = &ref_diff_src_vec[0];
-
-    dnnl::impl::parallel_nd(pd.mb * pd.c * pd.id * pd.ih * pd.iw,
-            [&](memory::dim i) { ref_diff_src[i] = 0.; });
-
-    dnnl::impl::parallel_nd(pd.mb, pd.c, [&](memory::dim n, memory::dim c) {
-        for_(memory::dim od = 0; od < pd.od; od++)
-        for_(memory::dim oh = 0; oh < pd.oh; oh++)
-        for (memory::dim ow = 0; ow < pd.ow; ow++) {
-            memory::dim oidx = n * pd.c * pd.od * pd.oh * pd.ow
-                    + c * pd.od * pd.oh * pd.ow + od * pd.oh * pd.ow
-                    + oh * pd.ow + ow;
-            data_t diff_dst = diff_dst_data[diff_dst_mdw.off_l(oidx, true)];
-            for_(memory::dim kd = 0; kd < pd.kd; kd++)
-            for_(memory::dim kh = 0; kh < pd.kh; kh++)
-            for (memory::dim kw = 0; kw < pd.kw; kw++) {
-                memory::dim iw = ow * pd.strw - pd.padl + kw * (pd.dw + 1);
-                memory::dim ih = oh * pd.strh - pd.padt + kh * (pd.dh + 1);
-                memory::dim id = od * pd.strd - pd.padf + kd * (pd.dd + 1);
-                if (iw < 0 || iw >= pd.iw) continue;
-                if (ih < 0 || ih >= pd.ih) continue;
-                if (id < 0 || id >= pd.id) continue;
-                memory::dim iidx = n * pd.c * pd.id * pd.ih * pd.iw
-                        + c * pd.id * pd.ih * pd.iw + id * pd.ih * pd.iw
-                        + ih * pd.iw + iw;
-                if (p.aalgorithm == algorithm::pooling_max) {
-                    memory::dim kw_max
-                            = ws_data(ws_mdw.off_l(oidx, true)) % pd.kw;
-                    memory::dim kh_max
-                            = (ws_data(ws_mdw.off_l(oidx, true)) / pd.kw)
-                            % pd.kh;
-                    memory::dim kd_max
-                            = (ws_data(ws_mdw.off_l(oidx, true)) / pd.kw)
-                            / pd.kh;
-                    if (kh == kh_max && kw == kw_max && kd == kd_max)
-                        ref_diff_src[iidx] += diff_dst;
-                } else {
-                    auto id_start = od * pd.strd - pd.padf;
-                    auto ih_start = oh * pd.strh - pd.padt;
-                    auto iw_start = ow * pd.strw - pd.padl;
-                    auto id_end = od * pd.strd - pd.padf + (pd.kd - 1) * pd.dd
-                            + pd.kd;
-                    auto ih_end = oh * pd.strh - pd.padt + (pd.kh - 1) * pd.dh
-                            + pd.kh;
-                    auto iw_end = ow * pd.strw - pd.padl + (pd.kw - 1) * pd.dw
-                            + pd.kw;
-
-                    auto id_start_excluded = id_start < 0
-                            ? (0 - id_start - 1) / (pd.dd + 1) + 1
-                            : 0;
-                    auto ih_start_excluded = ih_start < 0
-                            ? (0 - ih_start - 1) / (pd.dh + 1) + 1
-                            : 0;
-                    auto iw_start_excluded = iw_start < 0
-                            ? (0 - iw_start - 1) / (pd.dw + 1) + 1
-                            : 0;
-                    auto id_end_excluded = id_end > pd.id
-                            ? (id_end - pd.id - 1) / (pd.dd + 1) + 1
-                            : 0;
-                    auto ih_end_excluded = ih_end > pd.ih
-                            ? (ih_end - pd.ih - 1) / (pd.dh + 1) + 1
-                            : 0;
-                    auto iw_end_excluded = iw_end > pd.iw
-                            ? (iw_end - pd.iw - 1) / (pd.dw + 1) + 1
-                            : 0;
-
-                    auto num_summands
-                            = (p.aalgorithm
-                                      != algorithm::pooling_avg_exclude_padding)
-                            ? pd.kw * pd.kh * pd.kd
-                            : (pd.kd - id_start_excluded - id_end_excluded)
-                                    * (pd.kh - ih_start_excluded
-                                            - ih_end_excluded)
-                                    * (pd.kw - iw_start_excluded
-                                            - iw_end_excluded);
-
-                    ref_diff_src[iidx] += diff_dst / num_summands;
-                }
-            }
-        }
-    });
-
-    dnnl::impl::parallel_nd(
-            pd.mb * pd.c * pd.id * pd.ih * pd.iw, [&](memory::dim i) {
-                if (is_current_test_failed()) return;
-
-                ASSERT_NEAR(ref_diff_src[i],
-                        diff_src_data[diff_src_mdw.off_l(i, true)], 1e-5f);
-            });
+bool generic_check_format_tags(memory::format_tag format) {
+    return impl::utils::one_of(format, memory::format_tag::a,
+            memory::format_tag::nchw, memory::format_tag::ncdhw,
+            memory::format_tag::nhwc, memory::format_tag::ndhwc,
+            memory::format_tag::any);
 }
 
 template <typename data_t>
@@ -261,20 +74,12 @@ private:
     std::shared_ptr<memory::desc> src_desc;
     std::shared_ptr<memory::desc> dst_desc;
     memory workspace;
-    union prim_desc_union {
-        pooling_forward::primitive_desc pool_prim_desc;
-        pooling_v2_forward::primitive_desc pool_v2_prim_desc;
-        prim_desc_union() {
-            new (&pool_v2_prim_desc) pooling_v2_forward::primitive_desc();
-        }
-        ~prim_desc_union() { pool_v2_prim_desc.~primitive_desc(); }
-    } prim_desc;
+    pooling_forward::primitive_desc pool_prim_desc;
     pool_bwd_test_params_t p;
     memory::dims strides, ker, dilation, pad_l, pad_r;
     engine eng;
     stream strm;
     memory::data_type data_type;
-    bool is_not_dilated;
 
 protected:
     void SetUp() override {
@@ -283,10 +88,29 @@ protected:
                 "Unsupported format tag");
         SKIP_IF_CUDA(!cuda_check_format_tags(p.diff_dst_format),
                 "Unsupported format tag");
+        // This test makes assumptions on workspace content for the max
+        // algorithm therefore it cannot be used for non-intel implementations.
         SKIP_IF_CUDA(p.aalgorithm == algorithm::pooling_max,
-                "Unsupported algorithm MAX");
+                "Test is not designed to test non-intel implementations of max "
+                "algorithm");
+
+        SKIP_IF_HIP(!hip_check_format_tags(p.diff_src_format),
+                "Unsupported format tag");
+        SKIP_IF_HIP(!hip_check_format_tags(p.diff_dst_format),
+                "Unsupported format tag");
+        // This test makes assumptions on workspace content for the max
+        // algorithm therefore it cannot be used for non-intel implementations.
+        SKIP_IF_HIP(p.aalgorithm == algorithm::pooling_max,
+                "Test is not designed to test non-intel implementations of max "
+                "algorithm");
+
+        SKIP_IF_GENERIC(!generic_check_format_tags(p.diff_src_format),
+                "Unsupported format tag");
+        SKIP_IF_GENERIC(!generic_check_format_tags(p.diff_dst_format),
+                "Unsupported format tag");
+
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
     void Test() {
@@ -298,15 +122,19 @@ protected:
         ASSERT_EQ(data_type, dnnl::memory::data_type::f32);
 
         if (p.ndims == 5) {
-            src_desc.reset(new memory::desc({pd.mb, pd.c, pd.id, pd.ih, pd.iw},
-                    data_type, p.diff_src_format));
-            dst_desc.reset(new memory::desc({pd.mb, pd.c, pd.od, pd.oh, pd.ow},
-                    data_type, p.diff_dst_format));
+            auto src_dims = {pd.mb, pd.c, pd.id, pd.ih, pd.iw};
+            auto dst_dims = {pd.mb, pd.c, pd.od, pd.oh, pd.ow};
+            src_desc = std::make_shared<memory::desc>(
+                    src_dims, data_type, p.diff_src_format);
+            dst_desc = std::make_shared<memory::desc>(
+                    dst_dims, data_type, p.diff_dst_format);
         } else {
-            src_desc.reset(new memory::desc(
-                    {pd.mb, pd.c, pd.ih, pd.iw}, data_type, p.diff_src_format));
-            dst_desc.reset(new memory::desc(
-                    {pd.mb, pd.c, pd.oh, pd.ow}, data_type, p.diff_dst_format));
+            auto src_dims = {pd.mb, pd.c, pd.ih, pd.iw};
+            auto dst_dims = {pd.mb, pd.c, pd.oh, pd.ow};
+            src_desc = std::make_shared<memory::desc>(
+                    src_dims, data_type, p.diff_src_format);
+            dst_desc = std::make_shared<memory::desc>(
+                    dst_dims, data_type, p.diff_dst_format);
         }
 
         if (p.ndims == 5) {
@@ -330,14 +158,12 @@ protected:
                             pd.iw, pd.ow, pd.kw, pd.padl, pd.strw, pd.dw)});
         }
 
-        is_not_dilated = pd.dd == 0 && pd.dh == 0 && pd.dw == 0;
-
         Forward();
         Backward();
     }
 
-    template <typename prim_desc>
-    void check_prim_desc(prim_desc pool_bwd_prim_desc) {
+    void check_prim_desc(
+            const pooling_backward::primitive_desc &pool_bwd_prim_desc) {
         ASSERT_TRUE(pool_bwd_prim_desc.query_md(
                             query::exec_arg_md, DNNL_ARG_DIFF_SRC)
                 == pool_bwd_prim_desc.diff_src_desc());
@@ -347,6 +173,19 @@ protected:
         ASSERT_TRUE(pool_bwd_prim_desc.query_md(
                             query::exec_arg_md, DNNL_ARG_WORKSPACE)
                 == pool_bwd_prim_desc.workspace_desc());
+
+        ASSERT_EQ(pool_bwd_prim_desc.get_prop_kind(), prop_kind::backward_data);
+        ASSERT_EQ(pool_bwd_prim_desc.get_algorithm(), p.aalgorithm);
+        ASSERT_EQ(pool_bwd_prim_desc.get_kernel(), ker);
+        ASSERT_EQ(pool_bwd_prim_desc.get_strides(), strides);
+        ASSERT_EQ(pool_bwd_prim_desc.get_padding_l(), pad_l);
+        ASSERT_EQ(pool_bwd_prim_desc.get_padding_r(), pad_r);
+
+        if (p.test_pd.dd == 0 && p.test_pd.dh == 0 && p.test_pd.dw == 0)
+            ASSERT_EQ(pool_prim_desc.get_dilations(),
+                    memory::dims(pool_prim_desc.src_desc().get_ndims() - 2));
+        else
+            ASSERT_EQ(pool_prim_desc.get_dilations(), dilation);
     }
 
     void Forward() {
@@ -358,43 +197,29 @@ protected:
         check_zero_tail<data_t>(1, src);
         check_zero_tail<data_t>(1, dst);
 
-        if (is_not_dilated) {
-            auto pool_desc = pooling_forward::desc(prop_kind::forward_training,
-                    p.aalgorithm, *src_desc, *dst_desc, strides, ker, pad_l,
-                    pad_r);
-            prim_desc.pool_prim_desc
-                    = pooling_forward::primitive_desc(pool_desc, eng);
+        pool_prim_desc = pooling_forward::primitive_desc(eng,
+                prop_kind::forward_training, p.aalgorithm, *src_desc, *dst_desc,
+                strides, ker, dilation, pad_l, pad_r);
 
-            auto p_workspace_desc = prim_desc.pool_prim_desc.workspace_desc();
-            workspace = test::make_memory(p_workspace_desc, eng);
+        auto p_workspace_desc = pool_prim_desc.workspace_desc();
+        workspace = test::make_memory(p_workspace_desc, eng);
 
-            pooling_forward(prim_desc.pool_prim_desc)
-                    .execute(strm,
-                            {{DNNL_ARG_SRC, src}, {DNNL_ARG_DST, dst},
-                                    {DNNL_ARG_WORKSPACE, workspace}});
-        } else {
-            auto pool_desc = pooling_v2_forward::desc(
-                    prop_kind::forward_training, p.aalgorithm, *src_desc,
-                    *dst_desc, strides, ker, dilation, pad_l, pad_r);
-            prim_desc.pool_v2_prim_desc
-                    = pooling_v2_forward::primitive_desc(pool_desc, eng);
+        EXPECT_ANY_THROW(pooling_forward(pool_prim_desc, {}));
+        pooling_forward(pool_prim_desc)
+                .execute(strm,
+                        {{DNNL_ARG_SRC, src}, {DNNL_ARG_DST, dst},
+                                {DNNL_ARG_WORKSPACE, workspace}});
 
-            auto p_workspace_desc
-                    = prim_desc.pool_v2_prim_desc.workspace_desc();
-            workspace = test::make_memory(p_workspace_desc, eng);
-
-            pooling_v2_forward(prim_desc.pool_v2_prim_desc)
-                    .execute(strm,
-                            {{DNNL_ARG_SRC, src}, {DNNL_ARG_DST, dst},
-                                    {DNNL_ARG_WORKSPACE, workspace}});
-        }
         strm.wait();
 
         check_zero_tail<data_t>(0, dst);
-        check_pool_fwd<data_t>(p, src, dst);
     }
 
     void Backward() {
+        // pooling specific types and values
+        using pd_t = pooling_backward::primitive_desc;
+        using hint_pd_t = pooling_forward::primitive_desc;
+
         auto diff_src = test::make_memory(*src_desc, eng);
         auto diff_dst = test::make_memory(*dst_desc, eng);
 
@@ -405,42 +230,23 @@ protected:
         check_zero_tail<data_t>(1, diff_dst);
         check_zero_tail<data_t>(1, diff_src);
 
-        if (is_not_dilated) {
-            auto pool_bwd_desc = pooling_backward::desc(p.aalgorithm, *src_desc,
-                    *dst_desc, strides, ker, pad_l, pad_r);
-            auto pool_bwd_prim_desc = pooling_backward::primitive_desc(
-                    pool_bwd_desc, eng, prim_desc.pool_prim_desc);
-            pool_bwd_prim_desc = pooling_backward::primitive_desc(
-                    pool_bwd_prim_desc.get()); // test construction from a C pd
-
-            check_prim_desc(pool_bwd_prim_desc);
-
-            pooling_backward(pool_bwd_prim_desc)
-                    .execute(strm,
-                            {{DNNL_ARG_DIFF_DST, diff_dst},
-                                    {DNNL_ARG_DIFF_SRC, diff_src},
-                                    {DNNL_ARG_WORKSPACE, workspace}});
-        } else {
-            auto pool_bwd_desc = pooling_v2_backward::desc(p.aalgorithm,
-                    *src_desc, *dst_desc, strides, ker, dilation, pad_l, pad_r);
-            auto pool_bwd_prim_desc = pooling_v2_backward::primitive_desc(
-                    pool_bwd_desc, eng, prim_desc.pool_v2_prim_desc);
-            pool_bwd_prim_desc = pooling_v2_backward::primitive_desc(
-                    pool_bwd_prim_desc.get()); // test construction from a C pd
-
-            check_prim_desc(pool_bwd_prim_desc);
-
-            pooling_v2_backward(pool_bwd_prim_desc)
-                    .execute(strm,
-                            {{DNNL_ARG_DIFF_DST, diff_dst},
-                                    {DNNL_ARG_DIFF_SRC, diff_src},
-                                    {DNNL_ARG_WORKSPACE, workspace}});
-        }
+        auto pool_bwd_prim_desc = pd_t(eng, p.aalgorithm, *src_desc, *dst_desc,
+                strides, ker, dilation, pad_l, pad_r, pool_prim_desc);
+        // test all pd ctors
+        allows_attr_t aa {false}; // doesn't support anything
+        test_bwd_pd_constructors<pd_t, hint_pd_t>(pool_bwd_prim_desc,
+                pool_prim_desc, aa, p.aalgorithm, *src_desc, *dst_desc, strides,
+                ker, dilation, pad_l, pad_r);
+        check_prim_desc(pool_bwd_prim_desc);
 
+        pooling_backward(pool_bwd_prim_desc)
+                .execute(strm,
+                        {{DNNL_ARG_DIFF_DST, diff_dst},
+                                {DNNL_ARG_DIFF_SRC, diff_src},
+                                {DNNL_ARG_WORKSPACE, workspace}});
         strm.wait();
 
         check_zero_tail<data_t>(0, diff_src);
-        check_pool_bwd<data_t>(p, diff_src, diff_dst, workspace);
     }
 };
 
@@ -460,6 +266,11 @@ TEST_P(pooling_bwd_test_float, TestsPoolingBackward) {}
 
 INSTANTIATE_TEST_SUITE_P(TestPoolingBackwardZeroDim, pooling_bwd_test_float,
         ::testing::Values(
+                pool_bwd_test_params_float {
+                        algorithm::pooling_avg_include_padding,
+                        memory::format_tag::nchw, memory::format_tag::nchw,
+                        EXPAND_SIZES_2D(
+                                2, 0, 4, 4, 4, 4, 3, 3, 0, 0, 1, 1, 1, 1)},
                 pool_bwd_test_params_float {algorithm::pooling_max,
                         memory::format_tag::nchw, memory::format_tag::nchw,
                         EXPAND_SIZES_2D(
