@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -51,14 +51,23 @@ protected:
                 "Engine does not support this data type.");
         SKIP_IF(get_test_engine().get_kind() != engine::kind::cpu,
                 "Engine does not support this primitive.");
+        SKIP_IF_CUDA(p.aalgorithm != algorithm::reduction_max
+                        && p.aalgorithm != algorithm::reduction_min
+                        && p.aalgorithm != algorithm::reduction_sum
+                        && p.aalgorithm != algorithm::reduction_mul
+                        && p.aalgorithm != algorithm::reduction_mean
+                        && p.aalgorithm != algorithm::reduction_norm_lp_max
+                        && p.aalgorithm
+                                != algorithm::reduction_norm_lp_power_p_max
+                        && p.eps != 0.0f,
+                "Unsupported algorithm type for CUDA");
 
         catch_expected_failures(
-                [=]() { Test(); }, p.expect_to_fail, p.expected_status);
+                [&]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
 
     void Test() {
         // reduction specific types and values
-        using op_desc_t = reduction::desc;
         using pd_t = reduction::primitive_desc;
         allows_attr_t allowed_attributes {false}; // doesn't support anything
         allowed_attributes.po_sum = true;
@@ -71,19 +80,15 @@ protected:
         auto desc_src = memory::desc(p.src_dims, src_dt, p.src_format);
         auto desc_dst = memory::desc(p.dst_dims, dst_dt, p.dst_format);
 
-        // default op desc ctor
-        auto op_desc = op_desc_t();
-        // regular op desc ctor
-        op_desc = op_desc_t(p.aalgorithm, desc_src, desc_dst, p.p, p.eps);
-
         // default pd ctor
         auto pd = pd_t();
         // regular pd ctor
-        ASSERT_NO_THROW(pd = pd_t(op_desc, eng));
+        pd = pd_t(eng, p.aalgorithm, desc_src, desc_dst, p.p, p.eps);
         // test all pd ctors
-        test_fwd_pd_constructors<op_desc_t, pd_t>(
-                op_desc, pd, allowed_attributes);
+        test_fwd_pd_constructors<pd_t>(pd, allowed_attributes, p.aalgorithm,
+                desc_src, desc_dst, p.p, p.eps);
 
+        EXPECT_ANY_THROW(reduction(pd, {}));
         // default primitive ctor
         auto prim = reduction();
         // regular primitive ctor
@@ -95,6 +100,10 @@ protected:
         ASSERT_TRUE(pd.query_md(query::exec_arg_md, DNNL_ARG_SRC) == src_desc);
         ASSERT_TRUE(pd.query_md(query::exec_arg_md, DNNL_ARG_DST) == dst_desc);
 
+        ASSERT_EQ(pd.get_algorithm(), p.aalgorithm);
+        ASSERT_EQ(pd.get_p(), p.p);
+        ASSERT_EQ(pd.get_epsilon(), p.eps);
+
         const auto test_engine = pd.get_engine();
 
         auto mem_src = memory(src_desc, test_engine);
@@ -127,7 +136,11 @@ static auto expected_failures = []() {
             // not supported p
             reduction_test_params_t {tag::nchw, tag::nchw,
                     algorithm::reduction_norm_lp_max, 0.5f, 0.0f, {1, 8, 4, 4},
-                    {1, 8, 4, 4}, true, dnnl_invalid_arguments});
+                    {1, 8, 4, 4}, true, dnnl_invalid_arguments},
+            // invalid tag
+            reduction_test_params_t {tag::any, tag::nchw,
+                    algorithm::reduction_sum, 0.0f, 0.0f, {1, 1, 1, 4},
+                    {1, 1, 1, 1}, true, dnnl_invalid_arguments});
 };
 
 static auto zero_dim = []() {
