@@ -1,5 +1,6 @@
 /*******************************************************************************
-* Copyright 2016-2020 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
+* Copyright 2023 Arm Ltd. and affiliates
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -26,20 +27,26 @@
 
 namespace dnnl {
 
+using f32_bf16 = std::pair<float, bfloat16_t>;
 using f32_f32 = std::pair<float, float>;
 using s32_s32 = std::pair<int32_t, int32_t>;
 using s8_s8 = std::pair<int8_t, int8_t>;
 
+using cfg_bf16 = test_simple_params<f32_bf16>;
 using cfg_f32 = test_simple_params<f32_f32>;
 using cfg_s32 = test_simple_params<s32_s32>;
 using cfg_s8 = test_simple_params<s8_s8>;
 
+using reorder_simple_test_f32_bf16 = reorder_simple_test<f32_bf16>;
 using reorder_simple_test_f32_f32 = reorder_simple_test<f32_f32>;
 using reorder_simple_test_s32_s32 = reorder_simple_test<s32_s32>;
 using reorder_simple_test_s8_s8 = reorder_simple_test<s8_s8>;
 
 using fmt = memory::format_tag;
 
+TEST_P(reorder_simple_test_f32_bf16, TestsReorder) {
+    Test();
+}
 TEST_P(reorder_simple_test_s32_s32, TestsReorder) {
     Test();
 }
@@ -50,9 +57,23 @@ TEST_P(reorder_simple_test_s8_s8, TestsReorder) {
     Test();
 }
 
+INSTANTIATE_TEST_SUITE_P(ACLCases, reorder_simple_test_f32_bf16,
+        ::testing::Values(cfg_bf16 {fmt::ab, fmt::BA4b4a, {128, 128}},
+                cfg_bf16 {fmt::ab, fmt::BA8b4a, {128, 128}}));
+
+INSTANTIATE_TEST_SUITE_P(ACLCases, reorder_simple_test_f32_f32,
+        ::testing::Values(cfg_f32 {fmt::cdba, fmt::Acdb8a, {16, 16, 1, 1}},
+                cfg_f32 {fmt::cdba, fmt::Acdb8a, {1, 16, 1, 1}},
+                cfg_f32 {fmt::ba, fmt::Ab4a, {128, 128}},
+                cfg_f32 {fmt::ba, fmt::Ab8a, {128, 128}}));
+
 INSTANTIATE_TEST_SUITE_P(CornerCases, reorder_simple_test_f32_f32,
         ::testing::Values(cfg_f32 {fmt::nchw, fmt::nc, {2, 16, 8, 8}, true,
                                   dnnl_invalid_arguments},
+                cfg_f32 {fmt::any, fmt::nchw, {2, 28, 3, 4}, true,
+                        dnnl_invalid_arguments},
+                cfg_f32 {fmt::nchw, fmt::any, {2, 28, 3, 4}, true,
+                        dnnl_invalid_arguments},
                 cfg_f32 {fmt::nchw, fmt::nchw, {0, 16, 8, 8}},
                 cfg_f32 {fmt::nchw, fmt::nChw8c, {0, 5, 8, 8}},
                 cfg_f32 {fmt::nchw, fmt::nChw16c, {0, 5, 8, 8}},
@@ -102,10 +123,13 @@ CPU_INSTANTIATE_TEST_SUITE_P(PaddedWeights, reorder_simple_test_f32_f32,
                 // OIhw16i16o
                 cfg_f32 {fmt::oihw, fmt::OIhw16i16o, {17, 23, 2, 3}},
                 cfg_f32 {fmt::OIhw16i16o, fmt::oihw, {17, 23, 2, 3}},
-                cfg_f32 {fmt::oihw, fmt::OIhw16o16i, {17, 23, 2, 3}},
-                cfg_f32 {fmt::OIhw16o16i, fmt::oihw, {17, 23, 2, 3}},
                 cfg_f32 {fmt::hwio, fmt::OIhw16i16o, {17, 23, 2, 3}},
                 cfg_f32 {fmt::OIhw16i16o, fmt::hwio, {17, 23, 2, 3}},
+                // OhwI16i16o
+                cfg_f32 {fmt::oihw, fmt::OhwI16i16o, {17, 23, 2, 3}},
+                cfg_f32 {fmt::OhwI16i16o, fmt::oihw, {17, 23, 2, 3}},
+                cfg_f32 {fmt::hwio, fmt::OhwI16i16o, {17, 23, 2, 3}},
+                cfg_f32 {fmt::OhwI16i16o, fmt::hwio, {17, 23, 2, 3}},
                 // OIhw16o16i
                 cfg_f32 {fmt::oihw, fmt::OIhw16o16i, {17, 23, 2, 3}},
                 cfg_f32 {fmt::OIhw16o16i, fmt::oihw, {17, 23, 2, 3}},
@@ -136,21 +160,31 @@ CPU_INSTANTIATE_TEST_SUITE_P(PaddedWeights, reorder_simple_test_f32_f32,
                 // OIhw8i8o
                 cfg_f32 {fmt::oihw, fmt::OIhw8i8o, {17, 23, 2, 3}},
                 cfg_f32 {fmt::OIhw8i8o, fmt::oihw, {17, 23, 2, 3}},
+                cfg_f32 {fmt::hwio, fmt::OIhw8i8o, {17, 23, 2, 3}},
+                cfg_f32 {fmt::OIhw8i8o, fmt::hwio, {17, 23, 2, 3}},
+                // OIhw8o8i
                 cfg_f32 {fmt::oihw, fmt::OIhw8o8i, {17, 23, 2, 3}},
                 cfg_f32 {fmt::OIhw8o8i, fmt::oihw, {17, 23, 2, 3}},
-                cfg_f32 {fmt::hwio, fmt::OIhw8i8o, {17, 23, 2, 3}},
-                cfg_f32 {fmt::OIhw8i8o, fmt::hwio, {17, 23, 2, 3}}));
+                // OhwI8i8o
+                cfg_f32 {fmt::oihw, fmt::OhwI8i8o, {17, 23, 2, 3}},
+                cfg_f32 {fmt::OhwI8i8o, fmt::oihw, {17, 23, 2, 3}},
+                cfg_f32 {fmt::hwio, fmt::OhwI8i8o, {17, 23, 2, 3}},
+                cfg_f32 {fmt::OhwI8i8o, fmt::hwio, {17, 23, 2, 3}}));
 
 CPU_INSTANTIATE_TEST_SUITE_P(Weights_3d, reorder_simple_test_f32_f32,
         ::testing::Values(
                 cfg_f32 {fmt::oidhw, fmt::OIdhw8i8o, {16, 24, 2, 3, 3}},
                 cfg_f32 {fmt::OIdhw8i8o, fmt::oidhw, {16, 24, 2, 3, 3}},
+                cfg_f32 {fmt::oidhw, fmt::OdhwI8i8o, {16, 24, 2, 3, 3}},
+                cfg_f32 {fmt::OdhwI8i8o, fmt::oidhw, {16, 24, 2, 3, 3}},
                 cfg_f32 {fmt::oidhw, fmt::OIdhw8o8i, {16, 24, 2, 3, 3}},
                 cfg_f32 {fmt::OIdhw8o8i, fmt::oidhw, {16, 24, 2, 3, 3}},
                 cfg_f32 {fmt::oidhw, fmt::OIdhw8o4i, {16, 24, 2, 3, 3}},
                 cfg_f32 {fmt::OIdhw8o4i, fmt::oidhw, {16, 24, 2, 3, 3}},
                 cfg_f32 {fmt::dhwio, fmt::OIdhw8i8o, {16, 24, 2, 3, 3}},
                 cfg_f32 {fmt::OIdhw8i8o, fmt::dhwio, {16, 24, 2, 3, 3}},
+                cfg_f32 {fmt::dhwio, fmt::OdhwI8i8o, {16, 24, 2, 3, 3}},
+                cfg_f32 {fmt::OdhwI8i8o, fmt::dhwio, {16, 24, 2, 3, 3}},
                 cfg_f32 {fmt::goidhw, fmt::gOdhwi8o, {2, 16, 24, 2, 2, 3}},
                 cfg_f32 {fmt::gOdhwi8o, fmt::goidhw, {2, 16, 24, 3, 2, 3}},
                 cfg_f32 {fmt::goidhw, fmt::gOIdhw8i8o, {2, 16, 24, 2, 2, 3}},
@@ -173,6 +207,11 @@ CPU_INSTANTIATE_TEST_SUITE_P(Weights_3d, reorder_simple_test_f32_f32,
                 cfg_f32 {fmt::IOdhw16o16i, fmt::dhwio, {64, 48, 2, 3, 4}},
                 cfg_f32 {fmt::OIdhw16i16o, fmt::IOdhw16o16i, {64, 48, 2, 3, 4}},
                 cfg_f32 {fmt::IOdhw16o16i, fmt::OIdhw16i16o, {64, 48, 2, 3, 4}},
+                // OdhwI16i16o
+                cfg_f32 {fmt::oidhw, fmt::OdhwI16i16o, {64, 48, 2, 3, 4}},
+                cfg_f32 {fmt::OdhwI16i16o, fmt::oidhw, {64, 48, 2, 3, 4}},
+                cfg_f32 {fmt::dhwio, fmt::OdhwI16i16o, {64, 48, 2, 3, 4}},
+                cfg_f32 {fmt::OdhwI16i16o, fmt::dhwio, {64, 48, 2, 3, 4}},
                 // gOIdhw16i16o and gIOdhw16o16i
                 cfg_f32 {fmt::goidhw, fmt::gOIdhw16i16o, {2, 64, 96, 2, 3, 4}},
                 cfg_f32 {fmt::gOIdhw16i16o, fmt::goidhw, {2, 64, 96, 2, 3, 4}},
@@ -217,6 +256,10 @@ CPU_INSTANTIATE_TEST_SUITE_P(Weights_0, reorder_simple_test_f32_f32,
                 cfg_f32 {fmt::OIhw8i8o, fmt::oihw, {32, 32, 3, 3}},
                 cfg_f32 {fmt::ihwo, fmt::OIhw8i8o, {32, 32, 3, 3}},
                 cfg_f32 {fmt::OIhw8i8o, fmt::ihwo, {32, 32, 3, 3}},
+                cfg_f32 {fmt::oihw, fmt::OhwI8i8o, {32, 32, 3, 3}},
+                cfg_f32 {fmt::OhwI8i8o, fmt::oihw, {32, 32, 3, 3}},
+                cfg_f32 {fmt::ihwo, fmt::OhwI8i8o, {32, 32, 3, 3}},
+                cfg_f32 {fmt::OhwI8i8o, fmt::ihwo, {32, 32, 3, 3}},
                 cfg_f32 {fmt::oihw, fmt::OIhw8o8i, {32, 32, 3, 3}},
                 cfg_f32 {fmt::OIhw8o8i, fmt::oihw, {32, 32, 3, 3}},
                 cfg_f32 {fmt::oihw, fmt::OIhw8o4i, {32, 32, 3, 3}},
@@ -225,6 +268,8 @@ CPU_INSTANTIATE_TEST_SUITE_P(Weights_0, reorder_simple_test_f32_f32,
                 cfg_f32 {fmt::OIhw8o8i, fmt::OIhw8i8o, {32, 32, 3, 3}},
                 cfg_f32 {fmt::hwio, fmt::OIhw8i8o, {32, 32, 3, 3}},
                 cfg_f32 {fmt::OIhw8i8o, fmt::hwio, {32, 32, 3, 3}},
+                cfg_f32 {fmt::hwio, fmt::OhwI8i8o, {32, 32, 3, 3}},
+                cfg_f32 {fmt::OhwI8i8o, fmt::hwio, {32, 32, 3, 3}},
                 cfg_f32 {fmt::goihw, fmt::hwigo, {2, 32, 32, 3, 3}},
                 cfg_f32 {fmt::hwigo, fmt::goihw, {2, 32, 32, 3, 3}},
                 cfg_f32 {fmt::goihw, fmt::gOIhw8i8o, {2, 32, 32, 3, 3}},
@@ -239,10 +284,16 @@ CPU_INSTANTIATE_TEST_SUITE_P(Weights_0, reorder_simple_test_f32_f32,
                 cfg_f32 {fmt::OIhw16i16o, fmt::oihw, {64, 64, 3, 3}},
                 cfg_f32 {fmt::ihwo, fmt::OIhw16i16o, {64, 64, 3, 3}},
                 cfg_f32 {fmt::OIhw16i16o, fmt::ihwo, {64, 64, 3, 3}},
+                cfg_f32 {fmt::oihw, fmt::OhwI16i16o, {64, 64, 3, 3}},
+                cfg_f32 {fmt::OhwI16i16o, fmt::oihw, {64, 64, 3, 3}},
+                cfg_f32 {fmt::ihwo, fmt::OhwI16i16o, {64, 64, 3, 3}},
+                cfg_f32 {fmt::OhwI16i16o, fmt::ihwo, {64, 64, 3, 3}},
                 cfg_f32 {fmt::oihw, fmt::OIhw16o16i, {64, 64, 3, 3}},
                 cfg_f32 {fmt::OIhw16o16i, fmt::oihw, {64, 64, 3, 3}},
                 cfg_f32 {fmt::hwio, fmt::OIhw16i16o, {64, 64, 3, 3}},
                 cfg_f32 {fmt::OIhw16i16o, fmt::hwio, {64, 64, 3, 3}},
+                cfg_f32 {fmt::hwio, fmt::OhwI16i16o, {64, 64, 3, 3}},
+                cfg_f32 {fmt::OhwI16i16o, fmt::hwio, {64, 64, 3, 3}},
                 cfg_f32 {fmt::goihw, fmt::gOIhw16i16o, {2, 64, 64, 3, 3}},
                 cfg_f32 {fmt::gOIhw16i16o, fmt::goihw, {2, 64, 64, 3, 3}},
                 cfg_f32 {fmt::goihw, fmt::gOIhw16o16i, {2, 64, 64, 3, 3}},
@@ -266,6 +317,10 @@ CPU_INSTANTIATE_TEST_SUITE_P(Weights_1, reorder_simple_test_f32_f32,
         ::testing::Values(
                 cfg_f32 {fmt::goihw, fmt::Goihw16g, {32, 32, 32, 3, 3}},
                 cfg_f32 {fmt::Goihw16g, fmt::goihw, {32, 32, 32, 3, 3}},
+                cfg_f32 {fmt::goihw, fmt::hwioG16g, {32, 32, 32, 3, 3}},
+                cfg_f32 {fmt::hwioG16g, fmt::goihw, {32, 32, 32, 3, 3}},
+                cfg_f32 {fmt::goihw, fmt::hwioG8g, {32, 32, 32, 3, 3}},
+                cfg_f32 {fmt::hwioG8g, fmt::goihw, {32, 32, 32, 3, 3}},
                 cfg_f32 {fmt::oihw, fmt::iohw, {32, 32, 3, 3}},
                 cfg_f32 {fmt::iohw, fmt::oihw, {32, 32, 3, 3}},
                 cfg_f32 {fmt::goihw, fmt::giohw, {2, 32, 32, 3, 3}},
@@ -276,6 +331,8 @@ CPU_INSTANTIATE_TEST_SUITE_P(Weights_IOhw16o16i, reorder_simple_test_f32_f32,
                 cfg_f32 {fmt::IOhw16o16i, fmt::oihw, {64, 64, 3, 3}},
                 cfg_f32 {fmt::OIhw16i16o, fmt::IOhw16o16i, {64, 64, 3, 3}},
                 cfg_f32 {fmt::IOhw16o16i, fmt::OIhw16i16o, {64, 64, 3, 3}},
+                cfg_f32 {fmt::OhwI16i16o, fmt::IOhw16o16i, {64, 64, 3, 3}},
+                cfg_f32 {fmt::IOhw16o16i, fmt::OhwI16i16o, {64, 64, 3, 3}},
                 cfg_f32 {fmt::goihw, fmt::gOIhw16o16i, {2, 64, 64, 3, 3}},
                 cfg_f32 {fmt::gIOhw16o16i, fmt::goihw, {2, 64, 64, 3, 3}},
                 cfg_f32 {fmt::gOIhw16i16o, fmt::gIOhw16o16i, {2, 64, 64, 3, 3}},
@@ -289,6 +346,8 @@ CPU_INSTANTIATE_TEST_SUITE_P(Simple, reorder_simple_test_s32_s32,
 CPU_INSTANTIATE_TEST_SUITE_P(Simple, reorder_simple_test_s8_s8,
         ::testing::Values(cfg_s8 {fmt::oihw, fmt::OIhw4i16o4i, {64, 64, 3, 3}},
                 cfg_s8 {fmt::OIhw4i16o4i, fmt::oihw, {64, 64, 3, 3}},
+                cfg_s8 {fmt::oihw, fmt::OhwI4i16o4i, {64, 64, 3, 3}},
+                cfg_s8 {fmt::OhwI4i16o4i, fmt::oihw, {64, 64, 3, 3}},
                 cfg_s8 {fmt::goihw, fmt::gOIhw4i16o4i, {2, 64, 64, 3, 3}},
                 cfg_s8 {fmt::gOIhw4i16o4i, fmt::goihw, {2, 64, 64, 3, 3}}));
 
@@ -322,16 +381,21 @@ GPU_INSTANTIATE_TEST_SUITE_P(Weights, reorder_simple_test_f32_f32,
                 cfg_f32 {fmt::OIhw16o16i, fmt::IOhw16i16o, {32, 48, 2, 3}},
                 cfg_f32 {fmt::gOIhw16o16i, fmt::gIOhw16i16o, {2, 64, 32, 3, 2}},
                 cfg_f32 {fmt::oidhw, fmt::OIdhw16i16o, {64, 32, 3, 9, 5}},
+                cfg_f32 {fmt::oidhw, fmt::OdhwI16i16o, {64, 32, 3, 9, 5}},
                 cfg_f32 {
                         fmt::goidhw, fmt::gOIdhw16i16o, {2, 32, 64, 2, 2, 7}}));
 
 GPU_INSTANTIATE_TEST_SUITE_P(weights_1D, reorder_simple_test_f32_f32,
         ::testing::Values(cfg_f32 {fmt::oiw, fmt::OIw8i16o2i, {32, 48, 7}},
                 cfg_f32 {fmt::OIw8i16o2i, fmt::oiw, {32, 48, 7}},
+                cfg_f32 {fmt::oiw, fmt::OwI8i16o2i, {32, 48, 7}},
+                cfg_f32 {fmt::OwI8i16o2i, fmt::oiw, {32, 48, 7}},
                 cfg_f32 {fmt::goiw, fmt::gOIw8i16o2i, {8, 32, 48, 7}},
                 cfg_f32 {fmt::gOIw8i16o2i, fmt::goiw, {8, 32, 48, 7}},
                 cfg_f32 {fmt::oiw, fmt::OIw16i16o, {32, 48, 7}},
+                cfg_f32 {fmt::oiw, fmt::OwI16i16o, {32, 48, 7}},
                 cfg_f32 {fmt::OIw16i16o, fmt::oiw, {32, 48, 7}},
+                cfg_f32 {fmt::OwI16i16o, fmt::oiw, {32, 48, 7}},
                 cfg_f32 {fmt::oiw, fmt::OIw8o4i, {32, 48, 7}},
                 cfg_f32 {fmt::OIw8o4i, fmt::oiw, {32, 48, 7}},
                 cfg_f32 {fmt::goiw, fmt::gOIw16i16o, {8, 32, 48, 7}},
