@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2024 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -35,13 +35,12 @@ inline void check_reorder(const memory::desc &md_i, const memory::desc &md_o,
     auto src_data = map_memory<data_i_t>(src);
     auto dst_data = map_memory<data_o_t>(dst);
 
-    const auto ndims = md_i.data.ndims;
-    const auto *dims = md_i.data.dims;
+    const auto dims = md_i.get_dims();
     const size_t nelems = std::accumulate(
-            dims, dims + ndims, size_t(1), std::multiplies<size_t>());
+            dims.begin(), dims.end(), size_t(1), std::multiplies<size_t>());
 
-    const dnnl::impl::memory_desc_wrapper mdw_i(md_i.data);
-    const dnnl::impl::memory_desc_wrapper mdw_o(md_o.data);
+    const dnnl::impl::memory_desc_wrapper mdw_i(md_i.get());
+    const dnnl::impl::memory_desc_wrapper mdw_o(md_o.get());
     for (size_t i = 0; i < nelems; ++i) {
         data_i_t s_raw = src_data[mdw_i.off_l(i, false)];
         data_o_t s = static_cast<data_o_t>(s_raw);
@@ -83,9 +82,17 @@ protected:
                              && (supported_format(p.fmt_o)
                                      || supported_blocking(prec_o, p.fmt_o))),
                 "Unsupported cuda format tag/ data type");
+        SKIP_IF_HIP(!((supported_format(p.fmt_i)
+                              || supported_blocking(prec_i, p.fmt_i))
+                            && (supported_format(p.fmt_o)
+                                    || supported_blocking(prec_o, p.fmt_o))),
+                "Unsupported cuda format tag/ data type");
+        SKIP_IF_GENERIC(
+                !(supported_format(p.fmt_i) && supported_format(p.fmt_o)),
+                "Unsupported generic format tag");
 
         catch_expected_failures(
-                [=]() {
+                [&]() {
                     engine eng = get_test_engine();
                     RunTest(eng, eng);
                 },
@@ -127,6 +134,17 @@ protected:
                                 || supported_blocking(prec_o, p.fmt_o))),
                 "Unsupported cuda format tag/ data type");
 #endif
+#ifdef DNNL_SYCL_HIP
+        SKIP_IF(!((supported_format(p.fmt_i)
+                          || supported_blocking(prec_i, p.fmt_i))
+                        && (supported_format(p.fmt_o)
+                                || supported_blocking(prec_o, p.fmt_o))),
+                "Unsupported hip format tag/ data type");
+#endif
+#ifdef DNNL_SYCL_GENERIC
+        SKIP_IF(!(supported_format(p.fmt_i) && supported_format(p.fmt_o)),
+                "Unsupported generic format tag");
+#endif
 
         catch_expected_failures([&]() { RunTest(eng_i, eng_o); },
                 p.expect_to_fail, p.expected_status);
@@ -147,17 +165,6 @@ protected:
         auto md_i = memory::desc(p.dims, prec_i, p.fmt_i);
         auto md_o = memory::desc(p.dims, prec_o, p.fmt_o);
 
-        auto src = test::make_memory(md_i, eng_i);
-        auto dst = test::make_memory(md_o, eng_o);
-
-        /* initialize input data */
-        const dnnl::impl::memory_desc_wrapper mdw_i(md_i.data);
-        {
-            auto src_data = map_memory<data_i_t>(src);
-            for (size_t i = 0; i < nelems; ++i)
-                src_data[mdw_i.off_l(i, false)] = data_i_t(i);
-        }
-
         reorder::primitive_desc r_pd(
                 eng_i, md_i, eng_o, md_o, primitive_attr());
         // test construction from a C pd
@@ -167,7 +174,22 @@ protected:
                 == r_pd.src_desc());
         ASSERT_TRUE(r_pd.query_md(query::exec_arg_md, DNNL_ARG_DST)
                 == r_pd.dst_desc());
+        if (p.fmt_i != memory::format_tag::any) {
+            ASSERT_TRUE(md_i == r_pd.src_desc());
+        }
 
+        auto src = test::make_memory(r_pd.src_desc(), eng_i);
+        auto dst = test::make_memory(r_pd.dst_desc(), eng_o);
+
+        /* initialize input data */
+        const dnnl::impl::memory_desc_wrapper mdw_i(md_i.get());
+        {
+            auto src_data = map_memory<data_i_t>(src);
+            for (size_t i = 0; i < nelems; ++i)
+                src_data[mdw_i.off_l(i, false)] = data_i_t(i);
+        }
+
+        EXPECT_ANY_THROW(reorder(r_pd, {}));
         auto r = reorder(r_pd);
         auto strm = make_stream(r_pd.get_engine());
         r.execute(strm, src, dst);
